@@ -142,16 +142,28 @@ REST_API_VERSION_HISTORY = """
     * 3.61 - Add ``cluster_name`` attribute to response body of volume details
              for admin.
     * 3.62 - Default volume type overrides
+    * 3.63 - Include volume type ID in the volume details JSON response. Before
+             this microversion (MV), Cinder returns only the volume type name
+             in the volume details. This MV affects the volume detail list
+             ("GET /v3/{project_id}/volumes/detail") and volume-show
+             ("GET /v3/{project_id}/volumes/{volume_id}") calls.
+    * 3.64 - Include 'encryption_key_id' in volume and backup details
+    * 3.65 - Include 'consumes_quota' in volume and snapshot details
+           - Accept 'consumes_quota' filter in volume and snapshot list
+             operation.
+    * 3.66 - Allow snapshotting in-use volumes without force flag.
+    * 3.67 - API URLs no longer need to include a project_id parameter.
+    * 3.68 - Support re-image volume
+    * 3.69 - Allow null value for shared_targets
+    * 3.70 - Support encrypted volume transfers
 """
 
 # The minimum and maximum versions of the API supported
 # The default api version request is defined to be the
 # minimum version of the API supported.
-# Explicitly using /v2 endpoints will still work
 _MIN_API_VERSION = "3.0"
-_MAX_API_VERSION = "3.62"
-_LEGACY_API_VERSION2 = "2.0"
-UPDATED = "2020-10-14T00:00:00Z"
+_MAX_API_VERSION = "3.70"
+UPDATED = "2022-08-31T00:00:00Z"
 
 
 # NOTE(cyeoh): min and max versions declared as functions so we can
@@ -165,10 +177,6 @@ def max_api_version():
     return APIVersionRequest(_MAX_API_VERSION)
 
 
-def legacy_api_version2():
-    return APIVersionRequest(_LEGACY_API_VERSION2)
-
-
 class APIVersionRequest(utils.ComparableMixin):
     """This class represents an API Version Request.
 
