@@ -43,27 +43,29 @@
                              certificate for SSL connections (default: False)
 """
 
-import hashlib
+import io
 import socket
 
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import secretutils
 from oslo_utils import timeutils
-import six
 from swiftclient import client as swift
+from swiftclient import exceptions as swift_exc
 
 from cinder.backup import chunkeddriver
 from cinder import exception
 from cinder.i18n import _
-from cinder.i18n import _LE
 from cinder import interface
+from cinder import service_auth
+from cinder.utils import retry
 
 LOG = logging.getLogger(__name__)
 
 swiftbackup_service_opts = [
-    cfg.StrOpt('backup_swift_url',
+    cfg.URIOpt('backup_swift_url',
                help='The URL of the Swift endpoint'),
-    cfg.StrOpt('backup_swift_auth_url',
+    cfg.URIOpt('backup_swift_auth_url',
                help='The URL of the Keystone endpoint'),
     cfg.StrOpt('swift_catalog_info',
                default='object-store:swift:publicURL',
@@ -79,7 +81,9 @@ swiftbackup_service_opts = [
                'Only used if backup_swift_auth_url is unset'),
     cfg.StrOpt('backup_swift_auth',
                default='per_user',
-               help='Swift authentication mechanism'),
+               choices=['per_user', 'single_user'],
+               help='Swift authentication mechanism (per_user or '
+               'single_user).'),
     cfg.StrOpt('backup_swift_auth_version',
                default='1',
                help='Swift authentication version. Specify "1" for auth 1.0'
@@ -102,10 +106,16 @@ swiftbackup_service_opts = [
     cfg.StrOpt('backup_swift_user',
                help='Swift user name'),
     cfg.StrOpt('backup_swift_key',
+               secret=True,
                help='Swift key for authentication'),
     cfg.StrOpt('backup_swift_container',
                default='volumebackups',
                help='The default Swift container to use'),
+    cfg.StrOpt('backup_swift_create_storage_policy',
+               default=None,
+               help='The storage policy to use when creating the Swift '
+                    'container. If the container already exists the '
+                    'storage policy cannot be enforced'),
     cfg.IntOpt('backup_swift_object_size',
                default=52428800,
                help='The size in bytes of Swift backup objects'),
@@ -133,6 +143,11 @@ swiftbackup_service_opts = [
                 default=False,
                 help='Bypass verification of server certificate when '
                      'making SSL connection to Swift.'),
+    cfg.BoolOpt('backup_swift_service_auth',
+                default=False,
+                help='Send a X-Service-Token header with service auth '
+                     'credentials. If enabled you also must set the '
+                     'service_user group and enable send_service_user_token.'),
 ]
 
 CONF = cfg.CONF
@@ -143,81 +158,84 @@ CONF.register_opts(swiftbackup_service_opts)
 class SwiftBackupDriver(chunkeddriver.ChunkedBackupDriver):
     """Provides backup, restore and delete of backup objects within Swift."""
 
-    def __init__(self, context, db_driver=None):
+    def __init__(self, context):
         chunk_size_bytes = CONF.backup_swift_object_size
         sha_block_size_bytes = CONF.backup_swift_block_size
         backup_default_container = CONF.backup_swift_container
         enable_progress_timer = CONF.backup_swift_enable_progress_timer
-        super(SwiftBackupDriver, self).__init__(context, chunk_size_bytes,
-                                                sha_block_size_bytes,
-                                                backup_default_container,
-                                                enable_progress_timer,
-                                                db_driver)
-        if CONF.backup_swift_url is None:
-            self.swift_url = None
-            info = CONF.swift_catalog_info
-            try:
-                service_type, service_name, endpoint_type = info.split(':')
-            except ValueError:
-                raise exception.BackupDriverException(_(
-                    "Failed to parse the configuration option "
-                    "'swift_catalog_info', must be in the form "
-                    "<service_type>:<service_name>:<endpoint_type>"))
-            for entry in context.service_catalog:
-                if entry.get('type') == service_type:
-                    # It is assumed that service_types are unique within
-                    # the service catalog, so once the correct one is found
-                    # it is safe to break out of the loop
-                    self.swift_url = entry.get(
-                        'endpoints')[0].get(endpoint_type)
-                    break
-        else:
-            self.swift_url = '%s%s' % (CONF.backup_swift_url,
-                                       context.project_id)
-        if self.swift_url is None:
-            raise exception.BackupDriverException(_(
-                "Could not determine which Swift endpoint to use. This can "
-                "either be set in the service catalog or with the "
-                "cinder.conf config option 'backup_swift_url'."))
-        if CONF.backup_swift_auth_url is None:
-            self.auth_url = None
-            info = CONF.keystone_catalog_info
-            try:
-                service_type, service_name, endpoint_type = info.split(':')
-            except ValueError:
-                raise exception.BackupDriverException(_(
-                    "Failed to parse the configuration option "
-                    "'keystone_catalog_info', must be in the form "
-                    "<service_type>:<service_name>:<endpoint_type>"))
-            for entry in context.service_catalog:
-                if entry.get('type') == service_type:
-                    # It is assumed that service_types are unique within
-                    # the service catalog, so once the correct one is found
-                    # it is safe to break out of the loop
-                    self.auth_url = entry.get(
-                        'endpoints')[0].get(endpoint_type)
-                    break
-        else:
-            self.auth_url = CONF.backup_swift_auth_url
+        super().__init__(
+            context,
+            chunk_size_bytes,
+            sha_block_size_bytes,
+            backup_default_container,
+            enable_progress_timer,
+        )
 
-        if self.auth_url is None:
-            raise exception.BackupDriverException(_(
-                "Could not determine which Keystone endpoint to use. This can "
-                "either be set in the service catalog or with the "
-                "cinder.conf config option 'backup_swift_auth_url'."))
-        LOG.debug("Using swift URL %s", self.swift_url)
-        LOG.debug("Using auth URL %s", self.auth_url)
+        # Do not intialize the instance created when the backup service
+        # starts up. The context will be missing information to do things
+        # like fetching endpoints from the service catalog.
+        if context and context.user_id:
+            self.initialize()
+
+    @staticmethod
+    def get_driver_options():
+        return swiftbackup_service_opts
+
+    @retry(Exception, retries=CONF.backup_swift_retry_attempts,
+           backoff_rate=CONF.backup_swift_retry_backoff)
+    def _headers(self, headers=None):
+        """Add service token to headers if its enabled"""
+        if not CONF.backup_swift_service_auth:
+            return headers
+
+        result = headers or {}
+
+        sa_plugin = service_auth.get_service_auth_plugin()
+        if sa_plugin is not None:
+            result['X-Service-Token'] = sa_plugin.get_token()
+
+        return result
+
+    def initialize(self):
         self.swift_attempts = CONF.backup_swift_retry_attempts
         self.swift_backoff = CONF.backup_swift_retry_backoff
-        LOG.debug('Connect to %s in "%s" mode', CONF.backup_swift_url,
-                  CONF.backup_swift_auth)
         self.backup_swift_auth_insecure = CONF.backup_swift_auth_insecure
+
         if CONF.backup_swift_auth == 'single_user':
             if CONF.backup_swift_user is None:
-                LOG.error(_LE("single_user auth mode enabled, "
-                              "but %(param)s not set"),
+                LOG.error("single_user auth mode enabled, "
+                          "but %(param)s not set",
                           {'param': 'backup_swift_user'})
                 raise exception.ParameterNotFound(param='backup_swift_user')
+            if CONF.backup_swift_auth_url is None:
+                self.auth_url = None
+                info = CONF.keystone_catalog_info
+                try:
+                    service_type, service_name, endpoint_type = info.split(':')
+                except ValueError:
+                    raise exception.BackupDriverException(_(
+                        "Failed to parse the configuration option "
+                        "'keystone_catalog_info', must be in the form "
+                        "<service_type>:<service_name>:<endpoint_type>"))
+                for entry in self.context.service_catalog:
+                    if entry.get('type') == service_type:
+                        # It is assumed that service_types are unique within
+                        # the service catalog, so once the correct one is found
+                        # it is safe to break out of the loop
+                        self.auth_url = entry.get(
+                            'endpoints')[0].get(endpoint_type)
+                        break
+            else:
+                self.auth_url = CONF.backup_swift_auth_url
+            if self.auth_url is None:
+                raise exception.BackupDriverException(_(
+                    "Could not determine which Keystone endpoint to use. This "
+                    "can either be set in the service catalog or with the "
+                    "cinder.conf config option 'backup_swift_auth_url'."))
+            LOG.debug("Using auth URL %s", self.auth_url)
+            LOG.debug('Connect to %s in "%s" mode', CONF.backup_swift_auth_url,
+                      CONF.backup_swift_auth)
+
             os_options = {}
             if CONF.backup_swift_user_domain is not None:
                 os_options['user_domain_name'] = CONF.backup_swift_user_domain
@@ -239,20 +257,51 @@ class SwiftBackupDriver(chunkeddriver.ChunkedBackupDriver):
                 insecure=self.backup_swift_auth_insecure,
                 cacert=CONF.backup_swift_ca_cert_file)
         else:
+            if CONF.backup_swift_url is None:
+                self.swift_url = None
+                info = CONF.swift_catalog_info
+                try:
+                    service_type, service_name, endpoint_type = info.split(':')
+                except ValueError:
+                    raise exception.BackupDriverException(_(
+                        "Failed to parse the configuration option "
+                        "'swift_catalog_info', must be in the form "
+                        "<service_type>:<service_name>:<endpoint_type>"))
+                for entry in self.context.service_catalog:
+                    if entry.get('type') == service_type:
+                        # It is assumed that service_types are unique within
+                        # the service catalog, so once the correct one is found
+                        # it is safe to break out of the loop
+                        self.swift_url = entry.get(
+                            'endpoints')[0].get(endpoint_type)
+                        break
+            else:
+                self.swift_url = '%s%s' % (CONF.backup_swift_url,
+                                           self.context.project_id)
+            if self.swift_url is None:
+                raise exception.BackupDriverException(_(
+                    "Could not determine which Swift endpoint to use. This "
+                    "can either be set in the service catalog or with the "
+                    "cinder.conf config option 'backup_swift_url'."))
+            LOG.debug("Using swift URL %s", self.swift_url)
+            LOG.debug('Connect to %s in "%s" mode', CONF.backup_swift_url,
+                      CONF.backup_swift_auth)
+
             self.conn = swift.Connection(retries=self.swift_attempts,
                                          preauthurl=self.swift_url,
                                          preauthtoken=self.context.auth_token,
                                          starting_backoff=self.swift_backoff,
-                                         insecure= (
+                                         insecure=(
                                              self.backup_swift_auth_insecure),
                                          cacert=CONF.backup_swift_ca_cert_file)
 
     class SwiftObjectWriter(object):
-        def __init__(self, container, object_name, conn):
+        def __init__(self, container, object_name, conn, headers_func=None):
             self.container = container
             self.object_name = object_name
             self.conn = conn
             self.data = bytearray()
+            self.headers_func = headers_func
 
         def __enter__(self):
             return self
@@ -264,30 +313,29 @@ class SwiftBackupDriver(chunkeddriver.ChunkedBackupDriver):
             self.data += data
 
         def close(self):
-            reader = six.BytesIO(self.data)
+            reader = io.BytesIO(self.data)
             try:
+                headers = self.headers_func() if self.headers_func else None
                 etag = self.conn.put_object(self.container, self.object_name,
                                             reader,
-                                            content_length=len(self.data))
+                                            content_length=len(self.data),
+                                            headers=headers)
             except socket.error as err:
                 raise exception.SwiftConnectionFailed(reason=err)
-            LOG.debug('swift MD5 for %(object_name)s: %(etag)s',
-                      {'object_name': self.object_name, 'etag': etag, })
-            md5 = hashlib.md5(self.data).hexdigest()
-            LOG.debug('backup MD5 for %(object_name)s: %(md5)s',
-                      {'object_name': self.object_name, 'md5': md5})
+            md5 = secretutils.md5(self.data, usedforsecurity=False).hexdigest()
             if etag != md5:
                 err = _('error writing object to swift, MD5 of object in '
                         'swift %(etag)s is not the same as MD5 of object sent '
-                        'to swift %(md5)s'), {'etag': etag, 'md5': md5}
+                        'to swift %(md5)s') % {'etag': etag, 'md5': md5}
                 raise exception.InvalidBackup(reason=err)
             return md5
 
     class SwiftObjectReader(object):
-        def __init__(self, container, object_name, conn):
+        def __init__(self, container, object_name, conn, headers_func=None):
             self.container = container
             self.object_name = object_name
             self.conn = conn
+            self.headers_func = headers_func
 
         def __enter__(self):
             return self
@@ -297,26 +345,50 @@ class SwiftBackupDriver(chunkeddriver.ChunkedBackupDriver):
 
         def read(self):
             try:
+                headers = self.headers_func() if self.headers_func else None
                 (_resp, body) = self.conn.get_object(self.container,
-                                                     self.object_name)
+                                                     self.object_name,
+                                                     headers=headers)
             except socket.error as err:
                 raise exception.SwiftConnectionFailed(reason=err)
             return body
 
     def put_container(self, container):
-        """Create the container if needed. No failure if it pre-exists."""
+        """Create the container if needed.
+
+        Check if the container exist by issuing a HEAD request, if
+        the container does not exist we create it.
+
+        We cannot enforce a new storage policy on an
+        existing container.
+        """
         try:
-            self.conn.put_container(container)
+            self.conn.head_container(container, headers=self._headers())
+        except swift_exc.ClientException as e:
+            if e.http_status == 404:
+                try:
+                    storage_policy = CONF.backup_swift_create_storage_policy
+                    headers = ({'X-Storage-Policy': storage_policy}
+                               if storage_policy else None)
+                    self.conn.put_container(container,
+                                            headers=self._headers(headers))
+                except socket.error as err:
+                    raise exception.SwiftConnectionFailed(reason=err)
+                return
+            LOG.warning("Failed to HEAD container to determine if it "
+                        "exists and should be created.")
+            raise exception.SwiftConnectionFailed(reason=e)
         except socket.error as err:
             raise exception.SwiftConnectionFailed(reason=err)
-        return
 
     def get_container_entries(self, container, prefix):
         """Get container entry names"""
         try:
+            headers = self._headers()
             swift_objects = self.conn.get_container(container,
                                                     prefix=prefix,
-                                                    full_listing=True)[1]
+                                                    full_listing=True,
+                                                    headers=headers)[1]
         except socket.error as err:
             raise exception.SwiftConnectionFailed(reason=err)
         swift_object_names = [swift_obj['name'] for swift_obj in swift_objects]
@@ -328,7 +400,8 @@ class SwiftBackupDriver(chunkeddriver.ChunkedBackupDriver):
         Returns a writer object that stores a chunk of volume data in a
         Swift object store.
         """
-        return self.SwiftObjectWriter(container, object_name, self.conn)
+        return self.SwiftObjectWriter(container, object_name, self.conn,
+                                      self._headers)
 
     def get_object_reader(self, container, object_name, extra_metadata=None):
         """Return reader object.
@@ -336,12 +409,14 @@ class SwiftBackupDriver(chunkeddriver.ChunkedBackupDriver):
         Returns a reader object that retrieves a chunk of backed-up volume data
         from a Swift object store.
         """
-        return self.SwiftObjectReader(container, object_name, self.conn)
+        return self.SwiftObjectReader(container, object_name, self.conn,
+                                      self._headers)
 
     def delete_object(self, container, object_name):
         """Deletes a backup object from a Swift object store."""
         try:
-            self.conn.delete_object(container, object_name)
+            self.conn.delete_object(container, object_name,
+                                    headers=self._headers())
         except socket.error as err:
             raise exception.SwiftConnectionFailed(reason=err)
 
@@ -363,6 +438,22 @@ class SwiftBackupDriver(chunkeddriver.ChunkedBackupDriver):
         """Swift driver does not use any extra metadata."""
         return None
 
-
-def get_backup_driver(context):
-    return SwiftBackupDriver(context)
+    def check_for_setup_error(self):
+        # Here we are trying to connect to swift backend service
+        # without any additional parameters.
+        # At the moment of execution we don't have any user data
+        # After just trying to do easiest operations, that will show
+        # that we've configured swift backup driver in right way
+        if not CONF.backup_swift_url:
+            LOG.warning("We will use endpoints from keystone. It is "
+                        "possible we could have problems because of it.")
+            return
+        conn = swift.Connection(retries=CONF.backup_swift_retry_attempts,
+                                preauthurl=CONF.backup_swift_url)
+        try:
+            conn.get_capabilities()
+            # TODO(e0ne) catch less general exception
+        except Exception:
+            LOG.exception("Can not get Swift capabilities during backup "
+                          "driver initialization.")
+            raise
