@@ -416,10 +416,15 @@ class BackupManager(manager.SchedulerDependentManager):
                     snapshot.status = fields.SnapshotStatus.AVAILABLE
                     snapshot.save()
                 else:
-                    self.db.volume_update(
-                        context, volume_id,
-                        {'status': previous_status,
-                         'previous_status': 'error_backing-up'})
+                    try:
+                        self.db.volume_update(
+                            context, volume_id,
+                            {'status': previous_status,
+                             'previous_status': 'error_backing-up'})
+                    except exception.VolumeNotFound:
+                        # If the volume was deleted we cannot update its
+                        # status but we still want to set the backup to error.
+                        pass
                 volume_utils.update_backup_error(backup, str(err))
 
     def _start_backup(self, context, backup, volume):
@@ -456,7 +461,8 @@ class BackupManager(manager.SchedulerDependentManager):
         previous_status = volume.get('previous_status', None)
 
         backup_service = self.service(context)
-        properties = volume_utils.brick_get_connector_properties()
+        properties = volume_utils.brick_get_connector_properties(
+            CONF.use_multipath_for_image_xfer, enforce_multipath=False)
 
         updates = {}
         try:
@@ -724,7 +730,8 @@ class BackupManager(manager.SchedulerDependentManager):
         orig_key_id = volume.encryption_key_id
         backup_service = self.service(context)
 
-        properties = volume_utils.brick_get_connector_properties()
+        properties = volume_utils.brick_get_connector_properties(
+            CONF.use_multipath_for_image_xfer, enforce_multipath=False)
         secure_enabled = (
             self.volume_rpcapi.secure_file_operations_enabled(context,
                                                               volume))
