@@ -30,10 +30,11 @@ eventlet.monkey_patch()
 # Monkey patch the original current_thread to use the up-to-date _active
 # global variable. See https://bugs.launchpad.net/bugs/1863021 and
 # https://github.com/eventlet/eventlet/issues/592
-import __original_module_threading as orig_threading
+import __original_module_threading as orig_threading  # pylint: disable=E0401
 import threading # noqa
 orig_threading.current_thread.__globals__['_active'] = threading._active
 
+import os_brick
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -69,11 +70,12 @@ CONF.register_opts(backup_cmd_opts)
 
 LOG = None
 
-# NOTE(mriedem): The default backup driver uses swift and performs read/write
+# NOTE: The default backup driver uses swift and performs read/write
 # operations in a thread. swiftclient will log requests and responses at DEBUG
 # level, which can cause a thread switch and break the backup operation. So we
-# set a default log level of WARN for swiftclient to try and avoid this issue.
-_EXTRA_DEFAULT_LOG_LEVELS = ['swiftclient=WARN']
+# set a default log level of WARN for swiftclient and boto to try and avoid
+# this issue.
+_EXTRA_DEFAULT_LOG_LEVELS = ['swiftclient=WARN', 'botocore=WARN']
 
 
 def _launch_backup_process(launcher, num_process, _semaphore):
@@ -81,7 +83,7 @@ def _launch_backup_process(launcher, num_process, _semaphore):
         server = service.Service.create(binary='cinder-backup',
                                         coordination=True,
                                         service_name='backup',
-                                        process_number=num_process,
+                                        process_number=num_process + 1,
                                         semaphore=_semaphore)
     except Exception:
         LOG.exception('Backup service %s failed to start.', CONF.host)
@@ -107,15 +109,26 @@ def main():
     priv_context.init(root_helper=shlex.split(utils.get_root_helper()))
     utils.monkey_patch()
     gmr.TextGuruMeditation.setup_autorun(version, conf=CONF)
+    os_brick.setup(CONF)
     global LOG
     LOG = logging.getLogger(__name__)
     semaphore = utils.semaphore_factory(CONF.backup_max_operations,
                                         CONF.backup_workers)
 
-    LOG.info('Backup running with %s processes.', CONF.backup_workers)
-    launcher = service.get_launcher()
+    if CONF.backup_workers > 1:
+        LOG.info('Backup running with %s processes.', CONF.backup_workers)
+        launcher = service.get_launcher()
 
-    for i in range(1, CONF.backup_workers + 1):
-        _launch_backup_process(launcher, i, semaphore)
+        for i in range(CONF.backup_workers):
+            _launch_backup_process(launcher, i, semaphore)
 
-    launcher.wait()
+        launcher.wait()
+    else:
+        LOG.info('Backup running in single process mode.')
+        server = service.Service.create(binary='cinder-backup',
+                                        coordination=True,
+                                        service_name='backup',
+                                        process_number=1,
+                                        semaphore=semaphore)
+        service.serve(server)
+        service.wait()
