@@ -50,18 +50,19 @@
 
 """CLI interface for cinder management."""
 
-try:
-    import collections.abc as collections
-except ImportError:
-    import collections
-
+import collections
+import collections.abc as collections_abc
+import errno
+import glob
+import itertools
 import logging as python_logging
+import os
+import re
 import sys
 import time
 
 from oslo_config import cfg
 from oslo_db import exception as db_exc
-from oslo_db.sqlalchemy import migration
 from oslo_log import log as logging
 from oslo_utils import timeutils
 import tabulate
@@ -69,7 +70,6 @@ import tabulate
 # Need to register global_opts
 from cinder.backup import rpcapi as backup_rpcapi
 from cinder.common import config  # noqa
-from cinder.common import constants
 from cinder import context
 from cinder import db
 from cinder.db import migration as db_migration
@@ -79,6 +79,7 @@ from cinder import exception
 from cinder.i18n import _
 from cinder import objects
 from cinder.objects import base as ovo_base
+from cinder import quota
 from cinder import rpc
 from cinder.scheduler import rpcapi as scheduler_rpcapi
 from cinder import version
@@ -99,43 +100,19 @@ RPC_VERSIONS = {
 OVO_VERSION = ovo_base.OBJ_VERSIONS.get_current()
 
 
-def _get_non_shared_target_hosts(ctxt):
-    hosts = []
-    numvols_needing_update = 0
-    rpc.init(CONF)
-    rpcapi = volume_rpcapi.VolumeAPI()
-
-    services = objects.ServiceList.get_all_by_topic(ctxt,
-                                                    constants.VOLUME_TOPIC)
-    for service in services:
-        capabilities = rpcapi.get_capabilities(ctxt, service.host, True)
-        # Select only non iSCSI connections and iSCSI that are explicit
-        if (capabilities.get('storage_protocol') != 'iSCSI' or
-                not capabilities.get('shared_targets', True)):
-            hosts.append(service.host)
-            numvols_needing_update += db_api.model_query(
-                ctxt, models.Volume).filter_by(
-                    shared_targets=True,
-                    service_uuid=service.uuid).count()
-    return hosts, numvols_needing_update
-
-
 # Decorators for actions
 def args(*args, **kwargs):
+    args = list(args)
+    if not args[0].startswith('-') and '-' in args[0]:
+        kwargs.setdefault('metavar', args[0])
+        args[0] = args[0].replace('-', '_')
+
     def _decorator(func):
         func.__dict__.setdefault('args', []).insert(0, (args, kwargs))
         return func
     return _decorator
 
 
-def _db_error(caught_exception):
-    print('%s' % caught_exception)
-    print(_("The above error may show that the database has not "
-            "been created.\nPlease create a database using "
-            "'cinder-manage db sync' before running this command."))
-    sys.exit(1)
-
-
 class HostCommands(object):
     """List hosts."""
 
@@ -169,13 +146,21 @@ class DbCommands(object):
     # NOTE: Online migrations cannot depend on having Cinder services running.
     # Migrations can be called during Fast-Forward Upgrades without having any
     # Cinder services up.
-    # NOTE; Online migrations must be removed at the beginning of the next
+    # NOTE: Online migrations must be removed at the beginning of the next
     # release to the one they've been introduced.  A comment with the release
     # a migration is introduced and the one where it must be removed must
     # preceed any element of the "online_migrations" tuple, like this:
     #    # Added in Queens remove in Rocky
     #    db.service_uuids_online_data_migration,
-    online_migrations = tuple()
+    online_migrations = (
+        # TODO: (Z Release) Remove next line and this comment
+        # TODO: (Y Release) Uncomment next line and remove this comment
+        # db.remove_temporary_admin_metadata_data_migration,
+
+        # TODO: (Y Release) Remove next 2 line and this comment
+        db.volume_use_quota_online_data_migration,
+        db.snapshot_use_quota_online_data_migration,
+    )
 
     def __init__(self):
         pass
@@ -194,7 +179,7 @@ class DbCommands(object):
                     '%(max_version)d.') % {'max_version': db.MAX_INT})
             sys.exit(1)
         try:
-            result = db_migration.db_sync(version)
+            db_migration.db_sync(version)
         except db_exc.DBMigrationError as ex:
             print("Error during database migration: %s" % ex)
             sys.exit(1)
@@ -214,13 +199,9 @@ class DbCommands(object):
             print(_('Error during service version bump: %s') % ex)
             sys.exit(2)
 
-        return result
-
     def version(self):
         """Print the current database version."""
-        print(migration.db_version(db_api.get_engine(),
-                                   db_migration.MIGRATE_REPO_PATH,
-                                   db_migration.INIT_VERSION))
+        print(db_migration.db_version())
 
     @args('age_in_days', type=int,
           help='Purge deleted rows older than age in days')
@@ -346,6 +327,240 @@ class DbCommands(object):
             sys.exit(1)
 
 
+class QuotaCommands(object):
+    """Class for managing quota issues."""
+
+    def __init__(self):
+        pass
+
+    @args('--project-id', default=None,
+          help=('The ID of the project where we want to sync the quotas '
+                '(defaults to all projects).'))
+    def check(self, project_id):
+        """Check if quotas and reservations are correct
+
+        This action checks quotas and reservations, for a specific project or
+        for all projects, to see if they are out of sync.
+
+        The check will also look for duplicated entries.
+
+        One way to use this check in combination with the sync action is to
+        run the check for all projects, take note of those that are out of
+        sync, and then sync them one by one at intervals to reduce stress on
+        the DB.
+        """
+        result = self._check_sync(project_id, do_fix=False)
+        if result:
+            sys.exit(1)
+
+    @args('--project-id', default=None,
+          help=('The ID of the project where we want to sync the quotas '
+                '(defaults to all projects).'))
+    def sync(self, project_id):
+        """Fix quotas and reservations
+
+        This action refreshes existing quota usage and reservation count for a
+        specific project or for all projects.
+
+        The refresh will also remove duplicated entries.
+
+        This operation is best executed when Cinder is not running, but it can
+        be run with cinder services running as well.
+
+        A different transaction is used for each project's quota sync, so an
+        action failure will only rollback the current project's changes.
+        """
+        self._check_sync(project_id, do_fix=True)
+
+    @db_api.main_context_manager.reader
+    def _get_quota_projects(self, context, project_id):
+        """Get project ids that have quota_usage entries."""
+        if project_id:
+            model = models.QuotaUsage
+            # If the project does not exist
+            if not context.session.query(
+                db_api.sql.exists()
+                .where(
+                    db_api.and_(
+                        model.project_id == project_id,
+                        ~model.deleted,
+                    ),
+                )
+            ).scalar():
+                print(
+                    'Project id %s has no quota usage. Nothing to do.' %
+                    project_id,
+                )
+                return []
+            return [project_id]
+
+        projects = db_api.model_query(
+            context,
+            models.QuotaUsage,
+            read_deleted="no"
+        ).with_entities('project_id').distinct().all()
+        project_ids = [row.project_id for row in projects]
+        return project_ids
+
+    def _get_usages(self, context, resources, project_id):
+        """Get data necessary to check out of sync quota usage.
+
+        Returns a list QuotaUsage instances for the specific project
+        """
+        usages = db_api.model_query(
+            context,
+            db_api.models.QuotaUsage,
+            read_deleted="no",
+        ).filter_by(project_id=project_id).with_for_update().all()
+        return usages
+
+    def _get_reservations(self, context, project_id, usage_id):
+        """Get reservations for a given project and usage id."""
+        reservations = (
+            db_api.model_query(
+                context,
+                models.Reservation,
+                read_deleted="no",
+            )
+            .filter_by(project_id=project_id, usage_id=usage_id)
+            .with_for_update()
+            .all()
+        )
+        return reservations
+
+    def _check_duplicates(self, context, usages, do_fix):
+        """Look for duplicated quota used entries (bug#1484343)
+
+        If we have duplicates and we are fixing them, then we reassign the
+        reservations of the usage we are removing.
+        """
+        resources = collections.defaultdict(list)
+        for usage in usages:
+            resources[usage.resource].append(usage)
+
+        duplicates_found = False
+        result = []
+        for resource_usages in resources.values():
+            keep_usage = resource_usages[0]
+            if len(resource_usages) > 1:
+                duplicates_found = True
+                print('\t%s: %s duplicated usage entries - ' %
+                      (keep_usage.resource, len(resource_usages) - 1),
+                      end='')
+
+                if do_fix:
+                    # Each of the duplicates can have reservations
+                    reassigned = 0
+                    for usage in resource_usages[1:]:
+                        reservations = self._get_reservations(
+                            context,
+                            usage.project_id,
+                            usage.id,
+                        )
+                        reassigned += len(reservations)
+                        for reservation in reservations:
+                            reservation.usage_id = keep_usage.id
+                        keep_usage.in_use += usage.in_use
+                        keep_usage.reserved += usage.reserved
+                        usage.delete(context.session)
+                    print('duplicates removed & %s reservations reassigned' %
+                          reassigned)
+                else:
+                    print('ignored')
+            result.append(keep_usage)
+        return result, duplicates_found
+
+    def _check_sync(self, project_id, do_fix):
+        """Check the quotas and reservations optionally fixing them."""
+
+        ctxt = context.get_admin_context()
+        # Get the quota usage types and their sync methods
+        resources = quota.QUOTAS.resources
+
+        # Get all project ids that have quota usage. Method doesn't lock
+        # projects, since newly added projects should not be out of sync and
+        # projects removed will just turn nothing on the quota usage.
+        projects = self._get_quota_projects(ctxt, project_id)
+
+        discrepancy = False
+
+        for project in projects:
+            discrepancy &= self._check_project_sync(
+                ctxt,
+                project,
+                do_fix,
+                resources,
+            )
+
+        print('Action successfully completed')
+        return discrepancy
+
+    @db_api.main_context_manager.reader
+    def _check_project_sync(self, context, project, do_fix, resources):
+        print('Processing quota usage for project %s' % project)
+
+        discrepancy = False
+        action_msg = ' - fixed' if do_fix else ''
+
+        # NOTE: It's important to always get the quota first and then the
+        # reservations to prevent deadlocks with quota commit and rollback from
+        # running Cinder services.
+
+        # We only want to sync existing quota usage rows
+        usages = self._get_usages(context, resources, project)
+
+        # Check for duplicated entries (bug#1484343)
+        usages, duplicates_found = self._check_duplicates(
+            context, usages, do_fix,
+        )
+        if duplicates_found:
+            discrepancy = True
+
+        # Check quota and reservations
+        for usage in usages:
+            resource_name = usage.resource
+            # Get the correct value for this quota usage resource
+            updates = db_api._get_sync_updates(
+                context,
+                project,
+                resources,
+                resource_name,
+            )
+            in_use = updates[resource_name]
+            if in_use != usage.in_use:
+                print(
+                    '\t%s: invalid usage saved=%s actual=%s%s' %
+                    (resource_name, usage.in_use, in_use, action_msg)
+                )
+                discrepancy = True
+                if do_fix:
+                    usage.in_use = in_use
+
+            reservations = self._get_reservations(
+                context,
+                project,
+                usage.id,
+            )
+            num_reservations = sum(
+                r.delta for r in reservations if r.delta > 0
+            )
+            if num_reservations != usage.reserved:
+                print(
+                    '\t%s: invalid reserved saved=%s actual=%s%s' %
+                    (
+                        resource_name,
+                        usage.reserved,
+                        num_reservations,
+                        action_msg,
+                    )
+                )
+                discrepancy = True
+                if do_fix:
+                    usage.reserved = num_reservations
+
+        return discrepancy
+
+
 class VersionCommands(object):
     """Class for exposing the codebase version."""
 
@@ -659,6 +874,116 @@ class ConsistencyGroupCommands(object):
             gr.save()
 
 
+class UtilCommands(object):
+    """Generic utils."""
+
+    @staticmethod
+    def _get_resources_locks():
+        """Get all vol/snap/backup file lock paths."""
+        backup_locks_prefix = 'cinder-cleanup_incomplete_backups_'
+        oslo_dir = os.path.abspath(cfg.CONF.oslo_concurrency.lock_path)
+        filenames = glob.glob(os.path.join(oslo_dir, 'cinder-*'))
+
+        backend_url = cfg.CONF.coordination.backend_url
+        if backend_url.startswith('file://'):
+            tooz_dir = os.path.abspath(backend_url[7:])
+            if tooz_dir != oslo_dir:
+                filenames += glob.glob(os.path.join(tooz_dir, 'cinder-*'))
+
+        volumes = collections.defaultdict(list)
+        snapshots = collections.defaultdict(list)
+        backups = collections.defaultdict(list)
+        matcher = re.compile('.*?([0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-'
+                             '[0-9a-f]{4}-[0-9a-f]{12}).*?', re.IGNORECASE)
+        for filename in filenames:
+            basename = os.path.basename(filename)
+            match = matcher.match(basename)
+            if match:
+                dest = snapshots if 'snapshot' in basename else volumes
+                res_id = match.group(1)
+                dest[res_id].append(filename)
+            elif basename.startswith(backup_locks_prefix):
+                pgrp = basename[34:]
+                backups[pgrp].append(filename)
+
+        return volumes, snapshots, backups
+
+    def _exclude_running_backups(self, backups):
+        """Remove backup entries from the dict for running backup services."""
+        for backup_pgrp in list(backups.keys()):
+            # The PGRP is the same as the PID of the parent process, so we know
+            # the lock could be in use if the process is running and it's the
+            # cinder-backup command (the PID could have been reused).
+            cmdline_file = os.path.join('/proc', backup_pgrp, 'cmdline')
+            try:
+                with open(cmdline_file, 'r') as f:
+                    if 'cinder-backup' in f.read():
+                        del backups[backup_pgrp]
+            except FileNotFoundError:
+                continue
+            except Exception:
+                # Unexpected error, leaving the lock file just in case
+                del backups[backup_pgrp]
+
+    @args('--services-offline', dest='online',
+          action='store_false', default=True,
+          help='All locks can be deleted as Cinder services are not running.')
+    def clean_locks(self, online):
+        """Clean file locks for vols, snaps, and backups on the current host.
+
+        Should be run on any host where we are running a Cinder service (API,
+        Scheduler, Volume, Backup) and can be run with the Cinder services
+        running or stopped.
+
+        If the services are running it will check existing resources in the
+        Cinder database in order to know which resources are still available
+        (it's not safe to remove their file locks) and will only remove the
+        file locks for the resources that are no longer present.  Deleting
+        locks while the services are offline is faster as there's no need to
+        check the database.
+
+        For backups, the way to know if we can remove the startup lock is by
+        checking if the PGRP in the file name is currently running
+        cinder-backup.
+
+        Default assumes that services are online, must pass
+        ``--services-offline`` to specify that they are offline.
+
+        Doesn't clean DLM locks (except when using file locks), as those don't
+        leave lock leftovers.
+        """
+        self.ctxt = context.get_admin_context()
+        # Find volume and snapshots ids, and backups PGRP based on the existing
+        # file locks
+        volumes, snapshots, backups = self._get_resources_locks()
+
+        # If services are online we cannot delete locks for existing resources
+        if online:
+            # We don't want to delete file locks for existing resources
+            volumes = {vol_id: files for vol_id, files in volumes.items()
+                       if not objects.Volume.exists(self.ctxt, vol_id)}
+            snapshots = {snap_id: files for snap_id, files in snapshots.items()
+                         if not objects.Snapshot.exists(self.ctxt, snap_id)}
+            self._exclude_running_backups(backups)
+
+        def _err(filename: str, exc: Exception) -> None:
+            print('Failed to cleanup lock %(name)s: %(exc)s',
+                  {'name': filename, 'exc': exc})
+
+        # Now clean
+        for filenames in itertools.chain(volumes.values(),
+                                         snapshots.values(),
+                                         backups.values()):
+            for filename in filenames:
+                try:
+                    os.remove(filename)
+                except OSError as exc:
+                    if (exc.errno != errno.ENOENT):
+                        _err(filename, exc)
+                except Exception as exc:
+                    _err(filename, exc)
+
+
 CATEGORIES = {
     'backup': BackupCommands,
     'config': ConfigCommands,
@@ -666,9 +991,11 @@ CATEGORIES = {
     'cg': ConsistencyGroupCommands,
     'db': DbCommands,
     'host': HostCommands,
+    'quota': QuotaCommands,
     'service': ServiceCommands,
     'version': VersionCommands,
     'volume': VolumeCommands,
+    'util': UtilCommands,
 }
 
 
@@ -681,17 +1008,25 @@ def methods_of(obj):
     result = []
     for i in dir(obj):
         if isinstance(getattr(obj, i),
-                      collections.Callable) and not i.startswith('_'):
+                      collections_abc.Callable) and not i.startswith('_'):
             result.append((i, getattr(obj, i)))
     return result
 
 
+def missing_action(help_func):
+    def wrapped():
+        help_func()
+        exit(2)
+    return wrapped
+
+
 def add_command_parsers(subparsers):
     for category in sorted(CATEGORIES):
         command_object = CATEGORIES[category]()
 
         parser = subparsers.add_parser(category)
         parser.set_defaults(command_object=command_object)
+        parser.set_defaults(action_fn=missing_action(parser.print_help))
 
         category_subparsers = parser.add_subparsers(dest='action')
 
@@ -720,7 +1055,7 @@ def get_arg_string(args):
             # This is long optional arg
             args = args[2:]
         else:
-            args = args[1:]
+            args = args[1:]  # pylint: disable=E1136
 
     # We convert dashes to underscores so we can have cleaner optional arg
     # names
