@@ -15,7 +15,13 @@
 
 """Coordination and locking utilities."""
 
+import errno
+import glob
 import inspect
+import os
+import re
+import sys
+from typing import Callable, Optional  # noqa: H301
 import uuid
 
 import decorator
@@ -26,6 +32,7 @@ from tooz import coordination
 
 from cinder import exception
 from cinder.i18n import _
+from cinder import utils
 
 LOG = log.getLogger(__name__)
 
@@ -50,31 +57,45 @@ class Coordinator(object):
         meaningful prefix.
     """
 
-    def __init__(self, agent_id=None, prefix=''):
+    def __init__(self, agent_id: Optional[str] = None, prefix: str = ''):
         self.coordinator = None
         self.agent_id = agent_id or str(uuid.uuid4())
         self.started = False
         self.prefix = prefix
+        self._file_path = None
 
-    def start(self):
+    def _get_file_path(self, backend_url):
+        if backend_url.startswith('file://'):
+            path = backend_url[7:]
+            # Copied from TooZ's _normalize_path to get the same path they use
+            if sys.platform == 'win32':
+                path = re.sub(r'\\(?=\w:\\)', '', os.path.normpath(path))
+            return os.path.abspath(os.path.join(path, self.prefix))
+        return None
+
+    def start(self) -> None:
         if self.started:
             return
 
+        backend_url = cfg.CONF.coordination.backend_url
+
         # NOTE(bluex): Tooz expects member_id as a byte string.
         member_id = (self.prefix + self.agent_id).encode('ascii')
-        self.coordinator = coordination.get_coordinator(
-            cfg.CONF.coordination.backend_url, member_id)
+        self.coordinator = coordination.get_coordinator(backend_url, member_id)
+        assert self.coordinator is not None
         self.coordinator.start(start_heart=True)
+        self._file_path = self._get_file_path(backend_url)
         self.started = True
 
-    def stop(self):
+    def stop(self) -> None:
         """Disconnect from coordination backend and stop heartbeat."""
         if self.started:
-            self.coordinator.stop()
+            if self.coordinator is not None:
+                self.coordinator.stop()
             self.coordinator = None
             self.started = False
 
-    def get_lock(self, name):
+    def get_lock(self, name: str):
         """Return a Tooz backend lock.
 
         :param str name: The lock name that is used to identify it
@@ -87,11 +108,36 @@ class Coordinator(object):
         else:
             raise exception.LockCreationFailed(_('Coordinator uninitialized.'))
 
+    def remove_lock(self, glob_name):
+        # Most locks clean up on release, but not the file lock, so we manually
+        # clean them.
+
+        def _err(file_name: str, exc: Exception) -> None:
+            LOG.warning('Failed to cleanup lock %(name)s: %(exc)s',
+                        {'name': file_name, 'exc': exc})
+
+        if self._file_path:
+            files = glob.glob(self._file_path + glob_name)
+            for file_name in files:
+                try:
+                    os.remove(file_name)
+                except OSError as exc:
+                    if (exc.errno != errno.ENOENT):
+                        _err(file_name, exc)
+                except Exception as exc:
+                    _err(file_name, exc)
+
 
 COORDINATOR = Coordinator(prefix='cinder-')
 
 
-def synchronized(lock_name, blocking=True, coordinator=COORDINATOR):
+def synchronized_remove(glob_name, coordinator=COORDINATOR):
+    coordinator.remove_lock(glob_name)
+
+
+def synchronized(lock_name: str,
+                 blocking: bool = True,
+                 coordinator: Coordinator = COORDINATOR) -> Callable:
     """Synchronization decorator.
 
     :param str lock_name: Lock name.
@@ -134,20 +180,23 @@ def synchronized(lock_name, blocking=True, coordinator=COORDINATOR):
     """
 
     @decorator.decorator
-    def _synchronized(f, *a, **k):
+    def _synchronized(f, *a, **k) -> Callable:
         call_args = inspect.getcallargs(f, *a, **k)
         call_args['f_name'] = f.__name__
         lock = coordinator.get_lock(lock_name.format(**call_args))
+        name = utils.convert_str(lock.name)
+        f_name = f.__name__
         t1 = timeutils.now()
         t2 = None
         try:
+            LOG.debug('Acquiring lock "%(name)s" by "%(f_name)s"',
+                      {'name': name, 'f_name': f_name})
             with lock(blocking):
                 t2 = timeutils.now()
-                LOG.debug('Lock "%(name)s" acquired by "%(function)s" :: '
-                          'waited %(wait_secs)0.3fs',
-                          {'name': lock.name,
-                           'function': f.__name__,
-                           'wait_secs': (t2 - t1)})
+                LOG.debug('Lock "%(name)s" acquired by "%(f_name)s" :: '
+                          'waited %(wait)s',
+                          {'name': name, 'f_name': f_name,
+                           'wait': "%0.3fs" % (t2 - t1)})
                 return f(*a, **k)
         finally:
             t3 = timeutils.now()
@@ -155,10 +204,8 @@ def synchronized(lock_name, blocking=True, coordinator=COORDINATOR):
                 held_secs = "N/A"
             else:
                 held_secs = "%0.3fs" % (t3 - t2)
-            LOG.debug('Lock "%(name)s" released by "%(function)s" :: held '
-                      '%(held_secs)s',
-                      {'name': lock.name,
-                       'function': f.__name__,
-                       'held_secs': held_secs})
+            LOG.debug(
+                'Lock "%(name)s" released by "%(f_name)s" :: held %(held)s',
+                {'name': name, 'f_name': f_name, 'held': held_secs})
 
     return _synchronized
