@@ -97,6 +97,159 @@ def dispose_engine():
 ###################
 
 
+def resource_exists(context, model, resource_id):
+    return IMPL.resource_exists(context, model, resource_id)
+
+
+def get_model_for_versioned_object(versioned_object):
+    return IMPL.get_model_for_versioned_object(versioned_object)
+
+
+def get_by_id(context, model, id, *args, **kwargs):
+    return IMPL.get_by_id(context, model, id, *args, **kwargs)
+
+
+class Condition(object):
+    """Class for normal condition values for conditional_update."""
+    def __init__(self, value, field=None):
+        self.value = value
+        # Field is optional and can be passed when getting the filter
+        self.field = field
+
+    def get_filter(self, model, field=None):
+        return IMPL.condition_db_filter(
+            model, self._get_field(field), self.value,
+        )
+
+    def _get_field(self, field=None):
+        # We must have a defined field on initialization or when called
+        field = field or self.field
+        if not field:
+            raise ValueError(_('Condition has no field.'))
+        return field
+
+
+class Not(Condition):
+    """Class for negated condition values for conditional_update.
+
+    By default NULL values will be treated like Python treats None instead of
+    how SQL treats it.
+
+    So for example when values are (1, 2) it will evaluate to True when we have
+    value 3 or NULL, instead of only with 3 like SQL does.
+    """
+    def __init__(self, value, field=None, auto_none=True):
+        super(Not, self).__init__(value, field)
+        self.auto_none = auto_none
+
+    def get_filter(self, model, field=None):
+        # If implementation has a specific method use it
+        if hasattr(IMPL, 'condition_not_db_filter'):
+            return IMPL.condition_not_db_filter(model, self._get_field(field),
+                                                self.value, self.auto_none)
+
+        # Otherwise non negated object must adming ~ operator for not
+        return ~super(Not, self).get_filter(model, field)
+
+
+class Case(object):
+    """Class for conditional value selection for conditional_update."""
+    def __init__(self, whens, value=None, else_=None):
+        self.whens = whens
+        self.value = value
+        self.else_ = else_
+
+
+def is_orm_value(obj):
+    """Check if object is an ORM field."""
+    return IMPL.is_orm_value(obj)
+
+
+def conditional_update(
+    context,
+    model,
+    values,
+    expected_values,
+    filters=(),
+    include_deleted='no',
+    project_only=False,
+    order=None,
+):
+    """Compare-and-swap conditional update.
+
+    Update will only occur in the DB if conditions are met.
+
+    We have 4 different condition types we can use in expected_values:
+
+    - Equality:  {'status': 'available'}
+    - Inequality: {'status': vol_obj.Not('deleting')}
+    - In range: {'status': ['available', 'error']
+    - Not in range: {'status': vol_obj.Not(['in-use', 'attaching'])
+
+    Method accepts additional filters, which are basically anything that can be
+    passed to a sqlalchemy query's filter method, for example:
+
+    .. code-block:: python
+
+        [~sql.exists().where(models.Volume.id == models.Snapshot.volume_id)]
+
+    We can select values based on conditions using Case objects in the 'values'
+    argument. For example:
+
+    .. code-block:: python
+
+        has_snapshot_filter = sql.exists().where(
+            models.Snapshot.volume_id == models.Volume.id
+        )
+        case_values = db.Case(
+            [(has_snapshot_filter, 'has-snapshot')],
+            else_='no-snapshot'
+        )
+        db.conditional_update(
+            context,
+            models.Volume,
+            {'status': case_values},
+            {'status': 'available'},
+        )
+
+    And we can use DB fields for example to store previous status in the
+    corresponding field even though we don't know which value is in the db from
+    those we allowed:
+
+    .. code-block:: python
+
+        db.conditional_update(
+            context,
+            models.Volume,
+            {'status': 'deleting', 'previous_status': models.Volume.status},
+            {'status': ('available', 'error')},
+        )
+
+    :param values: Dictionary of key-values to update in the DB.
+    :param expected_values: Dictionary of conditions that must be met for the
+        update to be executed.
+    :param filters: Iterable with additional filters.
+    :param include_deleted: Should the update include deleted items, this is
+        equivalent to read_deleted.
+    :param project_only: Should the query be limited to context's project.
+    :param order: Specific order of fields in which to update the values
+    :returns: Boolean indicating whether db rows were updated.
+    """
+    return IMPL.conditional_update(
+        context,
+        model,
+        values,
+        expected_values,
+        filters,
+        include_deleted,
+        project_only,
+        order,
+    )
+
+
+###################
+
+
 def service_destroy(context, service_id):
     """Destroy the service or raise if it does not exist."""
     return IMPL.service_destroy(context, service_id)
@@ -361,11 +514,9 @@ def volume_attachment_get(context, attachment_id):
     return IMPL.volume_attachment_get(context, attachment_id)
 
 
-def volume_attachment_get_all_by_volume_id(context, volume_id,
-                                           session=None):
+def volume_attachment_get_all_by_volume_id(context, volume_id):
     return IMPL.volume_attachment_get_all_by_volume_id(context,
-                                                       volume_id,
-                                                       session)
+                                                       volume_id)
 
 
 def volume_attachment_get_all_by_host(context, host, filters=None):
@@ -880,8 +1031,8 @@ def group_type_specs_update_or_create(context,
 ###################
 
 
-def volume_type_encryption_get(context, volume_type_id, session=None):
-    return IMPL.volume_type_encryption_get(context, volume_type_id, session)
+def volume_type_encryption_get(context, volume_type_id):
+    return IMPL.volume_type_encryption_get(context, volume_type_id)
 
 
 def volume_type_encryption_delete(context, volume_type_id):
@@ -898,13 +1049,12 @@ def volume_type_encryption_update(context, volume_type_id, encryption_specs):
                                               encryption_specs)
 
 
-def volume_type_encryption_volume_get(context, volume_type_id, session=None):
-    return IMPL.volume_type_encryption_volume_get(context, volume_type_id,
-                                                  session)
+def volume_type_encryption_volume_get(context, volume_type_id):
+    return IMPL.volume_type_encryption_volume_get(context, volume_type_id)
 
 
-def volume_encryption_metadata_get(context, volume_id, session=None):
-    return IMPL.volume_encryption_metadata_get(context, volume_id, session)
+def volume_encryption_metadata_get(context, volume_id):
+    return IMPL.volume_encryption_metadata_get(context, volume_id)
 
 
 ###################
@@ -1060,10 +1210,9 @@ def volume_glance_metadata_copy_from_volume_to_volume(context,
 ###################
 
 
-def quota_create(context, project_id, resource, limit, allocated=0):
+def quota_create(context, project_id, resource, limit):
     """Create a quota for the given project and resource."""
-    return IMPL.quota_create(context, project_id, resource, limit,
-                             allocated=allocated)
+    return IMPL.quota_create(context, project_id, resource, limit)
 
 
 def quota_get(context, project_id, resource):
@@ -1076,21 +1225,6 @@ def quota_get_all_by_project(context, project_id):
     return IMPL.quota_get_all_by_project(context, project_id)
 
 
-def quota_allocated_get_all_by_project(context, project_id):
-    """Retrieve all allocated quotas associated with a given project."""
-    return IMPL.quota_allocated_get_all_by_project(context, project_id)
-
-
-def quota_allocated_update(context, project_id,
-                           resource, allocated):
-    """Update allocated quota to subprojects or raise if it does not exist.
-
-    :raises cinder.exception.ProjectQuotaNotFound:
-    """
-    return IMPL.quota_allocated_update(context, project_id,
-                                       resource, allocated)
-
-
 def quota_update(context, project_id, resource, limit):
     """Update a quota or raise if it does not exist."""
     return IMPL.quota_update(context, project_id, resource, limit)
@@ -1166,12 +1300,10 @@ def quota_usage_get_all_by_project(context, project_id):
 
 
 def quota_reserve(context, resources, quotas, deltas, expire,
-                  until_refresh, max_age, project_id=None,
-                  is_allocated_reserve=False):
+                  until_refresh, max_age, project_id=None):
     """Check quotas and create appropriate reservations."""
     return IMPL.quota_reserve(context, resources, quotas, deltas, expire,
-                              until_refresh, max_age, project_id=project_id,
-                              is_allocated_reserve=is_allocated_reserve)
+                              until_refresh, max_age, project_id=project_id)
 
 
 def reservation_commit(context, reservations, project_id=None):
@@ -1652,9 +1784,6 @@ def driver_initiator_data_insert_by_key(context, initiator,
     """Updates DriverInitiatorData entry.
 
     Sets the value for the specified key within the namespace.
-
-    If the entry already exists return False, if it inserted successfully
-    return True.
     """
     return IMPL.driver_initiator_data_insert_by_key(context,
                                                     initiator,
@@ -1757,17 +1886,6 @@ def cleanup_expired_messages(context):
 ###################
 
 
-def workers_init():
-    """Check if DB supports subsecond resolution and set global flag.
-
-    MySQL 5.5 doesn't support subsecond resolution in datetime fields, so we
-    have to take it into account when working with the worker's table.
-
-    Once we drop support for MySQL 5.5 we can remove this method.
-    """
-    return IMPL.workers_init()
-
-
 def worker_create(context, **values):
     """Create a worker entry from optional arguments."""
     return IMPL.worker_create(context, **values)
@@ -1803,39 +1921,6 @@ def worker_destroy(context, **filters):
 ###################
 
 
-def resource_exists(context, model, resource_id):
-    return IMPL.resource_exists(context, model, resource_id)
-
-
-def get_model_for_versioned_object(versioned_object):
-    return IMPL.get_model_for_versioned_object(versioned_object)
-
-
-def get_by_id(context, model, id, *args, **kwargs):
-    return IMPL.get_by_id(context, model, id, *args, **kwargs)
-
-
-class Condition(object):
-    """Class for normal condition values for conditional_update."""
-    def __init__(self, value, field=None):
-        self.value = value
-        # Field is optional and can be passed when getting the filter
-        self.field = field
-
-    def get_filter(self, model, field=None):
-        return IMPL.condition_db_filter(model, self._get_field(field),
-                                        self.value)
-
-    def _get_field(self, field=None):
-        # We must have a defined field on initialization or when called
-        field = field or self.field
-        if not field:
-            raise ValueError(_('Condition has no field.'))
-        return field
-
-###################
-
-
 def attachment_specs_exist(context):
     """Check if there are attachment specs left."""
     return IMPL.attachment_specs_exist(context)
@@ -1867,94 +1952,17 @@ def attachment_specs_update_or_create(context,
 ###################
 
 
-class Not(Condition):
-    """Class for negated condition values for conditional_update.
-
-    By default NULL values will be treated like Python treats None instead of
-    how SQL treats it.
-
-    So for example when values are (1, 2) it will evaluate to True when we have
-    value 3 or NULL, instead of only with 3 like SQL does.
-    """
-    def __init__(self, value, field=None, auto_none=True):
-        super(Not, self).__init__(value, field)
-        self.auto_none = auto_none
-
-    def get_filter(self, model, field=None):
-        # If implementation has a specific method use it
-        if hasattr(IMPL, 'condition_not_db_filter'):
-            return IMPL.condition_not_db_filter(model, self._get_field(field),
-                                                self.value, self.auto_none)
-
-        # Otherwise non negated object must adming ~ operator for not
-        return ~super(Not, self).get_filter(model, field)
+# TODO: (Y Release) remove method and this comment
+def volume_use_quota_online_data_migration(context, max_count):
+    return IMPL.volume_use_quota_online_data_migration(context, max_count)
 
 
-class Case(object):
-    """Class for conditional value selection for conditional_update."""
-    def __init__(self, whens, value=None, else_=None):
-        self.whens = whens
-        self.value = value
-        self.else_ = else_
+# TODO: (Y Release) remove method and this comment
+def snapshot_use_quota_online_data_migration(context, max_count):
+    return IMPL.snapshot_use_quota_online_data_migration(context, max_count)
 
 
-def is_orm_value(obj):
-    """Check if object is an ORM field."""
-    return IMPL.is_orm_value(obj)
-
-
-def conditional_update(context, model, values, expected_values, filters=(),
-                       include_deleted='no', project_only=False, order=None):
-    """Compare-and-swap conditional update.
-
-    Update will only occur in the DB if conditions are met.
-
-    We have 4 different condition types we can use in expected_values:
-     - Equality:  {'status': 'available'}
-     - Inequality: {'status': vol_obj.Not('deleting')}
-     - In range: {'status': ['available', 'error']
-     - Not in range: {'status': vol_obj.Not(['in-use', 'attaching'])
-
-    Method accepts additional filters, which are basically anything that can be
-    passed to a sqlalchemy query's filter method, for example:
-
-    .. code-block:: python
-
-     [~sql.exists().where(models.Volume.id == models.Snapshot.volume_id)]
-
-    We can select values based on conditions using Case objects in the 'values'
-    argument. For example:
-
-    .. code-block:: python
-
-     has_snapshot_filter = sql.exists().where(
-         models.Snapshot.volume_id == models.Volume.id)
-     case_values = db.Case([(has_snapshot_filter, 'has-snapshot')],
-                           else_='no-snapshot')
-     db.conditional_update(context, models.Volume, {'status': case_values},
-                           {'status': 'available'})
-
-    And we can use DB fields for example to store previous status in the
-    corresponding field even though we don't know which value is in the db from
-    those we allowed:
-
-    .. code-block:: python
-
-     db.conditional_update(context, models.Volume,
-                           {'status': 'deleting',
-                            'previous_status': models.Volume.status},
-                           {'status': ('available', 'error')})
-
-    :param values: Dictionary of key-values to update in the DB.
-    :param expected_values: Dictionary of conditions that must be met for the
-                            update to be executed.
-    :param filters: Iterable with additional filters.
-    :param include_deleted: Should the update include deleted items, this is
-                            equivalent to read_deleted.
-    :param project_only: Should the query be limited to context's project.
-    :param order: Specific order of fields in which to update the values
-    :returns: Number of db rows that were updated.
-    """
-    return IMPL.conditional_update(context, model, values, expected_values,
-                                   filters, include_deleted, project_only,
-                                   order)
+# TODO: (Z Release) remove method and this comment
+# TODO: (Y Release) uncomment method
+# def remove_temporary_admin_metadata_data_migration(context, max_count):
+#     IMPL.remove_temporary_admin_metadata_data_migration(context, max_count)
