@@ -13,11 +13,14 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import annotations  # Remove when only supporting python 3.9+
+
 import operator
 import re
+import sys
+from typing import Callable
 
 import pyparsing
-import six
 
 from cinder import exception
 from cinder.i18n import _
@@ -40,7 +43,7 @@ class EvalConstant(object):
 
     def eval(self):
         result = self.value
-        if (isinstance(result, six.string_types) and
+        if (isinstance(result, str) and
                 re.match(r"^[a-zA-Z_]+\.[a-zA-Z_]+$", result)):
             (which_dict, entry) = result.split('.')
             try:
@@ -168,7 +171,7 @@ class EvalTernaryOp(object):
 
 
 class EvalFunction(object):
-    functions = {
+    functions: dict[str, Callable] = {
         "abs": abs,
         "max": max,
         "min": min,
@@ -237,7 +240,7 @@ def _def_parser():
 
     oneOf = pyparsing.oneOf
     opAssoc = pyparsing.opAssoc
-    operatorPrecedence = pyparsing.operatorPrecedence
+    infixNotation = pyparsing.infixNotation
     Word = pyparsing.Word
 
     integer = Word(nums)
@@ -257,7 +260,7 @@ def _def_parser():
     negateop = oneOf('NOT not !')
 
     operand.setParseAction(EvalConstant)
-    expr = operatorPrecedence(operand, [
+    expr = infixNotation(operand, [
         (fn, 1, opAssoc.RIGHT, EvalFunction),
         ("^", 2, opAssoc.RIGHT, EvalPowerOp),
         (signop, 1, opAssoc.RIGHT, EvalSignOp),
@@ -289,10 +292,18 @@ def evaluate(expression, **kwargs):
     global _vars
     _vars = kwargs
 
+    # Some reasonable formulas break with the default recursion limit of
+    # 1000.  Raise it here and reset it afterward.
+    orig_recursion_limit = sys.getrecursionlimit()
+    if orig_recursion_limit < 3000:
+        sys.setrecursionlimit(3000)
+
     try:
         result = _parser.parseString(expression, parseAll=True)[0]
     except pyparsing.ParseException as e:
         raise exception.EvaluatorParseException(
             _("ParseException: %s") % e)
+    finally:
+        sys.setrecursionlimit(orig_recursion_limit)
 
     return result.eval()
