@@ -21,33 +21,40 @@
 import inspect
 import os
 import random
+import subprocess
+import sys
+import time
+from typing import Optional
 
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_db import exception as db_exc
 from oslo_log import log as logging
 import oslo_messaging as messaging
-from oslo_service import loopingcall
 from oslo_service import service
 from oslo_service import wsgi
 from oslo_utils import importutils
-osprofiler_notifier = importutils.try_import('osprofiler.notifier')
+osprofiler_initializer = importutils.try_import('osprofiler.initializer')
 profiler = importutils.try_import('osprofiler.profiler')
-osprofiler_web = importutils.try_import('osprofiler.web')
 profiler_opts = importutils.try_import('osprofiler.opts')
 
 
-from cinder.backup import rpcapi as backup_rpcapi
+from cinder.common import constants
 from cinder import context
 from cinder import coordination
 from cinder import exception
-from cinder.i18n import _, _LE, _LI, _LW
+from cinder.i18n import _
 from cinder import objects
 from cinder.objects import base as objects_base
+from cinder.objects import fields
 from cinder import rpc
-from cinder.scheduler import rpcapi as scheduler_rpcapi
 from cinder import version
-from cinder.volume import rpcapi as volume_rpcapi
+from cinder.volume import volume_utils
+
+if os.name == 'nt':
+    from os_win import utilsfactory as os_win_utilsfactory
+else:
+    os_win_utilsfactory = None
 
 
 LOG = logging.getLogger(__name__)
@@ -86,32 +93,31 @@ if profiler_opts:
     profiler_opts.set_defaults(CONF)
 
 
-def setup_profiler(binary, host):
-    if (osprofiler_notifier is None or
+def setup_profiler(binary: str, host: str) -> None:
+    if (osprofiler_initializer is None or
             profiler is None or
-            osprofiler_web is None or
             profiler_opts is None):
         LOG.debug('osprofiler is not present')
         return
 
     if CONF.profiler.enabled:
-        _notifier = osprofiler_notifier.create(
-            "Messaging", messaging, context.get_admin_context().to_dict(),
-            rpc.TRANSPORT, "cinder", binary, host)
-        osprofiler_notifier.set(_notifier)
-        osprofiler_web.enable(CONF.profiler.hmac_keys)
+        osprofiler_initializer.init_from_conf(
+            conf=CONF,
+            context=context.get_admin_context().to_dict(),
+            project="cinder",
+            service=binary,
+            host=host
+        )
         LOG.warning(
-            _LW("OSProfiler is enabled.\nIt means that person who knows "
-                "any of hmac_keys that are specified in "
-                "/etc/cinder/cinder.conf can trace his requests. \n"
-                "In real life only operator can read this file so there "
-                "is no security issue. Note that even if person can "
-                "trigger profiler, only admin user can retrieve trace "
-                "information.\n"
-                "To disable OSprofiler set in cinder.conf:\n"
-                "[profiler]\nenabled=false"))
-    else:
-        osprofiler_web.disable()
+            "OSProfiler is enabled.\nIt means that person who knows "
+            "any of hmac_keys that are specified in "
+            "/etc/cinder/cinder.conf can trace his requests. \n"
+            "In real life only operator can read this file so there "
+            "is no security issue. Note that even if person can "
+            "trigger profiler, only admin user can retrieve trace "
+            "information.\n"
+            "To disable OSProfiler set in cinder.conf:\n"
+            "[profiler]\nenabled=false")
 
 
 class Service(service.Service):
@@ -121,11 +127,18 @@ class Service(service.Service):
     on topic. It also periodically runs tasks on the manager and reports
     it state to the database services table.
     """
+    # Make service_id a class attribute so it can be used for clean up
+    service_id = None
 
-    def __init__(self, host, binary, topic, manager, report_interval=None,
-                 periodic_interval=None, periodic_fuzzy_delay=None,
-                 service_name=None, coordination=False, cluster=None, *args,
-                 **kwargs):
+    def __init__(self, host: str, binary: str, topic: str,
+                 manager: str,
+                 report_interval: Optional[int] = None,
+                 periodic_interval: Optional[int] = None,
+                 periodic_fuzzy_delay: Optional[int] = None,
+                 service_name: Optional[str] = None,
+                 coordination: bool = False,
+                 cluster: Optional[str] = None,
+                 *args, **kwargs):
         super(Service, self).__init__()
 
         if not rpc.initialized():
@@ -141,24 +154,25 @@ class Service(service.Service):
         if CONF.profiler.enabled:
             manager_class = profiler.trace_cls("rpc")(manager_class)
 
+        self.service = None
+        self.manager = manager_class(host=self.host,
+                                     cluster=self.cluster,
+                                     service_name=service_name,
+                                     *args, **kwargs)
+        self.availability_zone: str = self.manager.availability_zone
+        self.model_disconnected: bool
+
         # NOTE(geguileo): We need to create the Service DB entry before we
         # create the manager, otherwise capped versions for serializer and rpc
         # client would use existing DB entries not including us, which could
         # result in us using None (if it's the first time the service is run)
         # or an old version (if this is a normal upgrade of a single service).
         ctxt = context.get_admin_context()
-        self.is_upgrading_to_n = self.is_svc_upgrading_to_n(binary)
         try:
             service_ref = objects.Service.get_by_args(ctxt, host, binary)
             service_ref.rpc_current_version = manager_class.RPC_API_VERSION
             obj_version = objects_base.OBJ_VERSIONS.get_current()
             service_ref.object_current_version = obj_version
-            # TODO(geguileo): In O we can remove the service upgrading part on
-            # the next equation, because by then all our services will be
-            # properly setting the cluster during volume migrations since
-            # they'll have the new Volume ORM model.  But until then we can
-            # only set the cluster in the DB and pass added_to_cluster to
-            # init_host when we have completed the rolling upgrade from M to N.
 
             # added_to_cluster attribute marks when we consider that we have
             # just added a host to a cluster so we can include resources into
@@ -167,92 +181,95 @@ class Service(service.Service):
             # configuration has a cluster value.  We don't want to do anything
             # automatic if the cluster is changed, in those cases we'll want
             # to use cinder manage command and to it manually.
-            self.added_to_cluster = (not service_ref.cluster_name and cluster
-                                     and not self.is_upgrading_to_n)
+            self.added_to_cluster = (not service_ref.cluster_name and cluster)
 
-            # TODO(geguileo): In O - Remove self.is_upgrading_to_n part
-            if (service_ref.cluster_name != cluster and
-                    not self.is_upgrading_to_n):
-                LOG.info(_LI('This service has been moved from cluster '
-                             '%(cluster_svc)s to %(cluster_cfg)s. Resources '
-                             'will %(opt_no)sbe moved to the new cluster'),
+            if service_ref.cluster_name != cluster:
+                LOG.info('This service has been moved from cluster '
+                         '%(cluster_svc)s to %(cluster_cfg)s. Resources '
+                         'will %(opt_no)sbe moved to the new cluster',
                          {'cluster_svc': service_ref.cluster_name,
                           'cluster_cfg': cluster,
-                          'opt_no': '' if self.added_to_cluster else 'NO '})
+                          'opt_no': '' if self.added_to_cluster else 'NOT '})
 
             if self.added_to_cluster:
                 # We pass copy service's disable status in the cluster if we
                 # have to create it.
-                self._ensure_cluster_exists(ctxt, service_ref.disabled)
+                self._ensure_cluster_exists(ctxt, service_ref)
                 service_ref.cluster_name = cluster
             service_ref.save()
-            self.service_id = service_ref.id
+            Service.service_id = service_ref.id
+            self.origin_service_id = service_ref.id
         except exception.NotFound:
-            # We don't want to include cluster information on the service or
-            # create the cluster entry if we are upgrading.
             self._create_service_ref(ctxt, manager_class.RPC_API_VERSION)
-            # TODO(geguileo): In O set added_to_cluster to True
-            # We don't want to include resources in the cluster during the
-            # start while we are still doing the rolling upgrade.
-            self.added_to_cluster = not self.is_upgrading_to_n
+            # Service entry Entry didn't exist because it was manually removed
+            # or it's the first time running, to be on the safe side we say we
+            # were added if we are clustered.
+            self.added_to_cluster = bool(cluster)  # type: ignore
 
-        self.manager = manager_class(host=self.host,
-                                     cluster=self.cluster,
-                                     service_name=service_name,
-                                     *args, **kwargs)
         self.report_interval = report_interval
         self.periodic_interval = periodic_interval
         self.periodic_fuzzy_delay = periodic_fuzzy_delay
         self.basic_config_check()
         self.saved_args, self.saved_kwargs = args, kwargs
-        self.timers = []
 
         setup_profiler(binary, host)
-        self.rpcserver = None
-        self.cluster_rpcserver = None
+        self.rpcserver: Optional['messaging.rpc.RPCServer'] = None
+        self.backend_rpcserver: Optional['messaging.rpc.RPCServer'] = None
+        self.cluster_rpcserver: Optional['messaging.rpc.RPCServer'] = None
 
-    # TODO(geguileo): Remove method in O since it will no longer be used.
-    @staticmethod
-    def is_svc_upgrading_to_n(binary):
-        """Given an RPC API class determine if the service is upgrading."""
-        rpcapis = {'cinder-scheduler': scheduler_rpcapi.SchedulerAPI,
-                   'cinder-volume': volume_rpcapi.VolumeAPI,
-                   'cinder-backup': backup_rpcapi.BackupAPI}
-        rpc_api = rpcapis[binary]
-        # If we are pinned to 1.3, then we are upgrading from M to N
-        return rpc_api.determine_obj_version_cap() == '1.3'
-
-    def start(self):
+    def start(self) -> None:
         version_string = version.version_string()
-        LOG.info(_LI('Starting %(topic)s node (version %(version_string)s)'),
+        LOG.info('Starting %(topic)s node (version %(version_string)s)',
                  {'topic': self.topic, 'version_string': version_string})
         self.model_disconnected = False
 
         if self.coordination:
             coordination.COORDINATOR.start()
 
-        self.manager.init_host(added_to_cluster=self.added_to_cluster)
+        # NOTE(yikun): When re-spawning child process, we should set the class
+        # attribute back using the origin service_id, otherwise,
+        # the Service.service_id will be inherited from the parent process,
+        # and will be recorded as the last started service id by mistaken.
+        Service.service_id = self.origin_service_id
+
+        self.manager.init_host(added_to_cluster=self.added_to_cluster,
+                               service_id=Service.service_id)
 
         LOG.debug("Creating RPC server for service %s", self.topic)
 
         ctxt = context.get_admin_context()
-        target = messaging.Target(topic=self.topic, server=self.host)
         endpoints = [self.manager]
         endpoints.extend(self.manager.additional_endpoints)
         obj_version_cap = objects.Service.get_minimum_obj_version(ctxt)
         LOG.debug("Pinning object versions for RPC server serializer to %s",
                   obj_version_cap)
         serializer = objects_base.CinderObjectSerializer(obj_version_cap)
+
+        target = messaging.Target(topic=self.topic, server=self.host)
         self.rpcserver = rpc.get_server(target, endpoints, serializer)
         self.rpcserver.start()
 
-        # TODO(geguileo): In O - Remove the is_svc_upgrading_to_n part
-        if self.cluster and not self.is_svc_upgrading_to_n(self.binary):
-            LOG.info(_LI('Starting %(topic)s cluster %(cluster)s (version '
-                         '%(version)s)'),
+        # NOTE(dulek): Kids, don't do that at home. We're relying here on
+        # oslo.messaging implementation details to keep backward compatibility
+        # with pre-Ocata services. This will not matter once we drop
+        # compatibility with them.
+        if self.topic == constants.VOLUME_TOPIC:
+            target = messaging.Target(
+                topic='%(topic)s.%(host)s' % {'topic': self.topic,
+                                              'host': self.host},
+                server=volume_utils.extract_host(self.host, 'host'))
+            self.backend_rpcserver = rpc.get_server(target, endpoints,
+                                                    serializer)
+            self.backend_rpcserver.start()
+
+        if self.cluster:
+            LOG.info('Starting %(topic)s cluster %(cluster)s (version '
+                     '%(version)s)',
                      {'topic': self.topic, 'version': version_string,
                       'cluster': self.cluster})
-            target = messaging.Target(topic=self.topic, server=self.cluster)
+            target = messaging.Target(
+                topic='%s.%s' % (self.topic, self.cluster),
+                server=volume_utils.extract_host(self.cluster, 'host'))
             serializer = objects_base.CinderObjectSerializer(obj_version_cap)
             self.cluster_rpcserver = rpc.get_server(target, endpoints,
                                                     serializer)
@@ -261,52 +278,65 @@ class Service(service.Service):
         self.manager.init_host_with_rpc()
 
         if self.report_interval:
-            pulse = loopingcall.FixedIntervalLoopingCall(
-                self.report_state)
-            pulse.start(interval=self.report_interval,
-                        initial_delay=self.report_interval)
-            self.timers.append(pulse)
+            self.tg.add_timer(self.report_interval, self.report_state,
+                              initial_delay=self.report_interval)
 
         if self.periodic_interval:
+            initial_delay: Optional[int]
             if self.periodic_fuzzy_delay:
                 initial_delay = random.randint(0, self.periodic_fuzzy_delay)
             else:
                 initial_delay = None
+            self.tg.add_timer(self.periodic_interval, self.periodic_tasks,
+                              initial_delay=initial_delay)
 
-            periodic = loopingcall.FixedIntervalLoopingCall(
-                self.periodic_tasks)
-            periodic.start(interval=self.periodic_interval,
-                           initial_delay=initial_delay)
-            self.timers.append(periodic)
-
-    def basic_config_check(self):
+    def basic_config_check(self) -> None:
         """Perform basic config checks before starting service."""
         # Make sure report interval is less than service down time
         if self.report_interval:
             if CONF.service_down_time <= self.report_interval:
                 new_down_time = int(self.report_interval * 2.5)
                 LOG.warning(
-                    _LW("Report interval must be less than service down "
-                        "time. Current config service_down_time: "
-                        "%(service_down_time)s, report_interval for this: "
-                        "service is: %(report_interval)s. Setting global "
-                        "service_down_time to: %(new_down_time)s"),
+                    "Report interval must be less than service down "
+                    "time. Current config service_down_time: "
+                    "%(service_down_time)s, report_interval for this: "
+                    "service is: %(report_interval)s. Setting global "
+                    "service_down_time to: %(new_down_time)s",
                     {'service_down_time': CONF.service_down_time,
                      'report_interval': self.report_interval,
                      'new_down_time': new_down_time})
                 CONF.set_override('service_down_time', new_down_time)
 
-    def _ensure_cluster_exists(self, context, disabled=None):
+    def _ensure_cluster_exists(self,
+                               context: context.RequestContext,
+                               service: 'Service') -> None:
         if self.cluster:
             try:
-                objects.Cluster.get_by_id(context, None, name=self.cluster,
-                                          binary=self.binary)
+                cluster = objects.Cluster.get_by_id(context, None,
+                                                    name=self.cluster,
+                                                    binary=self.binary)
+                # If the cluster already exists, then the service replication
+                # fields must match those of the cluster unless the service
+                # is in error status.
+                error_states = (fields.ReplicationStatus.ERROR,
+                                fields.ReplicationStatus.FAILOVER_ERROR)
+                if service.replication_status not in error_states:
+                    for attr in ('replication_status', 'active_backend_id',
+                                 'frozen'):
+                        if getattr(service, attr) != getattr(cluster, attr):
+                            setattr(service, attr, getattr(cluster, attr))
+
             except exception.ClusterNotFound:
-                cluster = objects.Cluster(context=context, name=self.cluster,
-                                          binary=self.binary)
-                # If disabled has been specified overwrite default value
-                if disabled is not None:
-                    cluster.disabled = disabled
+                # Since the cluster didn't exist, we copy replication fields
+                # from the service.
+                cluster = objects.Cluster(
+                    context=context,
+                    name=self.cluster,
+                    binary=self.binary,
+                    disabled=service.disabled,
+                    replication_status=service.replication_status,
+                    active_backend_id=service.active_backend_id,
+                    frozen=service.frozen)
                 try:
                     cluster.create()
 
@@ -315,37 +345,45 @@ class Service(service.Service):
                 except exception.ClusterExists:
                     pass
 
-    def _create_service_ref(self, context, rpc_version=None):
-        zone = CONF.storage_availability_zone
+    def _create_service_ref(self,
+                            context: context.RequestContext,
+                            rpc_version: Optional[str] = None) -> None:
         kwargs = {
             'host': self.host,
             'binary': self.binary,
             'topic': self.topic,
             'report_count': 0,
-            'availability_zone': zone,
+            'availability_zone': self.availability_zone,
             'rpc_current_version': rpc_version or self.manager.RPC_API_VERSION,
             'object_current_version': objects_base.OBJ_VERSIONS.get_current(),
         }
-        # TODO(geguileo): In O unconditionally set cluster_name like above
-        # If we are upgrading we have to ignore the cluster value
-        if not self.is_upgrading_to_n:
-            kwargs['cluster_name'] = self.cluster
+        kwargs['cluster_name'] = self.cluster
         service_ref = objects.Service(context=context, **kwargs)
         service_ref.create()
-        self.service_id = service_ref.id
-        # TODO(geguileo): In O unconditionally ensure that the cluster exists
-        if not self.is_upgrading_to_n:
-            self._ensure_cluster_exists(context)
+        Service.service_id = service_ref.id
+        self.origin_service_id = service_ref.id
+        self._ensure_cluster_exists(context, service_ref)
+        # If we have updated the service_ref with replication data from
+        # the cluster it will be saved.
+        service_ref.save()
 
-    def __getattr__(self, key):
+    def __getattr__(self, key: str):
         manager = self.__dict__.get('manager', None)
         return getattr(manager, key)
 
     @classmethod
-    def create(cls, host=None, binary=None, topic=None, manager=None,
-               report_interval=None, periodic_interval=None,
-               periodic_fuzzy_delay=None, service_name=None,
-               coordination=False, cluster=None):
+    def create(cls,
+               host: Optional[str] = None,
+               binary: Optional[str] = None,
+               topic: Optional[str] = None,
+               manager: Optional[str] = None,
+               report_interval: Optional[int] = None,
+               periodic_interval: Optional[int] = None,
+               periodic_fuzzy_delay: Optional[int] = None,
+               service_name: Optional[str] = None,
+               coordination: bool = False,
+               cluster: Optional[str] = None,
+               **kwargs) -> 'Service':
         """Instantiates class and passes back application object.
 
         :param host: defaults to CONF.host
@@ -373,33 +411,32 @@ class Service(service.Service):
             periodic_interval = CONF.periodic_interval
         if periodic_fuzzy_delay is None:
             periodic_fuzzy_delay = CONF.periodic_fuzzy_delay
+
+        assert host is not None
+        assert manager is not None
         service_obj = cls(host, binary, topic, manager,
                           report_interval=report_interval,
                           periodic_interval=periodic_interval,
                           periodic_fuzzy_delay=periodic_fuzzy_delay,
                           service_name=service_name,
                           coordination=coordination,
-                          cluster=cluster)
+                          cluster=cluster, **kwargs)
 
         return service_obj
 
-    def stop(self):
+    def stop(self) -> None:
         # Try to shut the connection down, but if we get any sort of
         # errors, go ahead and ignore them.. as we're shutting down anyway
         try:
-            self.rpcserver.stop()
+            if self.rpcserver is not None:
+                self.rpcserver.stop()
+            if self.backend_rpcserver:
+                self.backend_rpcserver.stop()
             if self.cluster_rpcserver:
                 self.cluster_rpcserver.stop()
         except Exception:
             pass
 
-        self.timers_skip = []
-        for x in self.timers:
-            try:
-                x.stop()
-            except Exception:
-                self.timers_skip.append(x)
-
         if self.coordination:
             try:
                 coordination.COORDINATOR.stop()
@@ -407,77 +444,73 @@ class Service(service.Service):
                 pass
         super(Service, self).stop(graceful=True)
 
-    def wait(self):
-        skip = getattr(self, 'timers_skip', [])
-        for x in self.timers:
-            if x not in skip:
-                try:
-                    x.wait()
-                except Exception:
-                    pass
+    def wait(self) -> None:
         if self.rpcserver:
             self.rpcserver.wait()
+        if self.backend_rpcserver:
+            self.backend_rpcserver.wait()
         if self.cluster_rpcserver:
             self.cluster_rpcserver.wait()
         super(Service, self).wait()
 
-    def periodic_tasks(self, raise_on_error=False):
+    def periodic_tasks(self, raise_on_error: bool = False) -> None:
         """Tasks to be run at a periodic interval."""
         ctxt = context.get_admin_context()
-        self.manager.periodic_tasks(ctxt, raise_on_error=raise_on_error)
+        self.manager.run_periodic_tasks(ctxt, raise_on_error=raise_on_error)
 
-    def report_state(self):
+    def report_state(self) -> None:
         """Update the state of this service in the datastore."""
         if not self.manager.is_working():
             # NOTE(dulek): If manager reports a problem we're not sending
             # heartbeats - to indicate that service is actually down.
-            LOG.error(_LE('Manager for service %(binary)s %(host)s is '
-                          'reporting problems, not sending heartbeat. '
-                          'Service will appear "down".'),
+            LOG.error('Manager for service %(binary)s %(host)s is '
+                      'reporting problems, not sending heartbeat. '
+                      'Service will appear "down".',
                       {'binary': self.binary,
                        'host': self.host})
             return
 
         ctxt = context.get_admin_context()
-        zone = CONF.storage_availability_zone
         try:
             try:
-                service_ref = objects.Service.get_by_id(ctxt, self.service_id)
+                service_ref = objects.Service.get_by_id(ctxt,
+                                                        Service.service_id)
             except exception.NotFound:
                 LOG.debug('The service database object disappeared, '
                           'recreating it.')
                 self._create_service_ref(ctxt)
-                service_ref = objects.Service.get_by_id(ctxt, self.service_id)
+                service_ref = objects.Service.get_by_id(ctxt,
+                                                        Service.service_id)
 
             service_ref.report_count += 1
-            if zone != service_ref.availability_zone:
-                service_ref.availability_zone = zone
+            if self.availability_zone != service_ref.availability_zone:
+                service_ref.availability_zone = self.availability_zone
 
             service_ref.save()
 
             # TODO(termie): make this pattern be more elegant.
             if getattr(self, 'model_disconnected', False):
                 self.model_disconnected = False
-                LOG.error(_LE('Recovered model server connection!'))
+                LOG.error('Recovered model server connection!')
 
         except db_exc.DBConnectionError:
             if not getattr(self, 'model_disconnected', False):
                 self.model_disconnected = True
-                LOG.exception(_LE('model server went away'))
+                LOG.exception('model server went away')
 
         # NOTE(jsbryant) Other DB errors can happen in HA configurations.
         # such errors shouldn't kill this thread, so we handle them here.
         except db_exc.DBError:
             if not getattr(self, 'model_disconnected', False):
                 self.model_disconnected = True
-                LOG.exception(_LE('DBError encountered: '))
+                LOG.exception('DBError encountered: ')
 
         except Exception:
             if not getattr(self, 'model_disconnected', False):
                 self.model_disconnected = True
-                LOG.exception(_LE('Exception encountered: '))
+                LOG.exception('Exception encountered: ')
 
-    def reset(self):
+    def reset(self) -> None:
         self.manager.reset()
         super(Service, self).reset()
 
@@ -508,7 +541,7 @@ class WSGIService(service.ServiceBase):
                      "must be greater than 0.") %
                    {'worker_name': worker_name,
                     'workers': self.workers})
-            raise exception.InvalidInput(msg)
+            raise exception.InvalidConfigurationValue(msg)
         setup_profiler(name, self.host)
 
         self.server = wsgi.Server(CONF,
@@ -539,7 +572,7 @@ class WSGIService(service.ServiceBase):
         manager_class = importutils.import_class(manager_class_name)
         return manager_class()
 
-    def start(self):
+    def start(self) -> None:
         """Start serving this service using loaded configuration.
 
         Also, retrieve updated port number in case '0' was passed in, which
@@ -553,7 +586,7 @@ class WSGIService(service.ServiceBase):
         self.server.start()
         self.port = self.server.port
 
-    def stop(self):
+    def stop(self) -> None:
         """Stop serving this API.
 
         :returns: None
@@ -561,7 +594,7 @@ class WSGIService(service.ServiceBase):
         """
         self.server.stop()
 
-    def wait(self):
+    def wait(self) -> None:
         """Wait for the service to stop serving this API.
 
         :returns: None
@@ -569,7 +602,7 @@ class WSGIService(service.ServiceBase):
         """
         self.server.wait()
 
-    def reset(self):
+    def reset(self) -> None:
         """Reset server greenpool size to default.
 
         :returns: None
@@ -578,8 +611,8 @@ class WSGIService(service.ServiceBase):
         self.server.reset()
 
 
-def process_launcher():
-    return service.ProcessLauncher(CONF)
+def process_launcher() -> service.ProcessLauncher:
+    return service.ProcessLauncher(CONF, restart_method='mutate')
 
 
 # NOTE(vish): the global launcher is to maintain the existing
@@ -593,26 +626,17 @@ def serve(server, workers=None):
     if _launcher:
         raise RuntimeError(_('serve() can only be called once'))
 
-    _launcher = service.launch(CONF, server, workers=workers)
+    _launcher = service.launch(CONF, server, workers=workers,
+                               restart_method='mutate')
 
 
-def wait():
-    LOG.debug('Full set of CONF:')
-    for flag in CONF:
-        flag_get = CONF.get(flag, None)
-        # hide flag contents from log if contains a password
-        # should use secret flag when switch over to openstack-common
-        if ("_password" in flag or "_key" in flag or
-                (flag == "sql_connection" and
-                    ("mysql:" in flag_get or "postgresql:" in flag_get))):
-            LOG.debug('%s : FLAG SET ', flag)
-        else:
-            LOG.debug('%(flag)s : %(flag_get)s',
-                      {'flag': flag, 'flag_get': flag_get})
+def wait() -> None:
+    CONF.log_opt_values(LOG, logging.DEBUG)
+
     try:
-        _launcher.wait()
+        _launcher.wait()  # type: ignore
     except KeyboardInterrupt:
-        _launcher.stop()
+        _launcher.stop()  # type: ignore
     rpc.cleanup()
 
 
@@ -622,7 +646,7 @@ class Launcher(object):
         self.wait = wait
 
 
-def get_launcher():
+def get_launcher() -> service.ProcessLauncher:
     # Note(lpetrut): ProcessLauncher uses green pipes which fail on Windows
     # due to missing support of non-blocking I/O pipes. For this reason, the
     # service must be spawned differently on Windows, using the ServiceLauncher
@@ -631,3 +655,48 @@ def get_launcher():
         return Launcher()
     else:
         return process_launcher()
+
+
+class WindowsProcessLauncher(object):
+    def __init__(self):
+        self._processutils = os_win_utilsfactory.get_processutils()
+
+        self._workers = []
+        self._worker_job_handles = []
+        self._signal_handler = service.SignalHandler()
+        self._add_signal_handlers()
+
+    def add_process(self, cmd):
+        LOG.info("Starting subprocess: %s", cmd)
+
+        worker = subprocess.Popen(cmd)
+        try:
+            job_handle = self._processutils.kill_process_on_job_close(
+                worker.pid)
+        except Exception:
+            LOG.exception("Could not associate child process "
+                          "with a job, killing it.")
+            worker.kill()
+            raise
+
+        self._worker_job_handles.append(job_handle)
+        self._workers.append(worker)
+
+    def _add_signal_handlers(self):
+        self._signal_handler.add_handler('SIGINT', self._terminate)
+        self._signal_handler.add_handler('SIGTERM', self._terminate)
+
+    def _terminate(self, *args):
+        # We've already assigned win32 job objects to child processes,
+        # requesting them to stop once all the job handles are closed.
+        # When this process dies, so will the child processes.
+        LOG.info("Received request to terminate.")
+        sys.exit(1)
+
+    def wait(self):
+        pids = [worker.pid for worker in self._workers]
+        if pids:
+            self._processutils.wait_for_multiple_processes(pids,
+                                                           wait_all=True)
+        # By sleeping here, we allow signal handlers to be executed.
+        time.sleep(0)
