@@ -15,10 +15,10 @@
 
 """Tests for cgsnapshot code."""
 
+from http import HTTPStatus
 from unittest import mock
 
 from oslo_serialization import jsonutils
-from six.moves import http_client
 import webob
 
 from cinder import context
@@ -65,7 +65,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
             volume_id=volume_id,
             group_snapshot_id=cgsnapshot.id)['id']
 
-        req = webob.Request.blank('/v2/%s/cgsnapshots/%s' % (
+        req = webob.Request.blank('/v3/%s/cgsnapshots/%s' % (
             fake.PROJECT_ID, cgsnapshot.id))
         req.method = 'GET'
         req.headers['Content-Type'] = 'application/json'
@@ -73,7 +73,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.OK, res.status_int)
+        self.assertEqual(HTTPStatus.OK, res.status_int)
         self.assertEqual('this is a test group snapshot',
                          res_dict['cgsnapshot']['description'])
 
@@ -87,7 +87,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
         consistencygroup.destroy()
 
     def test_show_cgsnapshot_with_cgsnapshot_NotFound(self):
-        req = webob.Request.blank('/v2/%s/cgsnapshots/%s' % (
+        req = webob.Request.blank('/v3/%s/cgsnapshots/%s' % (
             fake.PROJECT_ID, fake.WILL_NOT_BE_FOUND_ID))
         req.method = 'GET'
         req.headers['Content-Type'] = 'application/json'
@@ -95,8 +95,8 @@ class CgsnapshotsAPITestCase(test.TestCase):
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.NOT_FOUND, res.status_int)
-        self.assertEqual(http_client.NOT_FOUND,
+        self.assertEqual(HTTPStatus.NOT_FOUND, res.status_int)
+        self.assertEqual(HTTPStatus.NOT_FOUND,
                          res_dict['itemNotFound']['code'])
         self.assertEqual('GroupSnapshot %s could not be found.' %
                          fake.WILL_NOT_BE_FOUND_ID,
@@ -123,14 +123,14 @@ class CgsnapshotsAPITestCase(test.TestCase):
             self.context, group_id=consistencygroup.id,
             group_type_id=fake.GROUP_TYPE_ID,)
 
-        req = webob.Request.blank('/v2/%s/cgsnapshots' % fake.PROJECT_ID)
+        req = webob.Request.blank('/v3/%s/cgsnapshots' % fake.PROJECT_ID)
         req.method = 'GET'
         req.headers['Content-Type'] = 'application/json'
         res = req.get_response(fakes.wsgi_app(
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.OK, res.status_int)
+        self.assertEqual(HTTPStatus.OK, res.status_int)
         self.assertEqual(cgsnapshot3.id,
                          res_dict['cgsnapshots'][0]['id'])
         self.assertEqual('test_group_snapshot',
@@ -171,7 +171,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
             self.context, group_id=consistencygroup.id,
             group_type_id=fake.GROUP_TYPE_ID,)
 
-        req = webob.Request.blank('/v2/%s/cgsnapshots/detail' %
+        req = webob.Request.blank('/v3/%s/cgsnapshots/detail' %
                                   fake.PROJECT_ID)
         req.method = 'GET'
         req.headers['Content-Type'] = 'application/json'
@@ -180,7 +180,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.OK, res.status_int)
+        self.assertEqual(HTTPStatus.OK, res.status_int)
         self.assertEqual('this is a test group snapshot',
                          res_dict['cgsnapshots'][0]['description'])
         self.assertEqual('test_group_snapshot',
@@ -232,7 +232,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
                                "description":
                                "CG Snapshot 1",
                                "consistencygroup_id": consistencygroup.id}}
-        req = webob.Request.blank('/v2/%s/cgsnapshots' % fake.PROJECT_ID)
+        req = webob.Request.blank('/v3/%s/cgsnapshots' % fake.PROJECT_ID)
         req.method = 'POST'
         req.headers['Content-Type'] = 'application/json'
         req.body = jsonutils.dump_as_bytes(body)
@@ -241,7 +241,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
 
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.ACCEPTED, res.status_int)
+        self.assertEqual(HTTPStatus.ACCEPTED, res.status_int)
         self.assertIn('id', res_dict['cgsnapshot'])
         self.assertTrue(mock_validate.called)
 
@@ -270,7 +270,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
                                "description":
                                "CG Snapshot 1",
                                "consistencygroup_id": consistencygroup.id}}
-        req = webob.Request.blank('/v2/%s/cgsnapshots' % fake.PROJECT_ID)
+        req = webob.Request.blank('/v3/%s/cgsnapshots' % fake.PROJECT_ID)
         req.method = 'POST'
         req.headers['Content-Type'] = 'application/json'
         req.body = jsonutils.dump_as_bytes(body)
@@ -278,8 +278,8 @@ class CgsnapshotsAPITestCase(test.TestCase):
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
-        self.assertEqual(http_client.BAD_REQUEST,
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST,
                          res_dict['badRequest']['code'])
         self.assertEqual(
             "Invalid volume: The snapshot cannot be created when the volume "
@@ -293,7 +293,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
 
     def test_create_cgsnapshot_with_no_body(self):
         # omit body from the request
-        req = webob.Request.blank('/v2/%s/cgsnapshots' % fake.PROJECT_ID)
+        req = webob.Request.blank('/v3/%s/cgsnapshots' % fake.PROJECT_ID)
         req.body = jsonutils.dump_as_bytes(None)
         req.method = 'POST'
         req.headers['Content-Type'] = 'application/json'
@@ -302,8 +302,8 @@ class CgsnapshotsAPITestCase(test.TestCase):
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
-        self.assertEqual(http_client.BAD_REQUEST,
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST,
                          res_dict['badRequest']['code'])
         self.assertEqual("Missing required element 'cgsnapshot' in "
                          "request body.",
@@ -327,7 +327,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
                                "description":
                                "CG Snapshot 1",
                                "consistencygroup_id": consistencygroup.id}}
-        req = webob.Request.blank('/v2/%s/cgsnapshots' % fake.PROJECT_ID)
+        req = webob.Request.blank('/v3/%s/cgsnapshots' % fake.PROJECT_ID)
         req.body = jsonutils.dump_as_bytes(body)
         req.method = 'POST'
         req.headers['Content-Type'] = 'application/json'
@@ -335,8 +335,8 @@ class CgsnapshotsAPITestCase(test.TestCase):
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
-        self.assertEqual(http_client.BAD_REQUEST,
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST,
                          res_dict['badRequest']['code'])
         self.assertEqual('Invalid GroupSnapshot: invalid group_snapshot',
                          res_dict['badRequest']['message'])
@@ -363,7 +363,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
                                "CG Snapshot 1",
                                "consistencygroup_id": consistencygroup.id}}
 
-        req = webob.Request.blank('/v2/%s/cgsnapshots' % fake.PROJECT_ID)
+        req = webob.Request.blank('/v3/%s/cgsnapshots' % fake.PROJECT_ID)
         req.method = 'POST'
         req.headers['Content-Type'] = 'application/json'
         req.body = jsonutils.dump_as_bytes(body)
@@ -371,8 +371,8 @@ class CgsnapshotsAPITestCase(test.TestCase):
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.NOT_FOUND, res.status_int)
-        self.assertEqual(http_client.NOT_FOUND,
+        self.assertEqual(HTTPStatus.NOT_FOUND, res.status_int)
+        self.assertEqual(HTTPStatus.NOT_FOUND,
                          res_dict['itemNotFound']['code'])
         self.assertEqual('GroupSnapshot invalid_id could not be found.',
                          res_dict['itemNotFound']['message'])
@@ -393,7 +393,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
                                "CG Snapshot 1",
                                "consistencygroup_id": consistencygroup.id}}
 
-        req = webob.Request.blank('/v2/%s/cgsnapshots' % fake.PROJECT_ID)
+        req = webob.Request.blank('/v3/%s/cgsnapshots' % fake.PROJECT_ID)
         req.method = 'POST'
         req.headers['Content-Type'] = 'application/json'
         req.body = jsonutils.dump_as_bytes(body)
@@ -401,8 +401,8 @@ class CgsnapshotsAPITestCase(test.TestCase):
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
-        self.assertEqual(http_client.BAD_REQUEST,
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST,
                          res_dict['badRequest']['code'])
         self.assertIsNotNone(res_dict['badRequest']['message'])
 
@@ -427,7 +427,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
             self.context, group_id=consistencygroup.id,
             group_type_id=fake.GROUP_TYPE_ID,
             status='available')
-        req = webob.Request.blank('/v2/%s/cgsnapshots/%s' %
+        req = webob.Request.blank('/v3/%s/cgsnapshots/%s' %
                                   (fake.PROJECT_ID, cgsnapshot.id))
         req.method = 'DELETE'
         req.headers['Content-Type'] = 'application/json'
@@ -436,7 +436,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
 
         cgsnapshot = objects.GroupSnapshot.get_by_id(self.context,
                                                      cgsnapshot.id)
-        self.assertEqual(http_client.ACCEPTED, res.status_int)
+        self.assertEqual(HTTPStatus.ACCEPTED, res.status_int)
         self.assertEqual('deleting', cgsnapshot.status)
 
         cgsnapshot.destroy()
@@ -463,7 +463,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
             self.context, status='creating',
             group_snapshot_id=cgsnapshot.id,
             group_type_id=fake.GROUP_TYPE_ID)
-        req = webob.Request.blank('/v2/fake/cgsnapshots/%s' %
+        req = webob.Request.blank('/v3/fake/cgsnapshots/%s' %
                                   cgsnapshot.id)
         req.method = 'DELETE'
         req.headers['Content-Type'] = 'application/json'
@@ -471,7 +471,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
 
         cgsnapshot = objects.GroupSnapshot.get_by_id(self.context,
                                                      cgsnapshot.id)
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
         self.assertEqual('available', cgsnapshot.status)
 
         cgsnapshot.destroy()
@@ -480,7 +480,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
         cg2.destroy()
 
     def test_delete_cgsnapshot_with_cgsnapshot_NotFound(self):
-        req = webob.Request.blank('/v2/%s/cgsnapshots/%s' %
+        req = webob.Request.blank('/v3/%s/cgsnapshots/%s' %
                                   (fake.PROJECT_ID, fake.WILL_NOT_BE_FOUND_ID))
         req.method = 'DELETE'
         req.headers['Content-Type'] = 'application/json'
@@ -488,8 +488,8 @@ class CgsnapshotsAPITestCase(test.TestCase):
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.NOT_FOUND, res.status_int)
-        self.assertEqual(http_client.NOT_FOUND,
+        self.assertEqual(HTTPStatus.NOT_FOUND, res.status_int)
+        self.assertEqual(HTTPStatus.NOT_FOUND,
                          res_dict['itemNotFound']['code'])
         self.assertEqual('GroupSnapshot %s could not be found.' %
                          fake.WILL_NOT_BE_FOUND_ID,
@@ -511,7 +511,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
             group_type_id=fake.GROUP_TYPE_ID,
             status='invalid')
 
-        req = webob.Request.blank('/v2/%s/cgsnapshots/%s' % (
+        req = webob.Request.blank('/v3/%s/cgsnapshots/%s' % (
             fake.PROJECT_ID, cgsnapshot.id))
         req.method = 'DELETE'
         req.headers['Content-Type'] = 'application/json'
@@ -519,8 +519,8 @@ class CgsnapshotsAPITestCase(test.TestCase):
             fake_auth_context=self.user_ctxt))
         res_dict = jsonutils.loads(res.body)
 
-        self.assertEqual(http_client.BAD_REQUEST, res.status_int)
-        self.assertEqual(http_client.BAD_REQUEST,
+        self.assertEqual(HTTPStatus.BAD_REQUEST, res.status_int)
+        self.assertEqual(HTTPStatus.BAD_REQUEST,
                          res_dict['badRequest']['code'])
         self.assertIsNotNone(res_dict['badRequest']['message'])
 
@@ -546,7 +546,7 @@ class CgsnapshotsAPITestCase(test.TestCase):
             status='available')
         mock_delete.side_effect = exception.PolicyNotAuthorized(
             message='PolicyNotAuthorized')
-        req = webob.Request.blank('/v2/%s/cgsnapshots/%s' %
+        req = webob.Request.blank('/v3/%s/cgsnapshots/%s' %
                                   (fake.PROJECT_ID, cgsnapshot.id))
         req.method = 'DELETE'
         req.headers['Content-Type'] = 'application/json'
