@@ -27,37 +27,139 @@ from oslo_db.sqlalchemy import enginefacade
 from oslo_db.sqlalchemy import test_fixtures
 from oslo_db.sqlalchemy import test_migrations
 from oslo_db.sqlalchemy import utils as db_utils
+from oslo_log.fixture import logging_error as log_fixture
 from oslotest import base as test_base
 import sqlalchemy
 from sqlalchemy.engine import reflection
 
 import cinder.db.legacy_migrations
 from cinder.db import migration
+from cinder.db.sqlalchemy import models
+from cinder.tests import fixtures as cinder_fixtures
 from cinder.tests.unit import utils as test_utils
 from cinder.volume import volume_types
 
 
+class CinderModelsMigrationsSync(test_migrations.ModelsMigrationsSync):
+    """Test sqlalchemy-migrate migrations."""
+
+    # Migrations can take a long time, particularly on underpowered CI nodes.
+    # Give them some breathing room.
+    TIMEOUT_SCALING_FACTOR = 4
+
+    def setUp(self):
+        # Ensure BaseTestCase's ConfigureLogging fixture is disabled since
+        # we're using our own (StandardLogging).
+        with fixtures.EnvironmentVariable('OS_LOG_CAPTURE', '0'):
+            super().setUp()
+
+        self.useFixture(log_fixture.get_logging_handle_error_fixture())
+        self.useFixture(cinder_fixtures.WarningsFixture())
+        self.useFixture(cinder_fixtures.StandardLogging())
+
+        self.engine = enginefacade.writer.get_engine()
+
+    def db_sync(self, engine):
+        migration.db_sync(engine=self.engine)
+
+    def get_engine(self):
+        return self.engine
+
+    def get_metadata(self):
+        return models.BASE.metadata
+
+    def include_object(self, object_, name, type_, reflected, compare_to):
+        if type_ == 'table':
+            # migrate_version is a sqlalchemy-migrate control table and
+            # isn't included in the model
+            if name == 'migrate_version':
+                return False
+
+        return True
+
+    def filter_metadata_diff(self, diff):
+        # Overriding the parent method to decide on certain attributes
+        # that maybe present in the DB but not in the models.py
+
+        def include_element(element):
+            """Determine whether diff element should be excluded."""
+            if element[0] == 'modify_nullable':
+                table_name, column = element[2], element[3]
+                return (table_name, column) not in {
+                    # NOTE(stephenfin): This field has nullable=True set, but
+                    # since it's also a primary key (primary_key=True) the
+                    # resulting schema will still end up being "NOT NULL". This
+                    # weird combination was deemed necessary because MySQL will
+                    # otherwise set this to "NOT NULL DEFAULT ''" which may be
+                    # harmless but is inconsistent with other models. See the
+                    # migration for more information.
+                    ('encryption', 'encryption_id'),
+                    # NOTE(stephenfin): The nullability of these fields is
+                    # dependent on the backend, for some reason
+                    ('encryption', 'provider'),
+                    ('encryption', 'control_location'),
+                }
+
+            return True
+
+        return [element for element in diff if include_element(element[0])]
+
+
+class TestModelsSyncSQLite(
+    CinderModelsMigrationsSync,
+    test_fixtures.OpportunisticDBTestMixin,
+    test_base.BaseTestCase,
+):
+    pass
+
+
+class TestModelsSyncMySQL(
+    CinderModelsMigrationsSync,
+    test_fixtures.OpportunisticDBTestMixin,
+    test_base.BaseTestCase,
+):
+    FIXTURE = test_fixtures.MySQLOpportunisticFixture
+
+
+class TestModelsSyncPostgreSQL(
+    CinderModelsMigrationsSync,
+    test_fixtures.OpportunisticDBTestMixin,
+    test_base.BaseTestCase,
+):
+    FIXTURE = test_fixtures.PostgresqlOpportunisticFixture
+
+
 class MigrationsWalk(
     test_fixtures.OpportunisticDBTestMixin, test_base.BaseTestCase,
 ):
+    # Migrations can take a long time, particularly on underpowered CI nodes.
+    # Give them some breathing room.
+    TIMEOUT_SCALING_FACTOR = 4
+    BOOL_TYPE = sqlalchemy.types.BOOLEAN
+
     def setUp(self):
         super().setUp()
         self.engine = enginefacade.writer.get_engine()
         self.config = migration._find_alembic_conf()
         self.init_version = migration.ALEMBIC_INIT_VERSION
 
-    def _migrate_up(self, revision):
-        if revision == self.init_version:  # no tests for the initial revision
-            return
+    def _migrate_up(self, revision, connection):
+        check_method = getattr(self, f'_check_{revision}', None)
+        if revision != self.init_version:  # no tests for the initial revision
+            self.assertIsNotNone(
+                check_method,
+                f"API DB Migration {revision} doesn't have a test; add one"
+            )
+
+        pre_upgrade = getattr(self, f'_pre_upgrade_{revision}', None)
+        if pre_upgrade:
+            pre_upgrade(connection)
 
-        self.assertIsNotNone(
-            getattr(self, '_check_%s' % revision, None),
-            (
-                'API DB Migration %s does not have a test; you must add one'
-            ) % revision,
-        )
         alembic_api.upgrade(self.config, revision)
 
+        if check_method:
+            check_method(connection)
+
     def test_single_base_revision(self):
         """Ensure we only have a single base revision.
 
@@ -84,9 +186,11 @@ class MigrationsWalk(
         with self.engine.begin() as connection:
             self.config.attributes['connection'] = connection
             script = alembic_script.ScriptDirectory.from_config(self.config)
-            for revision_script in script.walk_revisions():
-                revision = revision_script.revision
-                self._migrate_up(revision)
+            revisions = list(script.walk_revisions())
+            # Need revisions from older to newer so the walk works as intended
+            revisions.reverse()
+            for revision_script in revisions:
+                self._migrate_up(revision_script.revision, connection)
 
     def test_db_version_alembic(self):
         migration.db_sync()
@@ -95,6 +199,20 @@ class MigrationsWalk(
         ).get_current_head()
         self.assertEqual(head, migration.db_version())
 
+    def _pre_upgrade_c92a3e68beed(self, connection):
+        """Test shared_targets is nullable."""
+        table = db_utils.get_table(connection, 'volumes')
+        self._previous_type = type(table.c.shared_targets.type)
+
+    def _check_c92a3e68beed(self, connection):
+        """Test shared_targets is nullable."""
+        table = db_utils.get_table(connection, 'volumes')
+        self.assertIn('shared_targets', table.c)
+        # Type hasn't changed
+        self.assertIsInstance(table.c.shared_targets.type, self._previous_type)
+        # But it's nullable
+        self.assertTrue(table.c.shared_targets.nullable)
+
 
 class TestMigrationsWalkSQLite(
     MigrationsWalk,
