@@ -36,7 +36,6 @@ class PurgeDeletedTest(test.TestCase):
         super(PurgeDeletedTest, self).setUp()
         self.context = context.get_admin_context()
         self.engine = db_api.get_engine()
-        self.session = db_api.get_session()
         self.conn = self.engine.connect()
         self.volumes = sqlalchemyutils.get_table(
             self.engine, "volumes")
@@ -114,107 +113,110 @@ class PurgeDeletedTest(test.TestCase):
 
         make_vol_now = self.volumes.update().\
             where(self.volumes.c.id.in_(self.uuidstrs[0:1]))\
-            .values(deleted_at=now)
+            .values(deleted_at=now, deleted=True)
         make_vol_old = self.volumes.update().\
             where(self.volumes.c.id.in_(self.uuidstrs[1:3]))\
-            .values(deleted_at=old)
+            .values(deleted_at=old, deleted=True)
         make_vol_older = self.volumes.update().\
             where(self.volumes.c.id.in_(self.uuidstrs[4:6]))\
-            .values(deleted_at=older)
+            .values(deleted_at=older, deleted=True)
         make_vol_meta_now = self.vm.update().\
             where(self.vm.c.volume_id.in_(self.uuidstrs[0:1]))\
-            .values(deleted_at=now)
+            .values(deleted_at=now, deleted=True)
         make_vol_meta_old = self.vm.update().\
             where(self.vm.c.volume_id.in_(self.uuidstrs[1:3]))\
-            .values(deleted_at=old)
+            .values(deleted_at=old, deleted=True)
         make_vol_meta_older = self.vm.update().\
             where(self.vm.c.volume_id.in_(self.uuidstrs[4:6]))\
-            .values(deleted_at=older)
+            .values(deleted_at=older, deleted=True)
 
         make_vol_types_now = self.vol_types.update().\
             where(self.vol_types.c.id.in_(self.uuidstrs[0:1]))\
-            .values(deleted_at=now)
+            .values(deleted_at=now, deleted=True)
         make_vol_types_old = self.vol_types.update().\
             where(self.vol_types.c.id.in_(self.uuidstrs[1:3]))\
-            .values(deleted_at=old)
+            .values(deleted_at=old, deleted=True)
         make_vol_types_older = self.vol_types.update().\
             where(self.vol_types.c.id.in_(self.uuidstrs[4:6]))\
-            .values(deleted_at=older)
+            .values(deleted_at=older, deleted=True)
         make_vol_type_proj_now = self.vol_type_proj.update().\
             where(self.vol_type_proj.c.volume_type_id.in_(self.uuidstrs[0:1]))\
-            .values(deleted_at=now)
+            .values(deleted_at=now, deleted=True)
         make_vol_type_proj_old = self.vol_type_proj.update().\
             where(self.vol_type_proj.c.volume_type_id.in_(self.uuidstrs[1:3]))\
-            .values(deleted_at=old)
+            .values(deleted_at=old, deleted=True)
         make_vol_type_proj_older = self.vol_type_proj.update().\
             where(self.vol_type_proj.c.volume_type_id.in_(self.uuidstrs[4:6]))\
-            .values(deleted_at=older)
+            .values(deleted_at=older, deleted=True)
 
         make_snap_now = self.snapshots.update().\
             where(self.snapshots.c.id.in_(self.uuidstrs[0:1]))\
-            .values(deleted_at=now)
+            .values(deleted_at=now, deleted=True)
         make_snap_old = self.snapshots.update().\
             where(self.snapshots.c.id.in_(self.uuidstrs[1:3]))\
-            .values(deleted_at=old)
+            .values(deleted_at=old, deleted=True)
         make_snap_older = self.snapshots.update().\
             where(self.snapshots.c.id.in_(self.uuidstrs[4:6]))\
-            .values(deleted_at=older)
+            .values(deleted_at=older, deleted=True)
 
         make_snap_meta_now = self.sm.update().\
             where(self.sm.c.snapshot_id.in_(self.uuidstrs[0:1]))\
-            .values(deleted_at=now)
+            .values(deleted_at=now, deleted=True)
         make_snap_meta_old = self.sm.update().\
             where(self.sm.c.snapshot_id.in_(self.uuidstrs[1:3]))\
-            .values(deleted_at=old)
+            .values(deleted_at=old, deleted=True)
         make_snap_meta_older = self.sm.update().\
             where(self.sm.c.snapshot_id.in_(self.uuidstrs[4:6]))\
-            .values(deleted_at=older)
+            .values(deleted_at=older, deleted=True)
 
         make_vol_glance_meta_now = self.vgm.update().\
             where(self.vgm.c.volume_id.in_(self.uuidstrs[0:1]))\
-            .values(deleted_at=now)
+            .values(deleted_at=now, deleted=True)
         make_vol_glance_meta_old = self.vgm.update().\
             where(self.vgm.c.volume_id.in_(self.uuidstrs[1:3]))\
-            .values(deleted_at=old)
+            .values(deleted_at=old, deleted=True)
         make_vol_glance_meta_older = self.vgm.update().\
             where(self.vgm.c.volume_id.in_(self.uuidstrs[4:6]))\
-            .values(deleted_at=older)
+            .values(deleted_at=older, deleted=True)
         make_snap_glance_meta_now = self.vgm.update().\
             where(self.vgm.c.snapshot_id.in_(self.uuidstrs[0:1]))\
-            .values(deleted_at=now)
+            .values(deleted_at=now, deleted=True)
         make_snap_glance_meta_old = self.vgm.update().\
             where(self.vgm.c.snapshot_id.in_(self.uuidstrs[1:3]))\
-            .values(deleted_at=old)
+            .values(deleted_at=old, deleted=True)
         make_snap_glance_meta_older = self.vgm.update().\
             where(self.vgm.c.snapshot_id.in_(self.uuidstrs[4:6]))\
-            .values(deleted_at=older)
+            .values(deleted_at=older, deleted=True)
 
         make_qos_now = self.qos.update().where(
-            self.qos.c.id.in_(self.uuidstrs[0:1])).values(deleted_at=now)
+            self.qos.c.id.in_(self.uuidstrs[0:1])).values(deleted_at=now,
+                                                          deleted=True)
         make_qos_old = self.qos.update().where(
-            self.qos.c.id.in_(self.uuidstrs[1:3])).values(deleted_at=old)
+            self.qos.c.id.in_(self.uuidstrs[1:3])).values(deleted_at=old,
+                                                          deleted=True)
         make_qos_older = self.qos.update().where(
-            self.qos.c.id.in_(self.uuidstrs[4:6])).values(deleted_at=older)
+            self.qos.c.id.in_(self.uuidstrs[4:6])).values(deleted_at=older,
+                                                          deleted=True)
 
         make_qos_child_record_now = self.qos.update().where(
             self.qos.c.specs_id.in_(self.uuidstrs[0:1])).values(
-            deleted_at=now)
+            deleted_at=now, deleted=True)
         make_qos_child_record_old = self.qos.update().where(
             self.qos.c.specs_id.in_(self.uuidstrs[1:3])).values(
-            deleted_at=old)
+            deleted_at=old, deleted=True)
         make_qos_child_record_older = self.qos.update().where(
             self.qos.c.specs_id.in_(self.uuidstrs[4:6])).values(
-            deleted_at=older)
+            deleted_at=older, deleted=True)
 
         make_vol_types1_now = self.vol_types.update().where(
             self.vol_types.c.qos_specs_id.in_(self.uuidstrs[0:1])).values(
-            deleted_at=now)
+            deleted_at=now, deleted=True)
         make_vol_types1_old = self.vol_types.update().where(
             self.vol_types.c.qos_specs_id.in_(self.uuidstrs[1:3])).values(
-            deleted_at=old)
+            deleted_at=old, deleted=True)
         make_vol_types1_older = self.vol_types.update().where(
             self.vol_types.c.qos_specs_id.in_(self.uuidstrs[4:6])).values(
-            deleted_at=older)
+            deleted_at=older, deleted=True)
 
         self.conn.execute(make_vol_now)
         self.conn.execute(make_vol_old)
@@ -259,25 +261,21 @@ class PurgeDeletedTest(test.TestCase):
     def test_purge_deleted_rows_in_zero_age_in(self):
         dialect = self.engine.url.get_dialect()
         if dialect == sqlite.dialect:
-            # We're seeing issues with foreign key support in SQLite 3.6.20
-            # SQLAlchemy doesn't support it at all with < SQLite 3.6.19
-            # It works fine in SQLite 3.7.
-            # Force foreign_key checking if running SQLite >= 3.7
-            import sqlite3
-            tup = sqlite3.sqlite_version_info
-            if tup[0] > 3 or (tup[0] == 3 and tup[1] >= 7):
-                self.conn.execute("PRAGMA foreign_keys = ON")
+            self.conn.exec_driver_sql("PRAGMA foreign_keys = ON")
         # Purge at age_in_days=0, should delete one more row
         db.purge_deleted_rows(self.context, age_in_days=0)
 
-        vol_rows = self.session.query(self.volumes).count()
-        vol_meta_rows = self.session.query(self.vm).count()
-        vol_type_rows = self.session.query(self.vol_types).count()
-        vol_type_proj_rows = self.session.query(self.vol_type_proj).count()
-        snap_rows = self.session.query(self.snapshots).count()
-        snap_meta_rows = self.session.query(self.sm).count()
-        vol_glance_meta_rows = self.session.query(self.vgm).count()
-        qos_rows = self.session.query(self.qos).count()
+        with db_api.main_context_manager.writer.using(self.context):
+            vol_rows = self.context.session.query(self.volumes).count()
+            vol_meta_rows = self.context.session.query(self.vm).count()
+            vol_type_rows = self.context.session.query(self.vol_types).count()
+            vol_type_proj_rows = self.context.session.query(
+                self.vol_type_proj,
+            ).count()
+            snap_rows = self.context.session.query(self.snapshots).count()
+            snap_meta_rows = self.context.session.query(self.sm).count()
+            vol_glance_meta_rows = self.context.session.query(self.vgm).count()
+            qos_rows = self.context.session.query(self.qos).count()
 
         # Verify that we only have 1 rows now
         self.assertEqual(1, vol_rows)
@@ -292,25 +290,21 @@ class PurgeDeletedTest(test.TestCase):
     def test_purge_deleted_rows_old(self):
         dialect = self.engine.url.get_dialect()
         if dialect == sqlite.dialect:
-            # We're seeing issues with foreign key support in SQLite 3.6.20
-            # SQLAlchemy doesn't support it at all with < SQLite 3.6.19
-            # It works fine in SQLite 3.7.
-            # Force foreign_key checking if running SQLite >= 3.7
-            import sqlite3
-            tup = sqlite3.sqlite_version_info
-            if tup[0] > 3 or (tup[0] == 3 and tup[1] >= 7):
-                self.conn.execute("PRAGMA foreign_keys = ON")
+            self.conn.exec_driver_sql("PRAGMA foreign_keys = ON")
         # Purge at 30 days old, should only delete 2 rows
         db.purge_deleted_rows(self.context, age_in_days=30)
 
-        vol_rows = self.session.query(self.volumes).count()
-        vol_meta_rows = self.session.query(self.vm).count()
-        vol_type_rows = self.session.query(self.vol_types).count()
-        vol_type_proj_rows = self.session.query(self.vol_type_proj).count()
-        snap_rows = self.session.query(self.snapshots).count()
-        snap_meta_rows = self.session.query(self.sm).count()
-        vol_glance_meta_rows = self.session.query(self.vgm).count()
-        qos_rows = self.session.query(self.qos).count()
+        with db_api.main_context_manager.writer.using(self.context):
+            vol_rows = self.context.session.query(self.volumes).count()
+            vol_meta_rows = self.context.session.query(self.vm).count()
+            vol_type_rows = self.context.session.query(self.vol_types).count()
+            vol_type_proj_rows = self.context.session.query(
+                self.vol_type_proj,
+            ).count()
+            snap_rows = self.context.session.query(self.snapshots).count()
+            snap_meta_rows = self.context.session.query(self.sm).count()
+            vol_glance_meta_rows = self.context.session.query(self.vgm).count()
+            qos_rows = self.context.session.query(self.qos).count()
 
         # Verify that we only deleted 2
         self.assertEqual(4, vol_rows)
@@ -325,25 +319,21 @@ class PurgeDeletedTest(test.TestCase):
     def test_purge_deleted_rows_older(self):
         dialect = self.engine.url.get_dialect()
         if dialect == sqlite.dialect:
-            # We're seeing issues with foreign key support in SQLite 3.6.20
-            # SQLAlchemy doesn't support it at all with < SQLite 3.6.19
-            # It works fine in SQLite 3.7.
-            # Force foreign_key checking if running SQLite >= 3.7
-            import sqlite3
-            tup = sqlite3.sqlite_version_info
-            if tup[0] > 3 or (tup[0] == 3 and tup[1] >= 7):
-                self.conn.execute("PRAGMA foreign_keys = ON")
+            self.conn.exec_driver_sql("PRAGMA foreign_keys = ON")
         # Purge at 10 days old now, should delete 2 more rows
         db.purge_deleted_rows(self.context, age_in_days=10)
 
-        vol_rows = self.session.query(self.volumes).count()
-        vol_meta_rows = self.session.query(self.vm).count()
-        vol_type_rows = self.session.query(self.vol_types).count()
-        vol_type_proj_rows = self.session.query(self.vol_type_proj).count()
-        snap_rows = self.session.query(self.snapshots).count()
-        snap_meta_rows = self.session.query(self.sm).count()
-        vol_glance_meta_rows = self.session.query(self.vgm).count()
-        qos_rows = self.session.query(self.qos).count()
+        with db_api.main_context_manager.writer.using(self.context):
+            vol_rows = self.context.session.query(self.volumes).count()
+            vol_meta_rows = self.context.session.query(self.vm).count()
+            vol_type_rows = self.context.session.query(self.vol_types).count()
+            vol_type_proj_rows = self.context.session.query(
+                self.vol_type_proj,
+            ).count()
+            snap_rows = self.context.session.query(self.snapshots).count()
+            snap_meta_rows = self.context.session.query(self.sm).count()
+            vol_glance_meta_rows = self.context.session.query(self.vgm).count()
+            qos_rows = self.context.session.query(self.qos).count()
 
         # Verify that we only have 2 rows now
         self.assertEqual(2, vol_rows)
@@ -366,16 +356,7 @@ class PurgeDeletedTest(test.TestCase):
     def test_purge_deleted_rows_integrity_failure(self):
         dialect = self.engine.url.get_dialect()
         if dialect == sqlite.dialect:
-            # We're seeing issues with foreign key support in SQLite 3.6.20
-            # SQLAlchemy doesn't support it at all with < SQLite 3.6.19
-            # It works fine in SQLite 3.7.
-            # So return early to skip this test if running SQLite < 3.7
-            import sqlite3
-            tup = sqlite3.sqlite_version_info
-            if tup[0] < 3 or (tup[0] == 3 and tup[1] < 7):
-                self.skipTest(
-                    'sqlite version too old for reliable SQLA foreign_keys')
-            self.conn.execute("PRAGMA foreign_keys = ON")
+            self.conn.exec_driver_sql("PRAGMA foreign_keys = ON")
 
         # add new entry in volume and volume_admin_metadata for
         # integrity check
@@ -389,7 +370,8 @@ class PurgeDeletedTest(test.TestCase):
         # set volume record to deleted 20 days ago
         old = timeutils.utcnow() - datetime.timedelta(days=20)
         make_old = self.volumes.update().where(
-            self.volumes.c.id.in_([uuid_str])).values(deleted_at=old)
+            self.volumes.c.id.in_([uuid_str])).values(deleted_at=old,
+                                                      deleted=True)
         self.conn.execute(make_old)
 
         # Verify that purge_deleted_rows fails due to Foreign Key constraint
