@@ -252,6 +252,36 @@ class TestGlanceImageService(test.TestCase):
         actual = self.service.show(self.context, image_id)
         self.assertDictEqual(expected, actual)
 
+    def test_create_without_is_public(self):
+        """Test Creating images without is_public.
+
+        Ensure we can create an image without is_public attribute.
+        """
+        fixture = {'name': 'test image',
+                   'protected': False}
+        image_id = self.service.create(self.context, fixture)['id']
+
+        expected = {
+            'id': image_id,
+            'name': 'test image',
+            'protected': False,
+            'size': None,
+            'min_disk': None,
+            'min_ram': None,
+            'disk_format': None,
+            'container_format': None,
+            'checksum': None,
+            'created_at': self.NOW_DATETIME,
+            'updated_at': self.NOW_DATETIME,
+            'deleted': None,
+            'status': None,
+            'properties': {},
+            'owner': None,
+            'visibility': None,
+        }
+        actual = self.service.show(self.context, image_id)
+        self.assertDictEqual(expected, actual)
+
     def test_show_shared_image_membership_success(self):
         """Test Create Shared Image Membership Success
 
@@ -614,6 +644,61 @@ class TestGlanceImageService(test.TestCase):
         }
         self.assertEqual(expected, image_meta)
 
+    def test_show_passes_when_is_admin_in_the_context(self):
+        fixture = self._make_fixture(name='image2')
+        image_id = self.service.create(self.context, fixture)['id']
+        self.context.auth_token = False
+        self.context.is_admin = True
+
+        image_meta = self.service.show(self.context, image_id)
+        expected = {
+            'id': image_id,
+            'name': 'image2',
+            'protected': None,
+            'size': None,
+            'min_disk': None,
+            'min_ram': None,
+            'disk_format': None,
+            'container_format': None,
+            'checksum': None,
+            'created_at': self.NOW_DATETIME,
+            'updated_at': self.NOW_DATETIME,
+            'deleted': None,
+            'status': None,
+            'properties': {'properties': {}},
+            'owner': None,
+            'visibility': None
+        }
+        self.assertEqual(expected, image_meta)
+
+    def test_show_passes_when_is_public_in_visibility_param(self):
+        fixture = self._make_fixture(name='image3')
+        fixture['visibility'] = 'public'
+        image_id = self.service.create(self.context, fixture)['id']
+        self.context.auth_token = False
+        self.context.is_admin = False
+
+        image_meta = self.service.show(self.context, image_id)
+        expected = {
+            'id': image_id,
+            'name': 'image3',
+            'protected': None,
+            'size': None,
+            'min_disk': None,
+            'min_ram': None,
+            'disk_format': None,
+            'container_format': None,
+            'checksum': None,
+            'created_at': self.NOW_DATETIME,
+            'updated_at': self.NOW_DATETIME,
+            'deleted': None,
+            'status': None,
+            'properties': {'properties': {}},
+            'owner': None,
+            'visibility': 'public'
+        }
+        self.assertEqual(expected, image_meta)
+
     def test_show_raises_when_no_authtoken_in_the_context(self):
         fixture = self._make_fixture(name='image1',
                                      is_public=False,
@@ -625,6 +710,30 @@ class TestGlanceImageService(test.TestCase):
                           self.context,
                           image_id)
 
+    def test_show_raises_when_no_is_admin_in_the_context(self):
+        fixture = self._make_fixture(name='image2',
+                                     is_public=False,
+                                     protected=False)
+        image_id = self.service.create(self.context, fixture)['id']
+        self.context.auth_token = False
+        self.context.is_admin = False
+        self.assertRaises(exception.ImageNotFound,
+                          self.service.show,
+                          self.context,
+                          image_id)
+
+    def test_show_raises_when_is_private_in_visibility_param(self):
+        fixture = self._make_fixture(name='image3',
+                                     protected=False)
+        fixture['visibility'] = 'private'
+        image_id = self.service.create(self.context, fixture)['id']
+        self.context.auth_token = False
+        self.context.is_admin = False
+        self.assertRaises(exception.ImageNotFound,
+                          self.service.show,
+                          self.context,
+                          image_id)
+
     def test_detail_passes_through_to_client(self):
         fixture = self._make_fixture(name='image10', is_public=True)
         image_id = self.service.create(self.context, fixture)['id']
@@ -875,6 +984,29 @@ class TestGlanceImageService(test.TestCase):
         attribute as the client would return if they're not set in the
         database. Regression test for bug #1308058.
         """
+        def _extract_attributes(image):
+            IMAGE_ATTRIBUTES = ('size', 'disk_format', 'owner',
+                                'container_format', 'status', 'id',
+                                'name', 'created_at', 'updated_at',
+                                'deleted', 'deleted_at', 'checksum',
+                                'min_disk', 'min_ram', 'protected',
+                                'visibility',
+                                'cinder_encryption_key_id')
+
+            output = {}
+
+            for attr in IMAGE_ATTRIBUTES:
+                if attr == 'deleted_at' and not output['deleted']:
+                    output[attr] = None
+                elif attr == 'checksum' and output['status'] != 'active':
+                    output[attr] = None
+                else:
+                    output[attr] = getattr(image, attr, None)
+
+            output['properties'] = getattr(image, 'properties', {})
+
+            return output
+
         class MyFakeGlanceImage(glance_stubs.FakeImage):
             def __init__(self, metadata):
                 IMAGE_ATTRIBUTES = ['size', 'disk_format', 'owner',
@@ -892,7 +1024,7 @@ class TestGlanceImageService(test.TestCase):
             'updated_at': self.NOW_DATETIME,
         }
         image = MyFakeGlanceImage(metadata)
-        actual = glance._extract_attributes(image)
+        actual = _extract_attributes(image)
         expected = {
             'id': 1,
             'name': None,
@@ -1099,8 +1231,8 @@ class TestGlanceImageServiceClient(test.TestCase):
         self.flags(auth_strategy='keystone')
         self.flags(glance_request_timeout=None)
 
-        class MyGlanceStubClient(object):
-            def __init__(inst, version, *args, **kwargs):
+        class MyGlanceStubClient(test.TestCase):
+            def __init__(self, version, *args, **kwargs):
                 self.assertEqual('2', version)
                 self.assertEqual("http://fake_host:9292", args[0])
                 self.assertNotIn('timeout', kwargs)
@@ -1135,8 +1267,8 @@ class TestGlanceImageServiceClient(test.TestCase):
         self.flags(glance_certfile='/opt/stack/data/cert.pem')
         self.flags(glance_keyfile='/opt/stack/data/key.pem')
 
-        class MyGlanceStubClient(object):
-            def __init__(inst, version, *args, **kwargs):
+        class MyGlanceStubClient(test.TestCase):
+            def __init__(self, version, *args, **kwargs):
                 self.assertEqual('2', version)
                 self.assertEqual("https://fake_host:9292", args[0])
                 self.assertNotIn('timeout', kwargs)
@@ -1167,8 +1299,8 @@ class TestGlanceImageServiceClient(test.TestCase):
         self.flags(
             glance_ca_certificates_file='/opt/stack/data/ca-bundle.pem')
 
-        class MyGlanceStubClient(object):
-            def __init__(inst, version, *args, **kwargs):
+        class MyGlanceStubClient(test.TestCase):
+            def __init__(self, version, *args, **kwargs):
                 self.assertEqual('2', version)
                 self.assertEqual('https://fake_host:9292', args[0])
                 self.assertEqual(60, kwargs['timeout'])
@@ -1188,8 +1320,8 @@ class TestGlanceImageServiceClient(test.TestCase):
         self.flags(auth_strategy='noauth')
         self.flags(glance_request_timeout=None)
 
-        class MyGlanceStubClient(object):
-            def __init__(inst, version, *args, **kwargs):
+        class MyGlanceStubClient(test.TestCase):
+            def __init__(self, version, *args, **kwargs):
                 self.assertEqual('2', version)
                 self.assertEqual("http://fake_host:9292", args[0])
                 self.assertNotIn('timeout', kwargs)
