@@ -45,9 +45,9 @@ object_data = {
     'RequestSpec': '1.5-2f6efbb86107ee70cc1bb07f4bdb4ec7',
     'Service': '1.6-e881b6b324151dd861e09cdfffcdaccd',
     'ServiceList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
-    'Snapshot': '1.5-ac1cdbd5b89588f6a8f44afdf6b8b201',
+    'Snapshot': '1.6-a2a1b62ae7e8d2794359ae59aff47ff6',
     'SnapshotList': '1.0-15ecf022a68ddbb8c2a6739cfc9f8f5e',
-    'Volume': '1.8-6cf615b72269cef48702a2a5c2940159',
+    'Volume': '1.9-4e25e166fa38bfcf039dcac1b19465b1',
     'VolumeList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
     'VolumeAttachment': '1.3-e6a3f7c5590d19f1e3ff6f819fbe6593',
     'VolumeAttachmentList': '1.1-15ecf022a68ddbb8c2a6739cfc9f8f5e',
@@ -80,6 +80,10 @@ class TestObjectVersions(test.TestCase):
                          "and we just have to change the hash in this module.")
 
     def test_versions_history(self):
+        # If we inserted a fake element in history, remove it so we don't fail
+        if base.OBJ_VERSIONS.get_current() == self.FAKE_OVO_HISTORY_VERSION:
+            fake_version = base.OBJ_VERSIONS.versions.pop(-1)
+            del base.OBJ_VERSIONS[fake_version]
         classes = base.CinderObjectRegistry.obj_classes()
         versions = base.OBJ_VERSIONS.get_current_versions()
         expected = {}
@@ -101,24 +105,50 @@ class TestObjectVersions(test.TestCase):
         # db model and object match.
         def _check_table_matched(db_model, cls):
             for column in db_model.__table__.columns:
+                # Ignore columns that aren't reflected by the model
+                if column.name not in cls.fields:
+                    continue
 
                 # NOTE(jdg): Model and Object don't match intentionally here
-                if (column.name in cls.fields and
-                        (column.name == 'uuid' and name == 'Service')):
+                if name == 'Service' and column.name == 'uuid':
                     continue
 
                 # NOTE(xyang): Skip the comparison of the colume name
                 # group_type_id in table Group because group_type_id
                 # is in the object Group but it is stored in a different
                 # table in the database, not in the Group table.
-                if (column.name in cls.fields and
-                        (column.name != 'group_type_id' and name != 'Group')):
-                    self.assertEqual(
-                        column.nullable,
-                        cls.fields[column.name].nullable,
-                        'Column %(c)s in table %(t)s not match.'
-                        % {'c': column.name,
-                           't': name})
+                if name == 'Group' and column.name == 'group_type_id':
+                    continue
+
+                # TODO(stephenfin): Model and Object don't match here, but it
+                # wasn't intentional
+                if (name, column.name) in {
+                    ('Backup', 'project_id'),
+                    ('Backup', 'user_id'),
+                    ('BackupImport', 'project_id'),
+                    ('BackupImport', 'user_id'),
+                    ('CGSnapshot', 'project_id'),
+                    ('CGSnapshot', 'user_id'),
+                    ('CGSnapshot', 'consistencygroup_id'),
+                    ('ConsistencyGroup', 'project_id'),
+                    ('ConsistencyGroup', 'user_id'),
+                    ('Group', 'user_id'),
+                    ('Group', 'project_id'),
+                    ('GroupType', 'name'),
+                    ('Service', 'frozen'),
+                    ('Snapshot', 'volume_id'),
+                    ('Snapshot', 'volume_type_id'),
+                    ('Volume', 'volume_type_id'),
+                }:
+                    continue
+
+                self.assertEqual(
+                    column.nullable,
+                    cls.fields[column.name].nullable,
+                    'Column %(c)s in table %(t)s not match.' % {
+                        'c': column.name, 't': name,
+                    },
+                )
 
         classes = base.CinderObjectRegistry.obj_classes()
         for name, cls in classes.items():
