@@ -38,7 +38,6 @@ from oslo_messaging import conffixture as messaging_conffixture
 from oslo_serialization import jsonutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import six
 import testtools
 
 from cinder.api import common as api_common
@@ -52,8 +51,10 @@ from cinder.objects import base as objects_base
 from cinder import rpc
 from cinder import service
 from cinder.tests import fixtures as cinder_fixtures
+from cinder.tests import unit as cinder_unit
 from cinder.tests.unit import conf_fixture
 from cinder.tests.unit import fake_notifier
+from cinder.tests.unit import known_issues as issues
 from cinder.volume import configuration
 from cinder.volume import driver as vol_driver
 from cinder.volume import volume_types
@@ -62,6 +63,7 @@ from cinder.volume import volume_utils
 CONF = config.CONF
 
 _DB_CACHE = None
+DB_SCHEMA = None
 SESSION_CONFIGURED = False
 
 
@@ -71,32 +73,41 @@ class TestingException(Exception):
 
 class Database(fixtures.Fixture):
 
-    def __init__(self, db_api, db_migrate, sql_connection):
+    def __init__(self):
+        super().__init__()
+
         # NOTE(lhx_): oslo_db.enginefacade is configured in tests the same
         # way as it's done for any other services that uses the db
         global SESSION_CONFIGURED
         if not SESSION_CONFIGURED:
             sqla_api.configure(CONF)
             SESSION_CONFIGURED = True
-        self.sql_connection = sql_connection
 
         # Suppress logging for test runs
         migrate_logger = logging.getLogger('migrate')
         migrate_logger.setLevel(logging.WARNING)
 
-        self.engine = db_api.get_engine()
-        self.engine.dispose()
-        conn = self.engine.connect()
-        db_migrate.db_sync()
-        self._DB = "".join(line for line in conn.connection.iterdump())
-        self.engine.dispose()
-
     def setUp(self):
-        super(Database, self).setUp()
+        super().setUp()
+        engine = sqla_api.get_engine()
+        engine.dispose()
+        self._cache_schema()
+        conn = engine.connect()
+        conn.connection.executescript(DB_SCHEMA)
+        self.addCleanup(self.cleanup)
 
-        conn = self.engine.connect()
-        conn.connection.executescript(self._DB)
-        self.addCleanup(self.engine.dispose)
+    def _cache_schema(self):
+        global DB_SCHEMA
+        if not DB_SCHEMA:
+            engine = sqla_api.get_engine()
+            conn = engine.connect()
+            migration.db_sync()
+            DB_SCHEMA = "".join(line for line in conn.connection.iterdump())
+            engine.dispose()
+
+    def cleanup(self):
+        engine = sqla_api.get_engine()
+        engine.dispose()
 
 
 class TestCase(testtools.TestCase):
@@ -107,14 +118,16 @@ class TestCase(testtools.TestCase):
             os.path.dirname(__file__),
             '../../../'))
     POLICY_PATH = os.path.join(SOURCE_TREE_ROOT,
-                               'cinder/tests/unit/policy.json')
+                               'cinder/tests/unit/policy.yaml')
     RESOURCE_FILTER_FILENAME = 'etc/cinder/resource_filters.json'
     RESOURCE_FILTER_PATH = os.path.join(SOURCE_TREE_ROOT,
                                         RESOURCE_FILTER_FILENAME)
     MOCK_WORKER = True
     MOCK_TOOZ = True
+    FAKE_OVO_HISTORY_VERSION = '9999.999'
 
     def __init__(self, *args, **kwargs):
+
         super(TestCase, self).__init__(*args, **kwargs)
 
         # Suppress some log messages during test runs
@@ -230,11 +243,7 @@ class TestCase(testtools.TestCase):
         CONF.set_default('connection', 'sqlite://', 'database')
         CONF.set_default('sqlite_synchronous', False, 'database')
 
-        global _DB_CACHE
-        if not _DB_CACHE:
-            _DB_CACHE = Database(sqla_api, migration,
-                                 sql_connection=CONF.database.connection)
-        self.useFixture(_DB_CACHE)
+        self.useFixture(Database())
 
         # NOTE(blk-u): WarningsFixture must be after the Database fixture
         # because sqlalchemy-migrate messes with the warnings filters.
@@ -283,18 +292,6 @@ class TestCase(testtools.TestCase):
         coordination.COORDINATOR.start()
         self.addCleanup(coordination.COORDINATOR.stop)
 
-        if six.PY3:
-            # TODO(smcginnis) Python 3 deprecates assertRaisesRegexp to
-            # assertRaisesRegex, but Python 2 does not have the new name. This
-            # can be removed once we stop supporting py2 or the new name is
-            # added.
-            self.assertRaisesRegexp = self.assertRaisesRegex
-
-        # Ensure we have the default tpool size value and we don't carry
-        # threads from other test runs.
-        tpool.killall()
-        tpool._nthreads = 20
-
         # NOTE(mikal): make sure we don't load a privsep helper accidentally
         self.useFixture(cinder_fixtures.PrivsepNoHelperFixture())
 
@@ -303,6 +300,16 @@ class TestCase(testtools.TestCase):
 
         self.vt = volume_types.get_default_volume_type()
 
+        # Create fake RPC history if we don't have enough to do tests
+        obj_versions = objects_base.OBJ_VERSIONS
+        if len(obj_versions) == 1:
+            vol_vers = obj_versions.get_current_versions()['Volume'].split('.')
+            new_volume_version = '%s.%s' % (vol_vers[0], int(vol_vers[1]) + 1)
+            obj_versions.add(self.FAKE_OVO_HISTORY_VERSION,
+                             {'Volume': new_volume_version})
+
+        self.latest_ovo_version = obj_versions.get_current()
+
     def _restore_obj_registry(self):
         objects_base.CinderObjectRegistry._registry._obj_classes = \
             self._base_test_obj_backup
@@ -323,19 +330,28 @@ class TestCase(testtools.TestCase):
     def _common_cleanup(self):
         """Runs after each test method to tear down test environment."""
 
-        # Kill any services
+        # Stop any services (this stops RPC handlers)
         for x in self._services:
             try:
-                x.kill()
+                x.stop()
             except Exception:
                 pass
 
+        # Stop any looping call that has not yet been stopped
+        cinder_unit.stop_looping_calls()
+
         # Delete attributes that don't start with _ so they don't pin
         # memory around unnecessarily for the duration of the test
         # suite
         for key in [k for k in self.__dict__ if k[0] != '_']:
             del self.__dict__[key]
 
+        if not issues.TPOOL_KILLALL_ISSUE:
+            # Ensure we have the default tpool size value and we don't carry
+            # threads from other test runs.
+            tpool.killall()
+            tpool._nthreads = 20
+
     def override_config(self, name, override, group=None):
         """Cleanly override CONF variables."""
         CONF.set_override(name, override, group)
@@ -471,7 +487,7 @@ class ModelsObjectComparatorMixin(object):
 
         self.assertEqual(
             len(obj1), len(obj2),
-            "Keys mismatch: %s" % six.text_type(
+            "Keys mismatch: %s" % str(
                 set(obj1.keys()) ^ set(obj2.keys())))
         for key, value in obj1.items():
             self.assertEqual(value, obj2[key])
