@@ -13,20 +13,18 @@
 #    under the License.
 
 import datetime
-import functools
 import json
 import os
 import sys
-import time
 from unittest import mock
 
 import ddt
 from oslo_utils import timeutils
-import six
 import webob.exc
 
 import cinder
 from cinder.api import api_utils
+from cinder import context
 from cinder import exception
 from cinder.tests.unit import fake_constants as fake
 from cinder.tests.unit import test
@@ -110,16 +108,6 @@ class GenericUtilsTestCase(test.TestCase):
                           unit=False,
                           pretty_keys=False)
 
-    def test_require_driver_intialized(self):
-        driver = mock.Mock()
-        driver.initialized = True
-        utils.require_driver_initialized(driver)
-
-        driver.initialized = False
-        self.assertRaises(exception.DriverNotInitialized,
-                          utils.require_driver_initialized,
-                          driver)
-
     def test_hostname_unicode_sanitization(self):
         hostname = u"\u7684.test.example.com"
         self.assertEqual("test.example.com",
@@ -300,12 +288,12 @@ class TemporaryChownTestCase(test.TestCase):
         mock_stat.return_value.st_uid = 5678
         test_filename = 'a_file'
         with utils.temporary_chown(test_filename):
-            mock_exec.assert_called_once_with('chown', 1234, test_filename,
+            mock_exec.assert_called_once_with('chown', '1234', test_filename,
                                               run_as_root=True)
         mock_getuid.assert_called_once_with()
         mock_stat.assert_called_once_with(test_filename)
-        calls = [mock.call('chown', 1234, test_filename, run_as_root=True),
-                 mock.call('chown', 5678, test_filename, run_as_root=True)]
+        calls = [mock.call('chown', '1234', test_filename, run_as_root=True),
+                 mock.call('chown', '5678', test_filename, run_as_root=True)]
         mock_exec.assert_has_calls(calls)
 
     @mock.patch('os.stat')
@@ -315,12 +303,12 @@ class TemporaryChownTestCase(test.TestCase):
         mock_stat.return_value.st_uid = 5678
         test_filename = 'a_file'
         with utils.temporary_chown(test_filename, owner_uid=9101):
-            mock_exec.assert_called_once_with('chown', 9101, test_filename,
+            mock_exec.assert_called_once_with('chown', '9101', test_filename,
                                               run_as_root=True)
         self.assertFalse(mock_getuid.called)
         mock_stat.assert_called_once_with(test_filename)
-        calls = [mock.call('chown', 9101, test_filename, run_as_root=True),
-                 mock.call('chown', 5678, test_filename, run_as_root=True)]
+        calls = [mock.call('chown', '9101', test_filename, run_as_root=True),
+                 mock.call('chown', '5678', test_filename, run_as_root=True)]
         mock_exec.assert_has_calls(calls)
 
     @mock.patch('os.stat')
@@ -724,13 +712,13 @@ class AuditPeriodTest(test.TestCase):
 class BrickUtils(test.TestCase):
     """Unit test to test the brick utility wrapper functions."""
 
-    @mock.patch('cinder.utils.CONF')
+    @mock.patch('cinder.volume.volume_utils.CONF')
     @mock.patch('os_brick.initiator.connector.get_connector_properties')
     @mock.patch('cinder.utils.get_root_helper')
     def test_brick_get_connector_properties(self, mock_helper, mock_get,
                                             mock_conf):
         mock_conf.my_ip = '1.2.3.4'
-        output = utils.brick_get_connector_properties()
+        output = volume_utils.brick_get_connector_properties()
         mock_helper.assert_called_once_with()
         mock_get.assert_called_once_with(mock_helper.return_value, '1.2.3.4',
                                          False, False)
@@ -739,7 +727,7 @@ class BrickUtils(test.TestCase):
     @mock.patch('os_brick.initiator.connector.InitiatorConnector.factory')
     @mock.patch('cinder.utils.get_root_helper')
     def test_brick_get_connector(self, mock_helper, mock_factory):
-        output = utils.brick_get_connector('protocol')
+        output = volume_utils.brick_get_connector('protocol')
         mock_helper.assert_called_once_with()
         self.assertEqual(mock_factory.return_value, output)
         mock_factory.assert_called_once_with(
@@ -757,7 +745,9 @@ class BrickUtils(test.TestCase):
         ctxt = mock.Mock(name='context')
         mock_encryptor = mock.Mock()
         mock_get_encryptor.return_value = mock_encryptor
-        utils.brick_attach_volume_encryptor(ctxt, attach_info, encryption)
+        volume_utils.brick_attach_volume_encryptor(ctxt,
+                                                   attach_info,
+                                                   encryption)
 
         connection_info = attach_info['conn']
         connection_info['data']['device_path'] = attach_info['device']['path']
@@ -780,7 +770,7 @@ class BrickUtils(test.TestCase):
         encryption = {'encryption_key_id': fake.ENCRYPTION_KEY_ID}
         mock_encryptor = mock.Mock()
         mock_get_encryptor.return_value = mock_encryptor
-        utils.brick_detach_volume_encryptor(attach_info, encryption)
+        volume_utils.brick_detach_volume_encryptor(attach_info, encryption)
 
         mock_helper.assert_called_once_with()
         connection_info = attach_info['conn']
@@ -1086,339 +1076,17 @@ class TestRetryDecorator(test.TestCase):
 
 
 @ddt.ddt
-class LogTracingTestCase(test.TestCase):
-
-    def test_utils_setup_tracing(self):
-        self.mock_object(utils, 'LOG')
-
-        utils.setup_tracing(None)
-        self.assertFalse(utils.TRACE_API)
-        self.assertFalse(utils.TRACE_METHOD)
-        self.assertEqual(0, utils.LOG.warning.call_count)
-
-        utils.setup_tracing(['method'])
-        self.assertFalse(utils.TRACE_API)
-        self.assertTrue(utils.TRACE_METHOD)
-        self.assertEqual(0, utils.LOG.warning.call_count)
-
-        utils.setup_tracing(['method', 'api'])
-        self.assertTrue(utils.TRACE_API)
-        self.assertTrue(utils.TRACE_METHOD)
-        self.assertEqual(0, utils.LOG.warning.call_count)
-
-    def test_utils_setup_tracing_invalid_key(self):
-        self.mock_object(utils, 'LOG')
-
-        utils.setup_tracing(['fake'])
-
-        self.assertFalse(utils.TRACE_API)
-        self.assertFalse(utils.TRACE_METHOD)
-        self.assertEqual(1, utils.LOG.warning.call_count)
-
-    def test_utils_setup_tracing_valid_and_invalid_key(self):
-        self.mock_object(utils, 'LOG')
-
-        utils.setup_tracing(['method', 'fake'])
-
-        self.assertFalse(utils.TRACE_API)
-        self.assertTrue(utils.TRACE_METHOD)
-        self.assertEqual(1, utils.LOG.warning.call_count)
-
-    def test_trace_no_tracing(self):
-        self.mock_object(utils, 'LOG')
-
-        @utils.trace_method
-        def _trace_test_method(*args, **kwargs):
-            return 'OK'
-
-        utils.setup_tracing(None)
-
-        result = _trace_test_method()
-
-        self.assertEqual('OK', result)
-        self.assertEqual(0, utils.LOG.debug.call_count)
-
-    def test_utils_trace_method(self):
-        self.mock_object(utils, 'LOG')
-
-        @utils.trace_method
-        def _trace_test_method(*args, **kwargs):
-            return 'OK'
-
-        utils.setup_tracing(['method'])
-
-        result = _trace_test_method()
-        self.assertEqual('OK', result)
-        self.assertEqual(2, utils.LOG.debug.call_count)
-
-    def test_utils_trace_api(self):
-        self.mock_object(utils, 'LOG')
-
-        @utils.trace_api
-        def _trace_test_api(*args, **kwargs):
-            return 'OK'
-
-        utils.setup_tracing(['api'])
-
-        result = _trace_test_api()
-        self.assertEqual('OK', result)
-        self.assertEqual(2, utils.LOG.debug.call_count)
-
-    def test_utils_trace_api_filtered(self):
-        self.mock_object(utils, 'LOG')
-
-        def filter_func(all_args):
-            return False
-
-        @utils.trace_api(filter_function=filter_func)
-        def _trace_test_api(*args, **kwargs):
-            return 'OK'
-
-        utils.setup_tracing(['api'])
-
-        result = _trace_test_api()
-        self.assertEqual('OK', result)
-        self.assertEqual(0, utils.LOG.debug.call_count)
-
-    def test_utils_trace_filtered(self):
-        self.mock_object(utils, 'LOG')
-
-        def filter_func(all_args):
-            return False
-
-        @utils.trace(filter_function=filter_func)
-        def _trace_test(*args, **kwargs):
-            return 'OK'
-
-        utils.setup_tracing(['api'])
-
-        result = _trace_test()
-        self.assertEqual('OK', result)
-        self.assertEqual(0, utils.LOG.debug.call_count)
-
-    def test_utils_trace_method_default_logger(self):
-        mock_log = self.mock_object(utils, 'LOG')
-
-        @utils.trace_method
-        def _trace_test_method_custom_logger(*args, **kwargs):
-            return 'OK'
-        utils.setup_tracing(['method'])
-
-        result = _trace_test_method_custom_logger()
-
-        self.assertEqual('OK', result)
-        self.assertEqual(2, mock_log.debug.call_count)
-
-    def test_utils_trace_method_inner_decorator(self):
-        mock_logging = self.mock_object(utils, 'logging')
-        mock_log = mock.Mock()
-        mock_log.isEnabledFor = lambda x: True
-        mock_logging.getLogger = mock.Mock(return_value=mock_log)
-
-        def _test_decorator(f):
-            def blah(*args, **kwargs):
-                return f(*args, **kwargs)
-            return blah
-
-        @_test_decorator
-        @utils.trace_method
-        def _trace_test_method(*args, **kwargs):
-            return 'OK'
-
-        utils.setup_tracing(['method'])
-
-        result = _trace_test_method(self)
-
-        self.assertEqual('OK', result)
-        self.assertEqual(2, mock_log.debug.call_count)
-        # Ensure the correct function name was logged
-        for call in mock_log.debug.call_args_list:
-            self.assertIn('_trace_test_method', str(call))
-            self.assertNotIn('blah', str(call))
-
-    def test_utils_trace_method_outer_decorator(self):
-        mock_logging = self.mock_object(utils, 'logging')
-        mock_log = mock.Mock()
-        mock_log.isEnabledFor = lambda x: True
-        mock_logging.getLogger = mock.Mock(return_value=mock_log)
-
-        def _test_decorator(f):
-            def blah(*args, **kwargs):
-                return f(*args, **kwargs)
-            return blah
-
-        @utils.trace_method
-        @_test_decorator
-        def _trace_test_method(*args, **kwargs):
-            return 'OK'
-
-        utils.setup_tracing(['method'])
-
-        result = _trace_test_method(self)
-
-        self.assertEqual('OK', result)
-        self.assertEqual(2, mock_log.debug.call_count)
-        # Ensure the incorrect function name was logged
-        for call in mock_log.debug.call_args_list:
-            self.assertNotIn('_trace_test_method', str(call))
-            self.assertIn('blah', str(call))
-
-    def test_utils_trace_method_outer_decorator_with_functools(self):
-        mock_log = mock.Mock()
-        mock_log.isEnabledFor = lambda x: True
-        self.mock_object(utils.logging, 'getLogger', mock_log)
-        mock_log = self.mock_object(utils, 'LOG')
-
-        def _test_decorator(f):
-            @functools.wraps(f)
-            def wraps(*args, **kwargs):
-                return f(*args, **kwargs)
-            return wraps
-
-        @utils.trace_method
-        @_test_decorator
-        def _trace_test_method(*args, **kwargs):
-            return 'OK'
-
-        utils.setup_tracing(['method'])
-
-        result = _trace_test_method()
-
-        self.assertEqual('OK', result)
-        self.assertEqual(2, mock_log.debug.call_count)
-        # Ensure the incorrect function name was logged
-        for call in mock_log.debug.call_args_list:
-            self.assertIn('_trace_test_method', str(call))
-            self.assertNotIn('wraps', str(call))
-
-    def test_utils_trace_method_with_exception(self):
-        self.LOG = self.mock_object(utils, 'LOG')
-
-        @utils.trace_method
-        def _trace_test_method(*args, **kwargs):
-            raise exception.APITimeout('test message')
-
-        utils.setup_tracing(['method'])
-
-        self.assertRaises(exception.APITimeout, _trace_test_method)
-
-        exception_log = self.LOG.debug.call_args_list[1]
-        self.assertIn('exception', str(exception_log))
-        self.assertIn('test message', str(exception_log))
-
-    def test_utils_trace_method_with_time(self):
-        mock_logging = self.mock_object(utils, 'logging')
-        mock_log = mock.Mock()
-        mock_log.isEnabledFor = lambda x: True
-        mock_logging.getLogger = mock.Mock(return_value=mock_log)
-
-        mock_time = mock.Mock(side_effect=[3.1, 6])
-        self.mock_object(time, 'time', mock_time)
-
-        @utils.trace_method
-        def _trace_test_method(*args, **kwargs):
-            return 'OK'
-
-        utils.setup_tracing(['method'])
-
-        result = _trace_test_method(self)
-
-        self.assertEqual('OK', result)
-        return_log = mock_log.debug.call_args_list[1]
-        self.assertIn('2900', str(return_log))
-
-    def test_utils_trace_wrapper_class(self):
-        mock_logging = self.mock_object(utils, 'logging')
-        mock_log = mock.Mock()
-        mock_log.isEnabledFor = lambda x: True
-        mock_logging.getLogger = mock.Mock(return_value=mock_log)
-
-        utils.setup_tracing(['method'])
-
-        @six.add_metaclass(utils.TraceWrapperMetaclass)
-        class MyClass(object):
-            def trace_test_method(self):
-                return 'OK'
-
-        test_class = MyClass()
-        result = test_class.trace_test_method()
-
-        self.assertEqual('OK', result)
-        self.assertEqual(2, mock_log.debug.call_count)
-
-    def test_utils_trace_method_with_password_dict(self):
-        mock_logging = self.mock_object(utils, 'logging')
-        mock_log = mock.Mock()
-        mock_log.isEnabledFor = lambda x: True
-        mock_logging.getLogger = mock.Mock(return_value=mock_log)
-
-        @utils.trace_method
-        def _trace_test_method(*args, **kwargs):
-            return {'something': 'test',
-                    'password': 'Now you see me'}
-
-        utils.setup_tracing(['method'])
-        result = _trace_test_method(self)
-        expected_unmasked_dict = {'something': 'test',
-                                  'password': 'Now you see me'}
-
-        self.assertEqual(expected_unmasked_dict, result)
-        self.assertEqual(2, mock_log.debug.call_count)
-        self.assertIn("'password': '***'",
-                      str(mock_log.debug.call_args_list[1]))
-
-    def test_utils_trace_method_with_password_str(self):
-        mock_logging = self.mock_object(utils, 'logging')
-        mock_log = mock.Mock()
-        mock_log.isEnabledFor = lambda x: True
-        mock_logging.getLogger = mock.Mock(return_value=mock_log)
-
-        @utils.trace_method
-        def _trace_test_method(*args, **kwargs):
-            return "'adminPass': 'Now you see me'"
-
-        utils.setup_tracing(['method'])
-        result = _trace_test_method(self)
-        expected_unmasked_str = "'adminPass': 'Now you see me'"
-
-        self.assertEqual(expected_unmasked_str, result)
-        self.assertEqual(2, mock_log.debug.call_count)
-        self.assertIn("'adminPass': '***'",
-                      str(mock_log.debug.call_args_list[1]))
-
-    def test_utils_trace_method_with_password_in_formal_params(self):
-        mock_logging = self.mock_object(utils, 'logging')
-        mock_log = mock.Mock()
-        mock_log.isEnabledFor = lambda x: True
-        mock_logging.getLogger = mock.Mock(return_value=mock_log)
-
-        @utils.trace
-        def _trace_test_method(*args, **kwargs):
-            self.assertEqual('verybadpass',
-                             kwargs['test_args']['data']['password'])
-            pass
-
-        test_args = {
-            'data': {
-                'password': 'verybadpass'
-            }
-        }
-        _trace_test_method(self, test_args=test_args)
-
-        self.assertEqual(2, mock_log.debug.call_count)
-        self.assertIn("'password': '***'",
-                      str(mock_log.debug.call_args_list[0]))
-
+class TestCalculateVirtualFree(test.TestCase):
     @ddt.data(
         {'total': 30.01, 'free': 28.01, 'provisioned': 2.0, 'max_ratio': 1.0,
          'thin_support': False, 'thick_support': True,
          'is_thin_lun': False, 'expected': 27.01},
         {'total': 20.01, 'free': 18.01, 'provisioned': 2.0, 'max_ratio': 2.0,
          'thin_support': True, 'thick_support': False,
-         'is_thin_lun': True, 'expected': 37.02},
+         'is_thin_lun': True, 'expected': 36.02},
         {'total': 20.01, 'free': 18.01, 'provisioned': 2.0, 'max_ratio': 2.0,
          'thin_support': True, 'thick_support': True,
-         'is_thin_lun': True, 'expected': 37.02},
+         'is_thin_lun': True, 'expected': 36.02},
         {'total': 30.01, 'free': 28.01, 'provisioned': 2.0, 'max_ratio': 2.0,
          'thin_support': True, 'thick_support': True,
          'is_thin_lun': False, 'expected': 27.01},
@@ -1446,6 +1114,98 @@ class LogTracingTestCase(test.TestCase):
 
         self.assertEqual(expected, free_capacity)
 
+    @ddt.data(
+        {'total': 30.01, 'free': 28.01, 'provisioned': 2.0, 'max_ratio': 1.0,
+         'thin_support': False, 'thick_support': True,
+         'is_thin_lun': False, 'reserved_percentage': 5,
+         'expected_total_capacity': 30.01,
+         'expected_reserved_capacity': 1,
+         'expected_free_capacity': 28.01,
+         'expected_total_available_capacity': 29.01,
+         'expected_virtual_free': 27.01,
+         'expected_free_percent': 93.11,
+         'expected_provisioned_type': 'thick',
+         'expected_provisioned_ratio': 0.07},
+        {'total': 20.01, 'free': 18.01, 'provisioned': 2.0, 'max_ratio': 2.0,
+         'thin_support': True, 'thick_support': False,
+         'is_thin_lun': True, 'reserved_percentage': 10,
+         'expected_total_capacity': 20.01,
+         'expected_reserved_capacity': 2,
+         'expected_free_capacity': 18.01,
+         'expected_total_available_capacity': 36.02,
+         'expected_virtual_free': 34.02,
+         'expected_free_percent': 94.45,
+         'expected_provisioned_type': 'thin',
+         'expected_provisioned_ratio': 0.06},
+        {'total': 20.01, 'free': 18.01, 'provisioned': 2.0, 'max_ratio': 2.0,
+         'thin_support': True, 'thick_support': True,
+         'is_thin_lun': True, 'reserved_percentage': 20,
+         'expected_total_capacity': 20.01,
+         'expected_reserved_capacity': 4,
+         'expected_free_capacity': 18.01,
+         'expected_total_available_capacity': 32.02,
+         'expected_virtual_free': 30.02,
+         'expected_free_percent': 93.75,
+         'expected_provisioned_type': 'thin',
+         'expected_provisioned_ratio': 0.06},
+        {'total': 30.01, 'free': 28.01, 'provisioned': 2.0, 'max_ratio': 2.0,
+         'thin_support': True, 'thick_support': True,
+         'is_thin_lun': False, 'reserved_percentage': 10,
+         'expected_total_capacity': 30.01,
+         'expected_reserved_capacity': 3,
+         'expected_free_capacity': 28.01,
+         'expected_total_available_capacity': 27.01,
+         'expected_virtual_free': 25.01,
+         'expected_free_percent': 92.6,
+         'expected_provisioned_type': 'thick',
+         'expected_provisioned_ratio': 0.07},
+    )
+    @ddt.unpack
+    def test_utils_calculate_capacity_factors(
+            self, total, free, provisioned, max_ratio, thin_support,
+            thick_support, is_thin_lun, reserved_percentage,
+            expected_total_capacity,
+            expected_reserved_capacity,
+            expected_free_capacity,
+            expected_total_available_capacity,
+            expected_virtual_free,
+            expected_free_percent,
+            expected_provisioned_type,
+            expected_provisioned_ratio):
+        host_stat = {'total_capacity_gb': total,
+                     'free_capacity_gb': free,
+                     'provisioned_capacity_gb': provisioned,
+                     'max_over_subscription_ratio': max_ratio,
+                     'thin_provisioning_support': thin_support,
+                     'thick_provisioning_support': thick_support,
+                     'reserved_percentage': reserved_percentage}
+
+        factors = utils.calculate_capacity_factors(
+            host_stat['total_capacity_gb'],
+            host_stat['free_capacity_gb'],
+            host_stat['provisioned_capacity_gb'],
+            host_stat['thin_provisioning_support'],
+            host_stat['max_over_subscription_ratio'],
+            host_stat['reserved_percentage'],
+            is_thin_lun)
+
+        self.assertEqual(expected_total_capacity,
+                         factors['total_capacity'])
+        self.assertEqual(expected_reserved_capacity,
+                         factors['reserved_capacity'])
+        self.assertEqual(expected_free_capacity,
+                         factors['free_capacity'])
+        self.assertEqual(expected_total_available_capacity,
+                         factors['total_available_capacity'])
+        self.assertEqual(expected_virtual_free,
+                         factors['virtual_free_capacity'])
+        self.assertEqual(expected_free_percent,
+                         factors['free_percent'])
+        self.assertEqual(expected_provisioned_type,
+                         factors['provisioned_type'])
+        self.assertEqual(expected_provisioned_ratio,
+                         factors['provisioned_ratio'])
+
 
 class Comparable(utils.ComparableMixin):
     def __init__(self, value):
@@ -1738,3 +1498,102 @@ class LimitOperationsTestCase(test.TestCase):
             mock_exec.assert_called_once_with(mocked_semaphore.__enter__)
             mocked_semaphore.__exit__.assert_not_called()
         mocked_semaphore.__exit__.assert_called_once_with(None, None, None)
+
+
+class TestKeystoneProjectGet(test.TestCase):
+    class FakeProject(object):
+        def __init__(self, id='foo', name=None):
+            self.id = id
+            self.name = name
+            self.description = 'fake project description'
+            self.domain_id = 'default'
+
+    @mock.patch('keystoneclient.client.Client')
+    def test_get_project_keystoneclient_v2(self, ksclient_class):
+        self.context = context.RequestContext('fake_user', 'fake_proj_id')
+        keystoneclient = ksclient_class.return_value
+        keystoneclient.version = 'v2.0'
+        returned_project = self.FakeProject(self.context.project_id, 'bar')
+        keystoneclient.projects.get.return_value = returned_project
+        expected_project = api_utils.GenericProjectInfo(
+            self.context.project_id, 'v2.0', domain_id='default', name='bar',
+            description='fake project description')
+        project = api_utils.get_project(
+            self.context, self.context.project_id)
+        self.assertEqual(expected_project.__dict__, project.__dict__)
+
+    @mock.patch('keystoneclient.client.Client')
+    def test_get_project_keystoneclient_v3(self, ksclient_class):
+        self.context = context.RequestContext('fake_user', 'fake_proj_id')
+        keystoneclient = ksclient_class.return_value
+        keystoneclient.version = 'v3'
+        returned_project = self.FakeProject(self.context.project_id, 'bar')
+        keystoneclient.projects.get.return_value = returned_project
+        expected_project = api_utils.GenericProjectInfo(
+            self.context.project_id, 'v3', domain_id='default', name='bar',
+            description='fake project description')
+        project = api_utils.get_project(
+            self.context, self.context.project_id)
+        self.assertEqual(expected_project.__dict__, project.__dict__)
+
+
+class TestCleanFileLocks(test.TestCase):
+
+    @mock.patch('cinder.utils.LOG.warning')
+    @mock.patch('cinder.utils.synchronized_remove')
+    def test_clean_volume_file_locks(self, mock_remove, mock_log):
+        driver = mock.Mock()
+
+        utils.clean_volume_file_locks('UUID', driver)
+
+        self.assertEqual(3, mock_remove.call_count)
+        mock_remove.assert_has_calls([mock.call('UUID-delete_volume'),
+                                      mock.call('UUID'),
+                                      mock.call('UUID-detach_volume')])
+        driver.clean_volume_file_locks.assert_called_once_with('UUID')
+        mock_log.assert_not_called()
+
+    @mock.patch('cinder.utils.LOG.warning')
+    @mock.patch('cinder.utils.synchronized_remove')
+    def test_clean_volume_file_locks_errors(self, mock_remove, mock_log):
+        driver = mock.Mock()
+        driver.clean_volume_file_locks.side_effect = Exception
+        mock_remove.side_effect = [None, Exception, None]
+
+        utils.clean_volume_file_locks('UUID', driver)
+
+        self.assertEqual(3, mock_remove.call_count)
+        mock_remove.assert_has_calls([mock.call('UUID-delete_volume'),
+                                      mock.call('UUID'),
+                                      mock.call('UUID-detach_volume')])
+        driver.clean_volume_file_locks.assert_called_once_with('UUID')
+        self.assertEqual(2, mock_log.call_count)
+
+    @mock.patch('cinder.utils.LOG.warning')
+    @mock.patch('cinder.utils.synchronized_remove')
+    def test_clean_snapshot_file_locks(self, mock_remove, mock_log):
+        driver = mock.Mock()
+
+        utils.clean_snapshot_file_locks('UUID', driver)
+
+        mock_remove.assert_called_once_with('UUID-delete_snapshot')
+        driver.clean_snapshot_file_locks.assert_called_once_with('UUID')
+        mock_log.assert_not_called()
+
+    @mock.patch('cinder.utils.LOG.warning')
+    @mock.patch('cinder.utils.synchronized_remove')
+    def test_clean_snapshot_file_locks_failures(self, mock_remove, mock_log):
+        driver = mock.Mock()
+        driver.clean_snapshot_file_locks.side_effect = Exception
+        mock_remove.side_effect = Exception
+
+        utils.clean_snapshot_file_locks('UUID', driver)
+
+        mock_remove.assert_called_once_with('UUID-delete_snapshot')
+        driver.clean_snapshot_file_locks.assert_called_once_with('UUID')
+        self.assertEqual(2, mock_log.call_count)
+
+    @mock.patch('cinder.coordination.synchronized_remove')
+    def test_api_clean_volume_file_locks(self, mock_remove):
+        utils.api_clean_volume_file_locks('UUID')
+        mock_remove.assert_called_once_with('attachment_update-UUID-*')
