@@ -121,8 +121,10 @@ class VolumeTypeTestCase(test.TestCase):
                          'drive type was not deleted')
         # Assert that associated volume type access is deleted successfully
         # on destroying the volume type
-        vol_type_access = db_api._volume_type_access_query(
-            self.ctxt).filter_by(volume_type_id=type_ref['id']).all()
+        with db_api.main_context_manager.reader.using(self.ctxt):
+            vol_type_access = db_api._volume_type_access_query(
+                self.ctxt
+            ).filter_by(volume_type_id=type_ref['id']).all()
         self.assertEqual([], vol_type_access)
 
     @mock.patch('cinder.quota.VolumeTypeQuotaEngine.'
@@ -215,8 +217,10 @@ class VolumeTypeTestCase(test.TestCase):
 
     def test_get_all_volume_types(self):
         """Ensures that all volume types can be retrieved."""
-        session = db_api.get_session()
-        total_volume_types = session.query(models.VolumeType).count()
+        with db_api.main_context_manager.writer.using(self.ctxt):
+            total_volume_types = self.ctxt.session.query(
+                models.VolumeType,
+            ).count()
         vol_types = volume_types.get_all_types(self.ctxt)
         self.assertEqual(total_volume_types, len(vol_types))
 
@@ -469,7 +473,7 @@ class VolumeTypeTestCase(test.TestCase):
         self.assertEqual(('val1', 'val0'), diff['extra_specs']['key1'])
 
         # qos_ref 1 and 2 have the same specs, while 3 has different
-        qos_keyvals1 = {'k1': 'v1', 'k2': 'v2', 'k3': 'v3', 'created_at': 'v4'}
+        qos_keyvals1 = {'k1': 'v1', 'k2': 'v2', 'k3': 'v3', 'k4': 'v4'}
         qos_keyvals2 = {'k1': 'v0', 'k2': 'v2', 'k3': 'v3'}
         qos_ref1 = qos_specs.create(self.ctxt, 'qos-specs-1', qos_keyvals1)
         qos_ref2 = qos_specs.create(self.ctxt, 'qos-specs-2', qos_keyvals1)
@@ -483,6 +487,8 @@ class VolumeTypeTestCase(test.TestCase):
         diff, same = volume_types.volume_types_diff(self.ctxt, type_ref1['id'],
                                                     type_ref2['id'])
         self.assertTrue(same)
+        for k in ('id', 'name', 'created_at', 'updated_at', 'deleted_at'):
+            self.assertNotIn(k, diff['qos_specs'])
         self.assertEqual(('val1', 'val1'), diff['extra_specs']['key1'])
         self.assertEqual(('v1', 'v1'), diff['qos_specs']['k1'])
         qos_specs.disassociate_qos_specs(self.ctxt, qos_ref2['id'],
@@ -525,7 +531,7 @@ class VolumeTypeTestCase(test.TestCase):
                           'k1': (None, 'v1'),
                           'k2': (None, 'v2'),
                           'k3': (None, 'v3'),
-                          'created_at': (None, 'v4')}, diff['qos_specs'])
+                          'k4': (None, 'v4')}, diff['qos_specs'])
         self.assertEqual({'cipher': (None, 'c1'),
                           'control_location': (None, 'front-end'),
                           'deleted': (None, False),
