@@ -114,9 +114,9 @@ class PowerMaxData(object):
                  'host': 'HostX'}
 
     fabric_name_prefix = 'fakeFabric'
-    end_point_map = {connector['wwpns'][0]: [wwnn1],
-                     connector['wwpns'][1]: [wwnn1]}
-    target_wwns = [wwnn1]
+    end_point_map = {connector['wwpns'][0]: [wwpn1],
+                     connector['wwpns'][1]: [wwpn1]}
+    target_wwns = [wwpn1]
     target_wwns_multi = [wwnn1, wwnn2]
     zoning_mappings = {
         'array': u'000197800123',
@@ -412,6 +412,14 @@ class PowerMaxData(object):
                    'array': array,
                    'interval': 3,
                    'retries': 120}
+    extra_specs_no_workload = {
+        'pool_name': u'Diamond+NONE+SRP_1+000197800123',
+        'slo': slo,
+        'srp': srp,
+        'workload': 'NONE',
+        'array': array,
+        'interval': 3,
+        'retries': 120}
     extra_specs_optimized = {
         'pool_name': u'Optimized+None+SRP_1+000197800123',
         'slo': 'Optimized', 'workload': 'None',
@@ -475,6 +483,11 @@ class PowerMaxData(object):
     extra_specs_tags = deepcopy(extra_specs)
     extra_specs_tags.update({utils.STORAGE_GROUP_TAGS: sg_tags})
 
+    extra_specs_qos = deepcopy(extra_specs)
+    qos_dict = {
+        'total_iops_sec': '4000',
+        'DistributionType': 'Always'}
+    extra_specs_qos['qos'] = qos_dict
     rep_extra_specs_mgmt = deepcopy(rep_extra_specs)
     rep_extra_specs_mgmt['srp'] = srp
     rep_extra_specs_mgmt['mgmt_sg_name'] = rdf_managed_async_grp
@@ -546,6 +559,10 @@ class PowerMaxData(object):
         id=test_vol_grp_name_id_only, host=fake_host,
         replication_status=fields.ReplicationStatus.ENABLED)
 
+    test_rep_group2 = fake_group.fake_group_obj(
+        context=ctx,
+        replication_status=fields.ReplicationStatus.ENABLED)
+
     test_group = fake_group.fake_group_obj(
         context=ctx, name=storagegroup_name_source,
         id=test_vol_grp_name_id_only, host=fake_host)
@@ -583,7 +600,7 @@ class PowerMaxData(object):
         'connector': connector,
         'device_id': device_id,
         'init_group_name': initiatorgroup_name_f,
-        'initiator_check': None,
+        'initiator_check': False,
         'maskingview_name': masking_view_name_f,
         'parent_sg_name': parent_sg_f,
         'srp': srp,
@@ -625,12 +642,19 @@ class PowerMaxData(object):
     # vmax data
     # sloprovisioning
     compression_info = {'symmetrixId': ['000197800128']}
-    inititiatorgroup = [{'initiator': [wwpn1],
-                         'hostId': initiatorgroup_name_f,
-                         'maskingview': [masking_view_name_f]},
-                        {'initiator': [initiator],
-                         'hostId': initiatorgroup_name_i,
-                         'maskingview': [masking_view_name_i]}]
+    initiator_group_fc = {
+        'initiator': [wwpn1],
+        'hostId': initiatorgroup_name_f,
+        'maskingview': [masking_view_name_f]}
+    initiator_group_iscsi = {
+        'initiator': [initiator],
+        'hostId': initiatorgroup_name_i,
+        'maskingview': [masking_view_name_i]}
+    initiator_group_empty = {
+        'hostId': initiatorgroup_name_i,
+    }
+    initiator_group_list = [
+        initiator_group_fc, initiator_group_iscsi]
 
     initiator_list = [{'host': initiatorgroup_name_f,
                        'initiatorId': wwpn1,
@@ -656,6 +680,13 @@ class PowerMaxData(object):
                         {'host_lun_address': '0003'}]},
                    {}]
 
+    maskingview_no_lun = {
+        'maskingViewId': masking_view_name_f,
+        'portGroupId': port_group_name_f,
+        'storageGroupId': storagegroup_name_f,
+        'hostId': initiatorgroup_name_f,
+        'maskingViewConnection': []}
+
     portgroup = [{'portGroupId': port_group_name_f,
                   'symmetrixPortKey': [
                       {'directorId': 'FA-1D',
@@ -670,7 +701,7 @@ class PowerMaxData(object):
     port_list = [
         {'symmetrixPort': {'num_of_masking_views': 1,
                            'maskingview': [masking_view_name_f],
-                           'identifier': wwnn1,
+                           'identifier': wwpn1,
                            'symmetrixPortKey': {
                                'directorId': 'FA-1D',
                                'portId': '4'},
@@ -1265,6 +1296,12 @@ class PowerMaxData(object):
                 'getDynamicRDFCapability': 'RDF1_Capable', 'RDFA': False},
             'timeFinderInfo': {'snapVXTgt': False, 'snapVXSrc': False}}]
 
+    priv_vol_func_response_multi_sg = deepcopy(priv_vol_func_response_single)
+    priv_vol_func_response_multi_sg[0].get('volumeHeader').update(
+        {'numStorageGroups': 2})
+    priv_vol_func_response_multi_sg[0].get('volumeHeader').update(
+        {'storageGroup': ['SG1', 'SG2']})
+
     volume_create_info_dict = {utils.ARRAY: array, utils.DEVICE_ID: device_id}
 
     volume_info_dict = {
@@ -1515,7 +1552,7 @@ class PowerMaxData(object):
             'expandStorageGroupParam': {
                 'addVolumeParam': {
                     'emulation': 'FBA',
-                    'create_new_volumes': 'False',
+                    'create_new_volumes': 'True',
                     'volumeAttributes': [
                         {'num_of_vols': 1,
                          'volumeIdentifier': {
@@ -1649,3 +1686,36 @@ class PowerMaxData(object):
                          'SourceDeviceID': device_id,
                          'SourceDeviceLabel': device_label,
                          'SnapIdList': [snap_id]}
+
+    port_info = {
+        "symmetrixPort": {
+            "director_status": "Online",
+            "maskingview": [
+                "Test_MV",
+            ],
+            "port_status": "ON",
+            "symmetrixPortKey": {
+                "directorId": "FA-1D",
+                "portId": "4"
+            },
+            "portgroup": [
+                "Test_PG"
+            ]
+        }
+    }
+
+    port_info_off = deepcopy(port_info)
+    port_info_off.update({"symmetrixPort": {
+        "director_status": "Offline",
+        "port_status": "OFF"}})
+
+    port_info_no_status = deepcopy(port_info)
+    port_info_no_status.update({"symmetrixPort": {
+        "symmetrixPortKey": {
+            "directorId": "FA-1D",
+            "portId": "4"
+        }
+    }})
+
+    port_info_no_details = deepcopy(port_info)
+    port_info_no_details.pop("symmetrixPort")
