@@ -14,6 +14,7 @@
 #    under the License.
 
 from unittest import mock
+from unittest.mock import call
 
 from cinder import exception
 from cinder.tests.unit import test
@@ -32,14 +33,14 @@ class PowerMaxProvisionTest(test.TestCase):
     def setUp(self):
         self.data = tpd.PowerMaxData()
         super(PowerMaxProvisionTest, self).setUp()
-        volume_utils.get_max_over_subscription_ratio = mock.Mock()
+        self.mock_object(volume_utils, 'get_max_over_subscription_ratio')
         configuration = tpfo.FakeConfiguration(
             None, 'ProvisionTests', 1, 1, san_ip='1.1.1.1', san_login='smc',
             powermax_array=self.data.array, powermax_srp='SRP_1',
             san_password='smc', san_api_port=8443,
             powermax_port_groups=[self.data.port_group_name_i])
-        rest.PowerMaxRest._establish_rest_session = mock.Mock(
-            return_value=tpfo.FakeRequestsSession())
+        self.mock_object(rest.PowerMaxRest, '_establish_rest_session',
+                         return_value=tpfo.FakeRequestsSession())
         driver = iscsi.PowerMaxISCSIDriver(configuration=configuration)
         self.driver = driver
         self.common = self.driver.common
@@ -323,7 +324,7 @@ class PowerMaxProvisionTest(test.TestCase):
         # cannot retrieve srp
         array = self.data.array
         array_info = {'srpName': self.data.failed_resource}
-        ref_stats = (0, 0, 0, 0, False)
+        ref_stats = (0, 0, 0, 0)
         stats = self.provision.get_srp_pool_stats(array, array_info)
         self.assertEqual(ref_stats, stats)
         # cannot report on all stats
@@ -497,6 +498,23 @@ class PowerMaxProvisionTest(test.TestCase):
                 array, snap_name, source_group_name)
             mock_del.assert_not_called()
 
+    @mock.patch.object(rest.PowerMaxRest,
+                       'get_storage_group_snap_id_list',
+                       return_value=[tpd.PowerMaxData.snap_id])
+    @mock.patch.object(rest.PowerMaxRest,
+                       'delete_storagegroup_snap')
+    def test_delete_group_replica_exception_call_args(
+            self, mck_del_ss, mck_snap_list):
+        array = self.data.array
+        snap_name = self.data.group_snapshot_name
+        source_group_name = self.data.storagegroup_name_source
+        self.provision.delete_group_replica(
+            array, snap_name, source_group_name)
+
+        expected = [call('000197800123', 'Grp_source_sg',
+                         'Grp_snapshot', 118749976833, force=True)]
+        self.assertEqual(expected, mck_del_ss.call_args_list)
+
     def test_link_and_break_replica(self):
         array = self.data.array
         source_group_name = self.data.storagegroup_name_source
