@@ -39,7 +39,7 @@ from cinder.volume import volume_utils
 class PowerMaxUtilsTest(test.TestCase):
     def setUp(self):
         self.data = tpd.PowerMaxData()
-        volume_utils.get_max_over_subscription_ratio = mock.Mock()
+        self.mock_object(volume_utils, 'get_max_over_subscription_ratio')
         super(PowerMaxUtilsTest, self).setUp()
         self.replication_device = self.data.sync_rep_device
         configuration = tpfo.FakeConfiguration(
@@ -48,8 +48,8 @@ class PowerMaxUtilsTest(test.TestCase):
             san_password='smc', san_api_port=8443,
             powermax_port_groups=[self.data.port_group_name_i],
             replication_device=self.replication_device)
-        rest.PowerMaxRest._establish_rest_session = mock.Mock(
-            return_value=tpfo.FakeRequestsSession())
+        self.mock_object(rest.PowerMaxRest, '_establish_rest_session',
+                         return_value=tpfo.FakeRequestsSession())
         driver = iscsi.PowerMaxISCSIDriver(configuration=configuration)
         self.driver = driver
         self.common = self.driver.common
@@ -59,17 +59,18 @@ class PowerMaxUtilsTest(test.TestCase):
         with mock.patch.object(volume_types, 'get_volume_type_extra_specs',
                                return_value={'specs'}) as type_mock:
             # path 1: volume_type_id not passed in
-            self.data.test_volume.volume_type_id = (
-                self.data.test_volume_type.id)
-            self.utils.get_volumetype_extra_specs(self.data.test_volume)
+            volume = deepcopy(self.data.test_volume)
+            volume.volume_type_id = self.data.test_volume_type.id
+            self.utils.get_volumetype_extra_specs(volume)
             type_mock.assert_called_once_with(self.data.test_volume_type.id)
             type_mock.reset_mock()
             # path 2: volume_type_id passed in
-            self.utils.get_volumetype_extra_specs(self.data.test_volume, '123')
+            self.utils.get_volumetype_extra_specs(volume, '123')
             type_mock.assert_called_once_with('123')
             type_mock.reset_mock()
             # path 3: no type_id
-            self.utils.get_volumetype_extra_specs(self.data.test_clone_volume)
+            volume = deepcopy(self.data.test_clone_volume)
+            self.utils.get_volumetype_extra_specs(volume)
             type_mock.assert_not_called()
 
     def test_get_volumetype_extra_specs_exception(self):
@@ -220,42 +221,53 @@ class PowerMaxUtilsTest(test.TestCase):
     def test_is_compression_disabled_true(self):
         # Compression disabled in extra specs
         extra_specs = self.data.extra_specs_disable_compression
-        do_disable_compression = self.utils.is_compression_disabled(
-            extra_specs)
-        self.assertTrue(do_disable_compression)
+        self.assertTrue(self.utils.is_compression_disabled(extra_specs))
         # Compression disabled by no SL/WL combination
         extra_specs = deepcopy(self.data.vol_type_extra_specs_none_pool)
-        self.assertTrue(self.utils.is_compression_disabled(
-            extra_specs))
+        self.assertTrue(self.utils.is_compression_disabled(extra_specs))
+        extra_specs3 = deepcopy(extra_specs)
+        extra_specs3.update({utils.DISABLECOMPRESSION: '<is> True'})
+        self.assertTrue(self.utils.is_compression_disabled(extra_specs3))
+        extra_specs4 = deepcopy(extra_specs)
+        extra_specs4.update({utils.DISABLECOMPRESSION: 'True'})
+        self.assertTrue(self.utils.is_compression_disabled(extra_specs4))
 
     def test_is_compression_disabled_false(self):
         # Path 1: no compression extra spec set
         extra_specs = self.data.extra_specs
-        do_disable_compression = self.utils.is_compression_disabled(
-            extra_specs)
-        self.assertFalse(do_disable_compression)
+        self.assertFalse(self.utils.is_compression_disabled(extra_specs))
         # Path 2: compression extra spec set to false
         extra_specs2 = deepcopy(extra_specs)
         extra_specs2.update({utils.DISABLECOMPRESSION: 'false'})
-        do_disable_compression2 = self.utils.is_compression_disabled(
-            extra_specs)
-        self.assertFalse(do_disable_compression2)
+        self.assertFalse(self.utils.is_compression_disabled(extra_specs2))
+        extra_specs3 = deepcopy(extra_specs)
+        extra_specs3.update({utils.DISABLECOMPRESSION: '<is> False'})
+        self.assertFalse(self.utils.is_compression_disabled(extra_specs3))
+        extra_specs4 = deepcopy(extra_specs)
+        extra_specs4.update({utils.DISABLECOMPRESSION: 'False'})
+        self.assertFalse(self.utils.is_compression_disabled(extra_specs4))
 
     def test_change_compression_type_true(self):
         source_compr_disabled = True
-        new_type_compr_disabled = {
+        new_type_compr_disabled_1 = {
             'extra_specs': {utils.DISABLECOMPRESSION: 'false'}}
-        ans = self.utils.change_compression_type(
-            source_compr_disabled, new_type_compr_disabled)
-        self.assertTrue(ans)
+        self.assertTrue(self.utils.change_compression_type(
+            source_compr_disabled, new_type_compr_disabled_1))
+        new_type_compr_disabled_2 = {
+            'extra_specs': {utils.DISABLECOMPRESSION: '<is> False'}}
+        self.assertTrue(self.utils.change_compression_type(
+            source_compr_disabled, new_type_compr_disabled_2))
 
     def test_change_compression_type_false(self):
         source_compr_disabled = True
         new_type_compr_disabled = {
             'extra_specs': {utils.DISABLECOMPRESSION: 'true'}}
-        ans = self.utils.change_compression_type(
-            source_compr_disabled, new_type_compr_disabled)
-        self.assertFalse(ans)
+        self.assertFalse(self.utils.change_compression_type(
+            source_compr_disabled, new_type_compr_disabled))
+        new_type_compr_disabled_2 = {
+            'extra_specs': {utils.DISABLECOMPRESSION: '<is> True'}}
+        self.assertFalse(self.utils.change_compression_type(
+            source_compr_disabled, new_type_compr_disabled_2))
 
     def test_is_replication_enabled(self):
         is_re = self.utils.is_replication_enabled(
@@ -579,6 +591,11 @@ class PowerMaxUtilsTest(test.TestCase):
             self.assertFalse(
                 self.utils.is_volume_manageable(volume))
 
+    def test_is_volume_manageable_multi_sg(self):
+        for volume in self.data.priv_vol_func_response_multi_sg:
+            self.assertFalse(
+                self.utils.is_volume_manageable(volume))
+
     def test_is_snapshot_manageable(self):
         for volume in self.data.priv_vol_func_response_multi:
             self.assertTrue(
@@ -1229,7 +1246,7 @@ class PowerMaxUtilsTest(test.TestCase):
             'array': self.data.array,
             'editStorageGroupActionParam': {
                 'expandStorageGroupParam': {
-                    'addVolumeParam': {}}}}
+                    'addVolumeParam': {'create_new_volumes': 'False'}}}}
 
         updated_payload = self.utils.update_payload_for_rdf_vol_create(
             payload, self.data.remote_array, self.data.storagegroup_name_f)
@@ -1238,6 +1255,7 @@ class PowerMaxUtilsTest(test.TestCase):
             'editStorageGroupActionParam': {
                 'expandStorageGroupParam': {
                     'addVolumeParam': {
+                        'create_new_volumes': 'True',
                         'remoteSymmSGInfoParam': {
                             'force': 'true',
                             'remote_symmetrix_1_id': self.data.remote_array,
@@ -1248,7 +1266,7 @@ class PowerMaxUtilsTest(test.TestCase):
     def test_is_retype_supported(self):
         # Volume source type not replicated, target type Metro replicated,
         # volume is detached, host-assisted retype supported
-        volume = self.data.test_volume
+        volume = deepcopy(self.data.test_volume)
         volume.attach_status = 'detached'
 
         src_extra_specs = deepcopy(self.data.extra_specs)
@@ -1603,7 +1621,7 @@ class PowerMaxUtilsTest(test.TestCase):
             self.assertEqual(expected_msg, e.msg)
 
     def test_get_migration_delete_extra_specs_replicated(self):
-        volume = self.data.test_volume
+        volume = deepcopy(self.data.test_volume)
         metadata = deepcopy(self.data.volume_metadata)
         metadata[utils.IS_RE_CAMEL] = 'True'
         metadata['ReplicationMode'] = utils.REP_SYNC
@@ -1622,7 +1640,7 @@ class PowerMaxUtilsTest(test.TestCase):
         self.assertEqual(ref_extra_specs, updated_extra_specs)
 
     def test_get_migration_delete_extra_specs_non_replicated(self):
-        volume = self.data.test_volume
+        volume = deepcopy(self.data.test_volume)
         volume.metadata = self.data.volume_metadata
         extra_specs = deepcopy(self.data.extra_specs)
         extra_specs[utils.IS_RE] = True
