@@ -1,4 +1,4 @@
-# Copyright (C) 2020, Hitachi, Ltd.
+# Copyright (C) 2020, 2021, Hitachi, Ltd.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -20,19 +20,23 @@ from oslo_config import cfg
 import requests
 
 from cinder import context as cinder_context
-from cinder import db
 from cinder.db.sqlalchemy import api as sqlalchemy_api
+from cinder import exception
+from cinder.objects import group_snapshot as obj_group_snap
 from cinder.objects import snapshot as obj_snap
+from cinder.tests.unit import fake_group
+from cinder.tests.unit import fake_group_snapshot
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
 from cinder.tests.unit import test
-from cinder import utils
 from cinder.volume import configuration as conf
 from cinder.volume import driver
 from cinder.volume.drivers.hitachi import hbsd_common
 from cinder.volume.drivers.hitachi import hbsd_iscsi
 from cinder.volume.drivers.hitachi import hbsd_rest
+from cinder.volume.drivers.hitachi import hbsd_rest_api
 from cinder.volume import volume_types
+from cinder.volume import volume_utils
 
 # Configuration parameter values
 CONFIG_MAP = {
@@ -63,17 +67,22 @@ DEFAULT_CONNECTOR = {
 CTXT = cinder_context.get_admin_context()
 
 TEST_VOLUME = []
-for i in range(3):
+for i in range(4):
     volume = {}
     volume['id'] = '00000000-0000-0000-0000-{0:012d}'.format(i)
     volume['name'] = 'test-volume{0:d}'.format(i)
-    volume['provider_location'] = '{0:d}'.format(i)
+    volume['volume_type_id'] = '00000000-0000-0000-0000-{0:012d}'.format(i)
+    if i == 3:
+        volume['provider_location'] = None
+    else:
+        volume['provider_location'] = '{0:d}'.format(i)
     volume['size'] = 128
     if i == 2:
         volume['status'] = 'in-use'
     else:
         volume['status'] = 'available'
     volume = fake_volume.fake_volume_obj(CTXT, **volume)
+    volume.volume_type = fake_volume.fake_volume_type_obj(CTXT)
     TEST_VOLUME.append(volume)
 
 
@@ -97,6 +106,23 @@ snapshot = obj_snap.Snapshot._from_db_object(
     fake_snapshot.fake_db_snapshot(**snapshot))
 TEST_SNAPSHOT.append(snapshot)
 
+TEST_GROUP = []
+for i in range(2):
+    group = {}
+    group['id'] = '20000000-0000-0000-0000-{0:012d}'.format(i)
+    group['status'] = 'available'
+    group = fake_group.fake_group_obj(CTXT, **group)
+    TEST_GROUP.append(group)
+
+TEST_GROUP_SNAP = []
+group_snapshot = {}
+group_snapshot['id'] = '30000000-0000-0000-0000-{0:012d}'.format(0)
+group_snapshot['status'] = 'available'
+group_snapshot = obj_group_snap.GroupSnapshot._from_db_object(
+    CTXT, obj_group_snap.GroupSnapshot(),
+    fake_group_snapshot.fake_db_group_snapshot(**group_snapshot))
+TEST_GROUP_SNAP.append(group_snapshot)
+
 # Dummy response for REST API
 POST_SESSIONS_RESULT = {
     "token": "b74777a3-f9f0-4ea8-bd8f-09847fac48d3",
@@ -205,6 +231,18 @@ GET_SNAPSHOTS_RESULT = {
     ],
 }
 
+GET_SNAPSHOTS_RESULT_PAIR = {
+    "data": [
+        {
+            "primaryOrSecondary": "S-VOL",
+            "status": "PAIR",
+            "pvolLdevId": 0,
+            "muNumber": 1,
+            "svolLdevId": 1,
+        },
+    ],
+}
+
 GET_LDEVS_RESULT = {
     "data": [
         {
@@ -292,6 +330,10 @@ class HBSDRESTISCSIDriverTest(test.TestCase):
             CONFIG_MAP['port_id']]
         self.configuration.hitachi_group_create = True
         self.configuration.hitachi_group_delete = True
+        self.configuration.hitachi_copy_speed = 3
+        self.configuration.hitachi_copy_check_interval = 3
+        self.configuration.hitachi_async_copy_check_interval = 10
+        self.configuration.hitachi_port_scheduler = False
 
         self.configuration.san_login = CONFIG_MAP['user_id']
         self.configuration.san_password = CONFIG_MAP['user_pass']
@@ -299,8 +341,41 @@ class HBSDRESTISCSIDriverTest(test.TestCase):
             'rest_server_ip_addr']
         self.configuration.san_api_port = CONFIG_MAP[
             'rest_server_ip_port']
+        self.configuration.hitachi_rest_disable_io_wait = True
         self.configuration.hitachi_rest_tcp_keepalive = True
         self.configuration.hitachi_discard_zero_page = True
+        self.configuration.hitachi_rest_number = "0"
+        self.configuration.hitachi_lun_timeout = hbsd_rest._LUN_TIMEOUT
+        self.configuration.hitachi_lun_retry_interval = (
+            hbsd_rest._LUN_RETRY_INTERVAL)
+        self.configuration.hitachi_restore_timeout = hbsd_rest._RESTORE_TIMEOUT
+        self.configuration.hitachi_state_transition_timeout = (
+            hbsd_rest._STATE_TRANSITION_TIMEOUT)
+        self.configuration.hitachi_lock_timeout = hbsd_rest_api._LOCK_TIMEOUT
+        self.configuration.hitachi_rest_timeout = hbsd_rest_api._REST_TIMEOUT
+        self.configuration.hitachi_extend_timeout = (
+            hbsd_rest_api._EXTEND_TIMEOUT)
+        self.configuration.hitachi_exec_retry_interval = (
+            hbsd_rest_api._EXEC_RETRY_INTERVAL)
+        self.configuration.hitachi_rest_connect_timeout = (
+            hbsd_rest_api._DEFAULT_CONNECT_TIMEOUT)
+        self.configuration.hitachi_rest_job_api_response_timeout = (
+            hbsd_rest_api._JOB_API_RESPONSE_TIMEOUT)
+        self.configuration.hitachi_rest_get_api_response_timeout = (
+            hbsd_rest_api._GET_API_RESPONSE_TIMEOUT)
+        self.configuration.hitachi_rest_server_busy_timeout = (
+            hbsd_rest_api._REST_SERVER_BUSY_TIMEOUT)
+        self.configuration.hitachi_rest_keep_session_loop_interval = (
+            hbsd_rest_api._KEEP_SESSION_LOOP_INTERVAL)
+        self.configuration.hitachi_rest_another_ldev_mapped_retry_timeout = (
+            hbsd_rest_api._ANOTHER_LDEV_MAPPED_RETRY_TIMEOUT)
+        self.configuration.hitachi_rest_tcp_keepidle = (
+            hbsd_rest_api._TCP_KEEPIDLE)
+        self.configuration.hitachi_rest_tcp_keepintvl = (
+            hbsd_rest_api._TCP_KEEPINTVL)
+        self.configuration.hitachi_rest_tcp_keepcnt = (
+            hbsd_rest_api._TCP_KEEPCNT)
+        self.configuration.hitachi_host_mode_options = []
 
         self.configuration.use_chap_auth = True
         self.configuration.chap_username = CONFIG_MAP['auth_user']
@@ -330,13 +405,13 @@ class HBSDRESTISCSIDriverTest(test.TestCase):
 
     @mock.patch.object(requests.Session, "request")
     @mock.patch.object(
-        utils, 'brick_get_connector_properties',
+        volume_utils, 'brick_get_connector_properties',
         side_effect=_brick_get_connector_properties)
     def _setup_driver(
             self, brick_get_connector_properties=None, request=None):
         """Set up the driver environment."""
         self.driver = hbsd_iscsi.HBSDISCSIDriver(
-            configuration=self.configuration, db=db)
+            configuration=self.configuration)
         request.side_effect = [FakeResponse(200, POST_SESSIONS_RESULT),
                                FakeResponse(200, GET_PORTS_RESULT),
                                FakeResponse(200, GET_PORT_RESULT),
@@ -360,11 +435,11 @@ class HBSDRESTISCSIDriverTest(test.TestCase):
     # API test cases
     @mock.patch.object(requests.Session, "request")
     @mock.patch.object(
-        utils, 'brick_get_connector_properties',
+        volume_utils, 'brick_get_connector_properties',
         side_effect=_brick_get_connector_properties)
     def test_do_setup(self, brick_get_connector_properties, request):
         drv = hbsd_iscsi.HBSDISCSIDriver(
-            configuration=self.configuration, db=db)
+            configuration=self.configuration)
         self._setup_config()
         request.side_effect = [FakeResponse(200, POST_SESSIONS_RESULT),
                                FakeResponse(200, GET_PORTS_RESULT),
@@ -386,12 +461,12 @@ class HBSDRESTISCSIDriverTest(test.TestCase):
 
     @mock.patch.object(requests.Session, "request")
     @mock.patch.object(
-        utils, 'brick_get_connector_properties',
+        volume_utils, 'brick_get_connector_properties',
         side_effect=_brick_get_connector_properties)
     def test_do_setup_create_hg(self, brick_get_connector_properties, request):
         """Normal case: The host group not exists."""
         drv = hbsd_iscsi.HBSDISCSIDriver(
-            configuration=self.configuration, db=db)
+            configuration=self.configuration)
         self._setup_config()
         request.side_effect = [FakeResponse(200, POST_SESSIONS_RESULT),
                                FakeResponse(200, GET_PORTS_RESULT),
@@ -425,14 +500,14 @@ class HBSDRESTISCSIDriverTest(test.TestCase):
     @mock.patch.object(driver.ISCSIDriver, "get_goodness_function")
     @mock.patch.object(driver.ISCSIDriver, "get_filter_function")
     @mock.patch.object(requests.Session, "request")
-    def test_get_volume_stats(
+    def test__update_volume_stats(
             self, request, get_filter_function, get_goodness_function):
         request.return_value = FakeResponse(200, GET_POOL_RESULT)
         get_filter_function.return_value = None
         get_goodness_function.return_value = None
-        stats = self.driver.get_volume_stats(True)
-        self.assertEqual('Hitachi', stats['vendor_name'])
-        self.assertTrue(stats["pools"][0]['multiattach'])
+        self.driver._update_volume_stats()
+        self.assertEqual('Hitachi', self.driver._stats['vendor_name'])
+        self.assertTrue(self.driver._stats["pools"][0]['multiattach'])
         self.assertEqual(1, request.call_count)
         self.assertEqual(1, get_filter_function.call_count)
         self.assertEqual(1, get_goodness_function.call_count)
@@ -497,7 +572,11 @@ class HBSDRESTISCSIDriverTest(test.TestCase):
         self.assertEqual(5, request.call_count)
 
     @mock.patch.object(requests.Session, "request")
-    def test_initialize_connection(self, request):
+    @mock.patch.object(volume_types, 'get_volume_type_extra_specs')
+    def test_initialize_connection(
+            self, get_volume_type_extra_specs, request):
+        extra_specs = {"hbsd:target_ports": "CL1-A"}
+        get_volume_type_extra_specs.return_value = extra_specs
         request.side_effect = [FakeResponse(200, GET_HOST_ISCSIS_RESULT),
                                FakeResponse(200, GET_HOST_GROUP_RESULT),
                                FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
@@ -517,11 +596,16 @@ class HBSDRESTISCSIDriverTest(test.TestCase):
         self.assertEqual(
             CONFIG_MAP['auth_password'], ret['data']['auth_password'])
         self.assertEqual(1, ret['data']['target_lun'])
+        self.assertEqual(1, get_volume_type_extra_specs.call_count)
         self.assertEqual(3, request.call_count)
 
     @mock.patch.object(requests.Session, "request")
-    def test_initialize_connection_shared_target(self, request):
+    @mock.patch.object(volume_types, 'get_volume_type_extra_specs')
+    def test_initialize_connection_shared_target(
+            self, get_volume_type_extra_specs, request):
         """Normal case: A target shared with other systems."""
+        extra_specs = {"hbsd:target_ports": "CL1-A"}
+        get_volume_type_extra_specs.return_value = extra_specs
         request.side_effect = [FakeResponse(200, NOTFOUND_RESULT),
                                FakeResponse(200, GET_HOST_GROUPS_RESULT),
                                FakeResponse(200, GET_HOST_ISCSIS_RESULT),
@@ -542,6 +626,7 @@ class HBSDRESTISCSIDriverTest(test.TestCase):
         self.assertEqual(
             CONFIG_MAP['auth_password'], ret['data']['auth_password'])
         self.assertEqual(1, ret['data']['target_lun'])
+        self.assertEqual(1, get_volume_type_extra_specs.call_count)
         self.assertEqual(4, request.call_count)
 
     @mock.patch.object(requests.Session, "request")
@@ -700,3 +785,151 @@ class HBSDRESTISCSIDriverTest(test.TestCase):
         self.driver.revert_to_snapshot(
             self.ctxt, TEST_VOLUME[0], TEST_SNAPSHOT[0])
         self.assertEqual(5, request.call_count)
+
+    def test_create_group(self):
+        ret = self.driver.create_group(self.ctxt, TEST_GROUP[0])
+        self.assertIsNone(ret)
+
+    @mock.patch.object(requests.Session, "request")
+    def test_delete_group(self, request):
+        request.side_effect = [FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
+        ret = self.driver.delete_group(
+            self.ctxt, TEST_GROUP[0], [TEST_VOLUME[0]])
+        self.assertEqual(4, request.call_count)
+        actual = (
+            {'status': TEST_GROUP[0]['status']},
+            [{'id': TEST_VOLUME[0]['id'], 'status': 'deleted'}]
+        )
+        self.assertTupleEqual(actual, ret)
+
+    @mock.patch.object(requests.Session, "request")
+    def test_create_group_from_src_volume(self, request):
+        request.side_effect = [FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
+        ret = self.driver.create_group_from_src(
+            self.ctxt, TEST_GROUP[1], [TEST_VOLUME[1]],
+            source_group=TEST_GROUP[0], source_vols=[TEST_VOLUME[0]]
+        )
+        self.assertEqual(5, request.call_count)
+        actual = (
+            None, [{'id': TEST_VOLUME[1]['id'], 'provider_location': '1'}])
+        self.assertTupleEqual(actual, ret)
+
+    @mock.patch.object(requests.Session, "request")
+    def test_create_group_from_src_snapshot(self, request):
+        request.side_effect = [FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
+        ret = self.driver.create_group_from_src(
+            self.ctxt, TEST_GROUP[0], [TEST_VOLUME[0]],
+            group_snapshot=TEST_GROUP_SNAP[0], snapshots=[TEST_SNAPSHOT[0]]
+        )
+        self.assertEqual(5, request.call_count)
+        actual = (
+            None, [{'id': TEST_VOLUME[0]['id'], 'provider_location': '1'}])
+        self.assertTupleEqual(actual, ret)
+
+    def test_create_group_from_src_volume_error(self):
+        self.assertRaises(
+            exception.VolumeDriverException, self.driver.create_group_from_src,
+            self.ctxt, TEST_GROUP[1], [TEST_VOLUME[1]],
+            source_group=TEST_GROUP[0], source_vols=[TEST_VOLUME[3]]
+        )
+
+    @mock.patch.object(volume_utils, 'is_group_a_cg_snapshot_type')
+    def test_update_group(self, is_group_a_cg_snapshot_type):
+        is_group_a_cg_snapshot_type.return_value = False
+        ret = self.driver.update_group(
+            self.ctxt, TEST_GROUP[0], add_volumes=[TEST_VOLUME[0]])
+        self.assertTupleEqual((None, None, None), ret)
+
+    @mock.patch.object(volume_utils, 'is_group_a_cg_snapshot_type')
+    def test_update_group_error(self, is_group_a_cg_snapshot_type):
+        is_group_a_cg_snapshot_type.return_value = True
+        self.assertRaises(
+            exception.VolumeDriverException, self.driver.update_group,
+            self.ctxt, TEST_GROUP[0], add_volumes=[TEST_VOLUME[3]],
+            remove_volumes=[TEST_VOLUME[0]]
+        )
+
+    @mock.patch.object(requests.Session, "request")
+    @mock.patch.object(sqlalchemy_api, 'volume_get', side_effect=_volume_get)
+    @mock.patch.object(volume_utils, 'is_group_a_cg_snapshot_type')
+    def test_create_group_snapshot_non_cg(
+            self, is_group_a_cg_snapshot_type, volume_get, request):
+        is_group_a_cg_snapshot_type.return_value = False
+        request.side_effect = [FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT)]
+        ret = self.driver.create_group_snapshot(
+            self.ctxt, TEST_GROUP_SNAP[0], [TEST_SNAPSHOT[0]]
+        )
+        self.assertEqual(4, request.call_count)
+        actual = (
+            {'status': 'available'},
+            [{'id': TEST_SNAPSHOT[0]['id'],
+              'provider_location': '1',
+              'status': 'available'}]
+        )
+        self.assertTupleEqual(actual, ret)
+
+    @mock.patch.object(requests.Session, "request")
+    @mock.patch.object(sqlalchemy_api, 'volume_get', side_effect=_volume_get)
+    @mock.patch.object(volume_utils, 'is_group_a_cg_snapshot_type')
+    def test_create_group_snapshot_cg(
+            self, is_group_a_cg_snapshot_type, volume_get, request):
+        is_group_a_cg_snapshot_type.return_value = True
+        request.side_effect = [FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT_PAIR),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT)]
+        ret = self.driver.create_group_snapshot(
+            self.ctxt, TEST_GROUP_SNAP[0], [TEST_SNAPSHOT[0]]
+        )
+        self.assertEqual(5, request.call_count)
+        actual = (
+            None,
+            [{'id': TEST_SNAPSHOT[0]['id'],
+              'provider_location': '1',
+              'status': 'available'}]
+        )
+        self.assertTupleEqual(actual, ret)
+
+    @mock.patch.object(requests.Session, "request")
+    def test_delete_group_snapshot(self, request):
+        request.side_effect = [FakeResponse(200, GET_LDEV_RESULT_PAIR),
+                               FakeResponse(200, NOTFOUND_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT),
+                               FakeResponse(200, GET_SNAPSHOTS_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT),
+                               FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(200, GET_LDEV_RESULT),
+                               FakeResponse(202, COMPLETED_SUCCEEDED_RESULT)]
+        ret = self.driver.delete_group_snapshot(
+            self.ctxt, TEST_GROUP_SNAP[0], [TEST_SNAPSHOT[0]])
+        self.assertEqual(10, request.call_count)
+        actual = (
+            {'status': TEST_GROUP_SNAP[0]['status']},
+            [{'id': TEST_SNAPSHOT[0]['id'], 'status': 'deleted'}]
+        )
+        self.assertTupleEqual(actual, ret)
+
+    @mock.patch.object(hbsd_iscsi.HBSDISCSIDriver, "_get_oslo_driver_opts")
+    def test_get_driver_options(self, _get_oslo_driver_opts):
+        _get_oslo_driver_opts.return_value = []
+        ret = self.driver.get_driver_options()
+        actual = (hbsd_common.COMMON_VOLUME_OPTS +
+                  hbsd_rest.REST_VOLUME_OPTS)
+        self.assertEqual(actual, ret)
