@@ -22,7 +22,6 @@ from unittest import mock
 
 import ddt
 import eventlet
-from oslo_config import cfg
 import requests
 
 from cinder import context
@@ -35,8 +34,6 @@ from cinder.volume.drivers.inspur.as13000 import as13000_driver
 from cinder.volume import volume_utils
 
 
-CONF = cfg.CONF
-
 test_config = configuration.Configuration(None)
 test_config.san_ip = 'some_ip'
 test_config.san_api_port = 'as13000_api_port'
@@ -688,8 +685,10 @@ class AS13000DriverTestCase(test.TestCase):
                           self.as13000_san.delete_snapshot, snapshot)
         mock_cv.assert_called_once_with(snapshot.volume)
 
-    @ddt.data((time.time() - 3000), (time.time() - 4000))
-    def test__update_volume_stats(self, time_token):
+    @mock.patch('time.time')
+    @ddt.data(2000, 1000)
+    def test__update_volume_stats(self, time_token, mock_time):
+        mock_time.return_value = 5000
         self.as13000_san.VENDOR = 'INSPUR'
         self.as13000_san.VERSION = 'V1.3.1'
         self.as13000_san.PROTOCOL = 'iSCSI'
@@ -708,7 +707,7 @@ class AS13000DriverTestCase(test.TestCase):
 
         self.as13000_san._update_volume_stats()
         backend_data = {'driver_version': 'V1.3.1',
-                        'pools': [{'pool_name': 'fake_pool'}],
+                        'pools': fake_pool_backend,
                         'storage_protocol': 'iSCSI',
                         'vendor_name': 'INSPUR',
                         'volume_backend_name': 'fake_backend_name'}
