@@ -89,10 +89,10 @@ class NetAppBaseClientTestCase(test.TestCase):
         self.assertIsNone(self.client.check_is_naelement(element))
         self.assertRaises(ValueError, self.client.check_is_naelement, None)
 
-    @ddt.data({'ontap_version': '9.4', 'space_reservation': 'true'},
-              {'ontap_version': '9.4', 'space_reservation': 'false'},
-              {'ontap_version': '9.6', 'space_reservation': 'true'},
-              {'ontap_version': '9.6', 'space_reservation': 'false'})
+    @ddt.data({'ontap_version': (9, 4, 0), 'space_reservation': 'true'},
+              {'ontap_version': (9, 4, 0), 'space_reservation': 'false'},
+              {'ontap_version': (9, 6, 0), 'space_reservation': 'true'},
+              {'ontap_version': (9, 6, 0), 'space_reservation': 'false'})
     @ddt.unpack
     def test_create_lun(self, ontap_version, space_reservation):
         expected_path = '/vol/%s/%s' % (self.fake_volume, self.fake_lun)
@@ -104,9 +104,11 @@ class NetAppBaseClientTestCase(test.TestCase):
             client_base.Client, 'do_direct_resize')
         mock_set_space_reservation = self.mock_object(
             client_base.Client, 'set_lun_space_reservation')
+        mock_validate_qos_policy_group = self.mock_object(
+            client_base.Client, '_validate_qos_policy_group')
         initial_size = self.fake_size
 
-        if ontap_version < '9.5':
+        if ontap_version < (9, 5, 0):
             initial_size = fake.MAX_SIZE_FOR_A_LUN
             expected_space_reservation = 'false'
 
@@ -118,6 +120,7 @@ class NetAppBaseClientTestCase(test.TestCase):
                                    self.fake_size,
                                    self.fake_metadata)
 
+            mock_validate_qos_policy_group.assert_called_once()
             mock_create_node.assert_called_with(
                 'lun-create-by-size',
                 **{'path': expected_path,
@@ -128,20 +131,20 @@ class NetAppBaseClientTestCase(test.TestCase):
             self.connection.invoke_successfully.assert_called_with(
                 mock.ANY, True)
 
-        if ontap_version < '9.5':
+        if ontap_version < (9, 5, 0):
             mock_resize_lun.assert_called_once_with(
                 expected_path, self.fake_size)
 
-        if ontap_version < '9.5' and space_reservation == 'true':
+        if ontap_version < (9, 5, 0) and space_reservation == 'true':
             mock_set_space_reservation.assert_called_once_with(
                 expected_path, True)
         else:
             mock_set_space_reservation.assert_not_called()
 
-    @ddt.data({'ontap_version': '9.4', 'space_reservation': 'true'},
-              {'ontap_version': '9.4', 'space_reservation': 'false'},
-              {'ontap_version': '9.6', 'space_reservation': 'true'},
-              {'ontap_version': '9.6', 'space_reservation': 'false'})
+    @ddt.data({'ontap_version': (9, 4, 0), 'space_reservation': 'true'},
+              {'ontap_version': (9, 4, 0), 'space_reservation': 'false'},
+              {'ontap_version': (9, 6, 0), 'space_reservation': 'true'},
+              {'ontap_version': (9, 6, 0), 'space_reservation': 'false'})
     @ddt.unpack
     def test_create_lun_exact_size(self, ontap_version, space_reservation):
         expected_path = '/vol/%s/%s' % (self.fake_volume, self.fake_lun)
@@ -154,9 +157,11 @@ class NetAppBaseClientTestCase(test.TestCase):
             client_base.Client, 'do_direct_resize')
         mock_set_space_reservation = self.mock_object(
             client_base.Client, 'set_lun_space_reservation')
+        mock_validate_qos_policy_group = self.mock_object(
+            client_base.Client, '_validate_qos_policy_group')
         initial_size = self.fake_size
 
-        if ontap_version < '9.5':
+        if ontap_version < (9, 5, 0):
             initial_size = fake.MAX_SIZE_FOR_A_LUN
             expected_space_reservation = 'false'
 
@@ -168,6 +173,7 @@ class NetAppBaseClientTestCase(test.TestCase):
                                    self.fake_size,
                                    self.fake_metadata)
 
+            mock_validate_qos_policy_group.assert_called_once()
             mock_create_node.assert_called_with(
                 'lun-create-by-size',
                 **{'path': expected_path,
@@ -179,20 +185,20 @@ class NetAppBaseClientTestCase(test.TestCase):
             self.connection.invoke_successfully.assert_called_with(
                 mock.ANY, True)
 
-        if ontap_version < '9.5':
+        if ontap_version < (9, 5, 0):
             mock_resize_lun.assert_called_once_with(
                 expected_path, self.fake_size)
 
-        if ontap_version < '9.5' and space_reservation == 'true':
+        if ontap_version < (9, 5, 0) and space_reservation == 'true':
             mock_set_space_reservation.assert_called_once_with(
                 expected_path, True)
         else:
             mock_set_space_reservation.assert_not_called()
 
-    @ddt.data({'ontap_version': '9.4', 'space_reservation': 'true'},
-              {'ontap_version': '9.4', 'space_reservation': 'false'},
-              {'ontap_version': '9.6', 'space_reservation': 'true'},
-              {'ontap_version': '9.6', 'space_reservation': 'false'})
+    @ddt.data({'ontap_version': (9, 4, 0), 'space_reservation': 'true'},
+              {'ontap_version': (9, 4, 0), 'space_reservation': 'false'},
+              {'ontap_version': (9, 6, 0), 'space_reservation': 'true'},
+              {'ontap_version': (9, 6, 0), 'space_reservation': 'false'})
     @ddt.unpack
     def test_create_lun_with_qos_policy_group_name(
             self, ontap_version, space_reservation):
@@ -208,9 +214,11 @@ class NetAppBaseClientTestCase(test.TestCase):
             client_base.Client, 'do_direct_resize')
         mock_set_space_reservation = self.mock_object(
             client_base.Client, 'set_lun_space_reservation')
+        mock_validate_qos_policy_group = self.mock_object(
+            client_base.Client, '_validate_qos_policy_group')
         initial_size = self.fake_size
 
-        if ontap_version < '9.5':
+        if ontap_version < (9, 5, 0):
             initial_size = fake.MAX_SIZE_FOR_A_LUN
             expected_space_reservation = 'false'
 
@@ -225,6 +233,7 @@ class NetAppBaseClientTestCase(test.TestCase):
                 self.fake_metadata,
                 qos_policy_group_name=expected_qos_group_name)
 
+            mock_validate_qos_policy_group.assert_called_once()
             mock_create_node.assert_called_with(
                 'lun-create-by-size',
                 **{'path': expected_path, 'size': initial_size,
@@ -236,11 +245,11 @@ class NetAppBaseClientTestCase(test.TestCase):
             self.connection.invoke_successfully.assert_called_with(
                 mock.ANY, True)
 
-        if ontap_version < '9.5':
+        if ontap_version < (9, 5, 0):
             mock_resize_lun.assert_called_once_with(
                 expected_path, self.fake_size)
 
-        if ontap_version < '9.5' and space_reservation == 'true':
+        if ontap_version < (9, 5, 0) and space_reservation == 'true':
             mock_set_space_reservation.assert_called_once_with(
                 expected_path, True)
         else:
@@ -249,19 +258,20 @@ class NetAppBaseClientTestCase(test.TestCase):
     def test_get_ontap_version(self):
         version_response = netapp_api.NaElement(
             fake.SYSTEM_GET_VERSION_RESPONSE)
-        self.connection.invoke_successfully.return_value = version_response
+        self.connection.invoke_successfully.return_value = (
+            version_response)
 
         result = self.client.get_ontap_version(cached=False)
 
-        self.assertEqual(('9.6'), result)
+        self.assertEqual((9, 6, 0), result)
 
     def test_get_ontap_version_cached(self):
-        self.connection.get_ontap_version.return_value = '9.6'
+        self.connection.get_ontap_version.return_value = (9, 6, 0)
 
         result = self.client.get_ontap_version()
 
         self.connection.get_ontap_version.assert_called_once_with()
-        self.assertEqual(('9.6'), result)
+        self.assertEqual((9, 6, 0), result)
 
     def test_set_lun_space_reservation(self):
         path = '/vol/%s/%s' % (self.fake_volume, self.fake_lun)
@@ -278,12 +288,14 @@ class NetAppBaseClientTestCase(test.TestCase):
             self.connection.invoke_successfully.assert_called_once_with(
                 mock.ANY, True)
 
-    @ddt.data('9.4', '9.6')
+    @ddt.data((9, 4, 0), (9, 6, 0))
     def test_create_lun_raises_on_failure(self, ontap_version):
         self.connection.invoke_successfully = mock.Mock(
             side_effect=netapp_api.NaApiError)
         self.mock_object(self.client, 'get_ontap_version',
                          return_value=ontap_version)
+        mock_validate_qos_policy_group = self.mock_object(
+            client_base.Client, '_validate_qos_policy_group')
 
         self.assertRaises(netapp_api.NaApiError,
                           self.client.create_lun,
@@ -291,6 +303,7 @@ class NetAppBaseClientTestCase(test.TestCase):
                           self.fake_lun,
                           self.fake_size,
                           self.fake_metadata)
+        mock_validate_qos_policy_group.assert_called_once()
 
     def test_destroy_lun(self):
         path = '/vol/%s/%s' % (self.fake_volume, self.fake_lun)
