@@ -45,13 +45,13 @@ class NetAppDriverUtilsTestCase(test.TestCase):
     def test_validate_instantiation_proxy(self):
         kwargs = {'netapp_mode': 'proxy'}
         na_utils.validate_instantiation(**kwargs)
-        self.assertEqual(0, na_utils.LOG.warning.call_count)
+        na_utils.LOG.warning.assert_not_called()
 
     @mock.patch.object(na_utils, 'LOG', mock.Mock())
     def test_validate_instantiation_no_proxy(self):
         kwargs = {'netapp_mode': 'asdf'}
         na_utils.validate_instantiation(**kwargs)
-        self.assertEqual(1, na_utils.LOG.warning.call_count)
+        na_utils.LOG.warning.assert_called_once()
 
     def test_check_flags(self):
 
@@ -138,11 +138,10 @@ class NetAppDriverUtilsTestCase(test.TestCase):
             [fake.ISCSI_FAKE_PORT, fake.ISCSI_FAKE_PORT])
 
         actual_properties_mapped = actual_properties['data']
-        expected = fake.ISCSI_MP_TARGET_INFO_DICT.copy()
+        expected = copy.deepcopy(fake.ISCSI_MP_TARGET_INFO_DICT)
         expected['target_iqns'][1] = expected['target_iqns'][0]
 
-        self.assertDictEqual(actual_properties_mapped,
-                             fake.ISCSI_MP_TARGET_INFO_DICT)
+        self.assertDictEqual(expected, actual_properties_mapped)
 
     def test_iscsi_connection_lun_id_type_str(self):
         FAKE_LUN_ID = '1'
@@ -228,7 +227,7 @@ class NetAppDriverUtilsTestCase(test.TestCase):
 
         na_utils.log_extra_spec_warnings({'netapp:raid_type': 'raid4'})
 
-        self.assertEqual(1, mock_log.call_count)
+        mock_log.assert_called_once()
 
     def test_log_extra_spec_warnings_deprecated_specs(self):
 
@@ -236,7 +235,13 @@ class NetAppDriverUtilsTestCase(test.TestCase):
 
         na_utils.log_extra_spec_warnings({'netapp_thick_provisioned': 'true'})
 
-        self.assertEqual(1, mock_log.call_count)
+        mock_log.assert_called_once()
+
+    def test_validate_qos_spec(self):
+        qos_spec = fake.QOS_SPEC
+
+        # Just return without raising an exception.
+        na_utils.validate_qos_spec(qos_spec)
 
     def test_validate_qos_spec_none(self):
         qos_spec = None
@@ -244,26 +249,79 @@ class NetAppDriverUtilsTestCase(test.TestCase):
         # Just return without raising an exception.
         na_utils.validate_qos_spec(qos_spec)
 
+    def test_validate_qos_spec_adaptive(self):
+        # Just return without raising an exception.
+        na_utils.validate_qos_spec(fake.ADAPTIVE_QOS_SPEC)
+
     def test_validate_qos_spec_keys_weirdly_cased(self):
-        qos_spec = {'mAxIopS': 33000}
+        qos_spec = {'mAxIopS': 33000, 'mInIopS': 0}
 
         # Just return without raising an exception.
         na_utils.validate_qos_spec(qos_spec)
 
-    def test_validate_qos_spec_bad_key(self):
+    def test_validate_qos_spec_bad_key_max_flops(self):
         qos_spec = {'maxFlops': 33000}
 
         self.assertRaises(exception.Invalid,
                           na_utils.validate_qos_spec,
                           qos_spec)
 
-    def test_validate_qos_spec_bad_key_combination(self):
+    def test_validate_qos_spec_bad_key_min_bps(self):
+        qos_spec = {'minBps': 33000}
+
+        self.assertRaises(exception.Invalid,
+                          na_utils.validate_qos_spec,
+                          qos_spec)
+
+    def test_validate_qos_spec_bad_key_min_bps_per_gib(self):
+        qos_spec = {'minBPSperGiB': 33000}
+
+        self.assertRaises(exception.Invalid,
+                          na_utils.validate_qos_spec,
+                          qos_spec)
+
+    def test_validate_qos_spec_bad_key_combination_max_iops_max_bps(self):
         qos_spec = {'maxIOPS': 33000, 'maxBPS': 10000000}
 
         self.assertRaises(exception.Invalid,
                           na_utils.validate_qos_spec,
                           qos_spec)
 
+    def test_validate_qos_spec_bad_key_combination_miniops_miniopspergib(self):
+        qos_spec = {'minIOPS': 33000, 'minIOPSperGiB': 10000000}
+        self.assertRaises(exception.Invalid,
+                          na_utils.validate_qos_spec,
+                          qos_spec)
+
+    def test_validate_qos_spec_bad_key_combination_aqos_qos_max(self):
+        qos_spec = {'peakIOPSperGiB': 33000, 'maxIOPS': 33000}
+        self.assertRaises(exception.Invalid,
+                          na_utils.validate_qos_spec,
+                          qos_spec)
+
+    def test_validate_qos_spec_bad_key_combination_aqos_qos_min(self):
+        qos_spec = {'absoluteMinIOPS': 33000, 'minIOPS': 33000}
+        self.assertRaises(exception.Invalid,
+                          na_utils.validate_qos_spec,
+                          qos_spec)
+
+    def test_validate_qos_spec_bad_key_combination_aqos_qos_min_max(self):
+        qos_spec = {
+            'expectedIOPSperGiB': 33000,
+            'minIOPS': 33000,
+            'maxIOPS': 33000,
+        }
+        self.assertRaises(exception.Invalid,
+                          na_utils.validate_qos_spec,
+                          qos_spec)
+
+    def test_validate_qos_spec_adaptive_and_non_adaptive(self):
+        qos_spec = fake.INVALID_QOS_POLICY_GROUP_INFO_STANDARD_AND_ADAPTIVE
+
+        self.assertRaises(exception.Invalid,
+                          na_utils.validate_qos_spec,
+                          qos_spec)
+
     def test_map_qos_spec_none(self):
         qos_spec = None
 
@@ -271,6 +329,30 @@ class NetAppDriverUtilsTestCase(test.TestCase):
 
         self.assertIsNone(result)
 
+    def test_map_qos_spec_bad_key_combination_miniops_maxbpspergib(self):
+        qos_spec = {'minIOPS': 33000, 'maxBPSperGiB': 10000000}
+
+        self.assertRaises(exception.Invalid,
+                          na_utils.map_qos_spec,
+                          qos_spec,
+                          fake.VOLUME)
+
+    def test_map_qos_spec_bad_key_combination_min_iops_max_bps(self):
+        qos_spec = {'minIOPS': 33000, 'maxBPS': 10000000}
+
+        self.assertRaises(exception.Invalid,
+                          na_utils.map_qos_spec,
+                          qos_spec,
+                          fake.VOLUME)
+
+    def test_map_qos_spec_miniops_greater_than_maxiops(self):
+        qos_spec = {'minIOPS': 33001, 'maxIOPS': 33000}
+
+        self.assertRaises(exception.Invalid,
+                          na_utils.map_qos_spec,
+                          qos_spec,
+                          fake.VOLUME)
+
     def test_map_qos_spec_maxiops(self):
         qos_spec = {'maxIOPs': 33000}
         mock_get_name = self.mock_object(na_utils, 'get_qos_policy_group_name')
@@ -297,6 +379,20 @@ class NetAppDriverUtilsTestCase(test.TestCase):
 
         self.assertEqual(expected, result)
 
+    def test_map_qos_spec_miniopspergib_maxiopspergib(self):
+        qos_spec = {'minIOPSperGiB': 1000, 'maxIOPSperGiB': 1000}
+        mock_get_name = self.mock_object(na_utils, 'get_qos_policy_group_name')
+        mock_get_name.return_value = 'fake_qos_policy'
+        expected = {
+            'policy_name': 'fake_qos_policy',
+            'min_throughput': '42000iops',
+            'max_throughput': '42000iops',
+        }
+
+        result = na_utils.map_qos_spec(qos_spec, fake.VOLUME)
+
+        self.assertEqual(expected, result)
+
     def test_map_qos_spec_maxbps(self):
         qos_spec = {'maxBPS': 1000000}
         mock_get_name = self.mock_object(na_utils, 'get_qos_policy_group_name')
@@ -329,13 +425,121 @@ class NetAppDriverUtilsTestCase(test.TestCase):
         mock_get_name.return_value = 'fake_qos_policy'
         expected = {
             'policy_name': 'fake_qos_policy',
-            'max_throughput': None,
         }
 
         result = na_utils.map_qos_spec(qos_spec, fake.VOLUME)
 
         self.assertEqual(expected, result)
 
+    def test_map_qos_spec_miniops_maxiops(self):
+        qos_spec = {'minIOPs': 25000, 'maxIOPs': 33000}
+        mock_get_name = self.mock_object(na_utils, 'get_qos_policy_group_name')
+        mock_get_name.return_value = 'fake_qos_policy'
+        expected = {
+            'policy_name': 'fake_qos_policy',
+            'min_throughput': '25000iops',
+            'max_throughput': '33000iops',
+        }
+
+        result = na_utils.map_qos_spec(qos_spec, fake.VOLUME)
+
+        self.assertEqual(expected, result)
+
+    def test_map_aqos_spec(self):
+        qos_spec = {
+            'expectedIOPSperGiB': '128',
+            'peakIOPSperGiB': '512',
+            'expectedIOPSAllocation': 'used-space',
+            'peakIOPSAllocation': 'used-space',
+            'absoluteMinIOPS': '75',
+            'blockSize': 'ANY',
+        }
+        mock_get_name = self.mock_object(na_utils, 'get_qos_policy_group_name')
+        mock_get_name.return_value = 'fake_qos_policy'
+        expected = {
+            'expected_iops': '128IOPS/GB',
+            'peak_iops': '512IOPS/GB',
+            'expected_iops_allocation': 'used-space',
+            'peak_iops_allocation': 'used-space',
+            'absolute_min_iops': '75IOPS',
+            'block_size': 'ANY',
+            'policy_name': 'fake_qos_policy',
+        }
+
+        result = na_utils.map_aqos_spec(qos_spec, fake.VOLUME)
+
+        self.assertEqual(expected, result)
+
+    @ddt.data({'expectedIOPSperGiB': '528', 'peakIOPSperGiB': '128'},
+              {'expectedIOPSperGiB': '528'})
+    def test_map_aqos_spec_error(self, qos_spec):
+        mock_get_name = self.mock_object(na_utils, 'get_qos_policy_group_name')
+        mock_get_name.return_value = 'fake_qos_policy'
+
+        self.assertRaises(exception.Invalid, na_utils.map_aqos_spec, qos_spec,
+                          fake.VOLUME)
+
+    def test_is_qos_adaptive_adaptive_spec(self):
+        aqos_spec = fake.ADAPTIVE_QOS_SPEC
+
+        self.assertTrue(na_utils.is_qos_adaptive(aqos_spec))
+
+    def test_is_qos_adaptive_weirdly_cased_adaptive_spec(self):
+        aqos_spec = {'expecTEDiopsPERgib': '128IOPS/GB'}
+
+        self.assertTrue(na_utils.is_qos_adaptive(aqos_spec))
+
+    def test_is_qos_adaptive_non_adaptive_spec(self):
+        qos_spec = fake.QOS_SPEC
+
+        self.assertFalse(na_utils.is_qos_adaptive(qos_spec))
+
+    def test_is_qos_policy_group_spec_adaptive_adaptive_spec(self):
+        aqos_spec = {
+            'spec': {
+                'expected_iops': '128IOPS/GB',
+                'peak_iops': '512IOPS/GB',
+                'expected_iops_allocation': 'used-space',
+                'absolute_min_iops': '75IOPS',
+                'block_size': 'ANY',
+                'policy_name': 'fake_policy_name',
+            }
+        }
+
+        self.assertTrue(na_utils.is_qos_policy_group_spec_adaptive(aqos_spec))
+
+    def test_is_qos_policy_group_spec_adaptive_none(self):
+        qos_spec = None
+
+        self.assertFalse(na_utils.is_qos_policy_group_spec_adaptive(qos_spec))
+
+    def test_is_qos_policy_group_spec_adaptive_legacy(self):
+        qos_spec = {
+            'legacy': fake.LEGACY_QOS,
+        }
+
+        self.assertFalse(na_utils.is_qos_policy_group_spec_adaptive(qos_spec))
+
+    def test_is_qos_policy_group_spec_adaptive_non_adaptive_spec(self):
+        qos_spec = {
+            'spec': {
+                'max_throughput': '21834289B/s',
+                'policy_name': 'fake_policy_name',
+            }
+        }
+
+        self.assertFalse(na_utils.is_qos_policy_group_spec_adaptive(qos_spec))
+
+    def test_policy_group_qos_spec_is_adaptive_invalid_spec(self):
+        qos_spec = {
+            'spec': {
+                'max_flops': '512',
+                'policy_name': 'fake_policy_name',
+            }
+        }
+
+        self.assertFalse(na_utils.is_qos_policy_group_spec_adaptive(qos_spec))
+
     def test_map_dict_to_lower(self):
         original = {'UPperKey': 'Value'}
         expected = {'upperkey': 'Value'}
@@ -437,7 +641,6 @@ class NetAppDriverUtilsTestCase(test.TestCase):
         mock_get_valid_qos_spec_from_volume_type = self.mock_object(
             na_utils, 'get_valid_backend_qos_spec_from_volume_type')
         mock_get_valid_qos_spec_from_volume_type.return_value = None
-        self.mock_object(na_utils, 'check_for_invalid_qos_spec_combination')
         expected = fake.QOS_POLICY_GROUP_INFO_NONE
 
         result = na_utils.get_valid_qos_policy_group_info(fake.VOLUME)
@@ -455,7 +658,6 @@ class NetAppDriverUtilsTestCase(test.TestCase):
         mock_get_valid_qos_spec_from_volume_type = self.mock_object(
             na_utils, 'get_valid_backend_qos_spec_from_volume_type')
         mock_get_valid_qos_spec_from_volume_type.return_value = None
-        self.mock_object(na_utils, 'check_for_invalid_qos_spec_combination')
 
         result = na_utils.get_valid_qos_policy_group_info(fake.VOLUME)
 
@@ -472,7 +674,6 @@ class NetAppDriverUtilsTestCase(test.TestCase):
             na_utils, 'get_valid_backend_qos_spec_from_volume_type')
         mock_get_valid_qos_spec_from_volume_type.return_value =\
             fake.QOS_POLICY_GROUP_SPEC
-        self.mock_object(na_utils, 'check_for_invalid_qos_spec_combination')
 
         result = na_utils.get_valid_qos_policy_group_info(fake.VOLUME)
 
@@ -482,25 +683,43 @@ class NetAppDriverUtilsTestCase(test.TestCase):
         mock_get_spec = self.mock_object(
             na_utils, 'get_backend_qos_spec_from_volume_type')
         mock_get_spec.return_value = None
-        mock_validate = self.mock_object(na_utils, 'validate_qos_spec')
+        mock_map_qos_spec = self.mock_object(
+            na_utils, 'map_qos_spec')
+        mock_map_aqos_spec = self.mock_object(
+            na_utils, 'map_aqos_spec')
 
         result = na_utils.get_valid_backend_qos_spec_from_volume_type(
             fake.VOLUME, fake.VOLUME_TYPE)
 
         self.assertIsNone(result)
-        self.assertEqual(0, mock_validate.call_count)
+        mock_map_qos_spec.assert_not_called()
+        mock_map_aqos_spec.assert_not_called()
 
     def test_get_valid_backend_qos_spec_from_volume_type(self):
         mock_get_spec = self.mock_object(
             na_utils, 'get_backend_qos_spec_from_volume_type')
         mock_get_spec.return_value = fake.QOS_SPEC
-        mock_validate = self.mock_object(na_utils, 'validate_qos_spec')
+        mock_map_aqos_spec = self.mock_object(
+            na_utils, 'map_aqos_spec')
 
         result = na_utils.get_valid_backend_qos_spec_from_volume_type(
             fake.VOLUME, fake.VOLUME_TYPE)
 
         self.assertEqual(fake.QOS_POLICY_GROUP_SPEC, result)
-        self.assertEqual(1, mock_validate.call_count)
+        mock_map_aqos_spec.assert_not_called()
+
+    def test_get_valid_backend_qos_spec_from_volume_type_adaptive(self):
+        mock_get_spec = self.mock_object(
+            na_utils, 'get_backend_qos_spec_from_volume_type')
+        mock_get_spec.return_value = fake.ADAPTIVE_QOS_SPEC
+        mock_map_qos_spec = self.mock_object(
+            na_utils, 'map_qos_spec')
+
+        result = na_utils.get_valid_backend_qos_spec_from_volume_type(
+            fake.VOLUME, fake.VOLUME_TYPE)
+
+        self.assertEqual(fake.ADAPTIVE_QOS_POLICY_GROUP_SPEC, result)
+        mock_map_qos_spec.assert_not_called()
 
     def test_get_backend_qos_spec_from_volume_type_no_qos_specs_id(self):
         volume_type = copy.deepcopy(fake.VOLUME_TYPE)
@@ -510,7 +729,7 @@ class NetAppDriverUtilsTestCase(test.TestCase):
         result = na_utils.get_backend_qos_spec_from_volume_type(volume_type)
 
         self.assertIsNone(result)
-        self.assertEqual(0, mock_get_context.call_count)
+        mock_get_context.assert_not_called()
 
     def test_get_backend_qos_spec_from_volume_type_no_qos_spec(self):
         volume_type = fake.VOLUME_TYPE
@@ -552,11 +771,20 @@ class NetAppDriverUtilsTestCase(test.TestCase):
 
         self.assertEqual(fake.QOS_SPEC, result)
 
-    def test_check_for_invalid_qos_spec_combination(self):
+    def test_check_for_invalid_qos_spec_combination_legacy(self):
+        na_utils.check_for_invalid_qos_spec_combination(
+            fake.LEGACY_QOS_POLICY_GROUP_INFO,
+            fake.VOLUME_TYPE)
 
+    def test_check_for_invalid_qos_spec_combination_spec(self):
+        na_utils.check_for_invalid_qos_spec_combination(
+            fake.QOS_POLICY_GROUP_INFO,
+            fake.VOLUME_TYPE)
+
+    def test_check_for_invalid_qos_spec_combination_legacy_and_spec(self):
         self.assertRaises(exception.Invalid,
                           na_utils.check_for_invalid_qos_spec_combination,
-                          fake.INVALID_QOS_POLICY_GROUP_INFO,
+                          fake.INVALID_QOS_POLICY_GROUP_INFO_LEGACY_AND_SPEC,
                           fake.VOLUME_TYPE)
 
     def test_get_legacy_qos_policy(self):
@@ -597,6 +825,21 @@ class NetAppDriverUtilsTestCase(test.TestCase):
                           na_utils.get_export_host_junction_path,
                           share)
 
+    @ddt.data(True, False)
+    def test_qos_min_feature_name(self, is_nfs):
+        name = 'node'
+        feature_name = na_utils.qos_min_feature_name(is_nfs, name)
+
+        if is_nfs:
+            self.assertEqual('QOS_MIN_NFS_' + name, feature_name)
+        else:
+            self.assertEqual('QOS_MIN_BLOCK_' + name, feature_name)
+
+        self.assertEqual('QOS_MIN_NFS_',
+                         na_utils.qos_min_feature_name(True, None))
+        self.assertEqual('QOS_MIN_BLOCK_',
+                         na_utils.qos_min_feature_name(False, None))
+
 
 class OpenStackInfoTestCase(test.TestCase):
 
