@@ -18,7 +18,6 @@ import time
 from unittest import mock
 
 import ddt
-from oslo_config import cfg
 from oslo_utils import units
 
 from cinder import context
@@ -28,15 +27,12 @@ from cinder.objects import fields
 from cinder.tests.unit import fake_snapshot
 from cinder.tests.unit import fake_volume
 from cinder.tests.unit import test
-from cinder import utils
 from cinder.volume import configuration
 from cinder.volume.drivers.kaminario import kaminario_common
 from cinder.volume.drivers.kaminario import kaminario_fc
 from cinder.volume.drivers.kaminario import kaminario_iscsi
 from cinder.volume import volume_utils
 
-CONF = cfg.CONF
-
 CONNECTOR = {'initiator': 'iqn.1993-08.org.debian:01:12aa12aa12aa',
              'ip': '192.168.2.5', 'platform': 'x86_64', 'host': 'test-k2',
              'wwpns': ['12341a2a00001234', '12341a2a00001235'],
@@ -147,6 +143,8 @@ class TestKaminarioCommon(test.TestCase):
         self.conf.volume_dd_blocksize = 2
         self.conf.disable_discovery = False
         self.conf.unique_fqdn_network = True
+        self.conf.use_multipath_for_image_xfer = False
+        self.conf.enforce_multipath_for_image_xfer = False
 
     def _setup_driver(self):
         self.driver = (kaminario_iscsi.
@@ -207,7 +205,7 @@ class TestKaminarioCommon(test.TestCase):
         self.assertRaises(kaminario_common.KaminarioCinderDriverException,
                           self.driver.delete_snapshot, self.snap)
 
-    @mock.patch.object(utils, 'brick_get_connector_properties')
+    @mock.patch.object(volume_utils, 'brick_get_connector_properties')
     @mock.patch.object(volume_utils, 'copy_volume')
     def test_create_volume_from_snapshot(self, mock_copy_volume,
                                          mock_brick_get):
@@ -218,7 +216,7 @@ class TestKaminarioCommon(test.TestCase):
         result = self.driver.create_volume_from_snapshot(self.vol, self.snap)
         self.assertIsNone(result)
 
-    @mock.patch.object(utils, 'brick_get_connector_properties')
+    @mock.patch.object(volume_utils, 'brick_get_connector_properties')
     @mock.patch.object(volume_utils, 'copy_volume')
     def test_create_volume_from_snapshot_with_exception(self, mock_copy_volume,
                                                         mock_brick_get):
@@ -230,7 +228,7 @@ class TestKaminarioCommon(test.TestCase):
                           self.driver.create_volume_from_snapshot, self.vol,
                           self.snap)
 
-    @mock.patch.object(utils, 'brick_get_connector_properties')
+    @mock.patch.object(volume_utils, 'brick_get_connector_properties')
     @mock.patch.object(volume_utils, 'copy_volume')
     def test_create_cloned_volume(self, mock_copy_volume, mock_brick_get):
         """Test create_cloned_volume."""
@@ -240,7 +238,7 @@ class TestKaminarioCommon(test.TestCase):
         result = self.driver.create_cloned_volume(self.vol, self.vol)
         self.assertIsNone(result)
 
-    @mock.patch.object(utils, 'brick_get_connector_properties')
+    @mock.patch.object(volume_utils, 'brick_get_connector_properties')
     @mock.patch.object(volume_utils, 'copy_volume')
     def test_create_cloned_volume_with_exception(self, mock_copy_volume,
                                                  mock_brick_get):
