@@ -15,9 +15,18 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
-"""Utilities and helper functions."""
+"""Utilities and helper functions for all Cinder code.
+
+   This file is for utilities useful in all of Cinder,
+   including cinder-manage, the api service, the scheduler,
+   etc.
+
+   Code related to volume drivers and connecting to volumes
+   should be placed in volume_utils instead.
+"""
+
+from __future__ import annotations  # Remove when only supporting python 3.9+
 
-import abc
 from collections import OrderedDict
 import contextlib
 import datetime
@@ -34,26 +43,23 @@ import shutil
 import stat
 import sys
 import tempfile
-import time
-import types
+import typing
+from typing import Callable, Iterable, Iterator  # noqa: H301
+from typing import Optional, Type, Union  # noqa: H301
 
-from castellan import key_manager
 import eventlet
 from eventlet import tpool
-from os_brick import encryptors
-from os_brick.initiator import connector
 from oslo_concurrency import lockutils
 from oslo_concurrency import processutils
 from oslo_config import cfg
 from oslo_log import log as logging
-from oslo_utils import encodeutils
 from oslo_utils import excutils
 from oslo_utils import importutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
-import six
 import tenacity
 
+from cinder import coordination
 from cinder import exception
 from cinder.i18n import _
 
@@ -61,17 +67,56 @@ CONF = cfg.CONF
 LOG = logging.getLogger(__name__)
 ISO_TIME_FORMAT = "%Y-%m-%dT%H:%M:%S"
 PERFECT_TIME_FORMAT = "%Y-%m-%dT%H:%M:%S.%f"
-VALID_TRACE_FLAGS = {'method', 'api'}
-TRACE_METHOD = False
-TRACE_API = False
 INITIAL_AUTO_MOSR = 20
 INFINITE_UNKNOWN_VALUES = ('infinite', 'unknown')
 
 
 synchronized = lockutils.synchronized_with_prefix('cinder-')
+synchronized_remove = lockutils.remove_external_lock_file_with_prefix(
+    'cinder-')
 
 
-def as_int(obj, quiet=True):
+def clean_volume_file_locks(volume_id, driver):
+    """Remove file locks used by Cinder.
+
+    This doesn't take care of driver locks, those should be handled in driver's
+    delete_volume method.
+    """
+    for name in (volume_id + '-delete_volume', volume_id,
+                 volume_id + '-detach_volume'):
+        try:
+            synchronized_remove(name)
+        except Exception as exc:
+            LOG.warning('Failed to cleanup volume lock %(name)s: %(exc)s',
+                        {'name': name, 'exc': exc})
+
+    try:
+        driver.clean_volume_file_locks(volume_id)
+    except Exception as exc:
+        LOG.warning('Failed to cleanup driver locks for volume %(id)s: '
+                    '%(exc)s', {'id': volume_id, 'exc': exc})
+
+
+def api_clean_volume_file_locks(volume_id):
+    coordination.synchronized_remove('attachment_update-' + volume_id + '-*')
+
+
+def clean_snapshot_file_locks(snapshot_id, driver):
+    try:
+        name = snapshot_id + '-delete_snapshot'
+        synchronized_remove(name)
+    except Exception as exc:
+        LOG.warning('Failed to cleanup snapshot lock %(name)s: %(exc)s',
+                    {'name': name, 'exc': exc})
+
+    try:
+        driver.clean_snapshot_file_locks(snapshot_id)
+    except Exception as exc:
+        LOG.warning('Failed to cleanup driver locks for snapshot %(id)s: '
+                    '%(exc)s', {'id': snapshot_id, 'exc': exc})
+
+
+def as_int(obj: Union[int, float, str], quiet: bool = True) -> int:
     # Try "2" -> 2
     try:
         return int(obj)
@@ -85,10 +130,13 @@ def as_int(obj, quiet=True):
     # Eck, not sure what this is then.
     if not quiet:
         raise TypeError(_("Can not translate %s to integer.") % (obj))
+
+    obj = typing.cast(int, obj)
     return obj
 
 
-def check_exclusive_options(**kwargs):
+def check_exclusive_options(
+        **kwargs: Optional[Union[dict, str, bool]]) -> None:
     """Checks that only one of the provided options is actually not-none.
 
     Iterates over all the kwargs passed in and checks that only one of said
@@ -111,24 +159,24 @@ def check_exclusive_options(**kwargs):
         #
         # Ex: 'the_key' -> 'the key'
         if pretty_keys:
-            names = [k.replace('_', ' ') for k in kwargs]
+            tnames = [k.replace('_', ' ') for k in kwargs]
         else:
-            names = kwargs.keys()
-        names = ", ".join(sorted(names))
+            tnames = list(kwargs.keys())
+        names = ", ".join(sorted(tnames))
         msg = (_("May specify only one of %s") % (names))
         raise exception.InvalidInput(reason=msg)
 
 
-def execute(*cmd, **kwargs):
+def execute(*cmd: str, **kwargs: Union[bool, str]) -> tuple[str, str]:
     """Convenience wrapper around oslo's execute() method."""
     if 'run_as_root' in kwargs and 'root_helper' not in kwargs:
         kwargs['root_helper'] = get_root_helper()
     return processutils.execute(*cmd, **kwargs)
 
 
-def check_ssh_injection(cmd_list):
-    ssh_injection_pattern = ['`', '$', '|', '||', ';', '&', '&&', '>', '>>',
-                             '<']
+def check_ssh_injection(cmd_list: list[str]) -> None:
+    ssh_injection_pattern: tuple[str, ...] = ('`', '$', '|', '||', ';', '&',
+                                              '&&', '>', '>>', '<')
 
     # Check whether injection attacks exist
     for arg in cmd_list:
@@ -161,7 +209,8 @@ def check_ssh_injection(cmd_list):
                     raise exception.SSHInjectionThreat(command=cmd_list)
 
 
-def check_metadata_properties(metadata=None):
+def check_metadata_properties(
+        metadata: Optional[dict[str, str]]) -> None:
     """Checks that the volume metadata properties are valid."""
 
     if not metadata:
@@ -187,7 +236,9 @@ def check_metadata_properties(metadata=None):
             raise exception.InvalidVolumeMetadataSize(reason=msg)
 
 
-def last_completed_audit_period(unit=None):
+def last_completed_audit_period(unit: Optional[str] = None) -> \
+        tuple[Union[datetime.datetime, datetime.timedelta],
+              Union[datetime.datetime, datetime.timedelta]]:
     """This method gives you the most recently *completed* audit period.
 
     arguments:
@@ -208,11 +259,15 @@ def last_completed_audit_period(unit=None):
     if not unit:
         unit = CONF.volume_usage_audit_period
 
-    offset = 0
+    unit = typing.cast(str, unit)
+
+    offset: Union[str, int] = 0
     if '@' in unit:
         unit, offset = unit.split("@", 1)
         offset = int(offset)
 
+    offset = typing.cast(int, offset)
+
     rightnow = timeutils.utcnow()
     if unit not in ('month', 'day', 'year', 'hour'):
         raise ValueError('Time period must be hour, day, month or year')
@@ -274,7 +329,7 @@ def last_completed_audit_period(unit=None):
     return (begin, end)
 
 
-def monkey_patch():
+def monkey_patch() -> None:
     """Patches decorators for all functions in a specified module.
 
     If the CONF.monkey_patch set as True,
@@ -309,7 +364,7 @@ def monkey_patch():
             if isinstance(module_data[key], pyclbr.Class):
                 clz = importutils.import_class("%s.%s" % (module, key))
                 # On Python 3, unbound methods are regular functions
-                predicate = inspect.isfunction if six.PY3 else inspect.ismethod
+                predicate = inspect.isfunction
                 for method, func in inspect.getmembers(clz, predicate):
                     setattr(
                         clz, method,
@@ -321,7 +376,9 @@ def monkey_patch():
                         decorator("%s.%s" % (module, key), func))
 
 
-def make_dev_path(dev, partition=None, base='/dev'):
+def make_dev_path(dev: str,
+                  partition: Optional[str] = None,
+                  base: str = '/dev') -> str:
     """Return a path to a particular device.
 
     >>> make_dev_path('xvdc')
@@ -336,7 +393,7 @@ def make_dev_path(dev, partition=None, base='/dev'):
     return path
 
 
-def robust_file_write(directory, filename, data):
+def robust_file_write(directory: str, filename: str, data: str) -> None:
     """Robust file write.
 
     Use "write to temp file and rename" model for writing the
@@ -372,15 +429,17 @@ def robust_file_write(directory, filename, data):
         with excutils.save_and_reraise_exception():
             LOG.error("Failed to write persistence file: %(path)s.",
                       {'path': os.path.join(directory, filename)})
-            if os.path.isfile(tempname):
-                os.unlink(tempname)
+            if tempname is not None:
+                if os.path.isfile(tempname):
+                    os.unlink(tempname)
     finally:
-        if dirfd:
+        if dirfd is not None:
             os.close(dirfd)
 
 
 @contextlib.contextmanager
-def temporary_chown(path, owner_uid=None):
+def temporary_chown(path: str,
+                    owner_uid: Optional[int] = None) -> Iterator[None]:
     """Temporarily chown a path.
 
     :params owner_uid: UID of temporary owner (defaults to current user)
@@ -398,16 +457,16 @@ def temporary_chown(path, owner_uid=None):
     orig_uid = os.stat(path).st_uid
 
     if orig_uid != owner_uid:
-        execute('chown', owner_uid, path, run_as_root=True)
+        execute('chown', str(owner_uid), path, run_as_root=True)
     try:
         yield
     finally:
         if orig_uid != owner_uid:
-            execute('chown', orig_uid, path, run_as_root=True)
+            execute('chown', str(orig_uid), path, run_as_root=True)
 
 
 @contextlib.contextmanager
-def tempdir(**kwargs):
+def tempdir(**kwargs) -> Iterator[str]:
     tmpdir = tempfile.mkdtemp(**kwargs)
     try:
         yield tmpdir
@@ -415,127 +474,31 @@ def tempdir(**kwargs):
         try:
             shutil.rmtree(tmpdir)
         except OSError as e:
-            LOG.debug('Could not remove tmpdir: %s',
-                      six.text_type(e))
+            LOG.debug('Could not remove tmpdir: %s', str(e))
 
 
-def get_root_helper():
+def get_root_helper() -> str:
     return 'sudo cinder-rootwrap %s' % CONF.rootwrap_config
 
 
-def brick_get_connector_properties(multipath=False, enforce_multipath=False):
-    """Wrapper to automatically set root_helper in brick calls.
-
-    :param multipath: A boolean indicating whether the connector can
-                      support multipath.
-    :param enforce_multipath: If True, it raises exception when multipath=True
-                              is specified but multipathd is not running.
-                              If False, it falls back to multipath=False
-                              when multipathd is not running.
-    """
-
-    root_helper = get_root_helper()
-    return connector.get_connector_properties(root_helper,
-                                              CONF.my_ip,
-                                              multipath,
-                                              enforce_multipath)
-
-
-def brick_get_connector(protocol, driver=None,
-                        use_multipath=False,
-                        device_scan_attempts=3,
-                        *args, **kwargs):
-    """Wrapper to get a brick connector object.
-
-    This automatically populates the required protocol as well
-    as the root_helper needed to execute commands.
-    """
-
-    root_helper = get_root_helper()
-    return connector.InitiatorConnector.factory(protocol, root_helper,
-                                                driver=driver,
-                                                use_multipath=use_multipath,
-                                                device_scan_attempts=
-                                                device_scan_attempts,
-                                                *args, **kwargs)
-
-
-def brick_get_encryptor(connection_info, *args, **kwargs):
-    """Wrapper to get a brick encryptor object."""
-
-    root_helper = get_root_helper()
-    km = key_manager.API(CONF)
-    return encryptors.get_volume_encryptor(root_helper=root_helper,
-                                           connection_info=connection_info,
-                                           keymgr=km,
-                                           *args, **kwargs)
-
-
-def brick_attach_volume_encryptor(context, attach_info, encryption):
-    """Attach encryption layer."""
-    connection_info = attach_info['conn']
-    connection_info['data']['device_path'] = attach_info['device']['path']
-    encryptor = brick_get_encryptor(connection_info,
-                                    **encryption)
-    encryptor.attach_volume(context, **encryption)
-
-
-def brick_detach_volume_encryptor(attach_info, encryption):
-    """Detach encryption layer."""
-    connection_info = attach_info['conn']
-    connection_info['data']['device_path'] = attach_info['device']['path']
-
-    encryptor = brick_get_encryptor(connection_info,
-                                    **encryption)
-    encryptor.detach_volume(**encryption)
-
-
-def require_driver_initialized(driver):
-    """Verifies if `driver` is initialized
-
-    If the driver is not initialized, an exception will be raised.
-
-    :params driver: The driver instance.
-    :raises: `exception.DriverNotInitialized`
-    """
-    # we can't do anything if the driver didn't init
-    if not driver.initialized:
-        driver_name = driver.__class__.__name__
-        LOG.error("Volume driver %s not initialized", driver_name)
-        raise exception.DriverNotInitialized()
-    else:
-        log_unsupported_driver_warning(driver)
-
-
-def log_unsupported_driver_warning(driver):
-    """Annoy the log about unsupported drivers."""
-    if not driver.supported:
-        # Check to see if the driver is flagged as supported.
-        LOG.warning("Volume driver (%(driver_name)s %(version)s) is "
-                    "currently unsupported and may be removed in the "
-                    "next release of OpenStack.  Use at your own risk.",
-                    {'driver_name': driver.__class__.__name__,
-                     'version': driver.get_version()},
-                    resource={'type': 'driver',
-                              'id': driver.__class__.__name__})
-
-
-def get_file_mode(path):
+def get_file_mode(path: str) -> int:
     """This primarily exists to make unit testing easier."""
     return stat.S_IMODE(os.stat(path).st_mode)
 
 
-def get_file_gid(path):
+def get_file_gid(path: str) -> int:
     """This primarily exists to make unit testing easier."""
     return os.stat(path).st_gid
 
 
-def get_file_size(path):
+def get_file_size(path: str) -> int:
     """Returns the file size."""
     return os.stat(path).st_size
 
 
-def _get_disk_of_partition(devpath, st=None):
+def _get_disk_of_partition(
+        devpath: str,
+        st: Optional[os.stat_result] = None) -> tuple[str, os.stat_result]:
     """Gets a disk device path and status from partition path.
 
     Returns a disk device path from a partition device path, and stat for
@@ -558,7 +521,9 @@ def _get_disk_of_partition(devpath, st=None):
     return (devpath, st)
 
 
-def get_bool_param(param_string, params, default=False):
+def get_bool_param(param_string: str,
+                   params: dict,
+                   default: bool = False) -> bool:
     param = params.get(param_string, default)
     if not strutils.is_valid_boolstr(param):
         msg = _("Value '%(param)s' for '%(param_string)s' is not "
@@ -568,7 +533,8 @@ def get_bool_param(param_string, params, default=False):
     return strutils.bool_from_string(param, strict=True)
 
 
-def get_blkdev_major_minor(path, lookup_for_file=True):
+def get_blkdev_major_minor(path: str,
+                           lookup_for_file: bool = True) -> Optional[str]:
     """Get 'major:minor' number of block device.
 
     Get the device's 'major:minor' number of a block device to control
@@ -596,8 +562,9 @@ def get_blkdev_major_minor(path, lookup_for_file=True):
         raise exception.CinderException(msg)
 
 
-def check_string_length(value, name, min_length=0, max_length=None,
-                        allow_all_spaces=True):
+def check_string_length(value: str, name: str, min_length: int = 0,
+                        max_length: Optional[int] = None,
+                        allow_all_spaces: bool = True) -> None:
     """Check the length of specified string.
 
     :param value: the value of the string
@@ -617,7 +584,7 @@ def check_string_length(value, name, min_length=0, max_length=None,
         raise exception.InvalidInput(reason=msg)
 
 
-def is_blk_device(dev):
+def is_blk_device(dev: str) -> bool:
     try:
         if stat.S_ISBLK(os.stat(dev).st_mode):
             return True
@@ -628,30 +595,30 @@ def is_blk_device(dev):
 
 
 class ComparableMixin(object):
-    def _compare(self, other, method):
+    def _compare(self, other: object, method: Callable):
         try:
-            return method(self._cmpkey(), other._cmpkey())
+            return method(self._cmpkey(), other._cmpkey())  # type: ignore
         except (AttributeError, TypeError):
             # _cmpkey not implemented, or return different type,
             # so I can't compare with "other".
             return NotImplemented
 
-    def __lt__(self, other):
+    def __lt__(self, other: object):
         return self._compare(other, lambda s, o: s < o)
 
-    def __le__(self, other):
+    def __le__(self, other: object):
         return self._compare(other, lambda s, o: s <= o)
 
-    def __eq__(self, other):
+    def __eq__(self, other: object):
         return self._compare(other, lambda s, o: s == o)
 
-    def __ge__(self, other):
+    def __ge__(self, other: object):
         return self._compare(other, lambda s, o: s >= o)
 
-    def __gt__(self, other):
+    def __gt__(self, other: object):
         return self._compare(other, lambda s, o: s > o)
 
-    def __ne__(self, other):
+    def __ne__(self, other: object):
         return self._compare(other, lambda s, o: s != o)
 
 
@@ -666,8 +633,16 @@ class retry_if_exit_code(tenacity.retry_if_exception):
                 exc.exit_code in self.codes)
 
 
-def retry(retry_param, interval=1, retries=3, backoff_rate=2,
-          wait_random=False, retry=tenacity.retry_if_exception_type):
+def retry(retry_param: Union[None,
+                             Type[Exception],
+                             tuple[Type[Exception], ...],
+                             int,
+                             tuple[int, ...]],
+          interval: float = 1,
+          retries: int = 3,
+          backoff_rate: float = 2,
+          wait_random: bool = False,
+          retry=tenacity.retry_if_exception_type) -> Callable:
 
     if retries < 1:
         raise ValueError('Retries must be greater than or '
@@ -679,9 +654,9 @@ def retry(retry_param, interval=1, retries=3, backoff_rate=2,
         wait = tenacity.wait_exponential(
             multiplier=interval, min=0, exp_base=backoff_rate)
 
-    def _decorator(f):
+    def _decorator(f: Callable) -> Callable:
 
-        @six.wraps(f)
+        @functools.wraps(f)
         def _wrapper(*args, **kwargs):
             r = tenacity.Retrying(
                 sleep=tenacity.nap.sleep,
@@ -691,14 +666,14 @@ def retry(retry_param, interval=1, retries=3, backoff_rate=2,
                 reraise=True,
                 retry=retry(retry_param),
                 wait=wait)
-            return r.call(f, *args, **kwargs)
+            return r(f, *args, **kwargs)
 
         return _wrapper
 
     return _decorator
 
 
-def convert_str(text):
+def convert_str(text: Union[str, bytes]) -> str:
     """Convert to native string.
 
     Convert bytes and Unicode strings to native strings:
@@ -707,173 +682,14 @@ def convert_str(text):
       encode Unicode using encodeutils.safe_encode()
     * convert to Unicode on Python 3: decode bytes from UTF-8
     """
-    if six.PY2:
-        return encodeutils.to_utf8(text)
+    if isinstance(text, bytes):
+        return text.decode('utf-8')
     else:
-        if isinstance(text, bytes):
-            return text.decode('utf-8')
-        else:
-            return text
+        return text
 
 
-def trace_method(f):
-    """Decorates a function if TRACE_METHOD is true."""
-    @functools.wraps(f)
-    def trace_method_logging_wrapper(*args, **kwargs):
-        if TRACE_METHOD:
-            return trace(f)(*args, **kwargs)
-        return f(*args, **kwargs)
-    return trace_method_logging_wrapper
-
-
-def trace_api(*dec_args, **dec_kwargs):
-    """Decorates a function if TRACE_API is true."""
-
-    def _decorator(f):
-        @functools.wraps(f)
-        def trace_api_logging_wrapper(*args, **kwargs):
-            if TRACE_API:
-                return trace(f, *dec_args, **dec_kwargs)(*args, **kwargs)
-            return f(*args, **kwargs)
-        return trace_api_logging_wrapper
-
-    if len(dec_args) == 0:
-        # filter_function is passed and args does not contain f
-        return _decorator
-    else:
-        # filter_function is not passed
-        return _decorator(dec_args[0])
-
-
-def trace(*dec_args, **dec_kwargs):
-    """Trace calls to the decorated function.
-
-    This decorator should always be defined as the outermost decorator so it
-    is defined last. This is important so it does not interfere
-    with other decorators.
-
-    Using this decorator on a function will cause its execution to be logged at
-    `DEBUG` level with arguments, return values, and exceptions.
-
-    :returns: a function decorator
-    """
-
-    def _decorator(f):
-
-        func_name = f.__name__
-
-        @functools.wraps(f)
-        def trace_logging_wrapper(*args, **kwargs):
-            filter_function = dec_kwargs.get('filter_function')
-
-            if len(args) > 0:
-                maybe_self = args[0]
-            else:
-                maybe_self = kwargs.get('self', None)
-
-            if maybe_self and hasattr(maybe_self, '__module__'):
-                logger = logging.getLogger(maybe_self.__module__)
-            else:
-                logger = LOG
-
-            # NOTE(ameade): Don't bother going any further if DEBUG log level
-            # is not enabled for the logger.
-            if not logger.isEnabledFor(py_logging.DEBUG):
-                return f(*args, **kwargs)
-
-            all_args = inspect.getcallargs(f, *args, **kwargs)
-
-            pass_filter = filter_function is None or filter_function(all_args)
-
-            if pass_filter:
-                logger.debug('==> %(func)s: call %(all_args)r',
-                             {'func': func_name,
-                              'all_args': strutils.mask_password(
-                                  six.text_type(all_args))})
-
-            start_time = time.time() * 1000
-            try:
-                result = f(*args, **kwargs)
-            except Exception as exc:
-                total_time = int(round(time.time() * 1000)) - start_time
-                logger.debug('<== %(func)s: exception (%(time)dms) %(exc)r',
-                             {'func': func_name,
-                              'time': total_time,
-                              'exc': exc})
-                raise
-            total_time = int(round(time.time() * 1000)) - start_time
-
-            if isinstance(result, dict):
-                mask_result = strutils.mask_dict_password(result)
-            elif isinstance(result, six.string_types):
-                mask_result = strutils.mask_password(result)
-            else:
-                mask_result = result
-
-            if pass_filter:
-                logger.debug('<== %(func)s: return (%(time)dms) %(result)r',
-                             {'func': func_name,
-                              'time': total_time,
-                              'result': mask_result})
-            return result
-        return trace_logging_wrapper
-
-    if len(dec_args) == 0:
-        # filter_function is passed and args does not contain f
-        return _decorator
-    else:
-        # filter_function is not passed
-        return _decorator(dec_args[0])
-
-
-class TraceWrapperMetaclass(type):
-    """Metaclass that wraps all methods of a class with trace_method.
-
-    This metaclass will cause every function inside of the class to be
-    decorated with the trace_method decorator.
-
-    To use the metaclass you define a class like so:
-    @six.add_metaclass(utils.TraceWrapperMetaclass)
-    class MyClass(object):
-    """
-    def __new__(meta, classname, bases, classDict):
-        newClassDict = {}
-        for attributeName, attribute in classDict.items():
-            if isinstance(attribute, types.FunctionType):
-                # replace it with a wrapped version
-                attribute = functools.update_wrapper(trace_method(attribute),
-                                                     attribute)
-            newClassDict[attributeName] = attribute
-
-        return type.__new__(meta, classname, bases, newClassDict)
-
-
-class TraceWrapperWithABCMetaclass(abc.ABCMeta, TraceWrapperMetaclass):
-    """Metaclass that wraps all methods of a class with trace."""
-    pass
-
-
-def setup_tracing(trace_flags):
-    """Set global variables for each trace flag.
-
-    Sets variables TRACE_METHOD and TRACE_API, which represent
-    whether to log methods or api traces.
-
-    :param trace_flags: a list of strings
-    """
-    global TRACE_METHOD
-    global TRACE_API
-    try:
-        trace_flags = [flag.strip() for flag in trace_flags]
-    except TypeError:  # Handle when trace_flags is None or a test mock
-        trace_flags = []
-    for invalid_flag in (set(trace_flags) - VALID_TRACE_FLAGS):
-        LOG.warning('Invalid trace flag: %s', invalid_flag)
-    TRACE_METHOD = 'method' in trace_flags
-    TRACE_API = 'api' in trace_flags
-
-
-def build_or_str(elements, str_format=None):
+def build_or_str(elements: Union[None, str, Iterable[str]],
+                 str_format: Optional[str] = None) -> str:
     """Builds a string of elements joined by 'or'.
 
     Will join strings with the 'or' word and if a str_format is provided it
@@ -888,22 +704,129 @@ def build_or_str(elements, str_format=None):
     if not elements:
         return ''
 
-    if not isinstance(elements, six.string_types):
+    if not isinstance(elements, str):
         elements = _(' or ').join(elements)
 
+    elements = typing.cast(str, elements)
+
     if str_format:
         return str_format % elements
+
     return elements
 
 
-def calculate_virtual_free_capacity(total_capacity,
-                                    free_capacity,
-                                    provisioned_capacity,
-                                    thin_provisioning_support,
-                                    max_over_subscription_ratio,
-                                    reserved_percentage,
-                                    thin):
-    """Calculate the virtual free capacity based on thin provisioning support.
+def calculate_capacity_factors(total_capacity: float,
+                               free_capacity: float,
+                               provisioned_capacity: float,
+                               thin_provisioning_support: bool,
+                               max_over_subscription_ratio: float,
+                               reserved_percentage: int,
+                               thin: bool) -> dict:
+    """Create the various capacity factors of the a particular backend.
+
+    Based off of definition of terms
+    cinder-specs/specs/queens/provisioning-improvements.html
+    Description of factors calculated where units of gb are Gibibytes.
+    reserved_capacity - The amount of space reserved from the total_capacity
+    as reported by the backend.
+    total_reserved_available_capacity - The total capacity minus reserved
+    capacity
+    total_available_capacity - The total capacity available to cinder
+    calculated from total_reserved_available_capacity (for thick) OR
+    for thin total_reserved_available_capacity max_over_subscription_ratio
+    calculated_free_capacity - total_available_capacity - provisioned_capacity
+    virtual_free_capacity - The calculated free capacity available to cinder
+    to allocate new storage.
+    For thin: calculated_free_capacity
+    For thick: the reported free_capacity can be less than the calculated
+    capacity, so we use free_capacity - reserved_capacity.
+
+    free_percent - the percentage of the virtual_free_capacity and
+    total_available_capacity is left over
+    provisioned_ratio - The ratio of provisioned storage to
+    total_available_capacity
+
+    :param total_capacity: The reported total capacity in the backend.
+    :type total_capacity: float
+    :param free_capacity: The free space/capacity as reported by the backend.
+    :type free_capacity: float
+    :param provisioned_capacity: as reported by backend or volume manager from
+        allocated_capacity_gb
+    :type provisioned_capacity: float
+    :param thin_provisioning_support: Is thin provisioning supported?
+    :type thin_provisioning_support: bool
+    :param max_over_subscription_ratio: as reported by the backend
+    :type max_over_subscription_ratio: float
+    :param reserved_percentage: the % amount to reserve as unavailable. 0-100
+    :type reserved_percentage: int, 0-100
+    :param thin: calculate based on thin provisioning if enabled by
+        thin_provisioning_support
+    :type thin: bool
+    :return: A dictionary of all of the capacity factors.
+    :rtype: dict
+
+    """
+
+    total = float(total_capacity)
+    reserved = float(reserved_percentage) / 100
+    reserved_capacity = math.floor(total * reserved)
+    total_reserved_available = total - reserved_capacity
+
+    if thin and thin_provisioning_support:
+        total_available_capacity = (
+            total_reserved_available * max_over_subscription_ratio
+        )
+        calculated_free = total_available_capacity - provisioned_capacity
+        virtual_free = calculated_free
+        provisioned_type = 'thin'
+    else:
+        # Calculate how much free space is left after taking into
+        # account the reserved space.
+        total_available_capacity = total_reserved_available
+        calculated_free = total_available_capacity - provisioned_capacity
+        virtual_free = calculated_free
+        if free_capacity < calculated_free:
+            virtual_free = free_capacity
+
+        provisioned_type = 'thick'
+
+    if total_available_capacity:
+        provisioned_ratio = provisioned_capacity / total_available_capacity
+        free_percent = (virtual_free / total_available_capacity) * 100
+    else:
+        provisioned_ratio = 0
+        free_percent = 0
+
+    def _limit(x):
+        """Limit our floating points to 2 decimal places."""
+        return round(x, 2)
+
+    return {
+        "total_capacity": total,
+        "free_capacity": free_capacity,
+        "reserved_capacity": reserved_capacity,
+        "total_reserved_available_capacity": _limit(total_reserved_available),
+        "max_over_subscription_ratio": (
+            max_over_subscription_ratio if provisioned_type == 'thin' else None
+        ),
+        "total_available_capacity": _limit(total_available_capacity),
+        "provisioned_capacity": provisioned_capacity,
+        "calculated_free_capacity": _limit(calculated_free),
+        "virtual_free_capacity": _limit(virtual_free),
+        "free_percent": _limit(free_percent),
+        "provisioned_ratio": _limit(provisioned_ratio),
+        "provisioned_type": provisioned_type
+    }
+
+
+def calculate_virtual_free_capacity(total_capacity: float,
+                                    free_capacity: float,
+                                    provisioned_capacity: float,
+                                    thin_provisioning_support: bool,
+                                    max_over_subscription_ratio: float,
+                                    reserved_percentage: int,
+                                    thin: bool) -> float:
+    """Calculate the virtual free capacity based on multiple factors.
 
     :param total_capacity:  total_capacity_gb of a host_state or pool.
     :param free_capacity:   free_capacity_gb of a host_state or pool.
@@ -919,22 +842,21 @@ def calculate_virtual_free_capacity(total_capacity,
     :returns: the calculated virtual free capacity.
     """
 
-    total = float(total_capacity)
-    reserved = float(reserved_percentage) / 100
-
-    if thin and thin_provisioning_support:
-        free = (total * max_over_subscription_ratio
-                - provisioned_capacity
-                - math.floor(total * reserved))
-    else:
-        # Calculate how much free space is left after taking into
-        # account the reserved space.
-        free = free_capacity - math.floor(total * reserved)
-    return free
+    factors = calculate_capacity_factors(
+        total_capacity,
+        free_capacity,
+        provisioned_capacity,
+        thin_provisioning_support,
+        max_over_subscription_ratio,
+        reserved_percentage,
+        thin
+    )
+    return factors["virtual_free_capacity"]
 
 
-def calculate_max_over_subscription_ratio(capability,
-                                          global_max_over_subscription_ratio):
+def calculate_max_over_subscription_ratio(
+        capability: dict,
+        global_max_over_subscription_ratio: float) -> float:
     # provisioned_capacity_gb is the apparent total capacity of
     # all the volumes created on a backend, which is greater than
     # or equal to allocated_capacity_gb, which is the apparent
@@ -992,7 +914,7 @@ def calculate_max_over_subscription_ratio(capability,
     return max_over_subscription_ratio
 
 
-def validate_dictionary_string_length(specs):
+def validate_dictionary_string_length(specs: dict) -> None:
     """Check the length of each key and value of dictionary."""
     if not isinstance(specs, dict):
         msg = _('specs must be a dictionary.')
@@ -1008,7 +930,8 @@ def validate_dictionary_string_length(specs):
                                 min_length=0, max_length=255)
 
 
-def service_expired_time(with_timezone=False):
+def service_expired_time(
+        with_timezone: Optional[bool] = False) -> datetime.datetime:
     return (timeutils.utcnow(with_timezone=with_timezone) -
             datetime.timedelta(seconds=CONF.service_down_time))
 
@@ -1034,7 +957,7 @@ def notifications_enabled(conf):
     return notifications_driver and notifications_driver != {'noop'}
 
 
-def if_notifications_enabled(f):
+def if_notifications_enabled(f: Callable) -> Callable:
     """Calls decorated method only if notifications are enabled."""
     @functools.wraps(f)
     def wrapped(*args, **kwargs):
@@ -1047,7 +970,7 @@ def if_notifications_enabled(f):
 LOG_LEVELS = ('INFO', 'WARNING', 'ERROR', 'DEBUG')
 
 
-def get_log_method(level_string):
+def get_log_method(level_string: str) -> int:
     level_string = level_string or ''
     upper_level_string = level_string.upper()
     if upper_level_string not in LOG_LEVELS:
@@ -1056,7 +979,7 @@ def get_log_method(level_string):
     return getattr(logging, upper_level_string)
 
 
-def set_log_levels(prefix, level_string):
+def set_log_levels(prefix: str, level_string: str) -> None:
     level = get_log_method(level_string)
     prefix = prefix or ''
 
@@ -1065,18 +988,18 @@ def set_log_levels(prefix, level_string):
             v.logger.setLevel(level)
 
 
-def get_log_levels(prefix):
+def get_log_levels(prefix: str) -> dict:
     prefix = prefix or ''
-    return {k: logging.logging.getLevelName(v.logger.getEffectiveLevel())
+    return {k: py_logging.getLevelName(v.logger.getEffectiveLevel())
             for k, v in logging.get_loggers().items()
             if k and k.startswith(prefix)}
 
 
-def paths_normcase_equal(path_a, path_b):
+def paths_normcase_equal(path_a: str, path_b: str) -> bool:
     return os.path.normcase(path_a) == os.path.normcase(path_b)
 
 
-def create_ordereddict(adict):
+def create_ordereddict(adict: dict) -> OrderedDict:
     """Given a dict, return a sorted OrderedDict."""
     return OrderedDict(sorted(adict.items(),
                               key=operator.itemgetter(0)))
@@ -1099,7 +1022,9 @@ class Semaphore(object):
         return self.semaphore.__exit__(*args)
 
 
-def semaphore_factory(limit, concurrent_processes):
+def semaphore_factory(limit: int,
+                      concurrent_processes: int) -> Union[eventlet.Semaphore,
+                                                          Semaphore]:
     """Get a semaphore to limit concurrent operations.
 
     The semaphore depends on the limit we want to set and the concurrent
@@ -1116,7 +1041,7 @@ def semaphore_factory(limit, concurrent_processes):
     return contextlib.suppress()
 
 
-def limit_operations(func):
+def limit_operations(func: Callable) -> Callable:
     """Decorator to limit the number of concurrent operations.
 
      This method decorator expects to have a _semaphore attribute holding an
