@@ -1,4 +1,4 @@
-# Copyright (c) 2017 DataCore Software Corp. All Rights Reserved.
+# Copyright (c) 2022 DataCore Software Corp. All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -18,16 +18,17 @@ from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
 
+from cinder.common import constants
 from cinder import exception as cinder_exception
 from cinder.i18n import _
 from cinder import interface
 from cinder import utils as cinder_utils
+from cinder.volume import configuration
 from cinder.volume.drivers.datacore import driver
 from cinder.volume.drivers.datacore import exception as datacore_exception
 from cinder.volume.drivers.datacore import passwd
 from cinder.volume.drivers.datacore import utils as datacore_utils
-from cinder.volume import utils as volume_utils
-
+from cinder.volume import volume_utils as volume_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -41,16 +42,14 @@ datacore_iscsi_opts = [
                      'for each target as the value, such as '
                      '<iqn:target name>, <iqn:target name>, '
                      '<iqn:target name>.'),
-    cfg.BoolOpt('datacore_iscsi_chap_enabled',
-                default=False,
-                help='Configure CHAP authentication for iSCSI connections.'),
     cfg.StrOpt('datacore_iscsi_chap_storage',
-               default=None,
-               help='iSCSI CHAP authentication password storage file.'),
+               default='$state_path/.datacore_chap',
+               help='Fully qualified file name where dynamically generated '
+                    'iSCSI CHAP secrets are stored.'),
 ]
 
 CONF = cfg.CONF
-CONF.register_opts(datacore_iscsi_opts)
+CONF.register_opts(datacore_iscsi_opts, group=configuration.SHARED_CONF_GROUP)
 
 
 @interface.volumedriver
@@ -62,21 +61,27 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
     .. code-block:: none
 
         1.0.0 - Initial driver
+        2.0.0 - Reintroduce the driver
 
     """
 
-    VERSION = '1.0.0'
-    STORAGE_PROTOCOL = 'iSCSI'
+    VERSION = '2.0.0'
+    STORAGE_PROTOCOL = constants.ISCSI
     CI_WIKI_NAME = 'DataCore_CI'
 
-    # TODO(jsbryant) Remove driver in Stein if CI is not fixed
-    SUPPORTED = False
-
     def __init__(self, *args, **kwargs):
         super(ISCSIVolumeDriver, self).__init__(*args, **kwargs)
-        self.configuration.append_config_values(datacore_iscsi_opts)
+        self.configuration = kwargs.get('configuration', None)
+        if self.configuration:
+            self.configuration.append_config_values(datacore_iscsi_opts)
         self._password_storage = None
 
+    @classmethod
+    def get_driver_options(cls):
+        additional_opts = cls._get_oslo_driver_opts(
+            'san_ip', 'san_login', 'san_password')
+        return driver.datacore_opts + datacore_iscsi_opts + additional_opts
+
     def do_setup(self, context):
         """Perform validations and establish connection to server.
 
@@ -85,15 +90,8 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
 
         super(ISCSIVolumeDriver, self).do_setup(context)
 
-        password_storage_path = getattr(self.configuration,
-                                        'datacore_iscsi_chap_storage', None)
-        if (self.configuration.datacore_iscsi_chap_enabled
-                and not password_storage_path):
-            raise cinder_exception.InvalidInput(
-                _("datacore_iscsi_chap_storage not set."))
-        elif password_storage_path:
-            self._password_storage = passwd.PasswordFileStorage(
-                self.configuration.datacore_iscsi_chap_storage)
+        self._password_storage = passwd.PasswordFileStorage(
+            self.configuration.datacore_iscsi_chap_storage)
 
     def validate_connector(self, connector):
         """Fail if connector doesn't contain all the data needed by the driver.
@@ -119,7 +117,7 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
 
         LOG.debug("Initialize connection for volume %(volume)s for "
                   "connector %(connector)s.",
-                  {'volume': volume['id'], 'connector': connector})
+                  {'volume': volume.id, 'connector': connector})
 
         virtual_disk = self._get_virtual_disk_for(volume, raise_not_found=True)
 
@@ -139,13 +137,12 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
             iscsi_initiator = self._get_initiator(connector['host'],
                                                   connector['initiator'],
                                                   available_ports)
-
             iscsi_targets = self._get_targets(virtual_disk, available_ports)
 
             if not iscsi_targets:
                 msg = (_("Suitable targets not found for "
                          "virtual disk %(disk)s for volume %(volume)s.")
-                       % {'disk': virtual_disk.Id, 'volume': volume['id']})
+                       % {'disk': virtual_disk.Id, 'volume': volume.id})
                 LOG.error(msg)
                 raise cinder_exception.VolumeDriverException(message=msg)
 
@@ -160,10 +157,10 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
         targets, logical_units, chap_params = serve_virtual_disk()
 
         target_portal = datacore_utils.build_network_address(
-            targets[0].PortConfigInfo.PortalsConfig.iScsiPortalConfigInfo[0]
-            .Address.Address,
-            targets[0].PortConfigInfo.PortalsConfig.iScsiPortalConfigInfo[0]
-            .TcpPort)
+            targets[0].PortConfigInfo.PortalsConfig.iScsiPortalConfigInfo[
+                0].Address.Address,
+            targets[0].PortConfigInfo.PortalsConfig.iScsiPortalConfigInfo[
+                0].TcpPort)
 
         connection_data = {}
 
@@ -176,7 +173,7 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
         connection_data['target_iqn'] = targets[0].PortName
         connection_data['target_portal'] = target_portal
         connection_data['target_lun'] = logical_units[targets[0]].Lun.Quad
-        connection_data['volume_id'] = volume['id']
+        connection_data['volume_id'] = volume.id
         connection_data['access_mode'] = 'rw'
 
         LOG.debug("Connection data: %s", connection_data)
@@ -243,32 +240,32 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
 
         return logical_units
 
+    def terminate_connection(self, volume, connector, **kwargs):
+        super().unserve_virtual_disks_from_host(volume, connector)
+
     def _get_target_domain(self, target, initiator):
         target_domains = self._api.get_target_domains()
         target_domain = datacore_utils.get_first_or_default(
-            lambda domain: (domain.InitiatorHostId == initiator.HostId
-                            and domain.TargetHostId == target.HostId),
-            target_domains,
-            None)
+            lambda domain: (domain.InitiatorHostId == initiator.HostId and
+                            domain.TargetHostId == target.HostId),
+            target_domains, None)
         return target_domain
 
     def _get_target_device(self, target_domain, target, initiator):
         target_devices = self._api.get_target_devices()
         target_device = datacore_utils.get_first_or_default(
-            lambda device: (device.TargetDomainId == target_domain.Id
-                            and device.InitiatorPortId == initiator.Id
-                            and device.TargetPortId == target.Id),
-            target_devices,
-            None)
+            lambda device: (device.TargetDomainId == target_domain.Id and
+                            device.InitiatorPortId == initiator.Id and
+                            device.TargetPortId == target.Id),
+            target_devices, None)
         return target_device
 
     def _get_logical_unit(self, logical_disk, target_device):
         logical_units = self._api.get_logical_units()
         logical_unit = datacore_utils.get_first_or_default(
-            lambda unit: (unit.LogicalDiskId == logical_disk.Id
-                          and unit.VirtualTargetDeviceId == target_device.Id),
-            logical_units,
-            None)
+            lambda unit: (unit.LogicalDiskId == logical_disk.Id and
+                          unit.VirtualTargetDeviceId == target_device.Id),
+            logical_units, None)
         return logical_unit
 
     def _create_logical_unit(self, logical_disk, nexus, target_device):
@@ -280,7 +277,7 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
                                                   'Client')
         return logical_unit
 
-    def _check_iscsi_chap_configuration(self, iscsi_chap_enabled, targets):
+    def _check_iscsi_chap_configuration(self, chap, targets):
         logical_units = self._api.get_logical_units()
         target_devices = self._api.get_target_devices()
 
@@ -294,8 +291,10 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
                 lambda target_port, key=target_port_id: target_port.Id == key,
                 targets,
                 None)
-            if (target and iscsi_chap_enabled ==
-                    (target.ServerPortProperties.Authentication == 'None')):
+            if (target and
+                    chap == (target.ServerPortProperties.Authentication ==
+                             'None') and chap ==
+                    (target.ServerPortProperties.Authentication == 'Default')):
                 msg = _("iSCSI CHAP authentication can't be configured for "
                         "target %s. Device exists that served through "
                         "this target.") % target.PortName
@@ -303,7 +302,7 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
                 raise cinder_exception.VolumeDriverException(message=msg)
 
     def _setup_iscsi_chap_authentication(self, targets, initiator):
-        iscsi_chap_enabled = self.configuration.datacore_iscsi_chap_enabled
+        iscsi_chap_enabled = self.configuration.use_chap_auth
 
         self._check_iscsi_chap_configuration(iscsi_chap_enabled, targets)
 
@@ -311,11 +310,15 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
         update_access_token = False
         access_token = None
         chap_secret = None
+        chap_username = initiator.PortName
         if iscsi_chap_enabled:
             authentication = 'CHAP'
-            chap_secret = self._password_storage.get_password(
-                server_group.Id, initiator.PortName)
-            update_access_token = False
+            chap_username = self.configuration.chap_username
+            if not chap_username:
+                chap_username = initiator.PortName
+            chap_secret = (self.configuration.chap_password or
+                           self._password_storage.get_password(
+                               server_group.Id, initiator.PortName))
             if not chap_secret:
                 chap_secret = volume_utils.generate_password(length=15)
                 self._password_storage.set_password(
@@ -326,13 +329,17 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
                 None,
                 None,
                 False,
-                initiator.PortName,
+                chap_username,
                 chap_secret)
         else:
             authentication = 'None'
             if self._password_storage:
-                self._password_storage.delete_password(server_group.Id,
-                                                       initiator.PortName)
+                try:
+                    self._password_storage.delete_password(server_group.Id,
+                                                           initiator.PortName)
+                except Exception:
+                    pass
+
         changed_targets = {}
         try:
             for target in targets:
@@ -342,9 +349,9 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
                         lambda node: node.Name == initiator.PortName,
                         target_iscsi_nodes,
                         None)
-                    if (not iscsi_node
-                            or not iscsi_node.AccessToken.TargetUsername
-                            or update_access_token):
+                    if ((not iscsi_node) or not
+                            (iscsi_node.AccessToken.TargetUsername) or
+                            (update_access_token)):
                         self._api.set_access_token(target.Id, access_token)
                 properties = target.ServerPortProperties
                 if properties.Authentication != authentication:
@@ -369,7 +376,7 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
                                 "on initiator %(initiator)s: %(error)s.",
                                 {'initiator': initiator.PortName, 'error': e})
         if iscsi_chap_enabled:
-            return initiator.PortName, chap_secret
+            return chap_username, chap_secret
 
     def _get_initiator(self, host, iqn, available_ports):
         client = self._get_client(host, create_new=True)
@@ -381,7 +388,6 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
             lambda port: port.PortName == iqn,
             iscsi_initiator_ports,
             None)
-
         if not iscsi_initiator:
             scsi_port_data = self._api.build_scsi_port_data(
                 client.Id, iqn, 'Initiator', 'iSCSI')
@@ -411,18 +417,16 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
     def _get_logical_disk_on_host(virtual_disk_id,
                                   host_id, logical_disks):
         logical_disk = datacore_utils.get_first(
-            lambda disk: (disk.ServerHostId == host_id
-                          and disk.VirtualDiskId == virtual_disk_id),
+            lambda disk: (disk.ServerHostId == host_id and
+                          disk.VirtualDiskId == virtual_disk_id),
             logical_disks)
         return logical_disk
 
     @staticmethod
     def _is_iscsi_frontend_port(port):
-        if (port.PortType == 'iSCSI'
-                and port.PortMode == 'Target'
-                and port.HostId
-                and port.PresenceStatus == 'Present'
-                and hasattr(port, 'IScsiPortStateInfo')):
+        if (port.PortType == 'iSCSI' and port.PortMode == 'Target' and
+                port.HostId and port.PresenceStatus == 'Present' and
+                hasattr(port, 'IScsiPortStateInfo')):
             port_roles = port.ServerPortProperties.Role.split()
             port_state = (port.IScsiPortStateInfo.PortalsState
                           .PortalStateInfo[0].State)
@@ -437,7 +441,6 @@ class ISCSIVolumeDriver(driver.DataCoreVolumeDriver):
 
     @staticmethod
     def _get_host_iscsi_initiator_ports(host, ports):
-        return [port for port in ports
-                if port.PortType == 'iSCSI'
-                and port.PortMode == 'Initiator'
-                and port.HostId == host.Id]
+        return [port for port in ports if
+                port.PortType == 'iSCSI' and port.PortMode == 'Initiator' and
+                port.HostId == host.Id]
