@@ -29,6 +29,7 @@ from oslo_utils import units
 import six
 from six.moves import http_client
 
+from cinder.common import constants
 from cinder import context
 from cinder import exception
 from cinder.i18n import _
@@ -94,9 +95,10 @@ class PowerFlexDriver(driver.VolumeDriver):
           3.5.5 - Rebrand VxFlex OS to PowerFlex.
           3.5.6 - Fix for Bug #1897598 when volume can be migrated without
                   conversion of its type.
+          3.5.7 - Report trim/discard support.
     """
 
-    VERSION = "3.5.6"
+    VERSION = "3.5.7"
     # ThirdPartySystems wiki
     CI_WIKI_NAME = "DellEMC_PowerFlex_CI"
 
@@ -839,7 +841,14 @@ class PowerFlexDriver(driver.VolumeDriver):
         self._get_client().remove_volume(snapshot.provider_id)
 
     def initialize_connection(self, volume, connector, **kwargs):
-        return self._initialize_connection(volume, connector, volume.size)
+        res = self._initialize_connection(volume, connector, volume.size)
+
+        # TODO: Should probably be enabled for SSDs as well
+        # It is recommended not to trim volumes that contain snapshots as the
+        # logical capacity may not shrink.
+        if self.provisioning_type == 'thin' and not len(volume.snapshots):
+            res['data']['discard'] = True
+        return res
 
     def _initialize_connection(self, vol_or_snap, connector, vol_size):
         """Initialize connection and return connection info.
@@ -956,7 +965,7 @@ class PowerFlexDriver(driver.VolumeDriver):
         stats["volume_backend_name"] = backend_name or "powerflex"
         stats["vendor_name"] = "Dell EMC"
         stats["driver_version"] = self.VERSION
-        stats["storage_protocol"] = "scaleio"
+        stats["storage_protocol"] = constants.SCALEIO
         stats["reserved_percentage"] = 0
         stats["QoS_support"] = True
         stats["consistent_group_snapshot_enabled"] = True
@@ -1026,6 +1035,8 @@ class PowerFlexDriver(driver.VolumeDriver):
                      "prov": backend_provisioned_capacity,
                  })
         stats["pools"] = pools
+        # TODO: Should probably be enabled for SSDs as well
+        stats['sparse_copy_volume'] = self.provisioning_type == 'thin'
         self._stats = stats
 
     def _query_pool_stats(self, domain_name, pool_name):
@@ -1052,11 +1063,12 @@ class PowerFlexDriver(driver.VolumeDriver):
         total_capacity_gb, free_capacity_gb, provisioned_capacity = (
             self._compute_pool_stats(raw_pool_stats)
         )
-        LOG.info("Free capacity of Storage Pool %(pool)s: %(free)s, "
-                 "total capacity: %(total)s, "
+        LOG.info("Free capacity of Storage Pool %(domain)s:%(pool)s: "
+                 "%(free)s, total capacity: %(total)s, "
                  "provisioned capacity: %(prov)s.",
                  {
-                     "pool": "%s:%s" % (domain_name, pool_name),
+                     "domain": domain_name,
+                     "pool": pool_name,
                      "free": free_capacity_gb,
                      "total": total_capacity_gb,
                      "prov": provisioned_capacity,
@@ -1195,6 +1207,12 @@ class PowerFlexDriver(driver.VolumeDriver):
         connection_properties["scaleIO_volume_id"] = volume.provider_id
         connection_properties["config_group"] = self.configuration.config_group
         connection_properties["failed_over"] = self._is_failed_over
+        connection_properties["verify_certificate"] = (
+            self._get_client().verify_certificate
+        )
+        connection_properties["certificate_path"] = (
+            self._get_client().certificate_path
+        )
         device_info = self.connector.connect_volume(connection_properties)
         return device_info["path"]
 
@@ -1209,6 +1227,12 @@ class PowerFlexDriver(driver.VolumeDriver):
         connection_properties["scaleIO_volume_id"] = volume.provider_id
         connection_properties["config_group"] = self.configuration.config_group
         connection_properties["failed_over"] = self._is_failed_over
+        connection_properties["verify_certificate"] = (
+            self._get_client().verify_certificate
+        )
+        connection_properties["certificate_path"] = (
+            self._get_client().certificate_path
+        )
 
         self.connector.disconnect_volume(connection_properties, volume)
 
