@@ -17,6 +17,7 @@ import ast
 
 from oslo_log import log as logging
 
+from cinder.common import constants
 from cinder import exception
 from cinder import interface
 from cinder.volume import driver
@@ -129,15 +130,14 @@ class PowerMaxFCDriver(san.SanDriver, driver.FibreChannelDriver):
               - Fix to enable legacy volumes to live migrate (#1867163)
               - Use of snap id instead of generation (bp powermax-snapset-ids)
               - Support for Failover Abilities (bp/powermax-failover-abilities)
-        4.3.1 - Fix non-temporary snapshot delete (#1887962)
-        4.3.2 - Fix for create snapshot (#1939139)
-        4.3.3 - Allow for case mismatch in SGs (bug #1929429)
+        4.4.0 - Early check for status of port
+        4.4.1 - Report trim/discard support
     """
 
-    VERSION = "4.3.3"
+    VERSION = "4.4.1"
 
     # ThirdPartySystems wiki
-    CI_WIKI_NAME = "EMC_VMAX_CI"
+    CI_WIKI_NAME = "DellEMC_PowerMAX_CI"
 
     def __init__(self, *args, **kwargs):
 
@@ -312,6 +312,7 @@ class PowerMaxFCDriver(san.SanDriver, driver.FibreChannelDriver):
                 'data': {'target_lun': device_number,
                          'target_discovered': True,
                          'target_wwn': target_wwns,
+                         'discard': True,
                          'initiator_target_map': init_targ_map}}
 
         LOG.debug("Return FC data for zone addition: %(data)s.",
@@ -498,7 +499,7 @@ class PowerMaxFCDriver(san.SanDriver, driver.FibreChannelDriver):
                 # Get target WWN
                 port_details = self.rest.get_port(array_id, port)
                 port_info = port_details.get('symmetrixPort')
-                port_wwn = port_info.get('wwn_node')
+                port_wwn = port_info.get('identifier')
                 LOG.info("Port %(p)s WWN: %(wwn)s",
                          {'p': port, 'wwn': port_wwn})
                 # Set lowest load port WWN as FC target for connection
@@ -540,7 +541,7 @@ class PowerMaxFCDriver(san.SanDriver, driver.FibreChannelDriver):
         """Retrieve stats info from volume group."""
         LOG.debug("Updating volume stats")
         data = self.common.update_volume_stats()
-        data['storage_protocol'] = 'FC'
+        data['storage_protocol'] = constants.FC
         data['driver_version'] = self.VERSION
         self._stats = data
 
