@@ -22,6 +22,7 @@ from oslo_log import log as logging
 from oslo_utils import strutils
 import six
 
+from cinder.common import constants
 from cinder import exception
 from cinder.i18n import _
 from cinder import interface
@@ -135,15 +136,14 @@ class PowerMaxISCSIDriver(san.SanISCSIDriver):
               - Fix to enable legacy volumes to live migrate (#1867163)
               - Use of snap id instead of generation (bp powermax-snapset-ids)
               - Support for Failover Abilities (bp/powermax-failover-abilities)
-        4.3.1 - Fix non-temporary snapshot delete (#1887962)
-        4.3.2 - Fix for create snapshot (#1939139)
-        4.3.3 - Allow for case mismatch in SGs (bug #1929429)
+        4.4.0 - Early check for status of port
+        4.4.1 - Report trim/discard support
     """
 
-    VERSION = "4.3.3"
+    VERSION = "4.4.1"
 
     # ThirdPartySystems wiki
-    CI_WIKI_NAME = "EMC_VMAX_CI"
+    CI_WIKI_NAME = "DellEMC_PowerMAX_CI"
 
     def __init__(self, *args, **kwargs):
 
@@ -423,6 +423,7 @@ class PowerMaxISCSIDriver(san.SanISCSIDriver):
         properties['target_discovered'] = True
         properties['target_lun'] = host_lun_id
         properties['volume_id'] = volume.id
+        properties['discard'] = True
 
         if self.configuration.safe_get('use_chap_auth'):
             LOG.info("Chap authentication enabled.")
@@ -454,7 +455,7 @@ class PowerMaxISCSIDriver(san.SanISCSIDriver):
         """Retrieve stats info from volume group."""
         LOG.debug("Updating volume stats")
         data = self.common.update_volume_stats()
-        data['storage_protocol'] = 'iSCSI'
+        data['storage_protocol'] = constants.ISCSI
         data['driver_version'] = self.VERSION
         self._stats = data
 
