@@ -15,20 +15,27 @@
 
 """Utilities for Dell EMC PowerStore Cinder driver."""
 
+from distutils import version
+import functools
 import re
 
 from oslo_log import log as logging
 from oslo_utils import units
 
+from cinder.common import constants
 from cinder import exception
 from cinder.i18n import _
 from cinder.objects import fields
+from cinder.volume.drivers.dell_emc.powerstore import driver
 from cinder.volume import volume_utils
 
 
 LOG = logging.getLogger(__name__)
 CHAP_DEFAULT_USERNAME = "PowerStore_iSCSI_CHAP_Username"
 CHAP_DEFAULT_SECRET_LENGTH = 60
+PROTOCOL_FC = constants.FC
+PROTOCOL_ISCSI = constants.ISCSI
+PROTOCOL_NVME = "NVMe"
 
 
 def bytes_to_gib(size_in_bytes):
@@ -89,7 +96,7 @@ def powerstore_host_name(connector, protocol):
     """Generate PowerStore host name for connector.
 
     :param connector: connection properties
-    :param protocol: storage protocol (FC or iSCSI)
+    :param protocol: storage protocol (FC, iSCSI or NVMe)
     :return: unique host name
     """
 
@@ -151,3 +158,32 @@ def get_chap_credentials():
             CHAP_DEFAULT_SECRET_LENGTH
         )
     }
+
+
+def get_protection_policy_from_volume(volume):
+    """Get PowerStore Protection policy name from volume type.
+
+    :param volume: OpenStack Volume object
+    :return: Protection policy name
+    """
+
+    return volume.volume_type.extra_specs.get(driver.POWERSTORE_PP_KEY)
+
+
+def is_group_a_cg_snapshot_type(func):
+    """Check if group is a consistent snapshot group.
+
+    Fallback to generic volume group implementation if consistent group
+    snapshot is not enabled.
+    """
+
+    @functools.wraps(func)
+    def inner(self, *args, **kwargs):
+        if not volume_utils.is_group_a_cg_snapshot_type(args[0]):
+            raise NotImplementedError
+        return func(self, *args, **kwargs)
+    return inner
+
+
+def version_gte(ver1, ver2):
+    return version.LooseVersion(ver1) >= version.LooseVersion(ver2)
