@@ -17,6 +17,7 @@
 from oslo_log import log as logging
 from oslo_utils import excutils
 
+from cinder.common import constants
 from cinder import exception
 from cinder.i18n import _
 from cinder import interface
@@ -71,13 +72,13 @@ class SCFCDriver(storagecenter_common.SCCommonDriver,
 
     VERSION = '4.1.2'
 
-    CI_WIKI_NAME = "Dell_EMC_SC_Series_CI"
+    CI_WIKI_NAME = "DellEMC_SC_CI"
 
     def __init__(self, *args, **kwargs):
         super(SCFCDriver, self).__init__(*args, **kwargs)
         self.backend_name =\
             self.configuration.safe_get('volume_backend_name') or 'Dell-FC'
-        self.storage_protocol = 'FC'
+        self.storage_protocol = constants.FC
 
     def validate_connector(self, connector):
         """Fail if connector doesn't contain all the data needed by driver.
