@@ -32,6 +32,7 @@ Supports XtremIO version 2.4 and up.
   1.0.10 - option to clean unused IGs
   1.0.11 - add support for multiattach
   1.0.12 - add support for ports filtering
+  1.0.13 - add support for iSCSI IPv6
 """
 
 import json
@@ -41,12 +42,14 @@ import string
 
 from oslo_config import cfg
 from oslo_log import log as logging
+from oslo_utils import netutils
 from oslo_utils import strutils
 from oslo_utils import units
 import requests
 import six
 from six.moves import http_client
 
+from cinder.common import constants
 from cinder import context
 from cinder import exception
 from cinder.i18n import _
@@ -427,10 +430,10 @@ class XtremIOClient42(XtremIOClient4):
 class XtremIOVolumeDriver(san.SanDriver):
     """Executes commands relating to Volumes."""
 
-    VERSION = '1.0.12'
+    VERSION = '1.0.13'
 
     # ThirdPartySystems wiki
-    CI_WIKI_NAME = "EMC_XIO_CI"
+    CI_WIKI_NAME = "DellEMC_XtremIO_CI"
 
     driver_name = 'XtremIO'
     MIN_XMS_VERSION = [3, 0, 0]
@@ -1099,7 +1102,7 @@ class XtremIOISCSIDriver(XtremIOVolumeDriver, driver.ISCSIDriver):
 
     def __init__(self, *args, **kwargs):
         super(XtremIOISCSIDriver, self).__init__(*args, **kwargs)
-        self.protocol = 'iSCSI'
+        self.protocol = constants.ISCSI
 
     def _add_auth(self, data, login_chap, discovery_chap):
         login_passwd, discovery_passwd = None, None
@@ -1207,11 +1210,12 @@ class XtremIOISCSIDriver(XtremIOVolumeDriver, driver.ISCSIDriver):
             raise exception.VolumeBackendAPIException(data=msg)
         portal = RANDOM.choice(allowed_portals)
         portal_addr = ('%(ip)s:%(port)d' %
-                       {'ip': portal['ip-addr'],
+                       {'ip': netutils.escape_ipv6(portal['ip-addr']),
                         'port': portal['ip-port']})
 
-        tg_portals = ['%(ip)s:%(port)d' % {'ip': p['ip-addr'],
-                                           'port': p['ip-port']}
+        tg_portals = ['%(ip)s:%(port)d' %
+                      {'ip': netutils.escape_ipv6(p['ip-addr']),
+                       'port': p['ip-port']}
                       for p in allowed_portals]
         properties = {'target_discovered': False,
                       'target_iqn': portal['port-address'],
@@ -1237,7 +1241,7 @@ class XtremIOFCDriver(XtremIOVolumeDriver,
 
     def __init__(self, *args, **kwargs):
         super(XtremIOFCDriver, self).__init__(*args, **kwargs)
-        self.protocol = 'FC'
+        self.protocol = constants.FC
         self._targets = None
 
     def get_targets(self):
