@@ -1,4 +1,4 @@
-# Copyright (C) 2020, Hitachi, Ltd.
+# Copyright (C) 2020, 2021, Hitachi, Ltd.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
 #    not use this file except in compliance with the License. You may obtain
@@ -16,37 +16,32 @@
 
 import enum
 import logging as base_logging
-import os
 
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import excutils
-from oslo_utils import importutils
 from oslo_utils import timeutils
 from oslo_utils import units
 
 from cinder import exception
-from cinder.i18n import _
-
-_DRIVER_DIR = 'cinder.volume.drivers.hitachi'
-
-_DRIVERS = {
-    'REST': {
-        'FC': 'hbsd_rest_fc.HBSDRESTFC',
-        'iSCSI': 'hbsd_rest_iscsi.HBSDRESTISCSI',
-    },
-}
 
+VERSION = '2.2.3'
+CI_WIKI_NAME = 'Hitachi_VSP_CI'
+PARAM_PREFIX = 'hitachi'
+VENDOR_NAME = 'Hitachi'
+DRIVER_DIR_NAME = 'hbsd'
 DRIVER_PREFIX = 'HBSD'
+DRIVER_FILE_PREFIX = 'hbsd'
 TARGET_PREFIX = 'HBSD-'
+HDP_VOL_ATTR = 'HDP'
+HDT_VOL_ATTR = 'HDT'
+NVOL_LDEV_TYPE = 'DP-VOL'
 TARGET_IQN_SUFFIX = '.hbsd-target'
+PAIR_ATTR = 'HTI'
+
 GIGABYTE_PER_BLOCK_SIZE = units.Gi / 512
 
-MAX_PROCESS_WAITTIME = 24 * 60 * 60
-DEFAULT_PROCESS_WAITTIME = 15 * 60
-
 NORMAL_LDEV_TYPE = 'Normal'
-NVOL_LDEV_TYPE = 'DP-VOL'
 
 INFO_SUFFIX = 'I'
 WARNING_SUFFIX = 'W'
@@ -54,13 +49,7 @@ ERROR_SUFFIX = 'E'
 
 PORT_ID_LENGTH = 5
 
-
-class HBSDError(exception.VolumeDriverException):
-    message = _("HBSD error occurred. %(message)s")
-
-
-class HBSDBusy(HBSDError):
-    message = _("Device or resource is busy.")
+BUSY_MESSAGE = "Device or resource is busy."
 
 
 @enum.unique
@@ -184,6 +173,32 @@ class HBSDMsg(enum.Enum):
                'reason: %(reason)s)',
         'suffix': WARNING_SUFFIX,
     }
+    INVALID_EXTRA_SPEC_KEY_PORT = {
+        'msg_id': 330,
+        'loglevel': base_logging.WARNING,
+        'msg': 'The port name specified for the extra spec key '
+               '"%(target_ports_param)s" '
+               'of the volume type is not specified for the '
+               'target_ports or compute_target_ports '
+               'parameter in cinder.conf. (port: %(port)s, volume type: '
+               '%(volume_type)s)',
+        'suffix': WARNING_SUFFIX,
+    }
+    HOST_GROUP_NUMBER_IS_MAXIMUM = {
+        'msg_id': 335,
+        'loglevel': base_logging.WARNING,
+        'msg': 'Failed to create the host group because the host group '
+               'maximum of the port is exceeded. (port: %(port)s)',
+        'suffix': WARNING_SUFFIX,
+    }
+    WWN_NUMBER_IS_MAXIMUM = {
+        'msg_id': 336,
+        'loglevel': base_logging.WARNING,
+        'msg': 'Failed to add the wwns to the host group port because the '
+               'WWN maximum of the port is exceeded. '
+               '(port: %(port)s, WWN: %(wwn)s)',
+        'suffix': WARNING_SUFFIX,
+    }
     INVALID_PORT = {
         'msg_id': 339,
         'loglevel': base_logging.WARNING,
@@ -191,6 +206,13 @@ class HBSDMsg(enum.Enum):
                'invalid. (%(additional_info)s)',
         'suffix': WARNING_SUFFIX,
     }
+    INVALID_PORT_BY_ZONE_MANAGER = {
+        'msg_id': 340,
+        'loglevel': base_logging.WARNING,
+        'msg': 'Port %(port)s will not be used because it is not considered '
+               'to be active by the Fibre Channel Zone Manager.',
+        'suffix': WARNING_SUFFIX,
+    }
     STORAGE_COMMAND_FAILED = {
         'msg_id': 600,
         'loglevel': base_logging.ERROR,
@@ -427,6 +449,36 @@ class HBSDMsg(enum.Enum):
                '%(group_type)s, volume: %(volume)s, snapshot: %(snapshot)s)',
         'suffix': ERROR_SUFFIX,
     }
+    NO_ACTIVE_WWN = {
+        'msg_id': 747,
+        'loglevel': base_logging.ERROR,
+        'msg': 'Failed to initialize volume connection because no active WWN '
+               'was found for the connector. (WWN: %(wwn)s, volume: %(volume)s'
+               ')',
+        'suffix': ERROR_SUFFIX,
+    }
+    NO_PORT_WITH_ACTIVE_WWN = {
+        'msg_id': 748,
+        'loglevel': base_logging.ERROR,
+        'msg': 'Failed to initialize volume connection because no port with '
+               'an active WWN was found. (%(port_wwns)s, volume: %(volume)s)',
+        'suffix': ERROR_SUFFIX,
+    }
+    ZONE_MANAGER_IS_NOT_AVAILABLE = {
+        'msg_id': 749,
+        'loglevel': base_logging.ERROR,
+        'msg': 'The Fibre Channel Zone Manager is not available. The Fibre '
+               'Channel Zone Manager must be up and running when '
+               'port_scheduler parameter is set to True.',
+        'suffix': ERROR_SUFFIX,
+    }
+    HOST_GROUP_OR_WWN_IS_NOT_AVAILABLE = {
+        'msg_id': 750,
+        'loglevel': base_logging.ERROR,
+        'msg': 'Failed to initialize volume connection because no available '
+               'resource of host group or wwn was found. (ports: %(ports)s)',
+        'suffix': ERROR_SUFFIX,
+    }
 
     def __init__(self, error_info):
         """Initialize Enum attributes."""
@@ -467,18 +519,6 @@ def timed_out(start_time, timeout):
     return timeutils.is_older_than(start_time, timeout)
 
 
-def import_object(conf, driver_info, db):
-    """Import a class and return an instance of it."""
-    os.environ['LANG'] = 'C'
-    cli = _DRIVERS.get('REST')
-    return importutils.import_object(
-        '%(dir)s.%(proto)s' % {
-            'dir': _DRIVER_DIR,
-            'proto': cli[driver_info['proto']],
-        },
-        conf, driver_info, db)
-
-
 def check_opt_value(conf, names):
     """Check if the parameter names and values are valid."""
     for name in names:
@@ -489,24 +529,6 @@ def check_opt_value(conf, names):
                 output_log(MSG.INVALID_PARAMETER, param=name)
 
 
-def check_opts(conf, opts):
-    """Check if the specified configuration is valid."""
-    names = []
-    for opt in opts:
-        if opt.required and not conf.safe_get(opt.name):
-            msg = output_log(MSG.INVALID_PARAMETER, param=opt.name)
-            raise HBSDError(msg)
-        names.append(opt.name)
-    check_opt_value(conf, names)
-
-
-def require_target_existed(targets):
-    """Check if the target list includes one or more members."""
-    if not targets['list']:
-        msg = output_log(MSG.NO_CONNECTED_TARGET)
-        raise HBSDError(msg)
-
-
 def build_initiator_target_map(connector, target_wwns, lookup_service):
     """Return a dictionary mapping server-wwns and lists of storage-wwns."""
     init_targ_map = {}
@@ -544,6 +566,12 @@ def safe_get_message_id(errobj):
     return errobj.get('messageId', '')
 
 
+def safe_get_message(errobj):
+    if not errobj:
+        return ''
+    return errobj.get('message', '')
+
+
 def is_shared_connection(volume, connector):
     """Check if volume is multiattach to 1 node."""
     connection_count = 0
