@@ -15,13 +15,13 @@
 #
 import collections
 import copy
-import distutils.version as dist_version  # pylint: disable=E0611
 import math
 import os
 import string
 
 import eventlet
 from oslo_log import log as logging
+import packaging.version as dist_version  # pylint: disable=E0611
 import six
 
 from cinder import coordination
@@ -226,8 +226,8 @@ class DS8KCommonHelper(object):
                          % self.INVALID_STORAGE_VERSION))
         rest_ver = self.backend['rest_version'][0:2]
         if ('87' == rest_ver and
-           dist_version.LooseVersion(self.backend['rest_version']) <
-           dist_version.LooseVersion(self.VALID_REST_VERSION_87_51_MIN)):
+           dist_version.parse(self.backend['rest_version']) <
+           dist_version.parse(self.VALID_REST_VERSION_87_51_MIN)):
             raise exception.VolumeDriverException(
                 message=(_("REST version %(invalid)s is lower than "
                            "%(valid)s, please upgrade it in DS8K.")
@@ -242,12 +242,12 @@ class DS8KCommonHelper(object):
         valid_rest_version = None
         rest_ver = self.backend['rest_version'][0:2]
         if ('87' == rest_ver and
-           dist_version.LooseVersion(self.backend['rest_version']) <
-           dist_version.LooseVersion(self.REST_VERSION_87_51_MIN_PPRC_CG)):
+           dist_version.parse(self.backend['rest_version']) <
+           dist_version.parse(self.REST_VERSION_87_51_MIN_PPRC_CG)):
             valid_rest_version = self.REST_VERSION_87_51_MIN_PPRC_CG
         elif ('88' == rest_ver and
-              dist_version.LooseVersion(self.backend['rest_version']) <
-              dist_version.LooseVersion(self.REST_VERSION_88_20_MIN_PPRC_CG)):
+              dist_version.parse(self.backend['rest_version']) <
+              dist_version.parse(self.REST_VERSION_88_20_MIN_PPRC_CG)):
             valid_rest_version = self.REST_VERSION_88_20_MIN_PPRC_CG
 
         if valid_rest_version:
@@ -562,8 +562,9 @@ class DS8KCommonHelper(object):
             invalid_states += ('suspended',)
         elif state == 'suspended':
             invalid_states += ('valid',)
+
+        finished = False
         try:
-            finished = False
             while True:
                 eventlet.sleep(2)
                 pairs = self.get_pprc_pairs(min_vol_id, max_vol_id)
@@ -587,7 +588,7 @@ class DS8KCommonHelper(object):
 
     def _get_host(self, connector):
         # DS8K doesn't support hostname which is longer than 32 chars.
-        hname = ('OShost:%s' % filter_alnum(connector['host']))[:32]
+        hname = filter_alnum(connector['host'])[:32]
         os_type = connector.get('os_type')
         platform = connector.get('platform')
 
@@ -597,7 +598,7 @@ class DS8KCommonHelper(object):
             htype = 'iSeries'
         elif os_type == 'AIX':
             htype = 'pSeries'
-        elif platform in ('s390', 's390x') and os_type == 'linux2':
+        elif platform in ('s390', 's390x') and os_type in ('linux', 'linux2'):
             htype = 'zLinux'
         else:
             htype = 'LinuxRHEL'
@@ -678,7 +679,8 @@ class DS8KCommonHelper(object):
         }
 
     @coordination.synchronized('ibm-ds8k-{connector[host]}')
-    def terminate_connection(self, vol_id, host_id, connector, map_info):
+    def terminate_connection(self, volume, vol_id, host_id, connector,
+                             map_info):
         host = self._get_host(connector)
         host_ports = map_info['host_ports']
         lun_ids = map_info['lun_ids']
@@ -690,6 +692,27 @@ class DS8KCommonHelper(object):
                   {"host": host_ports,
                    "defined": host_id,
                    "delete": delete_ports})
+        host_name = host.name
+        if (host.name[:7] == "OShost:"):
+            host_name = host.name[7:]
+        attachment_count = 0
+        if hasattr(volume, 'multiattach') and volume.multiattach:
+            try:
+                attachment_list = volume.volume_attachment
+                for attachment in attachment_list:
+                    if (attachment.attach_status == "attached" and
+                       attachment.attached_host == host_name):
+                        attachment_count += 1
+            except AttributeError:
+                pass
+        if attachment_count > 1:
+            LOG.info("Volume %(volume)s is attached to multiple "
+                     "instances on host %(host)s, hence "
+                     "skipping delete host.",
+                     {'volume': volume.name,
+                      'host': host.name})
+            return
+
         for lun_id in lun_ids:
             self._delete_mappings(host_id, lun_id)
         if not lun_ids:
@@ -1092,17 +1115,17 @@ class DS8KECKDHelper(DS8KCommonHelper):
                            "please upgrade the CCL.")
                          % self.INVALID_STORAGE_VERSION))
         # DS8K supports ECKD ESE volume from 8.1
-        if (dist_version.LooseVersion(self.backend['storage_version']) <
-           dist_version.LooseVersion(self.MIN_VALID_STORAGE_VERSION)):
+        if (dist_version.parse(self.backend['storage_version']) <
+           dist_version.parse(self.MIN_VALID_STORAGE_VERSION)):
             self._disable_thin_provision = True
 
         rest_ver = self.backend['rest_version'][0:2]
         if (('87' == rest_ver and
-           dist_version.LooseVersion(self.backend['rest_version']) <
-           dist_version.LooseVersion(self.VALID_REST_VERSION_87_51_MIN)) or
+           dist_version.parse(self.backend['rest_version']) <
+           dist_version.parse(self.VALID_REST_VERSION_87_51_MIN)) or
            ('88' == rest_ver and
-           dist_version.LooseVersion(self.backend['rest_version']) <
-           dist_version.LooseVersion(self.VALID_REST_VERSION_88_20_MIN))):
+           dist_version.parse(self.backend['rest_version']) <
+           dist_version.parse(self.VALID_REST_VERSION_88_20_MIN))):
             raise exception.VolumeDriverException(
                 message=(_("REST version %(invalid)s is lower than "
                            "%(valid)s, please upgrade it in DS8K.")
@@ -1263,7 +1286,8 @@ class DS8KECKDHelper(DS8KCommonHelper):
             }
         }
 
-    def terminate_connection(self, vol_id, connector, force, **kwargs):
+    def terminate_connection(self, volume, vol_id, connector, force,
+                             **kwargs):
         return None
 
 
