@@ -27,6 +27,7 @@ from cinder.i18n import _
 from cinder import ssh_utils
 from cinder import utils
 from cinder.volume.drivers.ibm.storwize_svc import storwize_const
+from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -36,11 +37,40 @@ class StorwizeSVCReplication(object):
         self.driver = driver
         self.target = replication_target or {}
 
+    @volume_utils.trace
     def failover_volume_host(self, context, vref):
-        pass
+        # Make the aux volume writeable.
+        try:
+            tgt_volume = storwize_const.REPLICA_AUX_VOL_PREFIX + vref.name
+            self.target_helpers.stop_relationship(tgt_volume, access=True)
+            try:
+                self.target_helpers.start_relationship(tgt_volume, 'aux')
+            except exception.VolumeBackendAPIException as e:
+                LOG.error('Error running startrcrelationship due to %(err)s.',
+                          {'err': e})
+            return
+        except Exception as e:
+            msg = (_('Unable to fail-over the volume %(id)s to the '
+                   'secondary back-end, error: %(error)s') %
+                   {"id": vref['id'], "error": six.text_type(e)})
+            LOG.exception(msg)
+            raise exception.VolumeDriverException(message=msg)
 
+    @volume_utils.trace
     def replication_failback(self, volume):
-        pass
+        tgt_volume = storwize_const.REPLICA_AUX_VOL_PREFIX + volume['name']
+        rel_info = self.target_helpers.get_relationship_info(tgt_volume)
+        if rel_info:
+            try:
+                self.target_helpers.stop_relationship(tgt_volume, access=True)
+                self.target_helpers.start_relationship(tgt_volume, 'master')
+                return
+            except Exception as e:
+                msg = (_('Unable to fail-back the volume: %(vol)s to the '
+                         'master back-end, error: %(error)s') %
+                       {"vol": volume['name'], "error": six.text_type(e)})
+                LOG.exception(msg)
+                raise exception.VolumeDriverException(message=msg)
 
     def volume_replication_setup(self, context, vref):
         pass
@@ -81,7 +111,7 @@ class StorwizeSVCReplicationGlobalMirror(StorwizeSVCReplication):
 
             system_info = self.target_helpers.get_system_info()
             self.driver._helpers.create_relationship(
-                vref['name'], target_vol_name, system_info.get('system_name'),
+                vref['name'], target_vol_name, system_info.get('system_id'),
                 self.asyncmirror)
         except Exception as e:
             msg = (_("Unable to set up mirror mode replication for %(vol)s. "
@@ -91,55 +121,6 @@ class StorwizeSVCReplicationGlobalMirror(StorwizeSVCReplication):
             raise exception.VolumeDriverException(message=msg)
         LOG.debug('leave: volume_replication_setup:volume %s', vref['name'])
 
-    def failover_volume_host(self, context, vref):
-        LOG.debug('enter: failover_volume_host: vref=%(vref)s',
-                  {'vref': vref['name']})
-        target_vol = storwize_const.REPLICA_AUX_VOL_PREFIX + vref['name']
-
-        try:
-            rel_info = self.target_helpers.get_relationship_info(target_vol)
-            # Reverse the role of the primary and secondary volumes
-            self.target_helpers.switch_relationship(rel_info['name'])
-            return
-        except Exception as e:
-            LOG.exception('Unable to fail-over the volume %(id)s to the '
-                          'secondary back-end by switchrcrelationship '
-                          'command, error: %(error)s',
-                          {"id": vref['id'], "error": e})
-            # If the switch command fail, try to make the aux volume
-            # writeable again.
-            try:
-                self.target_helpers.stop_relationship(target_vol,
-                                                      access=True)
-                try:
-                    self.target_helpers.start_relationship(target_vol, 'aux')
-                except exception.VolumeBackendAPIException as e:
-                    LOG.error(
-                        'Error running startrcrelationship due to %(err)s.',
-                        {'err': e})
-                return
-            except Exception as e:
-                msg = (_('Unable to fail-over the volume %(id)s to the '
-                         'secondary back-end, error: %(error)s') %
-                       {"id": vref['id'], "error": e})
-                LOG.exception(msg)
-                raise exception.VolumeDriverException(message=msg)
-
-    def replication_failback(self, volume):
-        tgt_volume = storwize_const.REPLICA_AUX_VOL_PREFIX + volume['name']
-        rel_info = self.target_helpers.get_relationship_info(tgt_volume)
-        if rel_info:
-            try:
-                self.target_helpers.switch_relationship(rel_info['name'],
-                                                        aux=False)
-                return
-            except Exception as e:
-                msg = (_('Unable to fail-back the volume:%(vol)s to the '
-                         'master back-end, error:%(error)s') %
-                       {"vol": volume['name'], "error": e})
-                LOG.exception(msg)
-                raise exception.VolumeDriverException(message=msg)
-
 
 class StorwizeSVCReplicationMetroMirror(
         StorwizeSVCReplicationGlobalMirror):
@@ -159,7 +140,7 @@ class StorwizeSVCReplicationMetroMirror(
 
 
 class StorwizeSVCReplicationGMCV(StorwizeSVCReplicationGlobalMirror):
-    """Support for Storwize/SVC global mirror with change volumes mode replication.
+    """Support for Storwize/SVC GMCV mode replication.
 
     Global Mirror with Change Volumes(GMCV) provides asynchronous replication
     based on point-in-time copies of data. The volumes in a GMCV relationship
@@ -173,7 +154,7 @@ class StorwizeSVCReplicationGMCV(StorwizeSVCReplicationGlobalMirror):
         super(StorwizeSVCReplicationGMCV, self).__init__(
             driver, replication_target, target_helpers)
 
-    def volume_replication_setup(self, context, vref):
+    def volume_replication_setup(self, context, vref, new_type=None):
         LOG.debug('enter: volume_replication_setup: volume %s', vref['name'])
         source_change_vol_name = (storwize_const.REPLICA_CHG_VOL_PREFIX +
                                   vref['name'])
@@ -181,66 +162,98 @@ class StorwizeSVCReplicationGMCV(StorwizeSVCReplicationGlobalMirror):
         target_change_vol_name = (storwize_const.REPLICA_CHG_VOL_PREFIX +
                                   target_vol_name)
         try:
+            if new_type:
+                new_type_opts = self.driver._get_vdisk_params(
+                    new_type['id'], volume_type=new_type)
             src_attr = self.driver._helpers.get_vdisk_attributes(
                 vref['name'])
-            # Create source change volume if it doesn't exist
-            src_change_attr = self.driver._helpers.get_vdisk_attributes(
-                source_change_vol_name)
-            if not src_change_attr:
-                src_change_opts = self.driver._get_vdisk_params(
-                    vref['volume_type_id'])
-                src_change_opts['iogrp'] = src_attr['IO_group_id']
-                # Change volumes would usually be thin-provisioned
-                src_change_opts['autoexpand'] = True
+            # Source change volume creation
+            src_change_opts = self.driver._get_vdisk_params(
+                vref['volume_type_id'])
+            src_change_opts['iogrp'] = src_attr['IO_group_id']
+            # Change volumes would usually be thin-provisioned
+            src_change_opts['autoexpand'] = True
+            src_change_pool = src_attr['mdisk_grp_name']
+            if new_type:
+                src_child_pool = (
+                    new_type_opts['storwize_svc_src_child_pool'])
+            else:
+                src_child_pool = (
+                    src_change_opts['storwize_svc_src_child_pool'])
+            if src_child_pool:
+                src_change_pool = src_child_pool
+            try:
                 self.driver._helpers.create_vdisk(source_change_vol_name,
                                                   six.text_type(vref['size']),
                                                   'gb',
-                                                  src_attr['mdisk_grp_name'],
+                                                  src_change_pool,
                                                   src_change_opts)
-            # Create target volume if it doesn't exist
-            target_attr = self.target_helpers.get_vdisk_attributes(
-                target_vol_name)
-            if not target_attr:
-                target_opts = self.driver._get_vdisk_params(
-                    vref['volume_type_id'])
-                target_pool = self.target.get('pool_name')
-                target_opts['iogrp'] = src_attr['IO_group_id']
+            except exception.VolumeBackendAPIException as excp:
+                if "CMMVC6035E" in excp.msg:
+                    msg = ('Source change volume: %s already exists'
+                           % source_change_vol_name)
+                    LOG.info(msg)
+
+            # Target volume creation
+            target_opts = self.driver._get_vdisk_params(
+                vref['volume_type_id'])
+            target_pool = self.target.get('pool_name')
+            target_opts['iogrp'] = src_attr['IO_group_id']
+            try:
                 self.target_helpers.create_vdisk(target_vol_name,
                                                  six.text_type(vref['size']),
                                                  'gb',
                                                  target_pool,
                                                  target_opts)
+            except exception.VolumeBackendAPIException as excp:
+                if "CMMVC6035E" in excp.msg:
+                    msg = ('Target Volume: %s already exists'
+                           % target_vol_name)
+                    LOG.info(msg)
 
-            # Create target change volume if it doesn't exist
-            target_change_attr = self.target_helpers.get_vdisk_attributes(
-                target_change_vol_name)
-            if not target_change_attr:
-                target_change_opts = self.driver._get_vdisk_params(
-                    vref['volume_type_id'])
-                target_change_pool = self.target.get('pool_name')
-                target_change_opts['iogrp'] = src_attr['IO_group_id']
-                # Change Volumes would usually be thin-provisioned
-                target_change_opts['autoexpand'] = True
+            # Target change volume creation
+            target_change_opts = self.driver._get_vdisk_params(
+                vref['volume_type_id'])
+            target_change_pool = self.target.get('pool_name')
+            if new_type:
+                target_child_pool = (
+                    new_type_opts['storwize_svc_target_child_pool'])
+            else:
+                target_child_pool = (
+                    target_change_opts['storwize_svc_target_child_pool'])
+            if target_child_pool:
+                target_change_pool = target_child_pool
+            target_change_opts['iogrp'] = src_attr['IO_group_id']
+            # Change Volumes would usually be thin-provisioned
+            target_change_opts['autoexpand'] = True
+            try:
                 self.target_helpers.create_vdisk(target_change_vol_name,
                                                  six.text_type(vref['size']),
                                                  'gb',
                                                  target_change_pool,
                                                  target_change_opts)
+            except exception.VolumeBackendAPIException as excp:
+                if "CMMVC6035E" in excp.msg:
+                    msg = ('Target Change Volume: %s already exists'
+                           % target_change_vol_name)
+                    LOG.info(msg)
 
-            system_info = self.target_helpers.get_system_info()
+            target_system_id = self.driver._aux_state['system_id']
             # Get cycle_period_seconds
             src_change_opts = self.driver._get_vdisk_params(
                 vref['volume_type_id'])
             cycle_period_seconds = src_change_opts.get('cycle_period_seconds')
-            self.driver._helpers.create_relationship(
-                vref['name'], target_vol_name, system_info.get('system_name'),
+            rc_name = self.driver._helpers.create_relationship(
+                vref['name'], target_vol_name, target_system_id,
                 self.asyncmirror, True, source_change_vol_name,
                 cycle_period_seconds)
             # Set target change volume
             self.target_helpers.change_relationship_changevolume(
-                target_vol_name, target_change_vol_name, False)
+                target_vol_name, target_change_vol_name, False,
+                rc_name)
             # Start gmcv relationship
-            self.driver._helpers.start_relationship(vref['name'])
+            self.driver._helpers.start_relationship(vref['name'],
+                                                    rcrel=rc_name)
         except Exception as e:
             msg = (_("Unable to set up gmcv mode replication for %(vol)s. "
                      "Exception: %(err)s.") % {'vol': vref['id'],
@@ -249,43 +262,6 @@ class StorwizeSVCReplicationGMCV(StorwizeSVCReplicationGlobalMirror):
             raise exception.VolumeDriverException(message=msg)
         LOG.debug('leave: volume_replication_setup:volume %s', vref['name'])
 
-    def failover_volume_host(self, context, vref):
-        LOG.debug('enter: failover_volume_host: vref=%(vref)s',
-                  {'vref': vref['name']})
-        # Make the aux volume writeable.
-        try:
-            tgt_volume = storwize_const.REPLICA_AUX_VOL_PREFIX + vref.name
-            self.target_helpers.stop_relationship(tgt_volume, access=True)
-            try:
-                self.target_helpers.start_relationship(tgt_volume, 'aux')
-            except exception.VolumeBackendAPIException as e:
-                LOG.error('Error running startrcrelationship due to %(err)s.',
-                          {'err': e})
-            return
-        except Exception as e:
-            msg = (_('Unable to fail-over the volume %(id)s to the '
-                   'secondary back-end, error: %(error)s') %
-                   {"id": vref['id'], "error": six.text_type(e)})
-            LOG.exception(msg)
-            raise exception.VolumeDriverException(message=msg)
-
-    def replication_failback(self, volume):
-        LOG.debug('enter: replication_failback: volume=%(volume)s',
-                  {'volume': volume['name']})
-        tgt_volume = storwize_const.REPLICA_AUX_VOL_PREFIX + volume['name']
-        rel_info = self.target_helpers.get_relationship_info(tgt_volume)
-        if rel_info:
-            try:
-                self.target_helpers.stop_relationship(tgt_volume, access=True)
-                self.target_helpers.start_relationship(tgt_volume, 'master')
-                return
-            except Exception as e:
-                msg = (_('Unable to fail-back the volume:%(vol)s to the '
-                         'master back-end, error:%(error)s') %
-                       {"vol": volume['name'], "error": six.text_type(e)})
-                LOG.exception(msg)
-                raise exception.VolumeDriverException(message=msg)
-
 
 class StorwizeSVCReplicationManager(object):
 
