@@ -1,4 +1,4 @@
-# Copyright 2016 Infinidat Ltd.
+# Copyright 2022 Infinidat Ltd.
 # All Rights Reserved.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,20 +16,24 @@
 
 from contextlib import contextmanager
 import functools
+import math
 import platform
 import socket
 from unittest import mock
+import uuid
 
 from oslo_config import cfg
 from oslo_log import log as logging
 from oslo_utils import units
 
+from cinder.common import constants
+from cinder import context as cinder_context
 from cinder import coordination
 from cinder import exception
 from cinder.i18n import _
 from cinder import interface
+from cinder import objects
 from cinder.objects import fields
-from cinder import utils
 from cinder import version
 from cinder.volume import configuration
 from cinder.volume.drivers.san import san
@@ -117,10 +121,16 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         1.4 - added support for QoS
         1.5 - added support for volume compression
         1.6 - added support for volume multi-attach
+        1.7 - fixed iSCSI to return all portals
+        1.8 - added revert to snapshot
+        1.9 - added manage/unmanage/manageable-list volume/snapshot
+        1.10 - added support for TLS/SSL communication
+        1.11 - fixed generic volume migration
+        1.12 - fixed volume multi-attach
 
     """
 
-    VERSION = '1.6'
+    VERSION = '1.12'
 
     # ThirdPartySystems wiki page
     CI_WIKI_NAME = "INFINIDAT_CI"
@@ -137,11 +147,14 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
             'chap_username', 'chap_password', 'san_thin_provision',
             'use_multipath_for_image_xfer', 'enforce_multipath_for_image_xfer',
             'num_volume_device_scan_tries', 'volume_dd_blocksize',
+            'driver_use_ssl', 'suppress_requests_ssl_warnings',
             'max_over_subscription_ratio')
         return infinidat_opts + additional_opts
 
-    def _setup_and_get_system_object(self, management_address, auth):
-        system = infinisdk.InfiniBox(management_address, auth=auth)
+    def _setup_and_get_system_object(self, management_address, auth,
+                                     use_ssl=False):
+        system = infinisdk.InfiniBox(management_address, auth=auth,
+                                     use_ssl=use_ssl)
         system.api.add_auto_retry(
             lambda e: isinstance(
                 e, infinisdk.core.exceptions.APITransportFailure) and
@@ -158,19 +171,20 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
             raise exception.VolumeDriverException(message=msg)
         auth = (self.configuration.san_login,
                 self.configuration.san_password)
+        use_ssl = self.configuration.driver_use_ssl
         self.management_address = self.configuration.san_ip
-        self._system = (
-            self._setup_and_get_system_object(self.management_address, auth))
+        self._system = self._setup_and_get_system_object(
+            self.management_address, auth, use_ssl=use_ssl)
         backend_name = self.configuration.safe_get('volume_backend_name')
         self._backend_name = backend_name or self.__class__.__name__
         self._volume_stats = None
         if self.configuration.infinidat_storage_protocol.lower() == 'iscsi':
-            self._protocol = 'iSCSI'
+            self._protocol = constants.ISCSI
             if len(self.configuration.infinidat_iscsi_netspaces) == 0:
                 msg = _('No iSCSI network spaces configured')
                 raise exception.VolumeDriverException(message=msg)
         else:
-            self._protocol = 'FC'
+            self._protocol = constants.FC
         if (self.configuration.infinidat_use_compression and
            not self._system.compat.has_compression()):
             # InfiniBox systems support compression only from v3.0 and up
@@ -183,14 +197,24 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         LOG.debug('setup complete')
 
     def validate_connector(self, connector):
-        required = 'initiator' if self._protocol == 'iSCSI' else 'wwpns'
+        required = ('initiator' if self._protocol == constants.ISCSI
+                    else 'wwpns')
         if required not in connector:
             LOG.error('The volume driver requires %(data)s '
                       'in the connector.', {'data': required})
             raise exception.InvalidConnectorException(missing=required)
 
-    def _make_volume_name(self, cinder_volume):
-        return 'openstack-vol-%s' % cinder_volume.id
+    def _make_volume_name(self, cinder_volume, migration=False):
+        """Return the Infinidat volume name.
+
+        Use Cinder volume id in case of volume migration
+        and use Cinder volume name_id for all other cases.
+        """
+        if migration:
+            key = cinder_volume.id
+        else:
+            key = cinder_volume.name_id
+        return 'openstack-vol-%s' % key
 
     def _make_snapshot_name(self, cinder_snapshot):
         return 'openstack-snap-%s' % cinder_snapshot.id
@@ -220,21 +244,41 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
                 "host.created_by": _INFINIDAT_CINDER_IDENTIFIER}
         infinidat_object.set_metadata_from_dict(data)
 
+    def _get_infinidat_dataset_by_ref(self, existing_ref):
+        if 'source-id' in existing_ref:
+            kwargs = dict(id=existing_ref['source-id'])
+        elif 'source-name' in existing_ref:
+            kwargs = dict(name=existing_ref['source-name'])
+        else:
+            reason = _('dataset reference must contain '
+                       'source-id or source-name key')
+            raise exception.ManageExistingInvalidReference(
+                existing_ref=existing_ref, reason=reason)
+        return self._system.volumes.safe_get(**kwargs)
+
+    def _get_infinidat_volume_by_ref(self, existing_ref):
+        infinidat_volume = self._get_infinidat_dataset_by_ref(existing_ref)
+        if infinidat_volume is None:
+            raise exception.VolumeNotFound(volume_id=existing_ref)
+        return infinidat_volume
+
+    def _get_infinidat_snapshot_by_ref(self, existing_ref):
+        infinidat_snapshot = self._get_infinidat_dataset_by_ref(existing_ref)
+        if infinidat_snapshot is None:
+            raise exception.SnapshotNotFound(snapshot_id=existing_ref)
+        if not infinidat_snapshot.is_snapshot():
+            reason = (_('reference %(existing_ref)s is a volume')
+                      % {'existing_ref': existing_ref})
+            raise exception.InvalidSnapshot(reason=reason)
+        return infinidat_snapshot
+
     def _get_infinidat_volume_by_name(self, name):
-        volume = self._system.volumes.safe_get(name=name)
-        if volume is None:
-            msg = _('Volume "%s" not found') % name
-            LOG.error(msg)
-            raise exception.InvalidVolume(reason=msg)
-        return volume
+        ref = {'source-name': name}
+        return self._get_infinidat_volume_by_ref(ref)
 
     def _get_infinidat_snapshot_by_name(self, name):
-        snapshot = self._system.volumes.safe_get(name=name)
-        if snapshot is None:
-            msg = _('Snapshot "%s" not found') % name
-            LOG.error(msg)
-            raise exception.InvalidSnapshot(reason=msg)
-        return snapshot
+        ref = {'source-name': name}
+        return self._get_infinidat_snapshot_by_ref(ref)
 
     def _get_infinidat_volume(self, cinder_volume):
         volume_name = self._make_volume_name(cinder_volume)
@@ -257,9 +301,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         group_name = self._make_cg_name(cinder_group)
         infinidat_cg = self._system.cons_groups.safe_get(name=group_name)
         if infinidat_cg is None:
-            msg = _('Consistency group "%s" not found') % group_name
-            LOG.error(msg)
-            raise exception.InvalidGroup(message=msg)
+            raise exception.GroupNotFound(group_id=group_name)
         return infinidat_cg
 
     def _get_or_create_host(self, port):
@@ -335,8 +377,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
                     yield str(port.get_wwpn())
 
     def _initialize_connection_fc(self, volume, connector):
-        volume_name = self._make_volume_name(volume)
-        infinidat_volume = self._get_infinidat_volume_by_name(volume_name)
+        infinidat_volume = self._get_infinidat_volume(volume)
         ports = [wwn.WWN(wwpn) for wwpn in connector['wwpns']]
         for port in ports:
             infinidat_host = self._get_or_create_host(port)
@@ -365,19 +406,19 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
             raise exception.VolumeDriverException(message=msg)
         return netspace
 
-    def _get_iscsi_portal(self, netspace):
-        for netpsace_interface in netspace.get_ips():
-            if netpsace_interface.enabled:
-                port = netspace.get_properties().iscsi_tcp_port
-                return "%s:%s" % (netpsace_interface.ip_address, port)
+    def _get_iscsi_portals(self, netspace):
+        port = netspace.get_properties().iscsi_tcp_port
+        portals = ["%s:%s" % (interface.ip_address, port) for interface
+                   in netspace.get_ips() if interface.enabled]
+        if portals:
+            return portals
         # if we get here it means there are no enabled ports
         msg = (_('No available interfaces in iSCSI network space %s') %
                netspace.get_name())
         raise exception.VolumeDriverException(message=msg)
 
     def _initialize_connection_iscsi(self, volume, connector):
-        volume_name = self._make_volume_name(volume)
-        infinidat_volume = self._get_infinidat_volume_by_name(volume_name)
+        infinidat_volume = self._get_infinidat_volume(volume)
         port = iqn.IQN(connector['initiator'])
         infinidat_host = self._get_or_create_host(port)
         if self.configuration.use_chap_auth:
@@ -398,9 +439,11 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         target_luns = []
         for netspace_name in netspace_names:
             netspace = self._get_iscsi_network_space(netspace_name)
-            target_portals.append(self._get_iscsi_portal(netspace))
-            target_iqns.append(netspace.get_properties().iscsi_iqn)
-            target_luns.append(lun)
+            netspace_portals = self._get_iscsi_portals(netspace)
+            target_portals.extend(netspace_portals)
+            target_iqns.extend([netspace.get_properties().iscsi_iqn] *
+                               len(netspace_portals))
+            target_luns.extend([lun] * len(netspace_portals))
 
         result_data = dict(target_discovered=True,
                            target_portal=target_portals[0],
@@ -420,7 +463,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         if connector is None:
             # If no connector was provided it is a force-detach - remove all
             # host connections for the volume
-            if self._protocol == 'FC':
+            if self._protocol == constants.FC:
                 port_cls = wwn.WWN
             else:
                 port_cls = iqn.IQN
@@ -430,17 +473,43 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
                 host_ports = [port for port in host_ports
                               if isinstance(port, port_cls)]
                 ports.extend(host_ports)
-        elif self._protocol == 'FC':
+        elif self._protocol == constants.FC:
             ports = [wwn.WWN(wwpn) for wwpn in connector['wwpns']]
         else:
             ports = [iqn.IQN(connector['initiator'])]
         return ports
 
+    def _is_volume_multiattached(self, volume, connector):
+        """Returns whether the volume is multiattached.
+
+        Check if there are multiple attachments to the volume
+        from the same connector. Terminate connection only for
+        the last attachment from the corresponding host.
+        """
+        if not (connector and volume.multiattach and
+                volume.volume_attachment):
+            return False
+        keys = ['system uuid']
+        if self._protocol == constants.FC:
+            keys.append('wwpns')
+        else:
+            keys.append('initiator')
+        for key in keys:
+            if not (key in connector and connector[key]):
+                continue
+            if sum(1 for attachment in volume.volume_attachment if
+                   attachment.connector and key in attachment.connector and
+                   attachment.connector[key] == connector[key]) > 1:
+                LOG.debug('Volume %s is multiattached to %s %s',
+                          volume.name_id, key, connector[key])
+                return True
+        return False
+
     @infinisdk_to_cinder_exceptions
     @coordination.synchronized('infinidat-{self.management_address}-lock')
     def initialize_connection(self, volume, connector):
         """Map an InfiniBox volume to the host"""
-        if self._protocol == 'FC':
+        if self._protocol == constants.FC:
             return self._initialize_connection_fc(volume, connector)
         else:
             return self._initialize_connection_iscsi(volume, connector)
@@ -449,8 +518,10 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
     @coordination.synchronized('infinidat-{self.management_address}-lock')
     def terminate_connection(self, volume, connector, **kwargs):
         """Unmap an InfiniBox volume from the host"""
+        if self._is_volume_multiattached(volume, connector):
+            return True
         infinidat_volume = self._get_infinidat_volume(volume)
-        if self._protocol == 'FC':
+        if self._protocol == constants.FC:
             volume_type = 'fibre_channel'
         else:
             volume_type = 'iscsi'
@@ -470,7 +541,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
             # check if the host now doesn't have mappings
             if host is not None and len(host.get_luns()) == 0:
                 host.safe_delete()
-                if self._protocol == 'FC' and connector is not None:
+                if self._protocol == constants.FC and connector is not None:
                     # Create initiator-target mapping to delete host entry
                     # this is only relevant for regular (specific host) detach
                     target_wwpns = list(self._get_online_fc_ports())
@@ -479,11 +550,11 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
                                                          target_wwpns))
                     result_data = dict(target_wwn=target_wwpns,
                                        initiator_target_map=target_map)
-        conn_info = dict(driver_volume_type=volume_type,
-                         data=result_data)
-        if self._protocol == 'FC':
+        if self._protocol == constants.FC:
+            conn_info = dict(driver_volume_type=volume_type,
+                             data=result_data)
             fczm_utils.remove_fc_zone(conn_info)
-        return conn_info
+        return volume.volume_attachment and len(volume.volume_attachment) > 1
 
     @infinisdk_to_cinder_exceptions
     def get_volume_stats(self, refresh=False):
@@ -540,14 +611,13 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
     @infinisdk_to_cinder_exceptions
     def delete_volume(self, volume):
         """Delete a volume from the backend."""
-        volume_name = self._make_volume_name(volume)
         try:
-            infinidat_volume = self._get_infinidat_volume_by_name(volume_name)
-        except exception.InvalidVolume:
-            return      # volume not found
+            infinidat_volume = self._get_infinidat_volume(volume)
+        except exception.VolumeNotFound:
+            return
         if infinidat_volume.has_children():
             # can't delete a volume that has a live snapshot
-            raise exception.VolumeIsBusy(volume_name=volume_name)
+            raise exception.VolumeIsBusy(volume_name=volume.name)
         infinidat_volume.safe_delete()
 
     @infinisdk_to_cinder_exceptions
@@ -569,8 +639,9 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
     def _connection_context(self, volume):
         use_multipath = self.configuration.use_multipath_for_image_xfer
         enforce_multipath = self.configuration.enforce_multipath_for_image_xfer
-        connector = utils.brick_get_connector_properties(use_multipath,
-                                                         enforce_multipath)
+        connector = volume_utils.brick_get_connector_properties(
+            use_multipath,
+            enforce_multipath)
         connection = self.initialize_connection(volume, connector)
         try:
             yield connection
@@ -582,7 +653,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         use_multipath = self.configuration.use_multipath_for_image_xfer
         device_scan_attempts = self.configuration.num_volume_device_scan_tries
         protocol = connection['driver_volume_type']
-        connector = utils.brick_get_connector(
+        connector = volume_utils.brick_get_connector(
             protocol,
             use_multipath=use_multipath,
             device_scan_attempts=device_scan_attempts,
@@ -623,7 +694,8 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         # we need a cinder-volume-like object to map the clone by name
         # (which is derived from the cinder id) but the clone is internal
         # so there is no such object. mock one
-        clone = mock.Mock(id=str(volume.id) + '-internal')
+        clone = mock.Mock(name_id=str(volume.name_id) + '-internal',
+                          multiattach=False, volume_attachment=[])
         try:
             infinidat_volume = self._create_volume(volume)
             try:
@@ -647,11 +719,11 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         """Deletes a snapshot."""
         try:
             snapshot = self._get_infinidat_snapshot(snapshot)
-        except exception.InvalidSnapshot:
-            return      # snapshot not found
+        except exception.SnapshotNotFound:
+            return
         snapshot.safe_delete()
 
-    def _asssert_volume_not_mapped(self, volume):
+    def _assert_volume_not_mapped(self, volume):
         # copy is not atomic so we can't clone while the volume is mapped
         infinidat_volume = self._get_infinidat_volume(volume)
         if len(infinidat_volume.get_logical_units()) == 0:
@@ -677,7 +749,7 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         * copy data from source to new volume
         * unmap both volumes
         """
-        self._asssert_volume_not_mapped(src_vref)
+        self._assert_volume_not_mapped(src_vref)
         infinidat_volume = self._create_volume(volume)
         try:
             src_ctx = self._device_connect_context(src_vref)
@@ -742,8 +814,8 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
             raise NotImplementedError()
         try:
             infinidat_cg = self._get_infinidat_cg(group)
-        except exception.InvalidGroup:
-            pass      # group not found
+        except exception.GroupNotFound:
+            pass
         else:
             infinidat_cg.safe_delete()
         for volume in volumes:
@@ -831,3 +903,372 @@ class InfiniboxVolumeDriver(san.SanISCSIDriver):
         for snapshot in snapshots:
             self.delete_snapshot(snapshot)
         return None, None
+
+    def snapshot_revert_use_temp_snapshot(self):
+        """Disable the use of a temporary snapshot on revert."""
+        return False
+
+    @infinisdk_to_cinder_exceptions
+    def revert_to_snapshot(self, context, volume, snapshot):
+        """Revert volume to snapshot.
+
+        Note: the revert process should not change the volume's
+        current size, that means if the driver shrank
+        the volume during the process, it should extend the
+        volume internally.
+        """
+        infinidat_snapshot = self._get_infinidat_snapshot(snapshot)
+        infinidat_volume = self._get_infinidat_volume(snapshot.volume)
+        infinidat_volume.restore(infinidat_snapshot)
+        volume_size = infinidat_volume.get_size()
+        snapshot_size = snapshot.volume.size * capacity.GiB
+        if volume_size < snapshot_size:
+            self.extend_volume(volume, snapshot.volume.size)
+
+    @infinisdk_to_cinder_exceptions
+    def manage_existing(self, volume, existing_ref):
+        """Manage an existing Infinidat volume.
+
+        Checks if the volume is already managed.
+        Renames the Infinidat volume to match the expected name.
+        Updates QoS and metadata.
+
+        :param volume:       Cinder volume to manage
+        :param existing_ref: dictionary of the forms:
+                             {'source-name': 'Infinidat volume name'} or
+                             {'source-id': 'Infinidat volume serial number'}
+        """
+        infinidat_volume = self._get_infinidat_volume_by_ref(existing_ref)
+        infinidat_metadata = infinidat_volume.get_all_metadata()
+        if 'cinder_id' in infinidat_metadata:
+            cinder_id = infinidat_metadata['cinder_id']
+            if volume_utils.check_already_managed_volume(cinder_id):
+                raise exception.ManageExistingAlreadyManaged(
+                    volume_ref=cinder_id)
+        infinidat_pool = infinidat_volume.get_pool_name()
+        if infinidat_pool != self.configuration.infinidat_pool_name:
+            message = (_('unexpected pool name %(infinidat_pool)s')
+                       % {'infinidat_pool': infinidat_pool})
+            raise exception.InvalidConfigurationValue(message=message)
+        cinder_name = self._make_volume_name(volume)
+        infinidat_volume.update_name(cinder_name)
+        self._set_qos(volume, infinidat_volume)
+        self._set_cinder_object_metadata(infinidat_volume, volume)
+
+    @infinisdk_to_cinder_exceptions
+    def manage_existing_get_size(self, volume, existing_ref):
+        """Return size of an existing Infinidat volume.
+
+        When calculating the size, round up to the next GB.
+
+        :param volume:       Cinder volume to manage
+        :param existing_ref: dictionary of the forms:
+                             {'source-name': 'Infinidat volume name'} or
+                             {'source-id': 'Infinidat volume serial number'}
+        :returns size:       Volume size in GiB (integer)
+        """
+        infinidat_volume = self._get_infinidat_volume_by_ref(existing_ref)
+        return int(math.ceil(infinidat_volume.get_size() / capacity.GiB))
+
+    @infinisdk_to_cinder_exceptions
+    def get_manageable_volumes(self, cinder_volumes, marker, limit, offset,
+                               sort_keys, sort_dirs):
+        """List volumes on the Infinidat backend available for management.
+
+        Returns a list of dictionaries, each specifying a volume on the
+        Infinidat backend, with the following keys:
+        - reference (dictionary): The reference for a volume, which can be
+        passed to "manage_existing". Each reference contains keys:
+        Infinidat volume name and Infinidat volume serial number.
+        - size (int): The size of the volume according to the Infinidat
+        storage backend, rounded up to the nearest GB.
+        - safe_to_manage (boolean): Whether or not this volume is safe to
+        manage according to the storage backend. For example, is the volume
+        already managed, in use, has snapshots or active mappings.
+        - reason_not_safe (string): If safe_to_manage is False, the reason why.
+        - cinder_id (string): If already managed, provide the Cinder ID.
+        - extra_info (string): Extra information (pool name, volume type,
+        QoS and metadata) to return to the user.
+
+        :param cinder_volumes: A list of volumes in this host that Cinder
+                               currently manages, used to determine if
+                               a volume is manageable or not.
+        :param marker:    The last item of the previous page; we return the
+                          next results after this value (after sorting)
+        :param limit:     Maximum number of items to return
+        :param offset:    Number of items to skip after marker
+        :param sort_keys: List of keys to sort results by (valid keys are
+                          'identifier' and 'size')
+        :param sort_dirs: List of directions to sort by, corresponding to
+                          sort_keys (valid directions are 'asc' and 'desc')
+        """
+        manageable_volumes = []
+        cinder_ids = [cinder_volume.id for cinder_volume in cinder_volumes]
+        infinidat_pool = self._get_infinidat_pool()
+        infinidat_volumes = infinidat_pool.get_volumes()
+        for infinidat_volume in infinidat_volumes:
+            if infinidat_volume.is_snapshot():
+                continue
+            safe_to_manage = False
+            reason_not_safe = None
+            volume_id = infinidat_volume.get_id()
+            volume_name = infinidat_volume.get_name()
+            volume_size = infinidat_volume.get_size()
+            volume_type = infinidat_volume.get_type()
+            volume_pool = infinidat_volume.get_pool_name()
+            volume_qos = infinidat_volume.get_qos_policy()
+            volume_meta = infinidat_volume.get_all_metadata()
+            cinder_id = volume_meta.get('cinder_id')
+            volume_luns = infinidat_volume.get_logical_units()
+            if cinder_id and cinder_id in cinder_ids:
+                reason_not_safe = _('volume already managed')
+            elif volume_luns:
+                reason_not_safe = _('volume has mappings')
+            elif infinidat_volume.has_children():
+                reason_not_safe = _('volume has snapshots')
+            else:
+                safe_to_manage = True
+            reference = {
+                'source-name': volume_name,
+                'source-id': str(volume_id)
+            }
+            extra_info = {
+                'pool': volume_pool,
+                'type': volume_type,
+                'qos': str(volume_qos),
+                'meta': str(volume_meta)
+            }
+            manageable_volume = {
+                'reference': reference,
+                'size': int(math.ceil(volume_size / capacity.GiB)),
+                'safe_to_manage': safe_to_manage,
+                'reason_not_safe': reason_not_safe,
+                'cinder_id': cinder_id,
+                'extra_info': extra_info
+            }
+            manageable_volumes.append(manageable_volume)
+        return volume_utils.paginate_entries_list(
+            manageable_volumes, marker, limit,
+            offset, sort_keys, sort_dirs)
+
+    @infinisdk_to_cinder_exceptions
+    def unmanage(self, volume):
+        """Removes the specified volume from Cinder management.
+
+        Does not delete the underlying backend storage object.
+
+        For most drivers, this will not need to do anything.  However, some
+        drivers might use this call as an opportunity to clean up any
+        Cinder-specific configuration that they have associated with the
+        backend storage object.
+
+        :param volume: Cinder volume to unmanage
+        """
+        infinidat_volume = self._get_infinidat_volume(volume)
+        infinidat_volume.clear_metadata()
+
+    def _check_already_managed_snapshot(self, snapshot_id):
+        """Check cinder db for already managed snapshot.
+
+        :param snapshot_id snapshot id parameter
+        :returns: bool -- return True, if db entry with specified
+                          snapshot id exists, otherwise return False
+        """
+        try:
+            uuid.UUID(snapshot_id, version=4)
+        except ValueError:
+            return False
+        ctxt = cinder_context.get_admin_context()
+        return objects.Snapshot.exists(ctxt, snapshot_id)
+
+    @infinisdk_to_cinder_exceptions
+    def manage_existing_snapshot(self, snapshot, existing_ref):
+        """Manage an existing Infinidat snapshot.
+
+        Checks if the snapshot is already managed.
+        Renames the Infinidat snapshot to match the expected name.
+        Updates QoS and metadata.
+
+        :param snapshot:     Cinder snapshot to manage
+        :param existing_ref: dictionary of the forms:
+                             {'source-name': 'Infinidat snapshot name'} or
+                             {'source-id': 'Infinidat snapshot serial number'}
+        """
+        infinidat_snapshot = self._get_infinidat_snapshot_by_ref(existing_ref)
+        infinidat_metadata = infinidat_snapshot.get_all_metadata()
+        if 'cinder_id' in infinidat_metadata:
+            cinder_id = infinidat_metadata['cinder_id']
+            if self._check_already_managed_snapshot(cinder_id):
+                raise exception.ManageExistingAlreadyManaged(
+                    volume_ref=cinder_id)
+        infinidat_pool = infinidat_snapshot.get_pool_name()
+        if infinidat_pool != self.configuration.infinidat_pool_name:
+            message = (_('unexpected pool name %(infinidat_pool)s')
+                       % {'infinidat_pool': infinidat_pool})
+            raise exception.InvalidConfigurationValue(message=message)
+        cinder_name = self._make_snapshot_name(snapshot)
+        infinidat_snapshot.update_name(cinder_name)
+        self._set_qos(snapshot, infinidat_snapshot)
+        self._set_cinder_object_metadata(infinidat_snapshot, snapshot)
+
+    @infinisdk_to_cinder_exceptions
+    def manage_existing_snapshot_get_size(self, snapshot, existing_ref):
+        """Return size of an existing Infinidat snapshot.
+
+        When calculating the size, round up to the next GB.
+
+        :param snapshot:     Cinder snapshot to manage
+        :param existing_ref: dictionary of the forms:
+                             {'source-name': 'Infinidat snapshot name'} or
+                             {'source-id': 'Infinidat snapshot serial number'}
+        :returns size:       Snapshot size in GiB (integer)
+        """
+        infinidat_snapshot = self._get_infinidat_snapshot_by_ref(existing_ref)
+        return int(math.ceil(infinidat_snapshot.get_size() / capacity.GiB))
+
+    @infinisdk_to_cinder_exceptions
+    def get_manageable_snapshots(self, cinder_snapshots, marker, limit, offset,
+                                 sort_keys, sort_dirs):
+        """List snapshots on the Infinidat backend available for management.
+
+        Returns a list of dictionaries, each specifying a snapshot on the
+        Infinidat backend, with the following keys:
+        - reference (dictionary): The reference for a snapshot, which can be
+        passed to "manage_existing_snapshot". Each reference contains keys:
+        Infinidat snapshot name and Infinidat snapshot serial number.
+        - size (int): The size of the snapshot according to the Infinidat
+        storage backend, rounded up to the nearest GB.
+        - safe_to_manage (boolean): Whether or not this snapshot is safe to
+        manage according to the storage backend. For example, is the snapshot
+        already managed, has clones or active mappings.
+        - reason_not_safe (string): If safe_to_manage is False, the reason why.
+        - cinder_id (string): If already managed, provide the Cinder ID.
+        - extra_info (string): Extra information (pool name, snapshot type,
+        QoS and metadata) to return to the user.
+        - source_reference (string): Similar to "reference", but for the
+        snapshot's source volume. The source reference contains two keys:
+        Infinidat volume name and Infinidat volume serial number.
+
+        :param cinder_snapshots: A list of snapshots in this host that Cinder
+                                 currently manages, used to determine if
+                                 a snapshot is manageable or not.
+        :param marker:    The last item of the previous page; we return the
+                          next results after this value (after sorting)
+        :param limit:     Maximum number of items to return
+        :param offset:    Number of items to skip after marker
+        :param sort_keys: List of keys to sort results by (valid keys are
+                          'identifier' and 'size')
+        :param sort_dirs: List of directions to sort by, corresponding to
+                          sort_keys (valid directions are 'asc' and 'desc')
+        """
+        manageable_snapshots = []
+        cinder_ids = [cinder_snapshot.id for cinder_snapshot
+                      in cinder_snapshots]
+        infinidat_pool = self._get_infinidat_pool()
+        infinidat_snapshots = infinidat_pool.get_volumes()
+        for infinidat_snapshot in infinidat_snapshots:
+            if not infinidat_snapshot.is_snapshot():
+                continue
+            safe_to_manage = False
+            reason_not_safe = None
+            parent = infinidat_snapshot.get_parent()
+            parent_id = parent.get_id()
+            parent_name = parent.get_name()
+            snapshot_id = infinidat_snapshot.get_id()
+            snapshot_name = infinidat_snapshot.get_name()
+            snapshot_size = infinidat_snapshot.get_size()
+            snapshot_type = infinidat_snapshot.get_type()
+            snapshot_pool = infinidat_snapshot.get_pool_name()
+            snapshot_qos = infinidat_snapshot.get_qos_policy()
+            snapshot_meta = infinidat_snapshot.get_all_metadata()
+            cinder_id = snapshot_meta.get('cinder_id')
+            snapshot_luns = infinidat_snapshot.get_logical_units()
+            if cinder_id and cinder_id in cinder_ids:
+                reason_not_safe = _('snapshot already managed')
+            elif snapshot_luns:
+                reason_not_safe = _('snapshot has mappings')
+            elif infinidat_snapshot.has_children():
+                reason_not_safe = _('snapshot has clones')
+            else:
+                safe_to_manage = True
+            reference = {
+                'source-name': snapshot_name,
+                'source-id': str(snapshot_id)
+            }
+            source_reference = {
+                'source-name': parent_name,
+                'source-id': str(parent_id)
+            }
+            extra_info = {
+                'pool': snapshot_pool,
+                'type': snapshot_type,
+                'qos': str(snapshot_qos),
+                'meta': str(snapshot_meta)
+            }
+            manageable_snapshot = {
+                'reference': reference,
+                'size': int(math.ceil(snapshot_size / capacity.GiB)),
+                'safe_to_manage': safe_to_manage,
+                'reason_not_safe': reason_not_safe,
+                'cinder_id': cinder_id,
+                'extra_info': extra_info,
+                'source_reference': source_reference
+            }
+            manageable_snapshots.append(manageable_snapshot)
+        return volume_utils.paginate_entries_list(
+            manageable_snapshots, marker, limit,
+            offset, sort_keys, sort_dirs)
+
+    @infinisdk_to_cinder_exceptions
+    def unmanage_snapshot(self, snapshot):
+        """Removes the specified snapshot from Cinder management.
+
+        Does not delete the underlying backend storage object.
+
+        For most drivers, this will not need to do anything. However, some
+        drivers might use this call as an opportunity to clean up any
+        Cinder-specific configuration that they have associated with the
+        backend storage object.
+
+        :param snapshot: Cinder volume snapshot to unmanage
+        """
+        infinidat_snapshot = self._get_infinidat_snapshot(snapshot)
+        infinidat_snapshot.clear_metadata()
+
+    @infinisdk_to_cinder_exceptions
+    def update_migrated_volume(self, ctxt, volume, new_volume,
+                               original_volume_status):
+        """Return model update from Infinidat for migrated volume.
+
+        This method should rename the back-end volume name(id) on the
+        destination host back to its original name(id) on the source host.
+
+        :param ctxt: The context used to run the method update_migrated_volume
+        :param volume: The original volume that was migrated to this backend
+        :param new_volume: The migration volume object that was created on
+                           this backend as part of the migration process
+        :param original_volume_status: The status of the original volume
+        :returns: model_update to update DB with any needed changes
+        """
+        model_update = {'_name_id': new_volume.name_id,
+                        'provider_location': None}
+        new_volume_name = self._make_volume_name(new_volume, migration=True)
+        new_infinidat_volume = self._get_infinidat_volume(new_volume)
+        self._set_cinder_object_metadata(new_infinidat_volume, volume)
+        volume_name = self._make_volume_name(volume, migration=True)
+        try:
+            infinidat_volume = self._get_infinidat_volume(volume)
+        except exception.VolumeNotFound:
+            LOG.debug('Source volume %s not found', volume_name)
+        else:
+            volume_pool = infinidat_volume.get_pool_name()
+            LOG.debug('Found source volume %s in pool %s',
+                      volume_name, volume_pool)
+            return model_update
+        try:
+            new_infinidat_volume.update_name(volume_name)
+        except infinisdk.core.exceptions.InfiniSDKException as error:
+            LOG.error('Failed to rename destination volume %s -> %s: %s',
+                      new_volume_name, volume_name, error)
+            return model_update
+        return {'_name_id': None, 'provider_location': None}
