@@ -27,6 +27,7 @@ from oslo_utils import excutils
 from oslo_utils import strutils
 from oslo_utils import timeutils
 
+from cinder.common import constants
 from cinder import context
 from cinder.coordination import synchronized
 from cinder import exception
@@ -76,7 +77,7 @@ def replication_synced(params):
 class MacroSANBaseDriver(driver.VolumeDriver):
     """Base driver for MacroSAN SAN."""
 
-    CI_WIKI_NAME = 'MacroSAN Volume CI'
+    CI_WIKI_NAME = 'MacroSAN_Volume_CI'
 
     def __init__(self, *args, **kwargs):
         """Initialize the driver."""
@@ -411,7 +412,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def create_volume(self, volume):
         """Create a volume."""
         name = volume['name']
@@ -466,7 +467,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def delete_volume(self, volume):
         """Delete a volume."""
         name = self._volume_name(volume)
@@ -525,7 +526,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def create_snapshot(self, snapshot):
         """Create a snapshot."""
         volume = snapshot['volume']
@@ -554,7 +555,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def delete_snapshot(self, snapshot):
         """Delete a snapshot."""
         volume = snapshot['volume']
@@ -606,7 +607,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def create_volume_from_snapshot(self, volume, snapshot):
         """Create a volume from a snapshot."""
         snapshot_volume = snapshot['volume']
@@ -647,7 +648,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
             self._delete_snapshot(snp_name, src_vol_name, pointid)
 
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def create_cloned_volume(self, volume, src_vref):
         """Create a clone of the specified volume."""
         vol_name = volume['id']
@@ -687,7 +688,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def extend_volume(self, volume, new_size):
         """Extend a volume."""
         name = self._volume_name(volume)
@@ -747,7 +748,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
         self._stats = data
 
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def update_migrated_volume(self, ctxt, volume, new_volume,
                                original_volume_status=None):
         """Return model update for migrated volume."""
@@ -772,7 +773,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def initialize_connection_snapshot(self, snapshot, connector, **kwargs):
         volume = snapshot['volume']
         provider = snapshot['provider_location']
@@ -811,7 +812,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def manage_existing(self, volume, external_ref):
         vol_params = self._parse_volume_params(volume)
         self._check_volume_params(vol_params)
@@ -901,7 +902,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def manage_existing_snapshot(self, snapshot, existing_ref):
         volume = snapshot['volume']
         src_name = self._get_existing_snapname(existing_ref).lstrip('_')
@@ -948,7 +949,7 @@ class MacroSANBaseDriver(driver.VolumeDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def migrate_volume(self, ctxt, volume, host):
         if not self.migration_valid(volume, host):
             return False, None
@@ -1023,7 +1024,7 @@ class MacroSANISCSIDriver(MacroSANBaseDriver, driver.ISCSIDriver):
     def __init__(self, *args, **kwargs):
         """Initialize the driver."""
         super(MacroSANISCSIDriver, self).__init__(*args, **kwargs)
-        self.storage_protocol = 'iSCSI'
+        self.storage_protocol = constants.ISCSI
 
     def _do_setup(self):
         ports = self.client.get_iscsi_ports()
@@ -1148,7 +1149,7 @@ class MacroSANISCSIDriver(MacroSANBaseDriver, driver.ISCSIDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def initialize_connection(self, volume, connector):
         """Allow connection to connector and return connection info."""
 
@@ -1184,7 +1185,7 @@ class MacroSANISCSIDriver(MacroSANBaseDriver, driver.ISCSIDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def terminate_connection(self, volume, connector, **kwargs):
         """Disallow connection from connector."""
 
@@ -1224,7 +1225,7 @@ class MacroSANFCDriver(MacroSANBaseDriver, driver.FibreChannelDriver):
     def __init__(self, *args, **kwargs):
         """Initialize the driver."""
         super(MacroSANFCDriver, self).__init__(*args, **kwargs)
-        self.storage_protocol = 'FC'
+        self.storage_protocol = constants.FC
         self.fcsan_lookup_service = None
         self.use_sp_port_nr = self.configuration.macrosan_fc_use_sp_port_nr
         self.keep_mapped_ports = \
@@ -1433,7 +1434,7 @@ class MacroSANFCDriver(MacroSANBaseDriver, driver.FibreChannelDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def initialize_connection(self, volume, connector):
         """Allow connection to connector and return connection info."""
 
@@ -1496,7 +1497,7 @@ class MacroSANFCDriver(MacroSANBaseDriver, driver.FibreChannelDriver):
 
     @synchronized(lock_name)
     @record_request_id
-    @utils.trace
+    @volume_utils.trace
     def terminate_connection(self, volume, connector, **kwargs):
         """Disallow connection from connector."""
 
