@@ -38,7 +38,6 @@ import six
 
 from cinder import exception
 from cinder.i18n import _
-from cinder import utils
 from cinder.volume.drivers.netapp.dataontap.client import api as netapp_api
 from cinder.volume.drivers.netapp.dataontap.utils import loopingcalls
 from cinder.volume.drivers.netapp import options as na_opts
@@ -71,7 +70,7 @@ class NetAppLun(object):
                self.handle, self.name, self.size, self.metadata)
 
 
-@six.add_metaclass(utils.TraceWrapperMetaclass)
+@six.add_metaclass(volume_utils.TraceWrapperMetaclass)
 class NetAppBlockStorageLibrary(object):
     """NetApp block storage library for Data ONTAP."""
 
@@ -237,8 +236,10 @@ class NetAppBlockStorageLibrary(object):
         qos_policy_group_name = (
             na_utils.get_qos_policy_group_name_from_info(
                 qos_policy_group_info))
-        qos_policy_group_is_adaptive = volume_utils.is_boolean_str(
-            extra_specs.get('netapp:qos_policy_group_is_adaptive'))
+        qos_policy_group_is_adaptive = (volume_utils.is_boolean_str(
+            extra_specs.get('netapp:qos_policy_group_is_adaptive')) or
+            na_utils.is_qos_policy_group_spec_adaptive(
+                qos_policy_group_info))
 
         try:
             self._create_lun(pool_name, lun_name, size, metadata,
@@ -360,8 +361,10 @@ class NetAppBlockStorageLibrary(object):
         qos_policy_group_name = (
             na_utils.get_qos_policy_group_name_from_info(
                 qos_policy_group_info))
-        qos_policy_group_is_adaptive = volume_utils.is_boolean_str(
-            extra_specs.get('netapp:qos_policy_group_is_adaptive'))
+        qos_policy_group_is_adaptive = (volume_utils.is_boolean_str(
+            extra_specs.get('netapp:qos_policy_group_is_adaptive')) or
+            na_utils.is_qos_policy_group_spec_adaptive(
+                qos_policy_group_info))
 
         try:
             self._clone_lun(
@@ -407,12 +410,11 @@ class NetAppBlockStorageLibrary(object):
     def _extract_lun_info(self, lun):
         """Extracts the LUNs from API and populates the LUN table."""
 
-        meta_dict = self._create_lun_meta(lun)
-        path = lun.get_child_content('path')
+        path = lun['Path']
         (_rest, _splitter, name) = path.rpartition('/')
-        handle = self._create_lun_handle(meta_dict)
-        size = lun.get_child_content('size')
-        return NetAppLun(handle, name, size, meta_dict)
+        handle = self._create_lun_handle(lun)
+        size = lun['Size']
+        return NetAppLun(handle, name, size, lun)
 
     def _extract_and_populate_luns(self, api_luns):
         """Extracts the LUNs from API and populates the LUN table."""
@@ -476,16 +478,17 @@ class NetAppBlockStorageLibrary(object):
         Creates igroup if not already present with given host os type,
         igroup type and adds initiators.
         """
+        # Backend supports different igroups with the same initiators, so
+        # instead of reusing non OpenStack igroups, we make sure we only use
+        # our own, thus being compatible with custom igroups.
         igroups = self.zapi_client.get_igroup_by_initiators(initiator_list)
-        igroup_name = None
-
-        if igroups:
-            igroup = igroups[0]
+        for igroup in igroups:
             igroup_name = igroup['initiator-group-name']
-            host_os_type = igroup['initiator-group-os-type']
-            initiator_group_type = igroup['initiator-group-type']
-
-        if not igroup_name:
+            if igroup_name.startswith(na_utils.OPENSTACK_PREFIX):
+                host_os_type = igroup['initiator-group-os-type']
+                initiator_group_type = igroup['initiator-group-type']
+                break
+        else:
             igroup_name = self._create_igroup_add_initiators(
                 initiator_group_type, host_os_type, initiator_list)
         return igroup_name, host_os_type, initiator_group_type
@@ -500,6 +503,11 @@ class NetAppBlockStorageLibrary(object):
             self.zapi_client.add_igroup_initiator(igroup_name, initiator)
         return igroup_name
 
+    def _delete_lun_from_table(self, name):
+        """Deletes LUN from cache table."""
+        if self.lun_table.get(name, None):
+            self.lun_table.pop(name)
+
     def _add_lun_to_table(self, lun):
         """Adds LUN to cache table."""
         if not isinstance(lun, NetAppLun):
@@ -538,9 +546,6 @@ class NetAppBlockStorageLibrary(object):
             LOG.error("Error getting LUN attribute. Exception: %s", e)
         return None
 
-    def _create_lun_meta(self, lun):
-        raise NotImplementedError()
-
     def _get_fc_target_wwpns(self, include_partner=True):
         raise NotImplementedError()
 
@@ -606,7 +611,7 @@ class NetAppBlockStorageLibrary(object):
 
         ontap_version = self.zapi_client.get_ontap_version(cached=True)
 
-        if ontap_version >= '9.5':
+        if ontap_version >= (9, 5, 0):
             self.zapi_client.do_direct_resize(path, new_size_bytes)
         else:
             lun_geometry = self.zapi_client.get_lun_geometry(path)
@@ -716,8 +721,8 @@ class NetAppBlockStorageLibrary(object):
             msg = _('Failure getting LUN info for %s.')
             raise exception.VolumeBackendAPIException(data=msg % seg[-1])
         lun_info = lun_infos[-1]
-        bs = int(lun_info.get_child_content('block-size'))
-        ls = int(lun_info.get_child_content('size'))
+        bs = int(lun_info['BlockSize'])
+        ls = int(lun_info['Size'])
         block_count = ls / bs
         return block_count
 
@@ -742,6 +747,8 @@ class NetAppBlockStorageLibrary(object):
         qos_policy_group_name = (
             na_utils.get_qos_policy_group_name_from_info(
                 qos_policy_group_info))
+        is_adaptive = na_utils.is_qos_policy_group_spec_adaptive(
+            qos_policy_group_info)
 
         path = lun.get_metadata_property('Path')
         if lun.name == volume['name']:
@@ -757,7 +764,8 @@ class NetAppBlockStorageLibrary(object):
 
         if qos_policy_group_name is not None:
             self.zapi_client.set_lun_qos_policy_group(new_path,
-                                                      qos_policy_group_name)
+                                                      qos_policy_group_name,
+                                                      is_adaptive)
         self._add_lun_to_table(lun)
         LOG.info("Manage operation completed for LUN with new path"
                  " %(path)s and uuid %(uuid)s.",
