@@ -25,15 +25,17 @@ import os
 import uuid
 
 from oslo_log import log as logging
+from oslo_service import loopingcall
 from oslo_utils import excutils
+from oslo_utils import units
 import six
 
+from cinder.common import constants
 from cinder import exception
 from cinder.i18n import _
 from cinder.image import image_utils
 from cinder import interface
 from cinder.objects import fields
-from cinder import utils
 from cinder.volume.drivers.netapp.dataontap import nfs_base
 from cinder.volume.drivers.netapp.dataontap.performance import perf_cmode
 from cinder.volume.drivers.netapp.dataontap.utils import capabilities
@@ -49,10 +51,25 @@ LOG = logging.getLogger(__name__)
 
 
 @interface.volumedriver
-@six.add_metaclass(utils.TraceWrapperWithABCMetaclass)
+@six.add_metaclass(volume_utils.TraceWrapperWithABCMetaclass)
 class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                            data_motion.DataMotionMixin):
-    """NetApp NFS driver for Data ONTAP (Cluster-mode)."""
+    """NetApp NFS driver for Data ONTAP (Cluster-mode).
+
+    Version history:
+
+    .. code-block:: none
+
+        1.0.0 - Driver development before Wallaby
+        2.0.0 - Add support for QoS minimums specs
+                Add support for dynamic Adaptive QoS policy group creation
+                Implement FlexGroup pool
+        3.0.0 - Add support for Intra-cluster Storage assisted volume migration
+                Add support for revert to snapshot
+
+    """
+
+    VERSION = "3.0.0"
 
     REQUIRED_CMODE_FLAGS = ['netapp_vserver']
 
@@ -102,6 +119,12 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
     def check_for_setup_error(self):
         """Check that the driver is working and can communicate."""
         self._add_looping_tasks()
+
+        if (self.ssc_library.contains_flexgroup_pool() and
+                not self.zapi_client.features.FLEXGROUP):
+            msg = _('FlexGroup pool requires Data ONTAP 9.8 or later.')
+            raise na_utils.NetAppDriverException(msg)
+
         super(NetAppCmodeNfsDriver, self).check_for_setup_error()
 
     def _add_looping_tasks(self):
@@ -161,12 +184,17 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 self.ssc_library.get_ssc_flexvol_names())
 
     def _do_qos_for_volume(self, volume, extra_specs, cleanup=True):
-        qos_policy_group_is_adaptive = volume_utils.is_boolean_str(
-            extra_specs.get('netapp:qos_policy_group_is_adaptive'))
         try:
             qos_policy_group_info = na_utils.get_valid_qos_policy_group_info(
                 volume, extra_specs)
-            self.zapi_client.provision_qos_policy_group(qos_policy_group_info)
+            pool = volume_utils.extract_host(volume['host'], level='pool')
+            qos_min_support = self.ssc_library.is_qos_min_supported(pool)
+            qos_policy_group_is_adaptive = (volume_utils.is_boolean_str(
+                extra_specs.get('netapp:qos_policy_group_is_adaptive')) or
+                na_utils.is_qos_policy_group_spec_adaptive(
+                    qos_policy_group_info))
+            self.zapi_client.provision_qos_policy_group(qos_policy_group_info,
+                                                        qos_min_support)
             self._set_qos_policy_group_on_volume(volume, qos_policy_group_info,
                                                  qos_policy_group_is_adaptive)
         except Exception:
@@ -199,6 +227,80 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                                          qos_policy_group_is_adaptive,
                                          target_path)
 
+    def _revert_to_snapshot(self, volume, snapshot):
+        """Clone volume from snapshot to perform the file name swap."""
+        new_snap_name = 'new-%s' % snapshot['name']
+        self._clone_backing_file_for_volume(snapshot['name'],
+                                            new_snap_name,
+                                            snapshot['volume_id'],
+                                            is_snapshot=False)
+
+        (host_ip, junction_path) = self._get_export_ip_path(
+            volume_id=volume['id'])
+        vserver = self._get_vserver_for_ip(host_ip)
+        flexvol_name = self.zapi_client.get_vol_by_junc_vserver(vserver,
+                                                                junction_path)
+
+        try:
+            self._swap_files(flexvol_name, volume['name'], new_snap_name)
+        except Exception:
+            LOG.error("Swapping temporary reverted volume name from %s to %s "
+                      "failed.", new_snap_name, volume['name'])
+            with excutils.save_and_reraise_exception():
+                try:
+                    LOG.debug("Deleting temporary reverted volume file %s.",
+                              new_snap_name)
+                    file_path = '/vol/%s/%s' % (flexvol_name, new_snap_name)
+                    self.zapi_client.delete_file(file_path)
+                except Exception:
+                    LOG.error("Could not delete temporary reverted volume %s. "
+                              "A manual deletion is required.", new_snap_name)
+
+    def _swap_files(self, flexvol_name, original_file, new_file):
+        """Swaps cloned and original files using a temporary file.
+
+        Renames the original file path to a temporary path, then changes the
+        cloned file path to the original path (if this fails, change the
+        temporary file path back as original path) and finally deletes the
+        file with temporary path.
+        """
+        prefix_path_on_backend = '/vol/' + flexvol_name + '/'
+
+        new_file_path = prefix_path_on_backend + new_file
+        original_file_path = prefix_path_on_backend + original_file
+        tmp_file_path = prefix_path_on_backend + 'tmp-%s' % original_file
+
+        try:
+            self.zapi_client.rename_file(original_file_path, tmp_file_path)
+        except exception.VolumeBackendAPIException:
+            msg = _("Could not rename original volume from %s to %s.")
+            raise na_utils.NetAppDriverException(msg % (original_file_path,
+                                                        tmp_file_path))
+
+        try:
+            self.zapi_client.rename_file(new_file_path, original_file_path)
+        except exception.VolumeBackendAPIException:
+            try:
+                LOG.debug("Revert volume failed. Rolling back to its original"
+                          " name.")
+                self.zapi_client.rename_file(tmp_file_path, original_file_path)
+            except exception.VolumeBackendAPIException:
+                LOG.error("Could not rollback original volume name from %s "
+                          "to %s. Cinder may lose the volume management. "
+                          "Please, you should rename it back manually.",
+                          tmp_file_path, original_file_path)
+
+            msg = _("Could not rename temporary reverted volume from %s "
+                    "to original volume name %s.")
+            raise na_utils.NetAppDriverException(msg % (new_file_path,
+                                                        original_file_path))
+
+        try:
+            self.zapi_client.delete_file(tmp_file_path)
+        except exception.VolumeBackendAPIException:
+            LOG.error("Could not delete old volume %s. A manual deletion is "
+                      "required.", tmp_file_path)
+
     def _clone_backing_file_for_volume(self, volume_name, clone_name,
                                        volume_id, share=None,
                                        is_snapshot=False,
@@ -212,7 +314,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         """Gets the vserver and export volume for share."""
         (host_ip, export_path) = self._get_export_ip_path(volume_id, share)
         ifs = self.zapi_client.get_if_info_by_ip(host_ip)
-        vserver = ifs[0].get_child_content('vserver')
+        vserver = ifs[0].get('vserver')
         exp_volume = self.zapi_client.get_vol_by_junc_vserver(vserver,
                                                               export_path)
         return vserver, exp_volume
@@ -226,7 +328,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         data['volume_backend_name'] = backend_name or self.driver_name
         data['vendor_name'] = 'NetApp'
         data['driver_version'] = self.VERSION
-        data['storage_protocol'] = 'nfs'
+        data['storage_protocol'] = constants.NFS_VARIANT
         data['pools'] = self._get_pool_stats(
             filter_function=self.get_filter_function(),
             goodness_function=self.get_goodness_function())
@@ -235,7 +337,6 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         # Used for service state report
         data['replication_enabled'] = self.replication_enabled
 
-        self._spawn_clean_cache_job()
         self._stats = data
 
     def _get_pool_stats(self, filter_function=None, goodness_function=None):
@@ -280,12 +381,26 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             pool['multiattach'] = True
             pool['online_extend_support'] = False
 
+            is_flexgroup = ssc_vol_info.get('netapp_is_flexgroup') == 'true'
+            if is_flexgroup:
+                pool['consistencygroup_support'] = False
+                pool['consistent_group_snapshot_enabled'] = False
+                pool['multiattach'] = False
+
             # Add up-to-date capacity info
             nfs_share = ssc_vol_info['pool_name']
             capacity = self._get_share_capacity_info(nfs_share)
             pool.update(capacity)
+            if self.configuration.netapp_driver_reports_provisioned_capacity:
+                files = self.zapi_client.get_file_sizes_by_dir(ssc_vol_name)
+                provisioned_cap = 0
+                for f in files:
+                    if volume_utils.extract_id_from_volume_name(f['name']):
+                        provisioned_cap = provisioned_cap + f['file-size']
+                pool['provisioned_capacity_gb'] = na_utils.round_down(
+                    float(provisioned_cap) / units.Gi)
 
-            if self.using_cluster_credentials:
+            if self.using_cluster_credentials and not is_flexgroup:
                 dedupe_used = self.zapi_client.get_flexvol_dedupe_used_percent(
                     ssc_vol_name)
             else:
@@ -294,9 +409,24 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 dedupe_used)
 
             aggregate_name = ssc_vol_info.get('netapp_aggregate')
-            aggr_capacity = aggr_capacities.get(aggregate_name, {})
-            pool['netapp_aggregate_used_percent'] = aggr_capacity.get(
-                'percent-used', 0)
+            aggr_used = 0
+            if isinstance(aggregate_name, list):
+                # For FlexGroup, the aggregate percentage can be seen as the
+                # average of all aggregates.
+                aggr_used_total = 0
+                aggr_num = 0
+                for aggr in aggregate_name:
+                    aggr_capacity = aggr_capacities.get(aggr, {})
+                    aggr_used_total += aggr_capacity.get('percent-used', 0)
+                    aggr_num += 1
+
+                if aggr_num:
+                    aggr_used = aggr_used_total / aggr_num
+            else:
+                aggr_capacity = aggr_capacities.get(aggregate_name, {})
+                aggr_used = aggr_capacity.get('percent-used', 0)
+
+            pool['netapp_aggregate_used_percent'] = aggr_used
 
             # Add utilization data
             utilization = self.perf_library.get_node_utilization_for_pool(
@@ -381,7 +511,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         """Get vserver for the mentioned ip."""
         try:
             ifs = self.zapi_client.get_if_info_by_ip(ip)
-            vserver = ifs[0].get_child_content('vserver')
+            vserver = ifs[0].get('vserver')
             return vserver
         except Exception:
             return None
@@ -414,8 +544,10 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         try:
             qos_policy_group_info = na_utils.get_valid_qos_policy_group_info(
                 volume)
-            self.zapi_client.mark_qos_policy_group_for_deletion(
+            is_adaptive = na_utils.is_qos_policy_group_spec_adaptive(
                 qos_policy_group_info)
+            self.zapi_client.mark_qos_policy_group_for_deletion(
+                qos_policy_group_info, is_adaptive)
         except Exception:
             # Don't blow up here if something went wrong de-provisioning the
             # QoS policy for the volume.
@@ -423,18 +555,28 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
     def _delete_backing_file_for_volume(self, volume):
         """Deletes file on nfs share that backs a cinder volume."""
+        is_flexgroup = self._is_flexgroup(host=volume['host'])
         try:
             LOG.debug('Deleting backing file for volume %s.', volume['id'])
-            self._delete_file(volume['id'], volume['name'])
-        except Exception:
-            LOG.exception('Could not delete volume %s on backend, '
-                          'falling back to exec of "rm" command.',
-                          volume['id'])
-            try:
+            if (is_flexgroup and
+                    not self._is_flexgroup_clone_file_supported()):
                 super(NetAppCmodeNfsDriver, self).delete_volume(volume)
-            except Exception:
+            else:
+                self._delete_file(volume['id'], volume['name'])
+        except Exception:
+            if (is_flexgroup and
+                    not self._is_flexgroup_clone_file_supported()):
                 LOG.exception('Exec of "rm" command on backing file for '
                               '%s was unsuccessful.', volume['id'])
+            else:
+                LOG.exception('Could not delete volume %s on backend, '
+                              'falling back to exec of "rm" command.',
+                              volume['id'])
+                try:
+                    super(NetAppCmodeNfsDriver, self).delete_volume(volume)
+                except Exception:
+                    LOG.exception('Exec of "rm" command on backing file for '
+                                  '%s was unsuccessful.', volume['id'])
 
     def _delete_file(self, file_id, file_name):
         (host_ip, junction_path) = self._get_export_ip_path(volume_id=file_id)
@@ -448,7 +590,11 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
 
     def delete_snapshot(self, snapshot):
         """Deletes a snapshot."""
-        self._delete_backing_file_for_snapshot(snapshot)
+        if (self._is_flexgroup(snapshot['volume_id']) and
+                not self._is_flexgroup_clone_file_supported()):
+            super(NetAppCmodeNfsDriver, self).delete_snapshot(snapshot)
+        else:
+            self._delete_backing_file_for_snapshot(snapshot)
 
     def _delete_backing_file_for_snapshot(self, snapshot):
         """Deletes file on nfs share that backs a cinder volume."""
@@ -473,7 +619,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         if not vserver:
             raise exception.NotFound(_("Unable to locate an SVM that is "
                                        "managing the IP address '%s'") % ip)
-        return ip
+        return ip, vserver
 
     def _copy_from_cache(self, volume, image_id, cache_result):
         """Try copying image file_name from cached file_name."""
@@ -495,6 +641,11 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 LOG.debug("Trying copy from cache using copy offload.")
                 self._copy_from_remote_cache(volume, image_id, cache_copy)
                 copied = True
+            elif cache_copy:
+                LOG.debug("Trying copy from cache using file copy.")
+                self._copy_from_remote_cache(volume, image_id, cache_copy,
+                                             use_copyoffload_tool=False)
+                copied = True
         except Exception:
             LOG.exception('Error in workflow copy from cache.')
         return copied
@@ -523,41 +674,55 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 cache_copy = res
         return cache_copy, found_local_copy
 
-    def _copy_from_remote_cache(self, volume, image_id, cache_copy):
+    def _copy_from_remote_cache(self, volume, image_id, cache_copy,
+                                use_copyoffload_tool=True):
         """Copies the remote cached image to the provided volume.
 
-        Executes the copy offload binary which copies the cached image to
-        the destination path of the provided volume. Also registers the new
-        copy of the image as a cached image.
+        Executes either the copy offload binary or the file copy operation,
+        copying the cached image to the destination path of the provided
+        volume. Also registers the new copy of the image as a cached image.
         """
 
         (nfs_share, file_name) = cache_copy
-        col_path = self.configuration.netapp_copyoffload_tool_path
-        src_ip, src_path = self._get_source_ip_and_path(nfs_share, file_name)
-        dest_ip, dest_path = self._get_destination_ip_and_path(volume)
+        (src_ip, src_vserver, src_share_path, src_path) = (
+            self._get_source_ip_and_path(nfs_share, file_name))
+        (dest_ip, dest_vserver, dest_path) = (
+            self._get_destination_ip_and_path(volume))
 
-        # Always run copy offload as regular user, it's sufficient
-        # and rootwrap doesn't allow copy offload to run as root anyways.
-        self._execute(col_path, src_ip, dest_ip, src_path, dest_path,
-                      run_as_root=False, check_exit_code=0)
+        # NOTE(felipe_rodrigues): the copy offload tool code will be removed in
+        # the Antelope release.
+        col_path = self.configuration.netapp_copyoffload_tool_path
+        if use_copyoffload_tool and col_path:
+            # Always run copy offload as regular user, it's sufficient
+            # and rootwrap doesn't allow copy offload to run as root anyways.
+            self._execute(col_path, src_ip, dest_ip, src_path, dest_path,
+                          run_as_root=False, check_exit_code=0)
+            LOG.debug("Copied image from cache to volume %s using "
+                      "copy offload.", volume['id'])
+        else:
+            dest_share_path = dest_path.rsplit("/", 1)[0]
+            self._copy_file(file_name, file_name, src_share_path, src_vserver,
+                            dest_share_path, dest_vserver,
+                            dest_backend_name=self.backend_name,
+                            dest_file_name=volume.name)
+            LOG.debug("Copied image from cache to volume %s using "
+                      "file copy operation.", volume['id'])
         self._register_image_in_cache(volume, image_id)
-        LOG.debug("Copied image from cache to volume %s using copy offload.",
-                  volume['id'])
 
     def _get_source_ip_and_path(self, nfs_share, file_name):
         host, share_path = na_utils.get_export_host_junction_path(nfs_share)
-        src_ip = self._get_ip_verify_on_cluster(host)
+        (src_ip, src_vserver) = self._get_ip_verify_on_cluster(host)
         src_path = os.path.join(share_path, file_name)
 
-        return src_ip, src_path
+        return src_ip, src_vserver, share_path, src_path
 
     def _get_destination_ip_and_path(self, volume):
         share = volume_utils.extract_host(volume['host'], level='pool')
         share_ip, share_path = na_utils.get_export_host_junction_path(share)
-        dest_ip = self._get_ip_verify_on_cluster(share_ip)
+        (dest_ip, vserver) = self._get_ip_verify_on_cluster(share_ip)
         dest_path = os.path.join(share_path, volume['name'])
 
-        return dest_ip, dest_path
+        return dest_ip, vserver, dest_path
 
     def _clone_file_dst_exists(self, share, src_name, dst_name,
                                dest_exists=False):
@@ -567,38 +732,39 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                                     dest_exists=dest_exists)
 
     def _copy_from_img_service(self, context, volume, image_service,
-                               image_id):
-        """Copies from the image service using copy offload."""
+                               image_id, use_copyoffload_tool=True):
+        """Copies from the image service using copy offload or file copy."""
 
-        LOG.debug("Trying copy from image service using copy offload.")
         image_loc = image_service.get_location(context, image_id)
         locations = self._construct_image_nfs_url(image_loc)
         src_ip = None
+        src_vserver = None
+        src_volume = None
         selected_loc = None
         cloned = False
 
         # this will match the first location that has a valid IP on cluster
         for location in locations:
-            conn, dr = self._check_get_nfs_path_segs(location)
+            conn, src_volume = self._check_get_nfs_path_segs(location)
             if conn:
                 try:
-                    src_ip = self._get_ip_verify_on_cluster(conn.split(':')[0])
+                    (src_ip, src_vserver) = (
+                        self._get_ip_verify_on_cluster(conn.split(':')[0]))
                     selected_loc = location
                     break
                 except exception.NotFound:
                     pass
-        if src_ip is None:
+        if src_ip is None or src_vserver is None:
             raise exception.NotFound(_("Source host details not found."))
         (__, ___, img_file) = selected_loc.rpartition('/')
-        src_path = os.path.join(dr, img_file)
 
-        dst_ip, vol_path = self._get_destination_ip_and_path(volume)
-        share_path = vol_path.rsplit("/", 1)[0]
-        dst_share = dst_ip + ':' + share_path
+        (dst_ip, dest_vserver, vol_path) = (
+            self._get_destination_ip_and_path(volume))
+        dest_share_path = vol_path.rsplit("/", 1)[0]
+        dst_share = dst_ip + ':' + dest_share_path
 
         # tmp file is required to deal with img formats
         tmp_img_file = six.text_type(uuid.uuid4())
-        col_path = self.configuration.netapp_copyoffload_tool_path
         img_info = image_service.show(context, image_id)
         self._check_share_can_hold_size(dst_share, img_info['size'])
         run_as_root = self._execute_as_root
@@ -607,14 +773,27 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         dst_img_local = os.path.join(dst_dir, tmp_img_file)
 
         try:
-            dst_img_serv_path = os.path.join(
-                share_path, tmp_img_file)
-            # Always run copy offload as regular user, it's sufficient
-            # and rootwrap doesn't allow copy offload to run as root
-            # anyways.
-            self._execute(col_path, src_ip, dst_ip, src_path,
-                          dst_img_serv_path, run_as_root=False,
-                          check_exit_code=0)
+            # NOTE(felipe_rodrigues): the copy offload tool code will be
+            # removed in the AA release.
+            col_path = self.configuration.netapp_copyoffload_tool_path
+            if col_path and use_copyoffload_tool:
+                LOG.debug("Trying copy from image service using copy offload.")
+                dst_img_serv_path = os.path.join(dest_share_path, tmp_img_file)
+                src_path = os.path.join(src_volume, img_file)
+                # Always run copy offload as regular user, it's sufficient
+                # and rootwrap doesn't allow copy offload to run as root
+                # anyways.
+                self._execute(col_path, src_ip, dst_ip, src_path,
+                              dst_img_serv_path, run_as_root=False,
+                              check_exit_code=0)
+            else:
+                LOG.debug("Trying copy from image service using file copy.")
+                src_volume = ''.join(src_volume.split("/", 1))
+                dest_share_path = ''.join(dest_share_path.split("/", 1))
+                self._copy_file(img_file, img_file, src_volume, src_vserver,
+                                dest_share_path, dest_vserver,
+                                dest_backend_name=self.backend_name,
+                                dest_file_name=tmp_img_file)
 
             self._discover_file_till_timeout(dst_img_local, timeout=120)
             LOG.debug('Copied image %(img)s to tmp file %(tmp)s.',
@@ -732,6 +911,11 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         :returns: Hard-coded model update for generic volume group model.
         """
         model_update = {'status': fields.GroupStatus.AVAILABLE}
+        if (self._is_flexgroup(host=group['host']) and
+                volume_utils.is_group_a_cg_snapshot_type(group)):
+            msg = _("Cannot create %s consistency group on FlexGroup pool.")
+            raise na_utils.NetAppDriverException(msg % group['id'])
+
         return model_update
 
     def delete_group(self, context, group, volumes):
@@ -744,7 +928,7 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         volumes_model_update = []
         for volume in volumes:
             try:
-                self._delete_file(volume['id'], volume['name'])
+                self.delete_volume(volume)
                 volumes_model_update.append(
                     {'id': volume['id'], 'status': 'deleted'})
             except Exception:
@@ -763,6 +947,12 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         necessary to update any metadata on the backend. Since this is a NO-OP,
         there is guaranteed to be no change in any of the volumes' statuses.
         """
+        if volume_utils.is_group_a_cg_snapshot_type(group):
+            for vol in add_volumes:
+                if self._is_flexgroup(host=vol['host']):
+                    msg = _("Cannot add volume from FlexGroup pool to "
+                            "consistency group.")
+                    raise na_utils.NetAppDriverException(msg)
 
         return None, None, None
 
@@ -787,13 +977,20 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
         """
         try:
             if volume_utils.is_group_a_cg_snapshot_type(group_snapshot):
+                # NOTE(felipe_rodrigues): ONTAP FlexGroup does not support
+                # consistency group snapshot, so all members must be inside
+                # a FlexVol pool.
+                for snapshot in snapshots:
+                    if self._is_flexgroup(host=snapshot['volume']['host']):
+                        msg = _("Cannot create consistency group snapshot with"
+                                " volumes on a FlexGroup pool.")
+                        raise na_utils.NetAppDriverException(msg)
+
                 self._create_consistent_group_snapshot(group_snapshot,
                                                        snapshots)
             else:
                 for snapshot in snapshots:
-                    self._clone_backing_file_for_volume(
-                        snapshot['volume_name'], snapshot['name'],
-                        snapshot['volume_id'], is_snapshot=True)
+                    self.create_snapshot(snapshot)
         except Exception as ex:
             err_msg = (_("Create group snapshot failed (%s).") % ex)
             LOG.exception(err_msg, resource=group_snapshot)
@@ -847,7 +1044,22 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
                 volumes_model_update.append(update)
 
         elif source_group and sorted_source_vols:
-            hosts = [source_vol['host'] for source_vol in sorted_source_vols]
+            hosts = []
+            for source_vol in sorted_source_vols:
+                # NOTE(felipe_rodrigues): ONTAP FlexGroup does not support
+                # consistency group snapshot, so if any source volume is on a
+                # FlexGroup, the operation must be create from a not-cg,
+                # falling back to the generic group support.
+                if self._is_flexgroup(host=source_vol['host']):
+                    if volume_utils.is_group_a_cg_snapshot_type(group):
+                        msg = _("Cannot create consistency group with volume "
+                                "on a FlexGroup pool.")
+                        raise na_utils.NetAppDriverException(msg)
+                    else:
+                        # falls back to generic support
+                        raise NotImplementedError()
+                hosts.append(source_vol['host'])
+
             flexvols = self._get_flexvol_names_from_hosts(hosts)
 
             # Create snapshot for backing flexvol
@@ -879,3 +1091,162 @@ class NetAppCmodeNfsDriver(nfs_base.NetAppNfsDriver,
             model_update = {'status': fields.GroupStatus.ERROR}
 
         return model_update, volumes_model_update
+
+    def _is_flexgroup(self, vol_id=None, host=None):
+        """Discover if a volume is a FlexGroup or not"""
+        if host is None:
+            host = self._get_volume_host(vol_id)
+
+        pool_name = volume_utils.extract_host(host, level='pool')
+        return self.ssc_library.is_flexgroup(pool_name)
+
+    def _is_flexgroup_clone_file_supported(self):
+        """Check whether storage can perform clone file for FlexGroup"""
+        return self.zapi_client.features.FLEXGROUP_CLONE_FILE
+
+    def _cancel_file_copy(self, job_uuid, file_name, dest_pool,
+                          dest_backend_name=None):
+        """Cancel an on-going file copy operation."""
+        try:
+            # NOTE(sfernand): Another approach would be first checking if
+            # the copy operation isn't in `destroying` or `destroyed` states
+            # before issuing cancel.
+            self.zapi_client.destroy_file_copy(job_uuid)
+        except na_utils.NetAppDriverException:
+            dest_client = dot_utils.get_client_for_backend(dest_backend_name)
+            file_path = '%s/%s' % (dest_pool, file_name)
+            try:
+                dest_client.delete_file(file_path)
+            except Exception:
+                LOG.warning('Error cleaning up file %s in destination volume. '
+                            'Verify if destination volume still exists in '
+                            'pool %s and delete it manually to avoid unused '
+                            'resources.', file_path, dest_pool)
+
+    def _copy_file(self, file_name, volume_id, src_ontap_volume, src_vserver,
+                   dest_ontap_volume, dest_vserver, dest_file_name=None,
+                   dest_backend_name=None, cancel_on_error=False):
+        """Copies file from an ONTAP volume to another."""
+        job_uuid = self.zapi_client.start_file_copy(
+            file_name, dest_ontap_volume, src_ontap_volume=src_ontap_volume,
+            dest_file_name=dest_file_name)
+        LOG.debug('Start copying file %(file)s from '
+                  '%(src_vserver)s:%(src_ontap_vol)s to '
+                  '%(dest_vserver)s:%(dest_ontap_vol)s. Job UUID is %(job)s.',
+                  {'file': file_name, 'src_vserver': src_vserver,
+                   'src_ontap_vol': src_ontap_volume,
+                   'dest_vserver': dest_vserver,
+                   'dest_ontap_vol': dest_ontap_volume,
+                   'job': job_uuid})
+
+        def _wait_file_copy_complete():
+            copy_status = self.zapi_client.get_file_copy_status(job_uuid)
+            LOG.debug('Waiting for file copy job %s to complete. Current '
+                      'status is: %s.', job_uuid, copy_status['job-status'])
+            if not copy_status:
+                status_error_msg = (_("Error copying file %s. The "
+                                      "corresponding Job UUID % doesn't "
+                                      "exist."))
+                raise na_utils.NetAppDriverException(
+                    status_error_msg % (file_name, job_uuid))
+            elif copy_status['job-status'] == 'destroyed':
+                status_error_msg = (_('Error copying file %s. %s.'))
+                raise na_utils.NetAppDriverException(
+                    status_error_msg % (file_name,
+                                        copy_status['last-failure-reason']))
+            elif copy_status['job-status'] == 'complete':
+                raise loopingcall.LoopingCallDone()
+
+        try:
+            timer = loopingcall.FixedIntervalWithTimeoutLoopingCall(
+                _wait_file_copy_complete)
+            timer.start(
+                interval=10,
+                timeout=self.configuration.netapp_migrate_volume_timeout
+            ).wait()
+        except Exception as e:
+            with excutils.save_and_reraise_exception() as ctxt:
+                if cancel_on_error:
+                    try:
+                        self._cancel_file_copy(
+                            job_uuid, file_name, dest_ontap_volume,
+                            dest_backend_name=dest_backend_name)
+                    except na_utils.NetAppDriverException as ex:
+                        LOG.error("Failed to cancel file copy operation. %s",
+                                  ex)
+                if isinstance(e, loopingcall.LoopingCallTimeOut):
+                    ctxt.reraise = False
+                    msg = (_('Timeout waiting volume %s to complete '
+                             'migration.'))
+                    raise na_utils.NetAppDriverTimeout(msg % volume_id)
+
+    def _finish_volume_migration(self, src_volume, dest_pool):
+        """Finish volume migration to another ONTAP volume."""
+        # The source volume can be safely deleted after a successful migration.
+        self.delete_volume(src_volume)
+        # NFS driver requires the provider_location to be updated with the new
+        # destination.
+        updates = {'provider_location': dest_pool}
+        return updates
+
+    def _migrate_volume_to_vserver(self, volume, src_pool, src_vserver,
+                                   dest_pool, dest_vserver, dest_backend_name):
+        """Migrate volume to another vserver within the same cluster."""
+        LOG.info('Migrating volume %(vol)s from '
+                 '%(src_vserver)s:%(src_ontap_vol)s to '
+                 '%(dest_vserver)s:%(dest_ontap_vol)s.',
+                 {'vol': volume.id, 'src_vserver': src_vserver,
+                  'src_ontap_vol': src_pool, 'dest_vserver': dest_vserver,
+                  'dest_ontap_vol': dest_pool})
+        vserver_peer_application = 'file_copy'
+        self.create_vserver_peer(src_vserver, self.backend_name, dest_vserver,
+                                 [vserver_peer_application])
+        src_ontap_volume_name = src_pool.split(':/')[1]
+        dest_ontap_volume_name = dest_pool.split(':/')[1]
+        self._copy_file(volume.name, volume.id, src_ontap_volume_name,
+                        src_vserver, dest_ontap_volume_name, dest_vserver,
+                        dest_backend_name=dest_backend_name,
+                        cancel_on_error=True)
+        updates = self._finish_volume_migration(volume, dest_pool)
+        LOG.info('Successfully migrated volume %(vol)s from '
+                 '%(src_vserver)s:%(src_ontap_vol)s '
+                 'to %(dest_vserver)s:%(dest_ontap_vol)s.',
+                 {'vol': volume.id, 'src_vserver': src_vserver,
+                  'src_ontap_vol': src_pool, 'dest_vserver': dest_vserver,
+                  'dest_ontap_vol': dest_pool})
+        return updates
+
+    def _migrate_volume_to_pool(self, volume, src_pool, dest_pool, vserver,
+                                dest_backend_name):
+        """Migrate volume to another Cinder Pool within the same vserver."""
+        LOG.info('Migrating volume %(vol)s from pool %(src)s to '
+                 '%(dest)s within vserver %(vserver)s.',
+                 {'vol': volume.id, 'src': src_pool, 'dest': dest_pool,
+                  'vserver': vserver})
+        src_ontap_volume_name = src_pool.split(':/')[1]
+        dest_ontap_volume_name = dest_pool.split(':/')[1]
+        self._copy_file(volume.name, volume.id, src_ontap_volume_name, vserver,
+                        dest_ontap_volume_name, vserver,
+                        dest_backend_name=dest_backend_name,
+                        cancel_on_error=True)
+        updates = self._finish_volume_migration(volume, dest_pool)
+        LOG.info('Successfully migrated volume %(vol)s from pool %(src)s '
+                 'to %(dest)s within vserver %(vserver)s.',
+                 {'vol': volume.id, 'src': src_pool, 'dest': dest_pool,
+                  'vserver': vserver})
+        return updates
+
+    def migrate_volume(self, context, volume, host):
+        """Migrate Cinder volume to the specified pool or vserver."""
+        # NOTE(sfernand): the NetApp NFS driver relies only on coping
+        # operations for storage assisted migration which are always
+        # disruptive, as requires the destination volume to be added as a new
+        # block device to be the Nova instance.
+        if volume.status != fields.VolumeStatus.AVAILABLE:
+            LOG.info("Storage assisted migration requires volume to be in "
+                     "available status. Falling back to host assisted "
+                     "migration.")
+            return False, {}
+
+        return self.migrate_volume_ontap_assisted(
+            volume, host, self.backend_name, self.configuration.netapp_vserver)
