@@ -22,34 +22,9 @@ import re
 from oslo_log import log as logging
 import six
 
-from cinder import exception
-from cinder.i18n import _
-
 
 LOG = logging.getLogger(__name__)
 
-# NOTE(cknight): The keys in this map are tuples that contain arguments needed
-# for efficient use of the system-user-capability-get-iter cDOT API.  The
-# values are SSC extra specs associated with the APIs listed in the keys.
-SSC_API_MAP = {
-    ('storage.aggregate', 'show', 'aggr-options-list-info'): [
-        'netapp_raid_type',
-    ],
-    ('storage.disk', 'show', 'storage-disk-get-iter'): [
-        'netapp_disk_type',
-    ],
-    ('snapmirror', 'show', 'snapmirror-get-iter'): [
-        'netapp_mirrored',
-    ],
-    ('volume.efficiency', 'show', 'sis-get-iter'): [
-        'netapp_dedup',
-        'netapp_compression',
-    ],
-    ('volume', '*show', 'volume-get-iter'): [
-        'netapp_flexvol_encryption',
-    ],
-}
-
 
 class CapabilitiesLibrary(object):
 
@@ -64,30 +39,7 @@ class CapabilitiesLibrary(object):
         self.invalid_extra_specs = []
 
     def check_api_permissions(self):
-        """Check which APIs that support SSC functionality are available."""
-
-        inaccessible_apis = []
-        invalid_extra_specs = []
-
-        for api_tuple, extra_specs in SSC_API_MAP.items():
-            object_name, operation_name, api = api_tuple
-            if not self.zapi_client.check_cluster_api(object_name,
-                                                      operation_name,
-                                                      api):
-                inaccessible_apis.append(api)
-                invalid_extra_specs.extend(extra_specs)
-
-        if inaccessible_apis:
-            if 'volume-get-iter' in inaccessible_apis:
-                msg = _('User not permitted to query Data ONTAP volumes.')
-                raise exception.VolumeBackendAPIException(data=msg)
-            else:
-                LOG.warning('The configured user account does not have '
-                            'sufficient privileges to use all needed '
-                            'APIs. The following extra specs will fail '
-                            'or be ignored: %s.', invalid_extra_specs)
-
-        self.invalid_extra_specs = invalid_extra_specs
+        self.invalid_extra_specs = self.zapi_client.check_api_permissions()
 
     def cluster_user_supported(self):
         return not self.invalid_extra_specs
@@ -114,9 +66,22 @@ class CapabilitiesLibrary(object):
         aggregates = set()
         for __, flexvol_info in self.ssc.items():
             if 'netapp_aggregate' in flexvol_info:
-                aggregates.add(flexvol_info['netapp_aggregate'])
+                aggr = flexvol_info['netapp_aggregate']
+                if isinstance(aggr, list):
+                    aggregates.update(aggr)
+                else:
+                    aggregates.add(aggr)
         return list(aggregates)
 
+    def is_qos_min_supported(self, pool_name):
+        for __, flexvol_info in self.ssc.items():
+            if ('netapp_qos_min_support' in flexvol_info and
+                    'pool_name' in flexvol_info and
+                    flexvol_info['pool_name'] == pool_name):
+                return flexvol_info['netapp_qos_min_support'] == 'true'
+
+        return False
+
     def update_ssc(self, flexvol_map):
         """Periodically runs to update Storage Service Catalog data.
 
@@ -143,7 +108,13 @@ class CapabilitiesLibrary(object):
 
             # Get aggregate info
             aggregate_name = ssc_volume.get('netapp_aggregate')
-            ssc_volume.update(self._get_ssc_aggregate_info(aggregate_name))
+            is_flexgroup = isinstance(aggregate_name, list)
+            aggr_info = self._get_ssc_aggregate_info(
+                aggregate_name, is_flexgroup=is_flexgroup)
+            node_name = aggr_info.pop('netapp_node_name')
+            ssc_volume.update(aggr_info)
+
+            ssc_volume.update(self._get_ssc_qos_min_info(node_name))
 
             ssc[flexvol_name] = ssc_volume
 
@@ -163,12 +134,14 @@ class CapabilitiesLibrary(object):
                         (volume_info.get('space-guarantee') == 'file' or
                          volume_info.get('space-guarantee') == 'volume'))
         thick = self._get_thick_provisioning_support(netapp_thick)
+        is_flexgroup = volume_info.get('style-extended') == 'flexgroup'
 
         return {
             'netapp_thin_provisioned': six.text_type(not netapp_thick).lower(),
             'thick_provisioning_support': thick,
             'thin_provisioning_support': not thick,
             'netapp_aggregate': volume_info.get('aggregate'),
+            'netapp_is_flexgroup': six.text_type(is_flexgroup).lower(),
         }
 
     def _get_thick_provisioning_support(self, netapp_thick):
@@ -212,6 +185,24 @@ class CapabilitiesLibrary(object):
 
         return {'netapp_flexvol_encryption': six.text_type(encrypted).lower()}
 
+    def _get_ssc_qos_min_info(self, node_name):
+        """Gather Qos minimum info and recast into SSC-style stats."""
+        supported = True
+        is_nfs = self.protocol == 'nfs'
+        if isinstance(node_name, list):
+            # NOTE(felipe_rodrigues): it cannot choose which node the volume
+            # is created, so the pool must have all nodes as QoS min supported
+            # for enabling this feature.
+            for n_name in node_name:
+                if not self.zapi_client.is_qos_min_supported(is_nfs, n_name):
+                    supported = False
+                    break
+        else:
+            supported = self.zapi_client.is_qos_min_supported(is_nfs,
+                                                              node_name)
+
+        return {'netapp_qos_min_support': six.text_type(supported).lower()}
+
     def _get_ssc_mirror_info(self, flexvol_name):
         """Gather SnapMirror info and recast into SSC-style volume stats."""
 
@@ -220,15 +211,40 @@ class CapabilitiesLibrary(object):
 
         return {'netapp_mirrored': six.text_type(mirrored).lower()}
 
-    def _get_ssc_aggregate_info(self, aggregate_name):
-        """Gather aggregate info and recast into SSC-style volume stats."""
+    def _get_ssc_aggregate_info(self, aggregate_name, is_flexgroup=False):
+        """Gather aggregate info and recast into SSC-style volume stats.
+
+        :param aggregate_name: a list of aggregate names for FlexGroup or
+        a single aggregate name for FlexVol
+        :param is_flexgroup: bool informing the type of aggregate_name param
+        """
 
         if 'netapp_raid_type' in self.invalid_extra_specs:
             raid_type = None
             hybrid = None
             disk_types = None
+            node_name = None
+        elif is_flexgroup:
+            raid_type = set()
+            hybrid = set()
+            disk_types = set()
+            node_name = set()
+            for aggr in aggregate_name:
+                aggregate = self.zapi_client.get_aggregate(aggr)
+                node_name.add(aggregate.get('node-name'))
+                raid_type.add(aggregate.get('raid-type'))
+                hybrid.add((six.text_type(
+                    aggregate.get('is-hybrid')).lower()
+                    if 'is-hybrid' in aggregate else None))
+                disks = set(self.zapi_client.get_aggregate_disk_types(aggr))
+                disk_types = disk_types.union(disks)
+            node_name = list(node_name)
+            raid_type = list(raid_type)
+            hybrid = list(hybrid)
+            disk_types = list(disk_types)
         else:
             aggregate = self.zapi_client.get_aggregate(aggregate_name)
+            node_name = aggregate.get('node-name')
             raid_type = aggregate.get('raid-type')
             hybrid = (six.text_type(aggregate.get('is-hybrid')).lower()
                       if 'is-hybrid' in aggregate else None)
@@ -239,6 +255,7 @@ class CapabilitiesLibrary(object):
             'netapp_raid_type': raid_type,
             'netapp_hybrid_aggregate': hybrid,
             'netapp_disk_type': disk_types,
+            'netapp_node_name': node_name,
         }
 
     def get_matching_flexvols_for_extra_specs(self, extra_specs):
@@ -304,3 +321,20 @@ class CapabilitiesLibrary(object):
                     modified_extra_specs[key] = False
 
         return modified_extra_specs
+
+    def is_flexgroup(self, pool_name):
+        for __, flexvol_info in self.ssc.items():
+            if ('netapp_is_flexgroup' in flexvol_info and
+                    'pool_name' in flexvol_info and
+                    flexvol_info['pool_name'] == pool_name):
+                return flexvol_info['netapp_is_flexgroup'] == 'true'
+
+        return False
+
+    def contains_flexgroup_pool(self):
+        for __, flexvol_info in self.ssc.items():
+            if ('netapp_is_flexgroup' in flexvol_info and
+                    flexvol_info['netapp_is_flexgroup'] == 'true'):
+                return True
+
+        return False
