@@ -20,17 +20,16 @@ import json
 import socket
 
 from oslo_config import cfg
-from oslo_log import log
 
 from cinder import exception
 from cinder.i18n import _
-from cinder import utils
 from cinder.volume import configuration
 from cinder.volume import driver
 from cinder.volume.drivers.netapp.dataontap.client import client_cmode
+from cinder.volume.drivers.netapp.dataontap.client import client_cmode_rest
 from cinder.volume.drivers.netapp import options as na_opts
+from cinder.volume import volume_utils
 
-LOG = log.getLogger(__name__)
 CONF = cfg.CONF
 
 
@@ -58,6 +57,7 @@ def get_backend_configuration(backend_name):
     config.append_config_values(na_opts.netapp_san_opts)
     config.append_config_values(na_opts.netapp_replication_opts)
     config.append_config_values(na_opts.netapp_support_opts)
+    config.append_config_values(na_opts.netapp_migration_opts)
 
     return config
 
@@ -66,15 +66,28 @@ def get_client_for_backend(backend_name, vserver_name=None):
     """Get a cDOT API client for a specific backend."""
 
     config = get_backend_configuration(backend_name)
-    client = client_cmode.Client(
-        transport_type=config.netapp_transport_type,
-        username=config.netapp_login,
-        password=config.netapp_password,
-        hostname=config.netapp_server_hostname,
-        port=config.netapp_server_port,
-        vserver=vserver_name or config.netapp_vserver,
-        trace=utils.TRACE_API,
-        api_trace_pattern=config.netapp_api_trace_pattern)
+    if config.netapp_use_legacy_client:
+        client = client_cmode.Client(
+            transport_type=config.netapp_transport_type,
+            username=config.netapp_login,
+            password=config.netapp_password,
+            hostname=config.netapp_server_hostname,
+            port=config.netapp_server_port,
+            vserver=vserver_name or config.netapp_vserver,
+            trace=volume_utils.TRACE_API,
+            api_trace_pattern=config.netapp_api_trace_pattern)
+    else:
+        client = client_cmode_rest.RestClient(
+            transport_type=config.netapp_transport_type,
+            ssl_cert_path=config.netapp_ssl_cert_path,
+            username=config.netapp_login,
+            password=config.netapp_password,
+            hostname=config.netapp_server_hostname,
+            port=config.netapp_server_port,
+            vserver=vserver_name or config.netapp_vserver,
+            trace=volume_utils.TRACE_API,
+            api_trace_pattern=config.netapp_api_trace_pattern,
+            async_rest_timeout=config.netapp_async_rest_timeout)
 
     return client
 
