@@ -14,15 +14,16 @@
 #    under the License.
 
 import errno
-import hashlib
 import os
 import posixpath
 import uuid
 
 from oslo_log import log as logging
+from oslo_utils.secretutils import md5
 from oslo_utils import units
 import six
 
+from cinder.common import constants
 from cinder import context
 from cinder import coordination
 from cinder.i18n import _
@@ -88,7 +89,7 @@ class NexentaNfsDriver(nfs.NfsDriver):
 
     vendor_name = 'Nexenta'
     product_name = 'NexentaStor5'
-    storage_protocol = 'NFS'
+    storage_protocol = constants.NFS
     driver_volume_type = 'nfs'
 
     def __init__(self, *args, **kwargs):
@@ -770,7 +771,7 @@ class NexentaNfsDriver(nfs.NfsDriver):
         share = self._get_volume_share(volume)
         if isinstance(share, six.text_type):
             share = share.encode('utf-8')
-        path = hashlib.md5(share).hexdigest()
+        path = md5(share, usedforsecurity=False).hexdigest()
         return os.path.join(self.mount_point_base, path)
 
     def local_path(self, volume):
