@@ -25,15 +25,16 @@ import base64
 import errno
 import json
 import random
-import requests
 import time
 
 from oslo_log import log as logging
 from oslo_service import loopingcall
 from oslo_utils import units
+import requests
 import six
 from six.moves import http_client
 
+from cinder.common import constants
 from cinder import exception
 from cinder.i18n import _
 from cinder import objects
@@ -41,7 +42,7 @@ from cinder.objects import fields
 from cinder.volume import driver
 from cinder.volume.drivers.prophetstor import options
 from cinder.volume.drivers.san import san
-from cinder.volume import utils as volume_utils
+from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
@@ -85,8 +86,8 @@ class DPLCommand(object):
                   'Accept': 'application/cdmi-container',
                   'x-cdmi-specification-version': '1.0.2'}
         # base64 encode the username and password
-        auth = base64.encodestring('%s:%s'
-                                   % (self.username,
+        auth = base64.encodebytes('%s:%s'
+                                  % (self.username,
                                       self.password)).replace('\n', '')
         header['Authorization'] = 'Basic %s' % auth
 
@@ -612,14 +613,6 @@ class DPLVolume(object):
                              [http_client.OK, http_client.ACCEPTED,
                               http_client.CREATED])
 
-    def get_vg_list(self, vgtype=None):
-        method = 'GET'
-        if vgtype:
-            url = '/%s/?volume_group_type=%s' % (DPL_OBJ_VOLUMEGROUP, vgtype)
-        else:
-            url = '/%s/' % (DPL_OBJ_VOLUMEGROUP)
-        return self._execute(method, url, None, [http_client.OK])
-
     def get_vg(self, groupID):
         method = 'GET'
         url = '/%s/%s/' % (DPL_OBJ_VOLUMEGROUP, groupID)
@@ -668,6 +661,9 @@ class DPLCOMMONDriver(driver.CloneableImageVD,
     # ThirdPartySystems wiki page
     CI_WIKI_NAME = "ProphetStor_CI"
 
+    # TODO(jsbryant) Remove driver in the 'U' release if CI is not fixed.
+    SUPPORTED = False
+
     def __init__(self, *args, **kwargs):
         cert_path = None
         cert_verify = False
@@ -695,6 +691,10 @@ class DPLCOMMONDriver(driver.CloneableImageVD,
                              cert_path=cert_path)
         self._stats = {}
 
+    @staticmethod
+    def get_driver_options():
+        return options.DPL_OPTS
+
     def _convert_size_GB(self, size):
         s = round(float(size) / units.Gi, 2)
         if s > 0:
@@ -1396,16 +1396,6 @@ class DPLCOMMONDriver(driver.CloneableImageVD,
             LOG.info('Flexvisor succeeded to delete snapshot %(id)s.',
                      {'id': snapshot['id']})
 
-    def get_volume_stats(self, refresh=False):
-        """Get volume stats.
-
-        If 'refresh' is True, run update the stats first.
-        """
-        if refresh:
-            self._update_volume_stats()
-
-        return self._stats
-
     def _get_pools(self):
         pools = []
         qpools = []
@@ -1466,7 +1456,7 @@ class DPLCOMMONDriver(driver.CloneableImageVD,
             if ret == 0:
                 data['vendor_name'] = output['metadata']['vendor']
                 data['driver_version'] = output['metadata']['version']
-                data['storage_protocol'] = 'iSCSI'
+                data['storage_protocol'] = constants.ISCSI
                 data['location_info'] = location_info
                 data['consistencygroup_support'] = True
                 data['consistent_group_snapshot_enabled'] = True
