@@ -29,13 +29,13 @@ import six
 
 from cinder import coordination
 from cinder.i18n import _
-from cinder import utils
 import cinder.volume.drivers.stx.exception as stx_exception
+from cinder.volume import volume_utils
 
 LOG = logging.getLogger(__name__)
 
 
-@six.add_metaclass(utils.TraceWrapperMetaclass)
+@six.add_metaclass(volume_utils.TraceWrapperMetaclass)
 class STXClient(object):
     def __init__(self, host, login, password, protocol, ssl_verify):
         self._mgmt_ip_addrs = list(map(str.strip, host.split(',')))
@@ -113,6 +113,10 @@ class STXClient(object):
     def _get_session_key(self):
         """Retrieve a session key from the array."""
 
+        # TODO(alee): This appears to use md5 in a security related
+        # context in providing a session key and hashing a login and
+        # password.  This should likely be replaced by a version that
+        # does not use md5 here.
         self._session_key = None
         hash_ = "%s_%s" % (self._login, self._password)
         if six.PY3:
@@ -238,14 +242,6 @@ class STXClient(object):
     def logout(self):
         pass
 
-    def session_logout(self):
-        url = self._base_url + '/exit'
-        try:
-            requests.get(url, verify=self.ssl_verify, timeout=30)
-            return True
-        except Exception:
-            return False
-
     def is_titanium(self):
         """True for older array firmware."""
         return self._fw_type == 'T'
@@ -334,7 +330,7 @@ class STXClient(object):
             return False
 
     def _get_size(self, size):
-        return int(math.ceil(float(size) * 512 / (units.G)))
+        return int(math.ceil(float(size) * 512 / (units.Gi)))
 
     def backend_stats(self, backend_name, backend_type):
         stats = {'free_capacity_gb': 0,
@@ -699,3 +695,62 @@ class STXClient(object):
             LOG.debug("Array firmware is %s (%s%d)\n",
                       s, self._fw_type, self._fw_rev)
         return s
+
+    def get_volumes(self, filter_type=None):
+        """Get a list of volumes and snapshots"""
+
+        serial_number_to_name = {}
+        result = {}
+
+        # first get volume mappings so we note which volumes are mapped
+        mapping_list = self._request("/show/maps").xpath(
+            "//OBJECT[@name='volume-view']")
+        maps = {}
+        for m in mapping_list:
+            for el in m:
+                if el.attrib['name'] == 'volume-name':
+                    maps[el.text] = 1
+
+        volume_list = self._request("/show/volumes").xpath(
+            "//OBJECT[@name='volume']")
+        for v in volume_list:
+            vol = {}
+            for el in v:
+                key = el.attrib['name']
+                value = el.text
+                vol[key] = value
+
+            name = vol['volume-name']
+            type = vol['volume-type']
+            if type == 'base':
+                type = 'volume'
+            sn = vol['serial-number']
+            wwn = vol['wwn']
+            pool = vol['storage-pool-name']
+            size = int((int(vol['size-numeric']) * 512) / 2**30)
+            mapped = name in maps
+            parent_sn = vol['volume-parent']
+
+            serial_number_to_name[sn] = name
+
+            if filter_type:
+                if type != filter_type:
+                    continue
+
+            result[name] = {
+                'name': name,                 # 32-byte array volume name
+                'type': type,                 # 'volume' or 'snapshot'
+                'size': size,                 # size in GiB (int)
+                'serial': sn,                 # serial number
+                'wwn': wwn,                   # world wide name
+                'pool': pool,                 # storage pool name
+                'mapped': mapped,             # is mapped?
+                'parent_serial': parent_sn,   # parent serial number, or None
+            }
+
+        # Now that we've seen all the volumes, we can map the parent serial
+        # number to a name.
+        for v in result.values():
+            v['parent'] = serial_number_to_name.get(v['parent_serial'], None)
+
+        return result
