@@ -10,8 +10,12 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import annotations
+
 import binascii
 import traceback
+import typing
+from typing import Any, Optional  # noqa: H301
 
 from castellan import key_manager
 import os_brick.initiator.connectors
@@ -21,9 +25,9 @@ from oslo_log import log as logging
 from oslo_utils import excutils
 from oslo_utils import fileutils
 from oslo_utils import netutils
+from oslo_utils import strutils
 from oslo_utils import timeutils
 from oslo_utils import uuidutils
-import six
 import taskflow.engines
 from taskflow.patterns import linear_flow
 from taskflow.types import failure as ft
@@ -40,7 +44,6 @@ from cinder.image import image_utils
 from cinder.message import api as message_api
 from cinder.message import message_field
 from cinder import objects
-from cinder.objects import consistencygroup
 from cinder.objects import fields
 from cinder import utils
 from cinder.volume.flows import common
@@ -78,7 +81,7 @@ class OnFailureRescheduleTask(flow_utils.CinderTask):
     this volume elsewhere.
     """
 
-    def __init__(self, reschedule_context, db, driver, scheduler_rpcapi,
+    def __init__(self, reschedule_context, db, manager, scheduler_rpcapi,
                  do_reschedule):
         requires = ['filter_properties', 'request_spec', 'volume',
                     'context']
@@ -87,7 +90,7 @@ class OnFailureRescheduleTask(flow_utils.CinderTask):
         self.do_reschedule = do_reschedule
         self.scheduler_rpcapi = scheduler_rpcapi
         self.db = db
-        self.driver = driver
+        self.manager = manager
         self.reschedule_context = reschedule_context
         # These exception types will trigger the volume to be set into error
         # status rather than being rescheduled.
@@ -106,6 +109,7 @@ class OnFailureRescheduleTask(flow_utils.CinderTask):
             exception.VolumeNotFound,
             exception.SnapshotNotFound,
             exception.VolumeTypeNotFound,
+            exception.ImageConversionNotAllowed,
             exception.ImageUnacceptable,
             exception.ImageTooBig,
             exception.InvalidSignatureImage,
@@ -178,7 +182,7 @@ class OnFailureRescheduleTask(flow_utils.CinderTask):
         # host field will be erased. Just in case volume was already created at
         # the backend, we attempt to delete it.
         try:
-            self.driver.delete_volume(volume)
+            self.manager.driver_delete_volume(volume)
         except Exception:
             # Most likely the volume weren't created at the backend. We can
             # safely ignore this.
@@ -363,6 +367,9 @@ class NotifyVolumeActionTask(flow_utils.CinderTask):
         self.event_suffix = event_suffix
 
     def execute(self, context, volume):
+        if not self.event_suffix:
+            return
+
         try:
             volume_utils.notify_about_volume_usage(context, volume,
                                                    self.event_suffix,
@@ -384,15 +391,20 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
 
     default_provides = 'volume_spec'
 
-    def __init__(self, manager, db, driver, image_volume_cache=None):
+    def __init__(self, manager, db, driver, image_volume_cache=None) -> None:
         super(CreateVolumeFromSpecTask, self).__init__(addons=[ACTION])
         self.manager = manager
         self.db = db
         self.driver = driver
         self.image_volume_cache = image_volume_cache
-        self.message = message_api.API()
-        self.backup_api = backup_api.API()
-        self.backup_rpcapi = backup_rpcapi.BackupAPI()
+        self._message = None
+
+    @property
+    def message(self):
+        if self._message is None:
+            self._message = message_api.API()
+
+        return self._message
 
     def _handle_bootable_volume_glance_meta(self, context, volume,
                                             **kwargs):
@@ -453,8 +465,11 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                                                'vol_id': volume.id})
             raise exception.MetadataCopyFailure(reason=ex)
 
-    def _create_from_snapshot(self, context, volume, snapshot_id,
-                              **kwargs):
+    def _create_from_snapshot(self,
+                              context: cinder_context.RequestContext,
+                              volume: objects.Volume,
+                              snapshot_id: str,
+                              **kwargs: Any) -> dict:
         snapshot = objects.Snapshot.get_by_id(context, snapshot_id)
         try:
             model_update = self.driver.create_volume_from_snapshot(volume,
@@ -512,7 +527,14 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
 
         LOG.debug('rekey volume %s', volume.name)
 
-        properties = utils.brick_get_connector_properties(False, False)
+        # Rekeying writes very little data (KB), so doing a single pathed
+        # connection is more efficient, but it is less robust, because the
+        # driver could be returning a single path, and it happens to be down
+        # then the attach would fail where the multipathed connection would
+        # have succeeded.
+        properties = volume_utils.brick_get_connector_properties(
+            self.driver.configuration.use_multipath_for_image_xfer,
+            self.driver.configuration.enforce_multipath_for_image_xfer)
         LOG.debug("properties: %s", properties)
         attach_info = None
         model_update = {}
@@ -533,7 +555,7 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
             LOG.debug("attempting attach for rekey, attach_info: %s",
                       attach_info)
 
-            if (isinstance(attach_info['device']['path'], six.string_types)):
+            if (isinstance(attach_info['device']['path'], str)):
                 image_info = image_utils.qemu_img_info(
                     attach_info['device']['path'])
             else:
@@ -543,7 +565,6 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                 raise exception.RekeyNotSupported()
 
             encryption = volume_utils.check_encryption_provider(
-                self.db,
                 volume,
                 context)
 
@@ -552,7 +573,8 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                 volume,
                 encryption)
 
-            if image_info.encrypted == 'yes':
+            # see Bug #1942682 and Change I949f07582a708 for why we do this
+            if strutils.bool_from_string(image_info.encrypted):
                 key_str = source_pass + "\n" + new_pass + "\n"
                 del source_pass
 
@@ -621,7 +643,10 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
 
         return model_update
 
-    def _create_from_source_volume(self, context, volume, source_volid,
+    def _create_from_source_volume(self,
+                                   context: cinder_context.RequestContext,
+                                   volume: objects.Volume,
+                                   source_volid: str,
                                    **kwargs):
         # NOTE(harlowja): if the source volume has disappeared this will be our
         # detection of that since this database call should fail.
@@ -635,6 +660,7 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
             if model_update is None:
                 model_update = {}
             if volume.encryption_key_id is not None:
+                volume.update(model_update)
                 rekey_model_update = self._rekey_volume(context, volume)
                 model_update.update(rekey_model_update)
         finally:
@@ -647,8 +673,11 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                 context, volume, source_volid=srcvol_ref.id)
         return model_update
 
-    def _capture_volume_image_metadata(self, context, volume_id,
-                                       image_id, image_meta):
+    def _capture_volume_image_metadata(self,
+                                       context: cinder_context.RequestContext,
+                                       volume_id: str,
+                                       image_id: str,
+                                       image_meta: dict) -> None:
         volume_metadata = volume_utils.get_volume_image_metadata(
             image_id, image_meta)
         LOG.debug("Creating volume glance metadata for volume %(volume_id)s"
@@ -659,7 +688,7 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                                                    volume_metadata)
 
     @staticmethod
-    def _extract_cinder_ids(urls):
+    def _extract_cinder_ids(urls: list[str]) -> list[str]:
         """Process a list of location URIs from glance
 
         :param urls: list of glance location URIs
@@ -686,7 +715,11 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
 
         return ids
 
-    def _clone_image_volume(self, context, volume, image_location, image_meta):
+    def _clone_image_volume(self,
+                            context: cinder_context.RequestContext,
+                            volume: objects.Volume,
+                            image_location,
+                            image_meta: dict[str, Any]) -> tuple[None, bool]:
         """Create a volume efficiently from an existing image.
 
         Returns a dict of volume properties eg. provider_location,
@@ -743,8 +776,12 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                           {'id': image_volume['id']})
             return None, False
 
-    def _create_from_image_download(self, context, volume, image_location,
-                                    image_meta, image_service):
+    def _create_from_image_download(self,
+                                    context: cinder_context.RequestContext,
+                                    volume: objects.Volume,
+                                    image_location,
+                                    image_meta: dict[str, Any],
+                                    image_service) -> dict:
         # TODO(harlowja): what needs to be rolled back in the clone if this
         # volume create fails?? Likely this should be a subflow or broken
         # out task in the future. That will bring up the question of how
@@ -773,14 +810,20 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                               {'volume_id': volume.id})
         return model_update
 
-    def _create_from_image_cache(self, context, internal_context, volume,
-                                 image_id, image_meta):
+    def _create_from_image_cache(
+            self,
+            context: cinder_context.RequestContext,
+            internal_context: cinder_context.RequestContext,
+            volume: objects.Volume,
+            image_id: str,
+            image_meta: dict[str, Any]) -> tuple[None, bool]:
         """Attempt to create the volume using the image cache.
 
         Best case this will simply clone the existing volume in the cache.
         Worst case the image is out of date and will be evicted. In that case
         a clone will not be created and the image must be downloaded again.
         """
+        assert self.image_volume_cache is not None
         LOG.debug('Attempting to retrieve cache entry for image = '
                   '%(image_id)s on host %(host)s.',
                   {'image_id': image_id, 'host': volume.host})
@@ -817,9 +860,15 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
         return None, False
 
     @coordination.synchronized('{image_id}')
-    def _prepare_image_cache_entry(self, context, volume,
-                                   image_location, image_id,
-                                   image_meta, image_service):
+    def _prepare_image_cache_entry(self,
+                                   context: cinder_context.RequestContext,
+                                   volume: objects.Volume,
+                                   image_location: str,
+                                   image_id: str,
+                                   image_meta: dict[str, Any],
+                                   image_service) -> tuple[Optional[dict],
+                                                           bool]:
+        assert self.image_volume_cache is not None
         internal_context = cinder_context.get_internal_tenant_context()
         if not internal_context:
             return None, False
@@ -852,10 +901,15 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                 update_cache=True)
             return model_update, True
 
-    def _create_from_image_cache_or_download(self, context, volume,
-                                             image_location, image_id,
-                                             image_meta, image_service,
-                                             update_cache=False):
+    def _create_from_image_cache_or_download(
+            self,
+            context: cinder_context.RequestContext,
+            volume: objects.Volume,
+            image_location,
+            image_id: str,
+            image_meta: dict[str, Any],
+            image_service,
+            update_cache: bool = False) -> Optional[dict]:
         # NOTE(e0ne): check for free space in image_conversion_dir before
         # image downloading.
         # NOTE(mnaser): This check *only* happens if the backend is not able
@@ -880,7 +934,7 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
         should_create_cache_entry = False
         cloned = False
         model_update = None
-        if self.image_volume_cache:
+        if self.image_volume_cache is not None:
             internal_context = cinder_context.get_internal_tenant_context()
             if not internal_context:
                 LOG.info('Unable to get Cinder internal context, will '
@@ -972,6 +1026,14 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                             detail=
                             message_field.Detail.SIGNATURE_VERIFICATION_FAILED,
                             exception=err)
+                except exception.ImageConversionNotAllowed:
+                    with excutils.save_and_reraise_exception():
+                        self.message.create(
+                            context,
+                            message_field.Action.COPY_IMAGE_TO_VOLUME,
+                            resource_uuid=volume.id,
+                            detail=
+                            message_field.Detail.IMAGE_FORMAT_UNACCEPTABLE)
 
             if should_create_cache_entry:
                 # Update the newly created volume db entry before we clone it
@@ -998,9 +1060,14 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
         return model_update
 
     @utils.retry(exception.SnapshotLimitReached, retries=1)
-    def _create_from_image(self, context, volume,
-                           image_location, image_id, image_meta,
-                           image_service, **kwargs):
+    def _create_from_image(self,
+                           context: cinder_context.RequestContext,
+                           volume: objects.Volume,
+                           image_location,
+                           image_id: str,
+                           image_meta: dict[str, Any],
+                           image_service,
+                           **kwargs: Any) -> Optional[dict]:
         LOG.debug("Cloning %(volume_id)s from image %(image_id)s "
                   " at location %(image_location)s.",
                   {'volume_id': volume.id,
@@ -1066,9 +1133,14 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
         self._handle_bootable_volume_glance_meta(context, volume,
                                                  image_id=image_id,
                                                  image_meta=image_meta)
+        typing.cast(dict, model_update)
         return model_update
 
-    def _create_from_backup(self, context, volume, backup_id, **kwargs):
+    def _create_from_backup(self,
+                            context: cinder_context.RequestContext,
+                            volume: objects.Volume,
+                            backup_id: str,
+                            **kwargs) -> tuple[dict, bool]:
         LOG.info("Creating volume %(volume_id)s from backup %(backup_id)s.",
                  {'volume_id': volume.id,
                   'backup_id': backup_id})
@@ -1089,7 +1161,8 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
             volume.update(model_update)
             volume.save()
 
-            backup_host = self.backup_api.get_available_backup_service_host(
+            backupapi = backup_api.API()
+            backup_host = backupapi.get_available_backup_service_host(
                 backup.host, backup.availability_zone)
             updates = {'status': fields.BackupStatus.RESTORING,
                        'restore_volume_id': volume.id,
@@ -1097,8 +1170,9 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
             backup.update(updates)
             backup.save()
 
-            self.backup_rpcapi.restore_backup(context, backup.host, backup,
-                                              volume.id)
+            backuprpcapi = backup_rpcapi.BackupAPI()
+            backuprpcapi.restore_backup(context, backup.host, backup,
+                                        volume.id)
             need_update_volume = False
 
         LOG.info("Created volume %(volume_id)s from backup %(backup_id)s "
@@ -1107,7 +1181,10 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                   'backup_id': backup_id})
         return ret, need_update_volume
 
-    def _create_raw_volume(self, context, volume, **kwargs):
+    def _create_raw_volume(self,
+                           context: cinder_context.RequestContext,
+                           volume: objects.Volume,
+                           **kwargs: Any):
         try:
             ret = self.driver.create_volume(volume)
         except Exception as ex:
@@ -1122,7 +1199,10 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
             self._cleanup_cg_in_volume(volume)
         return ret
 
-    def execute(self, context, volume, volume_spec):
+    def execute(self,
+                context: cinder_context.RequestContext,
+                volume: objects.Volume,
+                volume_spec) -> dict:
         volume_spec = dict(volume_spec)
         volume_id = volume_spec.pop('volume_id', None)
         if not volume_id:
@@ -1135,20 +1215,15 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
                       "Volume driver %s not initialized", driver_name)
             raise exception.DriverNotInitialized()
 
-        # NOTE(xyang): Populate consistencygroup_id and consistencygroup
-        # fields before passing to the driver. This is to support backward
-        # compatibility of consistencygroup.
-        if volume.group_id:
-            volume.consistencygroup_id = volume.group_id
-            cg = consistencygroup.ConsistencyGroup()
-            cg.from_group(volume.group)
-            volume.consistencygroup = cg
+        # For backward compatibilty
+        volume.populate_consistencygroup()
 
         create_type = volume_spec.pop('type', None)
         LOG.info("Volume %(volume_id)s: being created as %(create_type)s "
                  "with specification: %(volume_spec)s",
                  {'volume_spec': volume_spec, 'volume_id': volume_id,
                   'create_type': create_type})
+        model_update: dict
         if create_type == 'raw':
             model_update = self._create_raw_volume(
                 context, volume, **volume_spec)
@@ -1185,7 +1260,7 @@ class CreateVolumeFromSpecTask(flow_utils.CinderTask):
             raise
         return volume_spec
 
-    def _cleanup_cg_in_volume(self, volume):
+    def _cleanup_cg_in_volume(self, volume: objects.Volume) -> None:
         # NOTE(xyang): Cannot have both group_id and consistencygroup_id.
         # consistencygroup_id needs to be removed to avoid DB reference
         # error because there isn't an entry in the consistencygroups table.
@@ -1287,19 +1362,24 @@ def get_flow(context, manager, db, driver, scheduler_rpcapi, host, volume,
     # status when reverting the flow. Meanwhile, no need to revert process of
     # ExtractVolumeRefTask.
     do_reschedule = allow_reschedule and request_spec and retry
-    volume_flow.add(OnFailureRescheduleTask(reschedule_context, db, driver,
+    volume_flow.add(OnFailureRescheduleTask(reschedule_context, db, manager,
                                             scheduler_rpcapi, do_reschedule))
 
     LOG.debug("Volume reschedule parameters: %(allow)s "
               "retry: %(retry)s", {'allow': allow_reschedule, 'retry': retry})
 
-    volume_flow.add(ExtractVolumeSpecTask(db),
-                    NotifyVolumeActionTask(db, "create.start"),
-                    CreateVolumeFromSpecTask(manager,
+    volume_flow.add(ExtractVolumeSpecTask(db))
+    # Temporary volumes created during migration should not be notified
+    end_notify_suffix = None
+    # TODO: (Y release) replace check with: if volume.use_quota:
+    if volume.use_quota or not volume.is_migration_target():
+        volume_flow.add(NotifyVolumeActionTask(db, 'create.start'))
+        end_notify_suffix = 'create.end'
+    volume_flow.add(CreateVolumeFromSpecTask(manager,
                                              db,
                                              driver,
                                              image_volume_cache),
-                    CreateVolumeOnFinishTask(db, "create.end"))
+                    CreateVolumeOnFinishTask(db, end_notify_suffix))
 
     # Now load (but do not run) the flow using the provided initial data.
     return taskflow.engines.load(volume_flow, store=create_what)
