@@ -12,12 +12,16 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+from __future__ import annotations
+
+from typing import Optional, Union  # noqa: H301
 
 from cinder.common import constants
+from cinder import context
 from cinder import objects
 from cinder import quota
 from cinder import rpc
-from cinder.volume import utils
+from cinder.volume import volume_utils
 
 
 QUOTAS = quota.QUOTAS
@@ -134,16 +138,21 @@ class VolumeAPI(rpc.RPCAPI):
                failover_replication, and list_replication_targets.
         3.15 - Add revert_to_snapshot method
         3.16 - Add no_snapshots to accept_transfer method
+        3.17 - Make get_backup_device a cast (async)
+        3.18 - Add reimage method
     """
 
-    RPC_API_VERSION = '3.16'
+    RPC_API_VERSION = '3.18'
     RPC_DEFAULT_VERSION = '3.0'
     TOPIC = constants.VOLUME_TOPIC
     BINARY = constants.VOLUME_BINARY
 
-    def _get_cctxt(self, host=None, version=None, **kwargs):
+    def _get_cctxt(self,
+                   host: Optional[str] = None,
+                   version: Optional[Union[str, tuple[str, ...]]] = None,
+                   **kwargs) -> rpc.RPCAPI:
         if host:
-            server = utils.extract_host(host)
+            server = volume_utils.extract_host(host)
 
             # TODO(dulek): If we're pinned before 3.6, we should send stuff the
             # old way - addressing server=host@backend, topic=cinder-volume.
@@ -153,13 +162,17 @@ class VolumeAPI(rpc.RPCAPI):
             if self.client.can_send_version('3.6'):
                 kwargs['topic'] = '%(topic)s.%(host)s' % {'topic': self.TOPIC,
                                                           'host': server}
-                server = utils.extract_host(server, 'host')
+                server = volume_utils.extract_host(server, 'host')
             kwargs['server'] = server
 
         return super(VolumeAPI, self)._get_cctxt(version=version, **kwargs)
 
-    def create_volume(self, ctxt, volume, request_spec, filter_properties,
-                      allow_reschedule=True):
+    def create_volume(self,
+                      ctxt: context.RequestContext,
+                      volume: 'objects.Volume',
+                      request_spec: Optional[dict],
+                      filter_properties: Optional[dict],
+                      allow_reschedule: bool = True) -> None:
         cctxt = self._get_cctxt(volume.service_topic_queue)
         cctxt.cast(ctxt, 'create_volume',
                    request_spec=request_spec,
@@ -174,7 +187,11 @@ class VolumeAPI(rpc.RPCAPI):
         cctxt.cast(ctxt, 'revert_to_snapshot', volume=volume,
                    snapshot=snapshot)
 
-    def delete_volume(self, ctxt, volume, unmanage_only=False, cascade=False):
+    def delete_volume(self,
+                      ctxt: context.RequestContext,
+                      volume: 'objects.Volume',
+                      unmanage_only: bool = False,
+                      cascade: bool = False) -> None:
         volume.create_worker()
         cctxt = self._get_cctxt(volume.service_topic_queue)
         msg_args = {
@@ -184,7 +201,10 @@ class VolumeAPI(rpc.RPCAPI):
 
         cctxt.cast(ctxt, 'delete_volume', **msg_args)
 
-    def create_snapshot(self, ctxt, volume, snapshot):
+    def create_snapshot(self,
+                        ctxt: context.RequestContext,
+                        volume: 'objects.Volume',
+                        snapshot: 'objects.Snapshot') -> None:
         snapshot.create_worker()
         cctxt = self._get_cctxt(volume.service_topic_queue)
         cctxt.cast(ctxt, 'create_snapshot', snapshot=snapshot)
@@ -231,9 +251,12 @@ class VolumeAPI(rpc.RPCAPI):
         return cctxt.call(ctxt, 'terminate_connection', volume_id=volume['id'],
                           connector=connector, force=force)
 
-    def remove_export(self, ctxt, volume):
+    def remove_export(self, ctxt, volume, sync=False):
         cctxt = self._get_cctxt(volume.service_topic_queue)
-        cctxt.cast(ctxt, 'remove_export', volume_id=volume['id'])
+        if sync:
+            cctxt.call(ctxt, 'remove_export', volume_id=volume.id)
+        else:
+            cctxt.cast(ctxt, 'remove_export', volume_id=volume.id)
 
     def publish_service_capabilities(self, ctxt):
         cctxt = self._get_cctxt(fanout=True)
@@ -341,8 +364,13 @@ class VolumeAPI(rpc.RPCAPI):
         return cctxt.call(ctxt, 'get_capabilities', discover=discover)
 
     def get_backup_device(self, ctxt, backup, volume):
-        cctxt = self._get_cctxt(volume.service_topic_queue, ('3.2', '3.0'))
-        if cctxt.can_send_version('3.2'):
+        cctxt = self._get_cctxt(volume.service_topic_queue, ('3.17', '3.2',
+                                                             '3.0'))
+        if cctxt.can_send_version('3.17'):
+            cctxt.cast(ctxt, 'get_backup_device', backup=backup,
+                       want_objects=True, async_call=True)
+            backup_obj = None
+        elif cctxt.can_send_version('3.2'):
             backup_obj = cctxt.call(ctxt, 'get_backup_device', backup=backup,
                                     want_objects=True)
         else:
@@ -390,7 +418,9 @@ class VolumeAPI(rpc.RPCAPI):
 
         return cctxt.call(ctxt, 'get_manageable_snapshots', **msg_args)
 
-    def create_group(self, ctxt, group):
+    def create_group(self,
+                     ctxt: context.RequestContext,
+                     group: 'objects.Group') -> None:
         cctxt = self._get_cctxt(group.service_topic_queue)
         cctxt.cast(ctxt, 'create_group', group=group)
 
@@ -435,9 +465,12 @@ class VolumeAPI(rpc.RPCAPI):
                           connector=connector, force=force)
 
     @rpc.assert_min_rpc_version('3.13')
-    def remove_export_snapshot(self, ctxt, snapshot):
+    def remove_export_snapshot(self, ctxt, snapshot, sync=False):
         cctxt = self._get_cctxt(snapshot.service_topic_queue, version='3.13')
-        cctxt.cast(ctxt, 'remove_export_snapshot', snapshot_id=snapshot.id)
+        if sync:
+            cctxt.call(ctxt, 'remove_export_snapshot', snapshot_id=snapshot.id)
+        else:
+            cctxt.cast(ctxt, 'remove_export_snapshot', snapshot_id=snapshot.id)
 
     @rpc.assert_min_rpc_version('3.9')
     def attachment_update(self, ctxt, vref, connector, attachment_id):
@@ -503,3 +536,8 @@ class VolumeAPI(rpc.RPCAPI):
         cctxt = self._get_cctxt(group.service_topic_queue, version='3.14')
         return cctxt.call(ctxt, 'list_replication_targets',
                           group=group)
+
+    @rpc.assert_min_rpc_version('3.18')
+    def reimage(self, ctxt, volume, image_meta):
+        cctxt = self._get_cctxt(volume.service_topic_queue, version='3.18')
+        cctxt.cast(ctxt, 'reimage', volume=volume, image_meta=image_meta)
